.TH SYNDOC LOCAL "21.5.1997"
.SH NAME
syndoc.tcl - Syntax-directed document prosessing system
.SH SYNOPSIS
.br
.B syndoc.tcl
.SH DESCRIPTION
SYNDOC is a prototype of a syntax-directed document prosessing
system. It uses grammars and syntax-directed translation schemas to
implement a declarative system for structured documents. The user
defines the structure and the layout of a document using extended
context-free grammars.

SYNDOC has two levels of usage. Expert users define a document
structure by a grammar. This part of job is done once for every
document type. The expert user can define different kinds of
structure grammars for different kinds of documents. Based on a
structure grammar, the user creates an input grammar. For formatting
document for printing, the user defines layout grammars for different
purposes, for example producing ASCII, LaTeX, SGML, or HTML files. 
The user can also search a set of documents according to conditions 
which integrate structure and content constraints. The structure of a 
filter defining the user's interest is made using the structure grammar. 
Existing documents with the particular structure can be transformed to
documents with another structure. For transformation, user defines a
new structure grammar. The system generates a tranformation program
based on two structure grammars.

The end users write and process their documents guided by these grammars.
At first, the end user selects some input grammar, which defines the
structure of the document and advice information for the input. 
The end user writes only the content of the document. System directs 
the input of the content by proposing the valid structure elements.
Thereafter, the user can preview or print documents using  various 
layouts, search required documents for a given grammar, and modify
a set of documents according to a new structure grammar.
More information of the system see references at the end of this manual.

STARTING SYNDOC SYSTEM of this manual tells, how to get started with
SYNDOC. In STRUCTURE OF DOCUMENTS IN SYNDOC there is a short
description about the structure of documents and forms of input and
layout grammars in SYNDOC.  It  is told for the expert user about
defining grammars of documents. PROCESSING DOCUMENTS describes 
for the end user how to make structured documents with SYNDOC, 
how to format existing documents, search a set of documents matching
given conditions and transform documents to get a new structure.  
COMMANDS IN MENU describes shortly SYNDOC's functions in the same 
order as they are in menus. DIAGNOSTICS explains error  and other
messages given by SYNDOC.



.SH OPTIONS
-
.br
.ne7
.SH "STARTING SYNDOC SYSTEM"
.SS "Installation"
SYNDOC system is implemented in BinProlog 5.75, Tcl 7.4b3 and Tk 4.0b3
( or Tcl 7.5 and Tk 4.1) toolkit script language and requires these or 
newer versions for running. The system has been tested in Sun workstations 
within SunOS 4.1 and SunOS 5.4 operating systems and in Linux. See more 
information about system requirements with BinProlog in BinProlog 5.75 
User Guide written by Paul Tarau, Department of Computer Science, 
University of Moncton, Canada, April 1996 (ftp clement.info.umoncton.ca).  

Program files of SYNDOC system are packed in the file syndocX.Y.Z.tar.gz
(X, Y and Z inform the version number). Unpack packed file for example 
in /usr/local/src directory. If there does not exist subdirectory syndocX.Y.Z/,
it is created when files are unpacked.

First, in some syndoc files (syndoc.tcl, syndocserver.tcl, 
syndocmakeindex.tcl, syndocmakematch.tcl and
syndocstart.pl) some paths have to 
be changed to correspond your working environment by editing 
Configure file in syndocX.Y.Z directory  or by changing the paths manually
(see details in INSTALLATION file).  Change the paths for 
variables mentioned in the beginning of the file  and then execute it.
Configure file uses gawk_script files which should be in the same
directory as Configure file.

.SS "Starting and exiting SYNDOC"
.PP
If the path to the syndoc source directory of the SYNDOC program files is set, 
type the command

	syndoc.tcl

to start SYNDOC system. To exit your SYNDOC-process invoke the Exit
button from the Exit menu.

To make your use of SYNDOC easy (and for example to try examples), 
you have to configure your system.
See Control menu of COMMANDS IN MENU  chapter for more information 
about this.

.SH STRUCTURE OF DOCUMENTS IN SYNDOC
.PP
Structured documents have a predefined, fixed structure. In SYNDOC
the hierarchical structure of a document is defined by an extended
context-free grammar. The grammar consists of rules. The left side of
the rule is a nonterminal. The right side of the rule contains
nonterminals and metasymbols |, *, + and brackets []. The metasymbol
| separates alternative nonterminals, * defines a list with zero or
more elements, + a list of one or more elements and brackets define
an optional nonterminal. The expert user writes grammars by
selecting Grammars menu from the main menu of SYNDOC.
.\"-----------------------------------------------------------------
.SS "Structure grammar"
First of all, the user or expert writes a new structure grammar, 
which is a kind of skeleton for the input and layout grammars as well 
as for filters to  define conditions for the search of documents.
The matesymbol for alternative is not used in the current version,
instead for a nonterminal several rules may exist. With the use of 
a single skeleton grammar the system guarantees the compatibility 
of the input and layout grammars and filters. A simple structure 
grammar for the article type of document is represented with the 
use of SYNDOC's notation as follows.

.na
.nf
	article. 
	text. 
	article -> authors [date] title content.
	authors -> (author)+. 
	author -> text. 
	date -> text. 
	title -> text. 
	content -> abstract (section)+. 
	abstract -> text.
	section -> heading (paragraph)+. 
	heading -> text. 
	paragraph -> textpara. 
	paragraph -> itemizelist. 
	itemizelist -> (itemize)+. 
	textpara -> text. 
	itemize -> text.
.fi
.ad

The first line of the grammar, in this case article, describes the
type of the document. The second line (text) is the name of the
special atomic nonterminal. It defines the content type, in this example
an unparsed string of characters. All other lines are rules of the grammar.
The left side of the rule with the nonterminal is separeted from the
right side of the rule by using an arrow ->. There is a period in the
end of every line. It is useful to select the names of the
nontermials to be intuitive enough for the end user. According to
this structure grammar, the article is composed of authors, an
optional date, a title and the content. The content contains an 
abstract and at least one section. In a section there can be one or 
more text paragraphs and itemizelists.
.\"-----------------------------------------------------------------
.SS "Input grammar"
Before writing a document user needs an input grammar based on the
structure grammar. Terminals to be used as advice information for the 
user in the input phase are added between nonterminals.  The system 
writes the name of the structure grammar file to the beginning of 
the new input grammar file and copies lines of the structure grammar 
file after that. Then the system opens an editor with the new file 
for modifications.

If the name of the structure grammar file above is article.gram, the
system opens editor with the new input grammar file containing
the following lines:

.na
.nf
	'article.gram'.
	article.
	text.
	article -> authors [date] title content.
	...
.fi
.ad

This version of SYNDOC does not use the terminals of the input
grammar for the interaction between the user and the system, thus the rules
of the input grammar should be identical with rules of the structure 
grammar. Instead, the version uses nonterminal names for the interaction.
.\"-----------------------------------------------------------------
.SS "Layout grammar"
A layout grammar is needed for formatting a set of documents. In the same 
way as described for the input grammar, system creates a new file 
based on the structure grammar file and opens an editor with
the new file. The user adds layout definitions as terminals between
nonterminals. There are some examples below about layout grammars for
different purposes.

A layout grammar for writing a document into an ASCII file in a form
with empty lines between elements:

.ne 7
.na
.nf
	'article.gram'.
	article.
	text.
	article -> authors [nl nl date] nl nl title nl nl 
                   content nl nl.
	authors -> (author)+{nl}.
	author -> text.
	date -> text.
	title -> text.
	content -> abstract nl nl (section)+{nl, nl}.
	abstract -> text.
	section -> heading nl nl (paragraph)+{nl, nl}.
	heading -> text.
	paragraph -> textpara.
	paragraph -> itemizelist.
	itemizelist -> (itemize)+{nl, nl}.
	textpara -> text.
	itemize -> '- ' text.
.fi
.ad

A layout grammar for the LaTeX formatter:

.na
.nf
	'article.gram'.
	article.
	text.
	article -> '\\documentstyle{article}' nl  
                   '\\author{' authors '}' 
		   nl  ['\\date{' date '}' nl] 
                   '\\title{' title '}' nl  
        	   '\\begin{document}' nl content 
                   '\\end{document}' nl 
		   '\\bye'.
	authors -> (author)+{nl, nl}.
	author -> '{\\bf ' text '}'.
	date -> '{\\bf ' text '}'.
	title -> '{\\bf ' text '}'.
	content ->  '\\maketitle' nl abstract nl nl 
		    (section)+{nl, nl} nl.
	abstract -> nl '{\\bf ' text '}'.
	section -> heading (paragraph)+{nl, nl}.
	heading -> '\\section{' text '}'.
	paragraph -> nl nl textpara.
	paragraph -> nl nl '\\begin{itemize}' nl itemizelist nl 
		     '\\end{itemize}'.
	itemizelist -> (itemize)+{nl}.
	textpara -> text.
	itemize -> '\\item ' text.
.fi
.ad

A layout grammar for making an HTML document:

.na
.nf
	'article.gram'.
	article.
	text.
	article -> '<HEAD><TITLE>Document</TITLE></HEAD>' nl 
		   '<BODY>' nl authors  nl nl [date] nl nl  
		   title nl nl content nl '</BODY>'.
	authors -> (author)+{nl}.
	author -> '<P>' text.
	date -> '<P>' text.
	title -> '<H1>' text '</H1>'.
	content -> abstract nl nl (section)+{nl, nl}.
	abstract -> '<P>' text.
	section -> '<H2>' heading '</H2>' (paragraph)+{nl}.
	heading -> text.
	paragraph -> textpara.
	paragraph -> '<UL>' itemizelist '</UL>'.
	itemizelist -> (itemize)+{nl}.
	textpara -> '<P>' text.
	itemize -> '<LI>' text.
.fi
.ad
.\"-----------------------------------------------------------------
.SH PROCESSING DOCUMENTS
.SS Inputting document
.PP
The end user can write a new documents or edit an old one by selecting 
the Input menu from the main menu of SYNDOC. The user selects a proper
input grammar for the structure definition of the document. The
system directs the input of the content by prompting nonterminals
according to the selected input grammar.

To the end user of the SYNDOC system, the derivation tree of the
current document is shown on the left side of the document window.
Nonterminals of the rules are labels of nodes of the tree. Some 
nonterminals on the right side of the rule are atomic nonterminals 
containing typically text, that is typed by the end user in the 
editor window. The contents of the document is shown in the right 
side of the document window. An unfinnished documents on the 
screen of SYNDOC is as follows. The arrow shows the active 
nonterminal.

.ne 7
.na
.nf
article
   authors 
      author                 Matti Mattinen
      author+ 
   [date]                    31.7.1996
 ->title                     
   content
.fi
.ad

The user can move in the tree structure by using the buttons in the
bottom of the document window of SYNDOC system (See Input functions 
in COMMANDS IN MENU Chapter). It is possible to move to a next or 
a previous node in depth first order, to a next or a previous sister, 
to the parent or to the first child of the current node. 

The user can also expand nodes and delete children of the current node.
When a node is expanded, system adds the nodes labelled by nonterminals 
on the right side of the grammar rule to the tree as subtrees of the 
node labelled by the nonterminal in the left side of the rule. If there
are many rules in the input grammar for the expanded node, system lists 
all of them and the user selects an appropriate rule. If the expanded 
node is atomic, the system opens the editor window (See Editor functions 
in COMMANDS IN MENU Chapter) and the user can write a content of the 
structure element.

If the current node is for a list element (there is + or * in the end of
the node label in the current node or some of next nodes with the same 
label), the functions for adding nodes to the list and deleting nodes 
from the list are available. Otherwise, these functions are disabled. 

The user can save document without stopping input. Cancel function 
destroys changes the user has made after last save. With exit function 
user can save the document and exit the input phase.
.\"-----------------------------------------------------------------
.SS "Layouting document"
After the document has been written, the user usually wants to 
format, preview and print it. By selecting the Output menu from the main 
menu of SYNDOC the user can start these functions. Also the user can
only create the source code for a formatter. The user selects 
a proper layout grammar according to what kind of document he wants 
to create. There can be, for example, HTML-, SGML-, TeX- (for LaTeX 
formatter) and ASCII text-types of layout grammars for the end user 
to select. In the first selection, the user can only add formatting 
commands to the document for a formatter. The formatting method 
determines the extension  of the file name for formatted  document.
Extension is taken automatically from the name of layout grammar. Second, 
before the user previews a document the system creates the formatted 
document or  the source code for a formatter program using the layout 
grammar and saves it to the file.  The preview program to show the 
documents in their final  forms is defined by the user. In the current 
version of SYNDOC the  user can print only the source code for a 
formatter  procuded by the program, not the final form shown by the 
previewer program.
.\"-----------------------------------------------------------------
.SS "Retrieving documents"
Using the document retrieval, the user finds a set of documents
which match the given conditions. In the retrieval based on filtering,
documents are considered as a hierarchy of parts which correspond 
instances of nonterminals of the structure grammar. Nonterminals are
called text types. The conditions to be true for parts for a text 
type are defined in a filter created from the same structure grammar 
as the searching documents.  A filter is either a simple filter 
containing one structure definition  or a compound filter containing 
a sequence of simple filters. A simple filter is a tree  whose nodes 
are labelled  by names of text types of the grammar and  associated 
with constraints. A simple filter defines the structure of documents 
at the required level of details. Constraints are associated with 
text types of the filter. A compound filter is a sequence of simple filters 
so that the last simple filter defines parts which must be true
for a filter. A compound filter the user will need when she or he
wants to define disjunctive conditions for parts of same text type, 
if the user wants to spilt a long filter and express its condition 
using several small filters, or if the user needs to compare the values 
of parts. For each filter, retrieved parts are 
defined by annotations and annotations of previous filters are 
used to bind filters in a compound filter.   

The retrieval by filtering consists of three phases: indexing of
documents, defining filters, and filtering. The filtering  contains the 
possibility to define the output form after founding a set of  documents.
To execute these  functions, the user has to select a structure grammar 
because the system applies all the functions only to documents 
for a selected grammar.

The user may index single documents or a set of documents by selecting
Indexing button in Document Retrieval submenu. The user defines the name of
a document file or selects all document files (denoted by *.doc as a file
name) in a given directory, and the control file for indexing. 
The control file contains words which are not indexed and the user can 
use or edit the default file or create her or his own file using a 
text editor. For each document file an indexed document file is 
generated separately.  The index is an inverted list which indicates 
occurrences of parts and words of a document and depicts the hierarchic 
structure of parts.

The end user can write a new filter or edit an old one by selecting from the
Defining Filters button in the Document Retrieval submenu. 
A filter as a tree is generated according to the selected structure 
grammar which directs the filter generation. The user selects a name for 
the filter file and a text type of the grammar for the type of the filter. 
The type is the label of the root of the filter tree.
 
On the screen, a filter is represented as a table with five columns.
The first column (RESULT) is for annotations of text types which define 
parts for which the filter must be true. An annotation is a similar name as
a text type of the grammar, however none of them. The second column 
(TYPE) shows the hierarchical structure, the root defines the type of 
the filter. Columns three, four and five (CONTAINS, POSITION and QUANTITY)
are for constraints to define the conditions for the text type on the
same line. A containment constraint defines which parts or characters
strings must be contained into a part for the text type. A containment
condition is a text type, = and a text type (not implemented when
filter is compared to documents), or a character string 
(enclosed in double parenthesis), or a Boolean combination of them 
(& for "AND", | for "OR" and ! for "NOT"). An annotation of a previous 
simple filters many be  used in containment constraints to bind 
simple filters of a compound  filter. A position constraint, associated 
only to an iterative text  type of the filter, defines for which of 
sequential parts conditions  must be true. A position constraint is of 
the form n or n..m where  n and m are positive or negative integers. 
One number n means parts  from the beginning or from the end of a 
sequence. The form n..m means  the parts in positions from n to m. 
A quantitative constraint associated  also only to an iterative text 
type is one of the following:  ALL, n, >n, >=n, <n, or <=n where 
n is positive integer.  It defines  how many sequential parts having 
given conditions must exist.  

The following filter specifies titles of  articles whose the first or 
second author is Mattinen and which contain sections, whose heading 
contains the  word "filter" and which  have more than 5 paragraphs, 
and the content having itemize-parts. The first simple filter annotates
only  sections which tell about filters  and have more than 5 paragaphs 
using the transient text type "filt_sec". The second filter checks 
only articles which contain these annotated parts and compares if other 
contraints  are true.
.na
.nf
RESULT     TYPE            CONTAINS     POSITION   QUANTITY
        *
filt_sec...section
              heading......"filter"
              paragraph............................>5

        *
           article.........filt_sec
             authors 
               author+...."Mattinen"....1..2 
             [date]                   
art_title..->title                     
             content......itemize 
.fi
.ad

The user can move in the tree structure of the filter by using the 
buttons at the bottom of the filter generation window of SYNDOC system 
(See Retrieval functions in COMMANDS IN MENU chapter). It is possible 
to move to a next or a previous node in depth first order, to the parent 
or to the first child, or to the next or the previous sister of the 
current node. 

The user can zoom nodes and unzoom children of the current node. 
When a node is zoomed, the system adds the nodes labelled by text 
types on the right side of the grammar rule to the tree as subtrees of 
the node labelled  by  the text type in the left side of the rule. 
If there are many  rules in the structure grammar for the zoomed node, 
system lists them all, each preceded by a | character. Atomic nodes are 
not zoomed. If the current node is an iterative text type (there is + 
or * in the end of the node label), it is listed  with its metasymbol 
to represent a sequence of parts with the same name in a document. 
The user can add  an existing simple filter to a compound filter or 
start to generate a new simple filter from its root type as part of 
a compound filter. A simple filter as part of a compound  filter 
may  also be deleted (except the last one). These two functions are 
enabled only when the arrow on the screen points at the asterics character 
which separates simple filters in a compound filter. The filter below 
the asterics is deleted or a new filter comes below the asterics. 
The user can also define constraints  and add annotations to the current 
node of a filter.

The user can save the filter without stopping the generation. Cancel 
function destroys changes which the user has made after the last saving. 
With exit function user can save the filter and exit the filter 
generation phase.

The user initiates the comparing of a filter in a file to a single document
or all the documents in the working directory using the Filtering button
from the Document Retrieval submenu. The user selects a filter and either
one indexed document or all the  indexed documents in the directory 
(denoted by *.doc.index). After the system has compared the
filter to documents, it displays  the number of retrieved
documents. The user can either preview them one after another or exit 
without the output.  The default form to  show selected documents is  
as Prolog terms.  However, the user can define other forms by selecting 
Define Output Form item in the menu of the message window. If the user 
wants to open  an input editor and edit the document the system asks 
the user to define an input grammar. To generate the formatted layout 
the system asks the user to  define the extension of the output file 
(should be a previewer).  If the user wants to modify the structures 
of documents before formatting the system asks the user to define 
a transformation program and the  layout grammar for the new structure 
and the extension of the output file. 
.\"-----------------------------------------------------------------
.SS "Transformations"
Transformations for existing documents are needed when the user
changes the structure grammar for a group of existing structured
documents. The user can, for example, insert new elements, rename,
delete or reorder existing elements and change an optional element
into a required one and a single element into a list of similar
elements. For a transformation, the user has to create a new
structure grammar, which corresponds a new structure of the
documents. 

The system forms a transformation grammar from an original and this 
new structure grammars. The system can derive all modifications, 
except renaming an element, from old and new grammar rules. If the 
system cannot find any nonterminal or rule from the new grammar 
corresponding to the old nonterminal or rule, it lists alternatives 
and user selects a proper one.

The transformation part of the system is under development. Currently
parts of two methods (syntax-directed translation schema (SDTS) and
tree transducer) which may execute restricted modifications has been
implemented. The transformtion grammars can be generated automatically
only to SDTS method, for tree transducer the expert user have to 
make them. At the beginning of the transformation grammar, file names
of a pair of structure grammars, the type of the documents and 
the content nonterminals is mentioned. A tranformation grammar file 
front_SPE_sdts.trgram containing an SDTS for structure grammars
front.gram and front_SPE.gram starts
.na
.nf
       'front.gram'.
       'front_SPE.gram'.
       front.
       string.
       front ; front -> title authors [footnote] ; title 
                        authors.
       title ; title -> string ; string.
       ....
.fi
.ad
A transformation grammar file front_EPODD_tr.gram containing a tree
transducer for documents between structure grammars front.gram and
front_EPODD.gram starts
.na
.nf
       'front.gram'.
       'front_EPODD.gram'.
       front.
       string.
       [p1,front,p2,authors,p3,names,
        p4,affiliations_addresses,p5,departments, 
        p6,universities,p7,postals,p8,emails,
        p9,title,p10,corr_name,p11,name,
        p12,corr_dept, p13,dept,p14,corr_university,
        p15,university,p16,corr_postal,p17,postal,
        p18,corr_email,p19,email,p20,string].
       [p1]. 
       p1(front(X1,X2,[X3])) -> front(p9(X1),
                                authors(p3(X2),p4(X2))).
       p9(title(X1)) -> title(p20(X1)).
       ....
.fi
.ad

.\"-----------------------------------------------------------------
.SH COMMANDS IN MENU
The window-based user interface of SYNDOC consists of menus and
question, document, filter, editor, and message windows. Menus are used 
to select operations. By using question windows, user gives information to the
system, for example about names of files or selected grammar rules.
Document window is for creating and editing document structures and
filter window for the same actions conserning filters. Through an 
editor window the content of a documents in written. The system informs 
the user through message windows.

In question windows
.in +6n
Go/Apply button starts function.

Done button closes the window. Entry fields that were modified 
during the user visiting in a window, are remembered.

Cancel button cancels the operation. All the work that was done 
in a window will be lost.

Help button gives more information about Syndoc by displaying
the manual pages.

Select buttons is used to select a file or another item to the
corresponding entry field from opened list window. 
.in -6n

In list windows 
.in +6n
Double-Click selects an active file and closes the window.

Show button displays contents of an active file on the
terminal.

Close button closes the list window without selecting file.
.in -6n

In message windows
.in +6n
OK allows the system to continue the processing.
.in -6n

.SS "Grammars menu"

.in +6n
.IP "  Create/Edit Structure Grammar"
Write a new file name or select an existing file from list with
Select-button.

.IP "  Input Grammars"
Select either  "Create Input Grammar" or "Edit Input Grammar" item.
.PP
.TP 16
.in +6n
Create Input Grammar
Select an existing structure grammar file with Select-button or write
a file name. Write a new input grammar file name. Existing input
grammar file with the same name will be destroyed.
.PP
.TP 16
.in +6n
Edit Input Grammar
Select an existing input grammar file with Select-button or write a
name of an existing file. 
.PP
.IP "  Layout Grammars"
Select either  "Create Layout Grammar" or "Edit Layout Grammar" item.
.PP
.TP 16
.in +6n
Create Layout Grammar
Select an existing structure grammar file with Select-button or write
a file name. Write a new layout grammar file name. An existing layout
grammar file with the same name will be destroyed.

.PP
.TP 16
.in +6n
Edit Layout Grammar
Select an existing layout grammar file with Select-button or write a
name of an existing file.
.in -8n
.\"-----------------------------------------------------------------
.SS Input menu
.in +8n
.IP "  Create New Document"
Give an existing input grammar file name and a name of a new document
file. An existing document file with the same name will be
overwritten.
.IP "  Open Document"
Give an existing input grammar file name and an existing document
file name.
.IP "  Input functions"
.PP
.TP 16
.in +6n
Arrow up
Moves to the previous node (in depth first order)	
.TP
.in +6n
Arrow down
Moves to the next node (in depth first order)
.TP
.in +6n
Parent
Moves to the parent node
.TP
.in +6n
Child
Moves to the first child
.TP
.in +6n
Previous
Moves to the previous sister
.TP
.in +6n
Next
Moves to the next sister
.TP
.in +6n
Expand
Expands tree at a node with the chosen rule
.TP
.in +6n
Unexpand
Deletes children of the current node (i.e cancels the application of
a rule)
.TP
.in +6n
List Add
Adds a new node in a list expressed as <list-element>+ or
<list-element>*. The node is added as a previous node of the current 
node
.TP
.in +6n
List Delete
Deletes the current node from the list expressed as <list-element>+
or <list-element>*
.TP
.in +6n
Save
Saves document without exit
.TP
.in +6n
Help
Help by dislaying manual pages
.TP
.in +6n
Cancel
Cancels editing without saving changes
.TP
.in +6n
Exit
Exits with save
.IP "  Editor functions"
.TP
.in +6n
Clear
Clears the text from window
.TP
.in +6n
Done
Saves text and exits the function
.TP
.in +6n
Cancel
Cancels the function without saving text
.ne 8
.\"-----------------------------------------------------------------
.SS Output menu
.IP "  Format Document"
Select an existing layout grammar file and document file with Select
button or write file names.  The name of the output file containing
the source code for a formatter is generated  by the system. The 
extension of the file is a character string which informs the method 
in the name  of the layout  grammar file (given between _ and . 
charactes before  the layoutgrammar extension).  An existing output 
file with the same  name will be destroyed. 
.IP "  Preview Document"
Select an existing layout grammar file and document file with Select
button or write file names. Check the name of the preview program and
edit if needed. The name of the output file is generated by the system.
The extension is a character string which informs the method in the name 
of the layout  grammar file (given between _ and . charactes before 
the layoutgrammar extension).  An existing output file with the same 
name will be destroyed. 
.IP "  Print Document"
Select an existing layout grammar file and document file with Select
button or write file names. The name of the output file is generated 
by the system. The extension is a character string which informs the 
method in the name of the layout grammar file (given between _ and . 
charactes before the layoutgrammar extension).  An existing output 
file with the same  name will be destroyed. Check print command and 
edit it if needed. 
.\"-----------------------------------------------------------------
.SS Retrieval menu
.IP "  Selecting structure grammar"
All functions in Retrieval phase of SYNDOC require the  user to give 
the name of a structure grammar file. Select a name of an existing
structure file or write a file name. After selection menu items
will be available.
.IP "  Indexing"
Select "Index Documents" entry.
.PP
.TP 16
.in +6n
Index Documents
Select or write names of the documents to be indexed and the name of 
control file for indexing. All documents in the directory can be 
expressed by an asterics, for example *.doc. Select either the default control 
file or your own (created by an text editor) for those words which are 
not to be indexed.  
.PP
.IP "  Defining Filters"
Select either "Create new filter" or "Open filter" entry.
.PP
.TP 16
.in +6n
Create New Filter
Select or give  an existing structure grammar file name, a name of a new
filter file, and a text type of the grammar as a type of the filter. 
An existing filter file with the same name will be overwritten.
.TP 16
.in +6n
Open Filter
Select or give an existing structure grammar file name and existing
filter file name.

.TP 16
.in +6n
Filter generation functions
.PP
.TP 16
.in +8n
Arrow up
Moves to the previous node (in depth first order)	
.TP 
.in +8n
Arrow down
Moves to the next node (in depth first order)
.TP 
.in +8n
Parent
Moves to the parent node
.TP 
.in +8n
Child
Moves to the first child
.TP
.in +8n
Previous
Moves to the previous sister
.TP
.in +8n
Next
Moves to the next sister
.TP
.in +8n
Zoom
Zooms tree at a node with the chosen rule(s)
.TP
.in +8n
Unzoom
Deletes children of the current node (i.e cancels the application of
a rule)
.TP
.in +8n
Add Filter
Adds a simple filter to a compound filter below an asteriks
.TP
.in +8n
Delete Filter
Deletes the simple filter below a star, except of the last one.
.TP
.in +8n
Constraints
Adds conditions associated to the current text type. For single 
text types containment condition may be given, for iterative text 
types also position and quantity constraints.
.TP
.in +8n
Results
Adds annotation associated to the current text type. 
.TP
.in +8n
Save
Saves filter without exit
.TP
.in +8n
Help
Help by dislaying manual pages
.TP
.in +8n
Cancel
Cancels editing without saving changes
.TP
.in +8n
Exit
Exits with save

.TP 16
.in +6n
Constraints functions:
.PP
.TP 16
.in +8n
Apply
Checks if containment condition is valid according to the grammmar.
If given text types are not reachable from the current text type or 
annotatations do not exit in the filter, opens a message window 
with a list of unvalid types. The user is allowed to correct the invalid 
condition by returning to the costraints definition window.
Otherwise, adds the condition to the filter.
.TP 
.in +8n
Help
Displays manual pages 
.TP 
.in +8n
Cancel
Cancels given conditions and restores the old ones.
.TP 16
.in +6n
Annotation functions:
.PP
.TP 16
.in +8n
Apply
Checks if annotation is valid according to the grammmar. If the given 
annotation is a text type of the grammar or  is already in the fitler, 
the process returns to filter definition window  and the user have to 
give the annotation again.
.TP 
.in +8n
Help
Displays manual pages
.TP 
.in +8n
Cancel
Cancels annotations


.IP "  Filtering"
Select "Compare filter to documents" entry.
.PP
.TP 16
.in +6n
Compare filter to documents
Select or write the name of an existing filter file and names of the 
documents to be searched or write the names. All documents in the 
directory can be expressed by star, for example *.doc.index.
Push Apply-button to start the comparing.
.TP 16
.in +6n
Result of comparison functions:
Shows number of matching documents and lets the user select type of 
output she wants. The default form is a Prolog term.
.TP 16
.in +6n
Define Output Form
Shows currently selected output form by a red block. Selecting an item 
from menu opens a window to ask parameters for selected output form. 
Each window has Apply button that makes that form currently selected 
output form.

.TP 16
.in +6n
Output form functions:
.TP 16
.in +10n
Default
Shows documents as Prolog terms  with a default previewer.
.TP
.in +10n
Input Form
Select the file name of an input grammar or wirte the name.
Documents are shown in the document window for the input and  can be 
edited.
.TP
.in +10n
Formatted
Select the file name of the layout grammar and the previewer or
write the names. Documents are formatted using the layout grammar and 
shown with the previewer.
.TP
.in +10n
Transformed and Formatted
Select the file names of transformation program and the layout grammar and
check the previewer.  Documents are transformed using the translator
formatted using the layout grammar and shown with previewer.

.TP 16
.in +6n
Preview
Shows the documents using the selected output form.
.TP 
.in +6n
Help
Displays manual pages
.TP 
.in +6n
No Output
Returns to filter comparing window, and the user can try a new matching. 
.\"-----------------------------------------------------------------
.SS Transform menu
.IP "  SDTS/Transducer - Modify"
Select an old structure grammar file with Select-button or write a
file name. Write a new structure grammar file name.  An existing 
structure grammar file with the same name will be overwritten. The
system copies contents of the old structure grammar file to the new
structure grammar file and opens an default editor with the new
file.
.IP "  SDTS/Transducer - Compare" 
Select a structure grammar file and a new structure grammar file with
Select-button or write names of files. Write a name of a
transformation grammar file. An existing file with the same name will
be overwritten. Implemented only for SDTS method.
.IP "  SDTS/Transducer - Transform"
Select a transformation grammar file and an old document file with
Select-button or write names of files. Write a name of a new document
file. An existing file with the same name will be overwritten.
.\"-----------------------------------------------------------------
.SS Control menu
In Control menu there are utility functions, which make use of SYNDOC
system easier.
.IP "  Defaults"
Enter default values so, that they corresponds to your directories and
available programs. System uses directory paths and extensions of
files as default values when it lists existing files or saves new
files. In the following there are an explanation and an example for
every field in the window.
.TP 16
.in +6n
Working directory
Main directory for your SYNDOC grammar and document files and directories:
/home/userid/
.TP
.in +6n
Structure grammar files
Possible subdirectory and file extension of structure grammar files:
grammars/*.gram
.TP
.in +6n
Input grammar files
Possible subdirectory and file extension of input grammar files:
grammars/*.inputgram
.TP
.in +6n
Input program files
Possible subdirectory and file extension of input program files:
grammars/*.inputgram.pl
.TP
.in +6n
Layout grammar files
Possible subdirectory and file extension of layout grammar files:
grammars/*.layoutgram
.TP
.in +6n
Layout program files
Possible subdirectory and file extension of layout program files:
grammars/*.layoutgram.pl
.TP
.in +6n
Document files
Possible subdirectory and file extension of document files: docs/*.doc
.TP
.in +6n
Indexed document files
Possible subdirectory and file extension of document index files: 
retrieval/*.doc.index
.TP
.in +6n
Index control files
Default control file or subdirectory and extension: 
retrieval/syndoccontrol.ctrl
.TP
.in +6n
Filters
Possible subdirectory and file extension of filters: retrieval/*.filt
.TP
.in +6n
Transformation grammar files
Possible subdirectory and file extension of transformation grammar
transform: transform/*.trgram
.TP
.in +6n
Transformation program files
Possible subdirectory and file extension of transformation program
files: transform/*.trgram.pl
.TP
.in +6n
Output files
Possible subdirectory and file extension of output files: outputs/*
.TP
.in +6n
Print command
Default printer command for function "Print Document": lpr -P
.TP
.in +6n
Default printer
Default printer for function "Print Document" : ps
.TP
.in +6n
Default editor
System uses value when opens editor with some grammar. If none is
given, system uses vi editor.
.TP
.in +6n
Default preview program
System uses value in "Preview Document" function: netscape
.TP
.in +6n
Default preview files 
Possible subdirectory and file extension of output files to be
previewed: outputs/*
.in -11n


Update defaults functions:
.TP
.in +6n
System defaults
Sets the default values of the system to the fields.
.TP
.in +6n
Help
Displays manual pages. 
.TP
.in +6n
Save and Exit
Saves values from the fields to the file .syndocdefaults in users
home directory and exits function.
.TP
.in +6n
Cancel
Cancels changes and exits the Defaults function. 
.IP "  System commands"
.in -11n
Write an unix system command. In this question window the Select
button adds a selected file to the end of the command line.
.\"-----------------------------------------------------------------
.SH DIAGNOSTICS 
In this Chapter, the error messages given by the SYNDOC system are
described in the alphabetical order.
.na
.nf
"<file> isn't a regular file!"
	Actions of the system:
	- The system doesn't execute a function.
	Reasons of the error: 
	- A file given by the user isn't exist. 
	How to recover from error:
	- The user have to give an existing file. 
	
"<path> isn't a regular directory!"
	Actions of the system:
	- The system doesn't execute a function. 
	Reasons of the error: 
	- A path given by the user isn't exist. 
	How to recover from error:
	- The user have to give an existing directory. 
	
"Already in the first sister!"
	Actions of the system:
	- The current node doesn't change. 
	Reasons of the error: 
	- The user has pushed Previous button in the first 
	  sister.
	How to recover from error:
	- The user can move up in the tree structure by 
	  using arrow up or Parent button.
	
"Already in the last sister!"
	Actions of the system:
	- The current node doesn't change. 
	Reasons of the error: 
	- The user has pushed Next button in the last sister.
	How to recover from error:
	- The user can move down in the tree structure by 
	  using arrow down or Child button.
	
"Already in the root !"
	Actions of the system:
	- The current node doesn't change. 
	Reasons of the error: 
	- The user has pushed Parent button in the root.
	How to recover from error:
	- In the root it is possible to use Expand button 
	  or Child and arrow down buttons, if the tree is 
	  already expanded.

"Can't find file <path>.syndocsysdefaults" 
	Actions of the system:
	- The system can't read default values from
	  .syndocsysdefaults or ~/.syndocdefaults and 
	  can't start the system. 
	Reasons of the error:
	- SYNDOC system file .syndocsysdefaults is 
	  missing. 
	How to recover from error:
	- The user have to contact to the supplier of 
	  the system. 
	
"Close editor !"
	Actions of the system:
	- The system doesn't exit a function before 
	  the editor is closed. 
	Reasons of the error:
	- The user has pushed Done button when the 
	  editor is open.
	How to recover from error:
	- The user have to close the editor. 
	
"Default path <path> isn't a regular directory!"
	Actions of the system:
	- The system doesn't execute a function. 
	Reasons of the error:
	- Value of the default path of files doesn't 
	  correspond directories of the user.
	How to recover from error:
	- The user have to corret default value or create
	  corresponding directory.

	
"Function not implemented"
	Actions of the system:
	- The system doesn't generate a list of reachable text 
          types. 
	Reasons of the error:
	- The user has tried to select a text type for  containment
          constraint from a list of reachable text types. 
	How to recover from error:
	- The user has to type the text type.

"Grammars are not same texttype!"
	Actions of the system:
	- System doesn't execute generation of the 
	  transformation  grammar.
	Reasons of the error: 
	- The user has given structure grammars with 
	  different kinds of text types in the second 
	  line of them.
	How to recover from error:
	- The user have to give structure grammars 
	  with same text type in the second lines of them.

"Grammars are not same type!"
	Actions of the system:
	- System doesn't execute generation of the 
	  transformation grammar.
	Reasons of the error: 
	- The user has given structure grammars with 
	  different kinds of document types in the 
	  first line of them.
	How to recover from error:
	- The user have to give structure grammars 
	  with same type in the first lines of them.

"Input grammar is not correct!"
	Actions of the system:
	- The system doesn't open the document window.               
	Reasons of the error:
	- The user has given the input grammar which 
	  is not correct to the document file.
	How to recover from error:
	- The user have to give a correct input grammar file. 

"Name is one of the names of text types in the grammar"
	Actions of the system:
	- The system doesn't accept the name for the annotation.               
	Reasons of the error:
	- The user has given a name of a text type in the grammar 
	  as a name for an annotation.
	How to recover from error:
	- The user have to give the annotation again. 

"No rule for this element."
	Actions of the system:
	- System doesn't expand node. 
	Reasons of the error: 
	- The user has pushed Expand button in the node, 
	  which doesn't have any rule in the grammar.
	How to recover from error:
	- The user can't expand this node. There can be 
	  some error in the grammar file.

"Structure grammars not comparable.!"
	Actions of the system:
	- System does not compare grammars in order to 
	  to make transformation grammar
	Reasons of the error: 
	- The user has given grammar whose types or 
	  content types are not the same.
	How to recover from error:
	- The ser has to select new grammars. 
 

"Text type(s): <list> not valid"
	Actions of the system:
	- System does not accept the constraint. 
        Reasons of the error: 
	- The user has written as a containment contraint of a
	text type 1)  a name of a text type which cannot define
	parts contained to parts for the text type, or 
	2) annotation which does not exist in the filter.
	How to recover from error:
	- The user can define the constraint again.

"The grammar of the old document is not correct!"
	Actions of the system:
	- System does not execute generation of the 
	  transformation grammar.
	Reasons of the error: 
	- The user has given structure grammars with 
	  different kinds of text types in the second 
	  line of them.
	How to recover from error:
	- The user have to give a new structure grammar 
	  based on an old structure grammar.

"Type of text is not correct !"
	Actions of the system:
	- System does not open document window. 
	Reasons of the error: 
	- There is not the same type of the text in 
	  the input grammar and document files given 
	  by the user.
	How to recover from error:
	- The user have to give a correct input grammar 
	  file. 

"Type of the structure grammar is not correct !"
	Actions of the system:
	- System does not open the document window. 
	Reasons of the error: 
	- There is not the same type of the structure 
	  grammar in the input grammar and document 
	  files given by the user.
	How to recover from error:
	- The user have to give a correct input grammar 
	  file. 
.fi
.ad
.\"-----------------------------------------------------------------
.SH RESTRICTIONS 
.PP
In this implementation of the system the size of the document files
is limited. Handling documents over 6 Kbytes is very slow and may
cause some unexpected errors, due to overflow of the memory.

Functions in menus Retrieval and Transform need more development.
.\"-----------------------------------------------------------------
.SH FILES
.TP 24
$HOME/.syndocdefaults
Users default values.
.TP
$HOME/mysyndoc/*
Users grammar and document files.
.\"-----------------------------------------------------------------
.SH REFERENCES
.TP 5
[1]
Kuikka E., Mykknen J., Ryynnen, A. and Salminen A., 
Implementation of two-dimensional filters for structured
documents in SYNDOC environment, Technical Report A-1995-4,
University of Joensuu, Department of Computer Science, 1995.
.TP
[2]
Kuikka E. and Penttonen M., Designing a syntax-directed text
processing system, in Proceedings of the Second Symposium on
Programming Languages and Software Tools, eds., Koskimies K.
and Rih K.-J., 191-204, University of Tampere, Department
of Computer Science, A-1991-5, Tampere, Finland, 1991.
.TP
[3]
Kuikka E., Penttonen M. and Visnen M.-K., Theory and
implementation of SYNDOC document prosessing system, in
Proceedings of the Second Int. Conference on Practical
Application of Prolog, PAP-94, London, UK pp. 311-327, April
1994.
.TP
[4]
Kuikka E. and Penttonen M., Tranformation of structured
documents with the use of grammar,  Electronic Publishing - 
Origination, Dissemination and Design, 6(3):373-383, 1993,  
.TP
[5]
Kuikka E. and Penttonen M., Tranformation of structured
documents,  will be published in Electronic Publishing - 
Origination, Dissemination and Design, 8(4), 1996.  
.TP
[6]
Kuikka E. and Salminen A., Filtering structured documents in
SYNDOC environment, International Conference on Electronic
Publishing - Origination, Dissemination and Design, Palo Alto,
California, September 23-26, 1996
.TP
[7]
Tarau P.: BinProlog 5.00 User Guide. Department of Computer
Science, University of Moncton, Canada, April 1996.
.TP
[8]
Tarau P. and Demoen B., Language Embedding by Dual Compilation
and State Mirroring, in Proceedings of 6th Workshop on Locig
Programming Environments, Santa Margherita Ligure, 1994.










