/*----------------------------------------------------------------------------
 * syndoc.pl
 * 
 * Compiles Prolog files
 * Predicates which are common for many SYNDOC files
 *----------------------------------------------------------------------------
*/

%Pred: single_quote
%Pred: double_quote
%Pred: left_parenthesis
%Pred: right_parenthesis
%Pred: right_bracket
%Pred: left_bracket
%Pred: lbrace
%Pred: rbrace
%Pred: single_star
%Pred: single_plus
%Pred: comma
%Pred: period
%Pred: semicolon
%Pred: greater_than
%Pred: procent
%Pred: hyphen
%Pred: underscore
%Pred: equal
%Pred: straight
%Pred: question_mark
%Pred: carriagereturn
%Pred: linefeed
%Pred: space
%Pred: endoffile
%Pred: empty_line_end
%Pred: rule_end 
%Pred: grammar_end
%Pred: isPoint
%Pred: endOfFile
%Pred: isEnd
%Pred: isLineFeed
%Pred: isNl
%Pred: isFullstop
%Pred: lc_letter                
%Pred: uc_letter                 
%Pred: letter                   
%Pred: letter_or_                
%Pred: letter_or_star_or_plus   
%Pred: any_but_single_quote     
%Pred: ant_but_right_parenthesis  
%Pred: any_but_right_bracket      
%Pred: any_but_rbrace             
%Pred: any_but_lbrace             
%Pred: any_but_semicolon         
%Pred: any_but_procent            
%Pred: ruleInput
%Pred: restCharacters
%Pred: restSingleQuote
%Pred: uc_to_lc
%Pred: rest_atom
%Pred: countLines
%Pred: findStarPlus
%Pred: findplainStar
%Pred: findStar
%Pred: findPlus
%Pred: findBracket
%Pred: findStraight
%Pred: findLBrace
%Pred: checkListSisters
%Pred: checkListSisters_r
%Pred: checkPlusSisters_r
%Pred: checkOptionalSisters
%Pred: exeCmd
%Pred: moreup
%Pred: moreup_r
%Pred: atRightst
%Pred: noSisters
%Pred: textEdited
%Pred: atSmallestGramEl
%Pred: atSmallestGramEl_r
%Pred: displayCurrentPlace
%Pred: displayContext
%Pred: displayTreeAndSisters
%Pred: writeLeftSisters
%Pred: writeRightSisters
%Pred: writeChildren
%Pred: writeArrow
%Pred: docmarginal
%Pred: structmarginal
%Pred: consmarginal
%Pred: consposmarginal
%Pred: consquamarginal
%Pred: calculateConsAndAnno
%Pred: separateCons
%Pred: sepContain
%Pred: sepPosition
%Pred: selectFirstChild
%Pred: setIndentSpaces
%Pred: setBetweenSpaces
%Pred: setBetweenPoints
%Pred: setSpacesToPpoints
%Pred: checkNl
%Pred: check_grams
%Pred: readGrammar_old







/******************************************
* Predicates for certain characters
*****************************************/


%Pred:single_quote
single_quote(39).               % yksinkertaiset lainausmerkit
single_quote(96).               
%Pred:double_quote
double_quote(34).               % kaksoislainausmerkki
double_quote(34,34).            
%Pred:left_parenthesis
left_parenthesis(40).           % vasen sulku
%Pred:right_parenthesis
right_parenthesis(41).          % oikea sulku
%Pred:right_bracket
right_bracket(93).              % oikea hakasulku
%Pred:left_bracket
left_bracket(91).               % vasen hakasulku
%Pred:lbrace
lbrace(123).                    % vasen aaltosulku
%Pred:rbrace
rbrace(125).                    % oikea aaltosulku
%Pred:single_star
single_star(42).                % tahti
%Pred:single_plus
single_plus(43).                % plus-merkki
%Pred:comma
comma(44).                      % pilkku
%Pred:period
period(46).                     % piste
%Pred:semicolon
semicolon(59).                  % puolipiste
%Pred:greater_than
greater_than(62).               % suurempi kuin merkki
%Pred:procent
procent(37).                    % prosenttimerkki
%Pred:hyphen
hyphen(45).                     % viiva
%Pred:underscore
underscore(95).                 % alaviiva
%Pred:equal
equal(61).                      % yhtasuuruusmerkki
%Pred:straight
straight(124).			% pystyviiva
%Pred:question_mark
question_mark(63).		% kysymysmerkki
%at_end_of_line                 % end_of_line  (Sicstus)
%Pred:carriagereturn
carriagereturn(13).             % CR  = ^M
%Pred:linefeed
linefeed(10).                   % LF = ^J
%Pred:space
space(32).                      % valimerkki
%Pred:endoffile
%at_end_of_stream.              % end_of_file (Sicstus)
endoffile(-1).
%Pred:empty_line_end
empty_line_end(C):-linefeed(C);carriagereturn(C).
%Pred:rule_end 
rule_end(C) :- linefeed(C);carriagereturn(C);space(C).
%Pred:grammar_end
grammar_end(C):-endoffile(C).

%poistettavat kunhan varmistetaan ettei tarvita
%Pred:isPoint
isPoint(46).                    % piste
%Pred:endOfFile
endOfFile([-1]).		% end_of_file
%Pred:isEnd
isEnd(-1).                      % end_of_file.
%Pred:isLineFeed
isLineFeed(13).  % ^M  = CR     % garriage return
isLineFeed(10).  % ^J  = LF     % rivinvaihto
%Pred:isNl
isNl(10).
%Pred:isFullstop
isFullstop(32).                 % tyhja
isFullstop(13).  % ^M  = CR     % garriage return
isFullstop(10).  % ^J  = LF     % rivinvaihto



%Pred:lc_letter                 % Merkki on pieni kirjain.
lc_letter(C):-C > 96, C < 123.
%Pred:uc_letter                 % Merkki on iso kirjain. 
uc_letter(C):- C > 64, C < 91.
%Pred:letter                    % Merkki on pieni tai iso kirjain.
letter(C):-lc_letter(C).
letter(C):-uc_letter(C).
%Pred:letter_or_                % Merkki on iso tai pieni kirjain tai 
                                % alaviiva.
letter_or_(C):- lc_letter(C).
letter_or_(C):-  uc_letter(C).
letter_or_(C):-  underscore(C).
%Pred:letter_or_star_or_plus    % Merkkijono on iso tai pieni kirjain, 
                                % alaviiva, tahti tai plus-merkki.
letter_or_star_or_plus(C):- lc_letter(C).
letter_or_star_or_plus(C):- uc_letter(C).
letter_or_star_or_plus(C):- underscore(C).
letter_or_star_or_plus(C):- single_star(C).
letter_or_star_or_plus(C):- single_plus(C).
%Pred:any_but_single_quote     % Merkki on mika tahansa muu merkki 
                               %kuin lainausmerkki. 
any_but_single_quote(C):-C=\=39,C=\=96.
%Pred:ant_but_right_parenthesis  % Merkki on mika tahansa muu merkki 
                                 % kuin oikea sulku.
any_but_right_parenthesis(C):-C=\=41.
%Pred:any_but_right_bracket      % Merkki on mika tahansa muu merkki 
                                 % kuin oikea hakasulku.
any_but_right_bracket(C):-C=\=93.
%Pred:any_but_rbrace             % Merkki on mika tahansa muu merkki 
                                 %kuin oikea aaltosulku.
any_but_rbrace(C):-C=\=125.
%Pred:any_but_lbrace             % Merkki on mika tahansa muu merkki 
                                 % kuin vasen aaltosulku.
any_but_lbrace(C):-C=\=123.
%Pred:any_but_semicolon          % Merkki on mika tahansa muu merkki 
                                 % kuin puolipiste.
any_but_semicolon(C):-C=\=59.
%Pred:any_but_procent            % Merkki on mika tahansa muu merkki 
                                 % kuin prosenttimerkki
any_but_procent(C):-C=\=37.

%************************************************


/*******************************************************
%Pred:ruleInput(-RuleChar)
*   
* Read grammar rules from a file a character by character
* A rule starts with  a character and ends with  a period
* Empty lines are not allowed
* After rules there can be extra lines containing spaces and a line feed
* or a line feed character
* File ends with a line containing spaces and end_of_file-character or
* end_of_file-character
*/ 
ruleInput(RuleChar):-get0(C),
		 restCharacters(C,[],RR),
		 reverse(RR,RuleChar).

%Pred:restCharacters(+ASCIIChar,+GenerateCharaterList,-RuleAsCharListInReversedOrder)
restCharacters(EndC,_,[end_of_file]):-grammar_end(EndC),!.
restCharacters(LineFeedC,_,[empty_line]):-empty_line_end(LineFeedC),!.
restCharacters(PointC,R,R):-period(PointC),!,get0(C1),rule_end(C1).
restCharacters(SingleQuote,R,R4):-single_quote(SingleQuote),!,get0(C1),
	                        restSingleQuote(C1,[39],R1),
				append(R1,R,R3),!,
				get0(C2),restCharacters(C2,R3,R4).
restCharacters(C,R,R1):-!,get0(C1),restCharacters(C1,[C|R],R1).

%Pred:restSingleQuote(+ASCIICharacter,+GeneratedCharacterList,-QuoteAsCharList)
restSingleQuote(SingleQuote,R,[SingleQuote|R]):-single_quote(SingleQuote),!.
restSingleQuote(C,R,R1):-!,get0(C1),restSingleQuote(C1,[C|R],R1).

%*********************************************************************
%Pred:uc_to_lc
% Muutetaan pienet kirjaimet isoiksi.
uc_to_lc([],[]):-!.
uc_to_lc([C|L],[D|L1]):-uc_letter(C),!,D is C+32,uc_to_lc(L,L1).
uc_to_lc([C|L],[C|L1]):-!,uc_to_lc(L,L1).


%**********************************************************************
%Pred:rest_atom(+CharList,-ModifiedAtomCharList,-RestCharList,Condition)
% Predikaattia kaytetaan atomin tutkimiseen.
% Condition muuttujan arvona on ehto, millaisia merkkeja atomi saa
% sisaltaa. 
rest_atom([C|P1],[C|Cs],P2,Condition):-
	Goal =.. [Condition,C],call(Goal),!,
        rest_atom(P1,Cs,P2,Condition).
rest_atom(P1,[],P1,_):-!.
rest_atom([],[],[[]|[]],_):-!.



%************************************************************************
%Pred:countLines
countLines([45,62|_],N,N):-!.
countLines([X|Xs],N1,N):-
	(same(X,10),N2 is N1 + 1 ;
	 same(N2,N1)),
	 countLines(Xs,N2,N).

%************************************************************************
%Pred:findStarPlus
% Tutkitaan, loytyyko merkkijonon lopusta tahti- tai plus-merkki.
% Predikaattia kaytetaan puun kasittelyssa (syndoc_input.pl).
findStarPlus([C|P],[C|Cs]):- letter(C),
                             rest_atom(P,Cs,P1,letter_or_),!,
                             ismember(P1).

ismember(P1):-member(42,P1).
ismember(P1):-member(43,P1).



% Tutkitaan sisaltaako merkkijono  vain tahti-merkin.
%kaytetaan retrieval-osassa filtterin merkkina
%Pred:findplainStar
findplainStar([C|P],[C|P]):- single_star(C).
% Tutkitaan sisaltaako merkkijono tahti-merkin.
% Kaytetaan, kun tutkitaan onko delete-toiminto mahdollinen
%Pred:findStar
findStar([C|P],[C|Cs]):- letter(C),
                         rest_atom(P,Cs,P1,letter_or_),!,member(42,P1).

% Tutkitaan sisaltaako merkkijono plus-merkin. 
% Kaytetaan, kun tutkitaan onko delete-toiminto mahdollinen.
%Pred:findPlus
findPlus([C|P],[C|Cs]):- letter(C),
                         rest_atom(P,Cs,P1,letter_or_),!,member(43,P1).

%Pred:findBracket
%Tutkii valinnaisen ja palauttaa ilman metasymboleja
% Tutkitaan, onko merkkijono hakasulkujen sisalla.
findBracket([C|P],Cs):-left_bracket(C),!,
                       rest_atom(P,Cs,P1,letter_or_),member(93,P1).
% Tutkitaan alkaako ?-merkilla
findBracket([C|P],Cs):-question_mark(C),!,
                       rest_atom(P,Cs,_,letter_or_).
% Tutkitaan sisaltaako merkkijono lopussa  ?-merkin. 
findBracket([C|P],[C|Cs]):- letter(C),
                         rest_atom(P,Cs,P1,letter_or_),!,member(63,P1).
%Tutkitaan, alkaako merkkijono pystyviivalla, palauttaa ilman pystyviivaa
%Pred:findStraight
findStraight([C|P],P):-straight(C).
%Tutkitaan onko nonterminaalinimessa retrievalin vaatima aaltosulku ja
%valitaan vain nonterminaalinimi
%Pred:findLBrace
findLBrace([C|P],[C|Cs]):-letter(C),
                          rest_atom(P,Cs,P1,letter_or_),!,member(123,P1).

%***********************************************************************
%Pred:checkListSisters
% Tutkitaan, onko listan puun solmun oikean sisaren nimi sama.
% palauttaa true, jos loppumerkki ja oikeat alipuuta samoja, muuten fail.
% Palautetaan vasemman juuren character-arvo ja oikeiden puiden listan loppu-
% merkki.
% Kaytetaan, kun tutkitaan onko delete- ja next-toiminto sallittu listalle. 
% ei oikeita alipuita, ei voi loytya listan loppumerkkia
checkListSisters([[_,_,[]],_],[],[]):-!,fail.
% vain oikeita alipuita, tutkitaan loytyyko listan loppumerkki ja 
% palautetaan se
checkListSisters([[[],T,[R|Rs]],_],[],RrootChar):-
	T=..[Root|_],!,
        R=..[Rroot|_],name(Rroot,RrootChar),!,
	(same(Rroot,Root),
         checkListSisters([[[],T,Rs],_],[],_)       
	;
	 findStarPlus(RrootChar,RrChar),name(Rr,RrChar),same(Rr,Root)).
% seka oikeita etta vasempia alipuita, lasketaan vasemman juuren arvo ja
% tutkitaan oikeita alipuita 
checkListSisters([[[L|_],T,[R|Rs]],_],LrootChar,RrootChar):-
	T=..[Root|_],!,
        L=..[Lroot|_],name(Lroot,LrootChar),
        R=..[Rroot|_],name(Rroot,RrootChar),!,
	(
	 same(Rroot,Root), 
	 checkListSisters([[[L|_],T,Rs],_],_,_)
         ;
	 findStarPlus(RrootChar,RrChar),name(Rr,RrChar),same(Rr,Root)).
% sama retrieval osassa, jossa nimessa an attribuutit
% Tutkitaan, onko listan puun solmun oikean sisaren nimi sama.
% palauttaa true, jos loppumerkki ja oikeat alipuuta samoja, muuten fail.
% Palautetaan vasemman juuren character-arvo ja oikeiden puiden listan loppu-
% merkki.
% Kaytetaan, kun tutkitaan onko delete- ja next-toiminto sallittu listalle. 

%Pred:checkListSisters_r
% ei oikeita alipuita, ei voi loytya listan loppumerkkia
checkListSisters_r([[_,_,[]],_],[],[]):-!,fail.
% vain oikeita alipuita, tutkitaan loytyyko listan loppumerkki ja 
% palautetaan se
checkListSisters_r([[[],T,[R|Rs]],_],[],RrootChar):-
	T=..[Root|_],!,
        R=..[Rroot|_],name(Rroot,RrootChar),!,
	(same(Rroot,Root),
         checkListSisters_r([[[],T,Rs],_],[],_)       
	;
	 findStarPlus(RrootChar,RrChar),
	name(Rr,RrChar),
	addAttributes(Rr,RrAttr),same(RrAttr,Root)).

% seka oikeita etta vasempia alipuita, lasketaan vasemman juuren arvo ja
% tutkitaan oikeita alipuita 
checkListSisters_r([[[L|_],T,[R|Rs]],_],LrootChar,RrootChar):-
	T=..[Root|_],!,
        L=..[Lroot|_],name(Lroot,LrootChar),
        R=..[Rroot|_],name(Rroot,RrootChar),!,
	(
	 same(Rroot,Root), 
	 checkListSisters_r([[[L|_],T,Rs],_],_,_)
         ;
	 findStarPlus(RrootChar,RrChar),
	name(Rr,RrChar),
	addAttributes(Rr,RrAttr),same(RrAttr,Root)).

%Pred:checkPlusSisters_r
% retrieval osassa, jossa nimessa an attribuutit
% Tutkitaan, onko + -listan puun loppusolmun vasemman sisaren nimi sama.
% palauttaa true, muuten jos vasenta sisarta ei ole, palauttaa fail.
 % ei vasempia alipuita, ei voi loytya sisarta
checkPlusSisters_r([[[],_,_],_]):-!,fail.
% seka oikeita etta vasempia alipuita, lasketaan vasemman juuren arvo ja
% ja puun listanlopputietueen arvo ja verrataan
checkPlusSisters_r([[[L|_],T,_],_]):-
	T=..[Root|_],name(Root,RootChar),!,
        L=..[Lroot|_],name(Lroot,LrootChar),
	 findPlus(RootChar,RChar),
	name(R,RChar),
	 findLBrace(LrootChar,LrChar),
        name(Lr,LrChar),
	same(R,Lr).

%Pred:checkOptionalSisters
% Tutkitaan, onko vaihtoehtoisen puun solmun oikean sisaren nimi sama.
% palauttaa true, jos jompikumpi sisar sama, muuten fail.
% Palautetaan vasemman ja oikean juuren character-arvo 
% Kaytetaan, kun tutkitaan onko delete sallittu valinnaiselle puulle. 
%ainoa alipuu, ei voida poistaa
checkOptionalSisters([[[],_,[]],_],[],[]):-!,fail.
% vain oikeita alipuita, tutkitaan onko valinnainen 
% palautetaan se oikea juuri
checkOptionalSisters([[[],_,[R|_]],_],[],RrootChar):-
        R=..[Rroot|_],name(Rroot,RrootChar),!,
	%(
        findStraight(RrootChar,_).
        % ;
	% fail).
% vain vasempia alipuita, ei tarvitse tutkia
checkOptionalSisters([[[L|_],_,[]],_],LrootChar,[]):-
        L=..[Lroot|_],name(Lroot,LrootChar),!,
	%(
        findStraight(LrootChar,_).
        % ;
	% fail).
% seka oikeita etta vasempia alipuita, lasketaan oikean vasemman juuren arvo ja
% tutkitaan onko jompikumpi sama
checkOptionalSisters([[[L|_],_,[R|_]],_],LrootChar,RrootChar):-
        L=..[Lroot|_],name(Lroot,LrootChar),
        R=..[Rroot|_],name(Rroot,RrootChar),!,
	(
	 (findStraight(LrootChar,_);findStraight(RrootChar,_))
         ;
	 fail).




/*
 *-----------------------------------------------------------------------------
 *    tree operations
 *-----------------------------------------------------------------------------
 */

%Pred:exeCmd
exeCmd(stop,CurrentPlace,CurrentPlace):-fail.

% Liikutaan puussa ylemmalle tasolle, solmun isaan.
% Jos ollaan jo puun juuressa, tulostuu ilmoitus 'Already in the root'.
exeCmd(up,CurrentPlace,NewCurrentPlace):-
	up(CurrentPlace,NewCurrentPlace).

%exeCmd(up_r,...) siirrytaan isaan 
% Liikutaan puussa ylemmalle tasolle, solmun isaan.
% Siirtymista jatketaan kunnes tullaan solmuun, jolla on sisaria tai ollaan
% jasennyspuun juurisolmussa.
% Jos ollaan jo puun juuressa, tulostuu ilmoitus 'Already in the root'.
exeCmd(up_r,CurrentPlace,NewCurrentPlace):-
	up_r(CurrentPlace,NewCurrentPlace).
%;
%	 same(CurrentPlace,NewCurrentPlace)).

% siirrytaan puussa ensimmaiseen lapseen, jos ei onnistu, pysytaan
% paikoillaan
exeCmd(low,CurrentPlace,NewCurrentPlace):-
	low(CurrentPlace,NewCurrentPlace).
exeCmd(low,CurrentPlace,NewCurrentPlace):-
	 same(CurrentPlace,NewCurrentPlace).

% siirrytaan puussa ensimmaiseen lapseen, jos ei onnistu, pysytaan
% paikoillaan
exeCmd(low_r,CurrentPlace,NewCurrentPlace):-
	low_r(CurrentPlace,NewCurrentPlace).
exeCmd(low_r,CurrentPlace,NewCurrentPlace):-
	 same(CurrentPlace,NewCurrentPlace).

% Kehitetaan puun solmuja. 
% Jos solmu on puun lehti, editoidaan solmun sisaltoa.
% Muutoin kehitetaan solmuja, kielioppisaantojen avulla.
exeCmd(down,CurrentPlace,NewCurrentPlace):-
	% Editor-predikaatissa on tieto siita, onko solmun sisaltoa editoitu.
%	retractall(editor(_)),
	assert(editor(no)),
	% Siirrytaan puussa ensimmaiseen lapseen.
	% Jos siirtyminen ei onnistu, pysytaan samassa paikassa.
        (down(CurrentPlace,NewCurrentPlace1);
	 same(CurrentPlace,NewCurrentPlace1)),
	% Tutkitaan, onko solmun tyyppi sama kuin dokumentin tekstityyppi.
	(atSmallestGramEl(NewCurrentPlace1,NewCurrentPlace2),
 	  % Jos solmulla ei ole oikeaa sisarta, siirrytaan naytossa 
           % alaspain seuraavaan solmuun
  	  (atRightst(NewCurrentPlace2),
 	   downscreen(NewCurrentPlace2,NewCurrentPlace);
	  % Jos solmulla on oikea sisar, siirrytaan siihen.
           right(NewCurrentPlace2,NewCurrentPlace));
	 % Onko solmun sisaltoa editoitu.
	 % Tarkistus tarvitaan, koska solmu voi olla muotoa 
         % author(text([teksti])).
	 textEdited(NewCurrentPlace1,NewCurrentPlace);
	 same(NewCurrentPlace1,NewCurrentPlace)),
	retractall(editor(_)).

% Liikutaan puussa solmun oikeaan sisareen.
exeCmd(right,CurrentPlace,NewCurrentPlace):-
	(right(CurrentPlace,NewCurrentPlace);
	same(CurrentPlace,NewCurrentPlace)).

% Liikutaan puussa solmun vasempaan sisareen.
exeCmd(left,CurrentPlace,NewCurrentPlace):-
	(left(CurrentPlace,NewCurrentPlace);
	same(CurrentPlace,NewCurrentPlace)).

% Liikutaan naytossa alempaan elementtiin.
exeCmd(downscreen,CurrentPlace,NewCurrentPlace):-
 	(downscreen(CurrentPlace,NewCurrentPlace);
 	same(CurrentPlace,NewCurrentPlace)).
 
% Liikutaan naytossa ylempaan elementiin.
exeCmd(upscreen,CurrentPlace,NewCurrentPlace):-
 	upscreen(CurrentPlace,NewCurrentPlace).
%;
% 	same(CurrentPlace,NewCurrentPlace)).
% Liikutaan naytossa ylempaan elementiin.
exeCmd(upscreen_r,CurrentPlace,NewCurrentPlace):-
 	upscreen_r(CurrentPlace,NewCurrentPlace).
%;
% 	same(CurrentPlace,NewCurrentPlace)).
 
% Kehitetaan solmusta uusi samanlainen solmu.
% Eli joko editoidaan solmun sisaltoa tai solmun lapsiksi kehtitetaan uusia solmuja.
exeCmd(next,CurrentPlace,NewCurrentPlace):-
       (next(CurrentPlace,NewCurrentPlace);
       same(CurrentPlace,NewCurrentPlace)).

% Poistetaan solmu
exeCmd(del,CurrentPlace,NewCurrentPlace):-
	del(CurrentPlace,NewCurrentPlace).
exeCmd(del,CurrentPlace,NewCurrentPlace):-
	 same(CurrentPlace,NewCurrentPlace).

% Poistetaan solmu ja sen vasemmat ja oikeat sisaret.
exeCmd(unexpand,CurrentPlace,NewCurrentPlace):-
	(unexpand(CurrentPlace,NewCurrentPlace);
	 same(CurrentPlace,NewCurrentPlace)).

% Poistetaan solmun lapset.
exeCmd(unexpandcurrent,CurrentPlace,NewCurrentPlace):-
	unexpandcurrent(CurrentPlace,NewCurrentPlace).
exeCmd(unexpandcurrent,CurrentPlace,NewCurrentPlace):-
	 same(CurrentPlace,NewCurrentPlace).


% kehitetaan atomi kieliopista ja siirrytaan ensimmaiseen lapseen,
% jos ei  onnistu pysytaan paikoillaan
% jos alimmassa alkiossa siirrytaan ylemmas tarpeeksi (kunnes sisaria,
% samalla poistetaan tyhja tekstityyppi-atomi
% jos  viimeisessa lapsessa, pysytaan paikoillaan, muuten tehdaan siirtyminen
% seuraavaan lapseen
exeCmd(zoom,CurrentPlace,NewCurrentPlace):-
	(zoom(CurrentPlace,NewCurrentPlace1);
	 same(CurrentPlace,NewCurrentPlace1)),
	(atSmallestGramEl_r(NewCurrentPlace1,NewCurrentPlace)
	 ;
	 same(NewCurrentPlace1,NewCurrentPlace)).


% poistetaan kehitetty solmun lapset puusta, solmusta tulee atomi puuhun 
exeCmd(unzoomcurrent,CurrentPlace,NewCurrentPlace):-
	unzoomcurrent(CurrentPlace,NewCurrentPlace).
%;
%	 same(CurrentPlace,NewCurrentPlace)).

%lisataan filtteri profiiliin
exeCmd(addfilter,CurrentPlace,NewCurrentPlace):-
%	exeCmd(next_r,CurrentPlace,NewCurrentPlace1),  
%	(addfilter(NewCurrentPlace1,NewCurrentPlace);
	(addfilter(CurrentPlace,NewCurrentPlace);
	 same(CurrentPlace,NewCurrentPlace)).

% kirjoitetaan tai korjataan ehto atomissa,
exeCmd(constraints,CurrentPlace,NewCurrentPlace):-
%	retractall(editor(_)),
	assert(editor(no)),
	(constraints(CurrentPlace,NewCurrentPlace);
	 same(CurrentPlace,NewCurrentPlace)),
	retractall(editor(_)).

%exeCmd(annotations,.. kirjoitetaan annotaatioehto nonterminaaliin
exeCmd(annotations,CurrentPlace,NewCurrentPlace):-
%	retractall(editor(_)),
	assert(editor(no)),
	annotations(CurrentPlace,NewCurrentPlace),
	retractall(editor(_)).
exeCmd(annotations,CurrentPlace,NewCurrentPlace):-
%	retractall(editor(_)),
	assert(editor(no)),
	 same(CurrentPlace,NewCurrentPlace),
	retractall(editor(_)).

%Pred:moreup
% Jos solmulla ei ole sisaria siirrytaan ylemalle tasolle eli isa-solmuun.
% Siirtymista lopetetaan, kun ollaan solmussa jolla on sisaria tai tullaan
% jasennyspuun juureen.
moreup([Tree,[[[],'document',[]]]],[Tree,[[[],'document',[]]]]):-!.
moreup(NewCurrentPlace,NewCurrentPlace1):-
	noSisters(NewCurrentPlace),
	 up(NewCurrentPlace,NewCurrentPlace2),
	  moreup(NewCurrentPlace2,NewCurrentPlace1).
moreup(NewCurrentPlace,NewCurrentPlace1):-
	noSisters(NewCurrentPlace),
	  same(NewCurrentPlace,NewCurrentPlace1).
moreup(NewCurrentPlace,NewCurrentPlace1):-
	 same(NewCurrentPlace,NewCurrentPlace1).



%Pred:moreup_r
%Jos solmulla ei ole sisaria siirrytaan ylemalle tasolle eli isa-solmuun.
% Siirtymista lopetetaan, kun ollaan solmussa jolla on sisaria tai tullaan
% jasennyspuun juureen.
moreup_r([Tree,[[[],'document{;;}{}',[]]]],[Tree,[[[],'document{;;}{}',[]]]]):-!.
moreup_r(NewCurrentPlace,NewCurrentPlace1):-
	noSisters(NewCurrentPlace),
	 (up_r(NewCurrentPlace,NewCurrentPlace2),
	  moreup_r(NewCurrentPlace2,NewCurrentPlace1);
	  same(NewCurrentPlace,NewCurrentPlace1)).
moreup_r(NewCurrentPlace,NewCurrentPlace1):-
	 same(NewCurrentPlace,NewCurrentPlace1).

% Onko solmulla oikean puoleisin.

%Pred:atRightst
atRightst([[_,_,[]],_]).

%Pred:noSisters
% Solmulla ei ole sisaria.
noSisters([[[],_,[]],_]).

%Pred:textEdited
% Solmun sisaltoa on editoitu editori-ikkunassa.
textEdited(NewCurrentPlace1,NewCurrentPlace):-
	editor(yes),
 	% Siirrytaan naytossa seuraavaan solmuun jos ei oikeaa sisarta.
 	(atRightst(NewCurrentPlace1),
 	 downscreen(NewCurrentPlace1,NewCurrentPlace);
         % jos oikea sisar siirrytaan siihen
 	 right(NewCurrentPlace1,NewCurrentPlace)).

%Pred:atSmallestGramEl
% Predikaattiin smallestGramEl on talletettu dokumentin tekstityypista.
atSmallestGramEl([[LS,T,RS],[[LS1,R1,RS2]|Context]],NewCurrentPlace2):-
    smallestGramEl(Element),
         % Jos puun juuri on sama kuin tekstityyppi, niin siirrytaan isaan
         % joka on naytoon kirjoitettava solmu
	(T=..[Root,_],same(Root,Element),
 	 up([[LS,T,RS],[[LS1,R1,RS2]|Context]],NewCurrentPlace2);
	% Jos solmu on atomi ja on sama kuin tekstityyppi, poistetaan solmu
	% ja sen sisaret puusta ja siirrytaan ylemmalle tasolle.
	 same(T,Element),
	 unexpand([[LS,T,RS],[[LS1,R1,RS2]|Context]],NewCurrentPlace2)).



%Pred:atSmallestGramEl_r
% Predikaattiin smallestGramEl_f on talletettu dokumentin tekstityypista.
atSmallestGramEl_r([[LS,T,RS],[[LS1,R1,RS2]|Context]],NewCurrentPlace):-
    smallestGramEl(Element),
	addAttributes(Element,Element1),        
	% Jos solmu  T on atomi ja on sama kuin tekstityyppi, poistetaan solmu
	% ja sen sisaret puusta ja siirrytaan ylemmalle tasolle.
	same(T,Element1),
	unzoom([[LS,T,RS],[[LS1,R1,RS2]|Context]],NewCurrentPlace).


/*------------------------------------------------------------------------*/

/*
 *-----------------------------------------------------------------------------
 *         Show the current place in the tree
 *-----------------------------------------------------------------------------
 * Muokataan jasennyspuu naytolle tulostettavaan muotoon:
 *  - kayttajan sijainti puussa osoitetaan nuolella (->),
 *  - solmujen sisallot tulostetaan alkamaan positiosta 40,
 *  - puun tasot sisennetaan.
 * TreeAndSisters muuttujassa on jasennyspuun osoitettu-osa.
 * Context muuttujassa on jasennyspuun konteksti.
 * TreeChar muuttujan arvona palautetaan tulostettava jasennyspuu.
 * ListElChar muuttujan arvona on solmu, jossa kayttaja sijaitsee.
*/
setmarginals:-
    (retract(pos(A)),asserta(pos(A))
     ;
    retractall(pos(_)),		 %dokumentin sisallon marginaali	
    retractall(indent(_)),           %alipuiden sisennys
    asserta(pos(40)),		 %dokumentin sisallon marginaali	
    asserta(indent([32,32,32])),     %alipuiden sisennys
    retractall(struct(_)),		 %rakenteen marginaali	
    retractall(cons(_)),		 %constraintin marginaali
    retractall(conspos(_)),          %position marginaali
    retractall(consqua(_)),          %quatityn marginaali
    asserta(struct(14)),		 %rakenteen marginaali	
    asserta(cons(44)),		 %constraintin marginaali
    asserta(conspos(60)),            %position marginaali
    asserta(consqua(70))).            %quatityn marginaali
/*
:-asserta(pos(40)).		 %dokumentin sisallon marginaali	
:-asserta(cons(44)).		%constraintin marginaali
:-asserta(struct(14)).		%rakenteen marginaali	
:-asserta(indent([32,32,32])).   %alipuiden sisennys
:-asserta(conspos(60)).          %position marginaali
:-asserta(consqua(70)).          %quatityn marginaali
*/
 
%Pred:displayCurrentPlace
displayCurrentPlace([TreeAndSisters,Context],TreeChar,ListElChar):-
	% Lasketaan kontekstin puun tasojen lukumaara.
        length(Context,ListLength2),
	% arrow ja arrowWritten predikaatteja kaytetaan predikaatissa 
        % writearrow,
	% kun merkitaan kayttajan sijanti puussa. Nuoli osoittaa 
        % sijainnin (->).
	assert(arrow(no)),
        assert(arrowWritten(no)),
	% Ensin kaydaan lapi jasennyspuun osoitettu-osa (pointed-part).
	% Muuttujassa ListElChar palautetaan solmu, jossa 
	% kayttaja silla hetkella
	% on puussa.
	displayTreeAndSisters(TreeAndSisters,[],TreeAndSistersChar1,
		              ListElChar,ListLength2),
	ListLength1 is ListLength2-1,
	% Muokataan konteksti.
	displayContext(Context,TreeAndSistersChar1,TreeChar,ListLength1),
	retractall(arrow(_)),
	retractall(arrowWritten(_)).

% Jasennyspuun konteksti-osan muokkaus.
%Pred:displayContext
displayContext([],T,T,_).
displayContext([[[],'document',[]]],T,T,_).
displayContext([[[],'document{;;}{}',[]]],T,T,_).
displayContext([Level|RestLevels],PartOfTreeChar,TreeChar,ListLength):-
	ListLength2 is ListLength-1,
	% Kontekstia kaydaan lapi taso kerrallaan.
        displayTreeAndSisters(Level,PartOfTreeChar,
		              PartOfTreeChar1,_,ListLength),
	displayContext(RestLevels,PartOfTreeChar1,TreeChar,ListLength2).


%Pred:displayTreeAndSisters
displayTreeAndSisters([[],Root,[]],TreeAndSistersChar,TreeAndSistersChar1,RootChar,ListLength) :-
	% vain atominen juuri, ei sisarpuita, tarkistetaan etta on atomi
	atom(Root),!,name(Root,RootChar),
	% erotetaan attribuutit nimen perasta
	(member(123,RootChar),
	%append([42],_,RootChar),
	separateConstraintAndAnnotation(Root,RootChar1,Cons,Anno);
	same(RootChar,RootChar1)),
	% Asetetaan tarvittavat sisennykset (tyhjat) solmun nimen eteen.
        setIndentSpaces(ListLength,SpacesBeforeRoot),
        retractall(arrow(_)),assert(arrow(yes)),
	% Merkitaan kayttajan sijainti jasennyspuussa.
	writeArrow(RootChar1,RootWithorWithoutArrowChar,SpacesBeforeRoot,SpacesToWrite),
	% atomisen juuren kirjoitus . 
        append([10|SpacesToWrite],RootWithorWithoutArrowChar,RootLineToWrite),
	% tutkitaan onko RootChar:ssa {, jos on niin kirjoitetaan
	% constraint ja annotation oikeisiin marginaaleihin
	(member(123,RootChar),
	 calculateConsAndAnno(RootLineToWrite,Cons,Anno,RootLineToWrite1);
	 same(RootLineToWrite,RootLineToWrite1)),
	append(RootLineToWrite1,TreeAndSistersChar,TreeAndSistersChar1). 


displayTreeAndSisters([[],Tree,[]],TreeAndSistersChar,TreeAndSistersChar1,RootChar1,ListLength):-
	% puu, ei sisarpuita
	% Poistetaan sisakkaiset listat
	checkislist(Tree,Tree1),
	% on puu, erotetaan juuri
	Tree1 =..[Root|Children],!,
	(name(Root,[RootChar]),
	%puu on lista, jonka juuri on piste
	period(RootChar),
	% ensimmainen lapsi valitaan
	selectFirstChild(Children,RootChar1),
        flatten(Children,Children1),
	setIndentSpaces(ListLength,SpacesBeforeChildren),
	writeChildren(Children1,ChildrenLinesChar,SpacesBeforeChildren,ListLength),
	append(TreeAndSistersChar,ChildrenLinesChar,TreeAndSistersChar1)
	;
	name(Root,RootChar),
	%puu 
	% erotetaan constraint ja annotation
	(member(123,RootChar),
%	append([42],_,RootChar),
	separateConstraintAndAnnotation(Root,RootChar1,Cons,Anno);
	same(RootChar,RootChar1)),
	% Asetetaan juuren nimen eteen tarvittavat sisennykset.
	% Alipuut sisennetaan ao. kohdissa.
	setIndentSpaces(ListLength,SpacesBeforeRoot),
	% Merkitaan kayttajan sijainti jasennyspuussa.
        retractall(arrow(_)),assert(arrow(yes)),
	writeArrow(RootChar1,RootWithorWithoutArrowChar,SpacesBeforeRoot,SpacesToWrite),
	% atomisen juuren kirjoitus . 
        append([10|SpacesToWrite],RootWithorWithoutArrowChar,RootLineToWrite),
	%maarataan kirjoitettavan tekstin pituus, jotta voidaan korjata marginaaleja
	length(RootLineToWrite,SpacesAndRootLength1),
	SpacesAndRootLength is SpacesAndRootLength1 - 1,
	docmarginal(SpacesAndRootLength),
	% tutkitaan onko RootChar:ssa {, jos on niin kirjoitetaan
	% constraint ja annotation oikeisiin marginaaleihin
	(member(123,RootChar),
	 calculateConsAndAnno(RootLineToWrite,Cons,Anno,RootLineToWrite1);
	  same(RootLineToWrite,RootLineToWrite1)),
	append(RootLineToWrite1,TreeAndSistersChar,TreeAndSistersChar2),	
	% Lasten kirjoitus .
	indent(IndentSpaces),
	append(IndentSpaces,SpacesBeforeRoot,SpacesBeforeChildren),
	writeChildren(Children,ChildrenLinesChar,SpacesBeforeChildren,SpacesAndRootLength),
	append(TreeAndSistersChar2,ChildrenLinesChar,TreeAndSistersChar1)
	).

displayTreeAndSisters([LS,Tree,RS],TreeAndSistersChar,TreeAndSistersChar1,RootChar,ListLength):-
	% Puu tai atomi, jolla on sisarpuita
	!,
	checkislist(Tree,Tree1),
	% Solmu ja sen alipuut.
	% Merkitaan kayttajan sijainti jasennyspuussa.
        retractall(arrow(_)),assert(arrow(yes)),
	displayTreeAndSisters([[],Tree1,[]],[],TreeLineChar,RootChar,ListLength),!,
	% Solmun vasemmat sisaret.
	writeLeftSisters(LS,LSistersLineChar,ListLength),!,
	% Solmun oikeat sisaret.
        writeRightSisters(RS,RSistersLineChar,ListLength),!,
	append(LSistersLineChar,TreeLineChar,S1),
	append(S1,TreeAndSistersChar,S2),
	append(S2,RSistersLineChar,TreeAndSistersChar1).

% Kaydaan lapi solmun vasemmat sisaret.
% Vasenten sisarten listassa ensimmainen sisar, joka pitaa tulostaa ensin
% on listan viimeisena. 
%Pred:writeLeftSisters
writeLeftSisters([],"",_):-!.
writeLeftSisters([L|Ls],SistersLineChar,ListLength):-!,
	writeLeftSisters(Ls,LsLineChar,ListLength),
	displayTreeAndSisters([[],L,[]],[],LLineChar,_,ListLength),
	append(LsLineChar,LLineChar,SistersLineChar).

% Kaydaan lapi solmun oikeat sisaret.
% Oikeiden sisarten listassa ensimmainen sisar, joka pitaa tulostaa ensin
% on listan alussa.
%Pred:writeRightSisters
writeRightSisters([],"",_):-!.
writeRightSisters([R|Rs],SistersLineChar,ListLength):-
        displayTreeAndSisters([[],R,[]],[],RLineChar,_,ListLength),!,
	writeRightSisters(Rs,RsLineChar,ListLength),
	append(RLineChar,RsLineChar,SistersLineChar).


% Jasennyspuun sisaret esitetaan sisallysluettelon muotoisena.
% Solmut on erotettu rivinvaihdolla ja niiden hierarkia kuvataan sisennyksilla.
% Solmun nimen eteen tulostetaan tarvittava maara tyhjia merkkeja.
% Solmujen sisallot tulostuvat sarakkeesta pos(_) alkaen.
%Pred:writeChildren
writeChildren([],"",_,_) :- !.
writeChildren([FirstSister|RestOfSisters],ChildrenChar,
               SpacesBeforeSisters,_) :-
	atom(FirstSister),!,
	% atomi
	name(FirstSister,FirstSisterChar),
	(member(123,FirstSisterChar),
	% erotetaan attribuutit nimen perasta
	separateConstraintAndAnnotation(FirstSister,FirstSisterChar1,Cons,Anno);
	same(FirstSisterChar,FirstSisterChar1)),
	%listataan nuoli tai jatetaan lisaamatta
	writeArrow(FirstSisterChar1,SisterWithorWithoutArrow,
	          SpacesBeforeSisters,SpacesToWrite),
	% Lisataan sisennykset (tyhjat) solmun nimen eteen. 
	append([10|SpacesToWrite],SisterWithorWithoutArrow,FirstSisterLineToWrite),
	% tutkitaan onko RootChar:ssa {, jos on niin kirjoitetaan
	% constraint ja annotation oikeisiin marginaaleihin
	(member(123,FirstSisterChar),
	 calculateConsAndAnno(FirstSisterLineToWrite,Cons,Anno,FirstSisterLineToWrite1)
	;
	 same(FirstSisterLineToWrite,FirstSisterLineToWrite1)),
	writeChildren(RestOfSisters,RestSistersLineChar,SpacesBeforeSisters,_),
	append(FirstSisterLineToWrite1,RestSistersLineChar,ChildrenChar).

writeChildren([FirstSister|RestOfSisters],ChildrenChar,
               SpacesBeforeSisters,SumLength) :-
	% Alipuu on lista
	 FirstSister =..[RootFirst|[RestFirst]],
	      % sisar, joka on text ja sen lapsena merkkijono
	      % text-sisar on aina ainoa lapsi
	     (smallestGramEl(Element),
	      % Tutkittavan puun juuri on tekstielementti.
	      same(RootFirst,Element),!,
	      name(RestFirst,RestFirstChar),
              % paikka josta dokumentin sisalto tulostuu
              pos(DocMargin),
	      setBetweenSpaces(DocMargin,SpacesBeforeText),
	      % Tarkistetaan, onko solmun sisallossa rivinvaihtomerkkeja.
	      checkNl(RestFirstChar,RestFirstChar1,SpacesBeforeText),
	      SumLength1 is DocMargin - SumLength,
	      setBetweenSpaces(SumLength1,SumSpaces),
	      append(SumSpaces,RestFirstChar1,ChildrenChar)
	      ;
	      % Tarkistetaan, onko puun listaesityksessa pisteita.
	      name(RootFirst,[Point]),
	      period(Point),
	      % Tarkistetaan, onko sisakkaisia listoja.
	      checkislist(RestFirst,RestFirst1),
	      writeChildren(RestFirst1,RestFirstLine,SpacesBeforeSisters,SumLength),
	      writeChildren(RestOfSisters,RestOfSistersLine,SpacesBeforeSisters,_),
	      append(RestFirstLine,RestOfSistersLine,ChildrenChar)
	      ;
	      % sisar, joka ei ole text
	      name(RootFirst,RootFirstChar),
	      % erotetaan attribuutit nimen perasta
	      (member(123,RootFirstChar),
	       separateConstraintAndAnnotation(RootFirst,RootFirstChar1,Cons,Anno);
	       same(RootFirstChar,RootFirstChar1)),
	      % Merkitaan kayttajan sijainti.
	      writeArrow(RootFirstChar1,RootWithorWithoutArrow,
	                SpacesBeforeSisters,SpacesToWrite),
	      % Lisataan solmun nimen eteen rivivaihtomerkki ja tyhjia.
              % Solmun alipuu sisennetaan alkamaan kolme merkkia oikelle.
	      indent(IndentSpaces),
	      append([10|SpacesToWrite],RootWithorWithoutArrow,
		      RootFirstLineToWrite),
	      length(RootFirstLineToWrite,SumLength2),
	      SumLength1 is SumLength2 - 1,
	      append(IndentSpaces,SpacesBeforeSisters,SpacesToWrite1),
	      % Poistetaan sisakkaiset listat.
	      checkislist(RestFirst,RestFirst1),
	      % tutkitaan onko RootFirstChar:ssa {, jos on niin kirjoitetaan
	      % constraint ja annotation oikeisiin marginaaleihin
	      (member(123,RootFirstChar),
	       calculateConsAndAnno(RootFirstLineToWrite,Cons,Anno,RootFirstLineToWrite1)
	      ;
	       same(RootFirstLineToWrite,RootFirstLineToWrite1)),
	      writeChildren([RestFirst1],RestFirstLines,
	                    SpacesToWrite1,SumLength1),
	      append(RootFirstLineToWrite1,RestFirstLines,S1),
	      writeChildren(RestOfSisters,RestOfSistersLine,SpacesBeforeSisters,_),
	      append(S1,RestOfSistersLine,ChildrenChar)
	      ).

writeChildren([FirstSister|RestOfSisters],ChildrenChar,
               SpacesBeforeSisters,SumLength) :-
	  % Alipuu ei ole lista
	  FirstSister =..[RootFirst|RestFirst],
	      % sisar, joka on text ja sen lapsena merkkijono
	      % text-sisar on aina ainoa lapsi
	     (smallestGramEl(Element),
	      % Tutkittavan puun juuri on tekstielementti.
	      same(RootFirst,Element),!,
		name(RestFirst,RestFirstChar),
              % paikka josta dokumentin sisalto tulostuu
              pos(DocMargin),
	      setBetweenSpaces(DocMargin,SpacesBeforeText),
	      % Tarkistetaan, onko solmun sisallossa rivinvaihtomerkkeja.
	      checkNl(RestFirstChar,RestFirstChar1,SpacesBeforeText),
	      SumLength1 is DocMargin - SumLength,
	      setBetweenSpaces(SumLength1,SumSpaces),
	      append(SumSpaces,RestFirstChar1,ChildrenChar)
	      ;
	      % Tarkistetaan, onko puun listaesityksessa pisteita.
	      name(RootFirst,[Point]),
	      period(Point),
	      % Tarkistetaan, onko sisakkaisia listoja.
	      checkislist(RestFirst,RestFirst1),
	      writeChildren(RestFirst1,RestFirstLine,SpacesBeforeSisters,SumLength),
	      writeChildren(RestOfSisters,RestOfSistersLine,SpacesBeforeSisters,_),
	      append(RestFirstLine,RestOfSistersLine,ChildrenChar)
	      ;
	      % sisar, joka ei ole text
	      name(RootFirst,RootFirstChar),
	      % erotetaan attribuutit nimen perasta
	      (member(123,RootFirstChar),
	       separateConstraintAndAnnotation(RootFirst,RootFirstChar1,Cons,Anno);
	       same(RootFirstChar,RootFirstChar1)),
	      % Merkitaan kayttajan sijainti.
	      writeArrow(RootFirstChar1,RootWithorWithoutArrow,
	                SpacesBeforeSisters,SpacesToWrite),
	      % Lisataan solmun nimen eteen rivivaihtomerkki ja tyhjia.
              % Solmun alipuu sisennetaan alkamaan kolme merkkia oikelle.
	      indent(IndentSpaces),
	      append([10|SpacesToWrite],RootWithorWithoutArrow,
		      RootFirstLineToWrite),
	      length(RootFirstLineToWrite,SumLength2),
	      SumLength1 is SumLength2 - 1,
	      append(IndentSpaces,SpacesBeforeSisters,SpacesToWrite1),
	      % Poistetaan sisakkaiset listat.
	      checkislist(RestFirst,RestFirst1),
	      % tutkitaan onko RootFirstChar:ssa {, jos on niin kirjoitetaan
	      % constraint ja annotation oikeisiin marginaaleihin
	      (member(123,RootFirstChar),
	       calculateConsAndAnno(RootFirstLineToWrite,Cons,Anno,RootFirstLineToWrite1)
	      ;
	       same(RootFirstLineToWrite,RootFirstLineToWrite1)),
	      writeChildren(RestFirst1,RestFirstLines,
	                    SpacesToWrite1,SumLength1),
	      append(RootFirstLineToWrite1,RestFirstLines,S1),
	      writeChildren(RestOfSisters,RestOfSistersLine,SpacesBeforeSisters,_),
	      append(S1,RestOfSistersLine,ChildrenChar)
	      ).




%Pred:writeArrow
writeArrow(TextList,TextList2,[S,S1|S3],S2) :-
	% Merkitaan kayttajan sijainti. Solmun nimen eteen lisataan nuoli, ->.
	(arrow(yes),arrowWritten(no),!,
	 append([45,62],TextList,TextList2),
	 same(S3,S2),
	 retractall(arrow(_)),assert(arrow(no)),
	 retractall(arrowWritten(_)),assert(arrowWritten(yes))
	;
	% Jos solmu ei ole se, jossa kayttaja talla hetkella sijaitsee
	% jasennyspuussa, solmun nimen eteen lisataan tyhja merkki.
	 arrow(yes),arrowWritten(yes),!,
%	 append("",TextList,TextList2),
	 append(TextList,[],TextList2),
	 append([S],[S1],S4),
	 append(S3,S4,S2),
	 retractall(arrow(_)),assert(arrow(no))
	;
	% Jos solmu ei ole se, jossa kayttaja talla hetkella sijaitsee
	% jasennyspuussa, solmun nimen eteen lisataan tyhja merkki.
	 arrow(no),
%	 append("",TextList,TextList2),
	 append(TextList,[],TextList2),
	 append([S],[S1],S4),
	 append(S3,S4,S2)).



%Pred:docmarginal
docmarginal(SpacesAndRootLength):-
	  pos(DocMargin),SpacesAndRootLength >= DocMargin,
	  DocMargin1 is SpacesAndRootLength + 10,
          retractall(pos(_)),
	  assert(pos(DocMargin1)).
docmarginal(_):-true.

%Pred:structmarginal
structmarginal(SpacesAndRootLength):-
	  struct(SMargin),SpacesAndRootLength >= SMargin,
	  SMargin1 is SpacesAndRootLength + 5,
          retractall(struct(_)),
	  assert(struct(SMargin1)),
          Diff is SMargin1 - SMargin,
          cons(CMargin),CMargin1 is CMargin + Diff,
          retractall(cons(_)),
	  assert(cons(CMargin1)),
          conspos(PMargin),PMargin1 is PMargin + Diff,
          retractall(conspos(_)),
	  assert(conspos(PMargin1)),
          consqua(QMargin),QMargin1 is QMargin + Diff,
          retractall(consqua(_)),
	  assert(consqua(QMargin1)).
structmarginal(_):-
          true.

%Pred:consmarginal
consmarginal(SpacesAndRootLength):-
	  cons(CMargin),SpacesAndRootLength >= CMargin,
	  CMargin1 is SpacesAndRootLength + 5,
          retractall(cons(_)),           
	  assert(cons(CMargin1)),
          Diff is CMargin1 - CMargin,
          conspos(PMargin),PMargin1 is PMargin + Diff,
          retractall(conspos(_)),
	  assert(conspos(PMargin1)),
          consqua(QMargin),QMargin1 is QMargin + Diff,
          retractall(consqua(_)),
	  assert(consqua(QMargin1)).
consmarginal(_):-
          true.

%Pred:consposmarginal
consposmarginal(SpacesAndRootLength):-
	  conspos(PMargin),SpacesAndRootLength >= PMargin,
	  PMargin1 is SpacesAndRootLength + 5,
          retractall(conspos(_)),
	  assert(conspos(PMargin1)),
          Diff is PMargin1 - PMargin,
          consqua(QMargin),QMargin1 is QMargin + Diff,
          retractall(consqua(_)),
	  assert(consqua(QMargin1)).
consposmarginal(_):-
          true.

%Pred:consquamarginal
consquamarginal(SpacesAndRootLength):-
	  consqua(QMargin),SpacesAndRootLength >= QMargin,
	  QMargin1 is SpacesAndRootLength + 5,
          retractall(consqua(_)),
	  assert(consqua(QMargin1)).
consquamarginal(_):-
          true.

%Pred:calculateConsAndAnno
calculateConsAndAnno(RootLineToWrite,Cons,Anno,RootLineToWrite1):-
	% Otetaan rivinvaihto ja valilyonti pois RootLineToWrite:sta
	append([10,32],RootLineToWrite2,RootLineToWrite),
	length(Anno,L1),
	structmarginal(L1),struct(StructMargin),
	SpacesBetweenAnnoAndStruct is StructMargin - L1 - 1,
	length(RootLineToWrite2,StructLength),
	L2 is StructMargin + StructLength,
	consmarginal(L2),cons(ConsMargin),
	SpacesBetweenStructAndCons is ConsMargin - L2,
 	(notempty(Cons),
	  (nonvar(Cons),
            separateCons(Cons,Contain,Position,Quantity),
            length(Contain,ContainLength),
              L3 is ConsMargin + ContainLength, 
              consposmarginal(L3),  
              conspos(PosMargin),
            length(Position,PositionLength),
              L4 is PosMargin + PositionLength, 
              consquamarginal(L4),  
              consqua(QuaMargin),
                SpacesBetweenPositionAndQuantity is 
                        QuaMargin - PosMargin - PositionLength,
                SpacesBetweenContainAndPosition is 
                        PosMargin - ConsMargin - ContainLength,
            (notempty(Contain),
              (notempty(Position),
               (notempty(Quantity),   
             % Contain, Position, Quantity ei-tyhjia ja arvot  on sidottu
      	     	setBetweenPoints(SpacesBetweenStructAndCons,BetweenContain),
                setBetweenPoints(SpacesBetweenContainAndPosition,
                                                         BetweenPosition),
                setBetweenPoints(SpacesBetweenPositionAndQuantity,
                                                         BetweenQuantity)
                ;
                % Contain, Position ei-tyhjia, Quantity tyhja
      	     	setBetweenPoints(SpacesBetweenStructAndCons,BetweenContain),
                setBetweenPoints(SpacesBetweenContainAndPosition,
                                                         BetweenPosition),
                setBetweenSpaces(SpacesBetweenPositionAndQuantity,
                                                         BetweenQuantity)
                )
                ;
                %Contain eityhja, Position tyhja testataan Quantity
                % Quantity eityhjia
                (notempty(Quantity),
      	     	setBetweenPoints(SpacesBetweenStructAndCons,BetweenContain),
                setBetweenPoints(SpacesBetweenContainAndPosition,
                                                         BetweenPosition),
                setBetweenPoints(SpacesBetweenPositionAndQuantity,
                                                         BetweenQuantity)
                ;
                % Quantity tyhja
      	     	setBetweenPoints(SpacesBetweenStructAndCons,BetweenContain),
                setBetweenSpaces(SpacesBetweenContainAndPosition,
                                                         BetweenPosition),
                setBetweenSpaces(SpacesBetweenPositionAndQuantity,
                                                         BetweenQuantity)
                )
              )
             ;
             %Contain tyhja, testataan Position, Quantity
             % Position ja Quantity eityhjia
              (notempty(Position), 
                (notempty(Quantity),
      	     	setBetweenPoints(SpacesBetweenStructAndCons,BetweenContain),
                setBetweenPoints(SpacesBetweenContainAndPosition,
                                                         BetweenPosition),
                setBetweenPoints(SpacesBetweenPositionAndQuantity,
                                                         BetweenQuantity)
               ;
               %Position ei-tyhja, Quantity tyhja
      	     	setBetweenPoints(SpacesBetweenStructAndCons,BetweenContain),
                setBetweenPoints(SpacesBetweenContainAndPosition,
                                                         BetweenPosition),
                setBetweenSpaces(SpacesBetweenPositionAndQuantity,
                                                         BetweenQuantity)
                ) 
                ;
                %Contain,Position  tyhjia
                %testataan Quantity
                (notempty(Quantity),
      	     	setBetweenPoints(SpacesBetweenStructAndCons,BetweenContain),
                setBetweenPoints(SpacesBetweenContainAndPosition,
                                                         BetweenPosition),
                setBetweenPoints(SpacesBetweenPositionAndQuantity,
                                                         BetweenQuantity)
                
                ;                
                %Contain, Quantity ja Position  tyhjia
      	     	setBetweenSpaces(SpacesBetweenStructAndCons,BetweenContain),
                setBetweenSpaces(SpacesBetweenContainAndPosition,
                                                         BetweenPosition),
                setBetweenSpaces(SpacesBetweenPositionAndQuantity,
                                                         BetweenQuantity)
                )
              ))
            ;
	    % Cons:n arvoa ei ole maaritelty
      	     	setBetweenSpaces(SpacesBetweenStructAndCons,BetweenContain),
                setBetweenSpaces(SpacesBetweenContainAndPosition,
                                                         BetweenPosition),
                setBetweenSpaces(SpacesBetweenPositionAndQuantity,
                                                         BetweenQuantity)
                
	    )
	    ;
	        % Cons tyhja 
      	     	setBetweenSpaces(SpacesBetweenStructAndCons,BetweenContain),
                setBetweenSpaces(SpacesBetweenContainAndPosition,
                                                         BetweenPosition),
                setBetweenSpaces(SpacesBetweenPositionAndQuantity,
                                                         BetweenQuantity)
      	),
	(notempty(Anno),
	    % Annotaatio ei-tyhja
	    (setBetweenPoints(SpacesBetweenAnnoAndStruct,BetweenAnno),
	    setSpacesToPoints(RootLineToWrite2,RootLineToWrite3)
	    )
	    ;
	    % Annotaatio tyhja
      	    (setBetweenSpaces(SpacesBetweenAnnoAndStruct,BetweenAnno),
	    append([],RootLineToWrite2,RootLineToWrite3))),
	% Annotaation eteen rivinvaihto ja valilyonti
	append([10,32|Anno],BetweenAnno,S1),
	append(S1,RootLineToWrite3,S2),
	append(S2,BetweenContain,S3),
	append(S3,Contain,S4),
        append(S4,BetweenPosition,S5),
        append(S5,Position,S6),
        append(S6,BetweenQuantity,S7),
        append(S7,Quantity,RootLineToWrite1).


%Pred:separateCons
separateCons(Cons,Contain1,Position1,Quantity):-
        sepContain(Cons,[],Contain,RestCons),reverse(Contain,Contain1),
        sepPosition(RestCons,[],Position,Quantity),
        reverse(Position,Position1).

%Pred:sepContain
sepContain([],C,C,_).
sepContain([C|Cons],Cont,Cont1,RCons) :-
        (same(C,59),same(Cont,Cont1),same(Cons,RCons);
         sepContain(Cons,[C|Cont],Cont1,RCons)).

%Pred:sepPosition
sepPosition([],C,C,_).
sepPosition([C|Cons],Pos,Pos1,RCons) :-
        (same(C,59),same(Pos,Pos1),same(Cons,RCons);
         sepContain(Cons,[C|Pos],Pos1,RCons)).

 
%Pred:selectFirstChild
selectFirstChild([First|_],FirstChar):-
	name(First,FirstChar).

% Asetetaan haluttu maara tyhjia merkkeja muuttujaan Spaces.
% Asetetaan tyhjat solmun nimen eteen, sisennykset.
%Pred:setIndentSpaces
setIndentSpaces(1,IndentSpaces) :- indent(IndentSpaces),!.
setIndentSpaces(ListLength,Spaces) :-
%	ListLength > 0,
	ListLength1 is ListLength-1,
	setIndentSpaces(ListLength1,Spaces1),
	indent(IndentSpaces),
	append(Spaces1,IndentSpaces,Spaces).
% setIndentSpaces(ListLength,Spaces) :-
%	errorHandler(ListLength,Spaces).


% Asetetaan haluttu maara tyhjia merkkeja muuttujaan Spaces.
% Asetetaan tyhjat solmun nimen ja tekstin valiin.
%Pred:setBetweenSpaces
setBetweenSpaces(1,[32]) :- !.
setBetweenSpaces(ListLength,Spaces) :-
	ListLength1 is ListLength-1,
	setBetweenSpaces(ListLength1,Spaces1),
	append(Spaces1,[32],Spaces).


% Asetetaan haluttu maara pisteita muuttujaan Spaces.
% Asetetaan tyhjat solmun nimen ja tekstin valiin.
%Pred:setBetweenPoints
setBetweenPoints(1,[46]) :- !.
setBetweenPoints(ListLength,Spaces) :-
	ListLength1 is ListLength-1,
	setBetweenPoints(ListLength1,Spaces1),
	append(Spaces1,[46],Spaces).




% Muutetaan rakenteen sisennystyhjeet pisteiksi.
%Pred:setSpacesToPpoints
setSpacesToPoints(List,NewList) :-
	append([32],List2,List),
	setSpacesToPoints(List2,List3),
	append([46],List3,NewList).
setSpacesToPoints(List,NewList) :- 
	append([],List,NewList).



% Tarkistetaan sisaltaako merkkijono rivinvaihtomerkkeja.
% Jos sisaltaa, uusi rivi sisennetaan alkamaan sarakkeesta pos(_).
%Pred:checkNl
checkNl([],"",_) :- !.
checkNl([X|RestFirstChar],RestFirstChar1,TextSpaces) :-
	checkNl(RestFirstChar,S1,TextSpaces),
	(isNl(X),!,
	append([X],TextSpaces,S3),
	append(S3,S1,RestFirstChar1);
	append([X],S1,RestFirstChar1)).


% Tarkistetaan, onko rakennekielioppitiedostot samat.
%Pred:check_grams
check_grams(Gram,Gram1):-
        (same(Gram,Gram1)
        ;
	% not correct grammar file
	seen,
        call_tcl(open_msg("Structure grammars are not same! ")),
        tcl_in(call_prolog(ok)),
	!,fail).





%Pred:readGrammar_old
readGrammar_old(File,NonTerm,Text1) :-
	    % Luetaan rakennekielioppitiedostosta File dokumentin tyyppi
            % ja tekstityyppi.
	    see(File),
	    read(Type),
	    read(Text),
	    % Filtterin  tyyppi talletetaan predikaatin nonTerm arvoksi
	    % ja tekstityyppi predikaatin smallestGramEl.
	    retractall(nonTerm_r(_)),
	    retractall(smallestGramEl_r(_)),
	    assert(nonTerm_r(Type)),
	    assert(smallestGramEl_r(Text)),
	    % Tuhotaan vanhat kielioppisaannot tietokannasta.
	    retractall(rule(Type,_,_)),
	    % Luetaan uudet kielioppisaannot tietokantaan.
	    read_and_assertRules(Type),
	    seen.









