# **********************************************************************
# syndocconfig.tcl 
#
# Function:   Updates configure file .syndocdefaults. Reads values of
#             global variables from ~/.syndocdefaults or if can't
#             find if, from $SYNDOCHOME/.syndocsysdefaults. 
#
# Date:       12.12.1995, 7.3.1996
# Author:     Paula Leinonen, Toni Ronkko
# **********************************************************************
#**************************************************************************
# Global variables - declarations in file  syndocglobals.tcl  
#**************************************************************************
# **********************************************************************
# Name:      edit_defaults
# Input:     String
# Output:    -
# Function:  Reads default values from .syndocdefaults file
# **********************************************************************
proc edit_defaults helpfile {
    global SYNDOCHOME;		# Directory for syndoc program files 
    global sysfile;		# .syndocsysdefaults-file 
    global ownfile;		# .syndocdefaults-file 
    global params;		# List for keys in .syndocdefaults

    set w .form
    catch {destroy $w}
    toplevel $w
    set name "Update defaults"
    wm title $w $name
    wm iconname $w $name
    dpos $w

    frame $w.buttons
    pack  $w.buttons -side bottom -expand y -fill x -pady 2m
    button $w.buttons.read -text "System defaults" -width 12 \
			 -command "read_defaults $sysfile {$params}"
    button $w.buttons.save -text "Save and Exit" -width 12 \
			 -command "save_and_exit $w $ownfile {$params}"
    button $w.buttons.cancel -text "Cancel" -width 12 \
					    -command "destroy $w"
    button $w.buttons.help -text "Help" -width 12 \
					-command "open_help $helpfile $w"
    pack $w.buttons.read $w.buttons.help $w.buttons.save $w.buttons.cancel \
					 -side left \
					 -expand 1

    foreach i {f1 f2 f3 f4 f5 f6 f7 f_index f_ctrl f_filter
               f8 f9 f10 f11 f12 f13 f14 f15 } {
        frame $w.$i -bd 2
        entry $w.$i.entry -relief sunken -width 40
        label $w.$i.label
        pack $w.$i.entry -side right
        pack $w.$i.label -side left
    }
    $w.f1.label config -text "Working directory: "
    $w.f2.label config -text "Structure grammar files: "
    $w.f3.label config -text "Input grammar files: " 
    $w.f4.label config -text "Input program files: " 
    $w.f5.label config -text "Layout grammar files: " 
    $w.f6.label config -text "Layout program files: " 
    $w.f7.label config -text "Document files: " 

    $w.f_index.label config -text "Indexed document files: "
    $w.f_ctrl.label config -text "Index control files: "
    $w.f_filter.label config -text "Filters: "

    $w.f8.label config -text "Transformation grammar files: " 
    $w.f9.label config -text "Transformation program files: "
    $w.f10.label config -text "Output files: "
    $w.f11.label config -text "Print command: "
    $w.f12.label config -text "Default printer: "
    $w.f13.label config -text "Default editor: "
    $w.f14.label config -text "Default preview program: "
    $w.f15.label config -text "Default preview files: "

    $w.f1.entry config -textvariable SYNDOCWORK
    $w.f2.entry config -textvariable STRUCTURE
    $w.f3.entry config -textvariable IGRAMMAR
    $w.f4.entry config -textvariable IPROGRAM
    $w.f5.entry config -textvariable LGRAMMAR
    $w.f6.entry config -textvariable LPROGRAM
    $w.f7.entry config -textvariable DOCUMENT

    $w.f_index.entry config -textvariable INDEX
    $w.f_ctrl.entry config -textvariable CONTROL
    $w.f_filter.entry config -textvariable FILTER
    
    $w.f8.entry config -textvariable TGRAMMAR
    $w.f9.entry config -textvariable TPROGRAM
    $w.f10.entry config -textvariable OUTPUT 
    $w.f11.entry config -textvariable PRINTCMD
    $w.f12.entry config -textvariable PRINTER 
    $w.f13.entry config -textvariable EDITORCMD
    $w.f14.entry config -textvariable PREVPROG
    $w.f15.entry config -textvariable PREVFILE

    pack $w.f1 $w.f2 $w.f3 $w.f4 $w.f5 -side top -fill x
    pack $w.f6 $w.f7 -side top -fill x
    pack $w.f_index $w.f_ctrl $w.f_filter -side top -fill x
    pack $w.f8 $w.f9 $w.f10 $w.f11 $w.f12 -side top -fill x
    pack $w.f13 $w.f14 $w.f15 -side top -fill x

# Default values from .syndocdefaults (ownfile) or from
# .syndocsysdefaults
    if [file isfile $ownfile] {
	read_defaults $ownfile $params 
    } else {
	if [file isfile $sysfile] {
            read_defaults $sysfile $params
	} else {
	    tk_dialog .d {} "Can't find file \"$sysfile\"" {} -1 OK
	}
    }
    focus $w.f1.entry
    grab $w
}

# **********************************************************************
# Name:      read_defaults
# Input:     String List
# Output:    -
# Function:  Reads default values from .syndocdefaults file
# **********************************************************************
proc  read_defaults { file l } {

    global CONTROL INDEX FILTER

# Keys for values in file .syndocdefaults
    global SYNDOCWORK;		# Users working directory
# Default paths and extensions for files 
    global STRUCTURE IGRAMMAR IPROGRAM
    global LGRAMMAR LPROGRAM DOCUMENT TGRAMMAR TPROGRAM OUTPUT 
    global PRINTER;	# Default printer 
    global PRINTCMD;	# Default printer command 
    global EDITORCMD;	# Default editor 
    global PREVPROG;	# Default preview program 
    global PREVFILE;	# Default path and extension for output
			# files to be previewed
# Initialization of the key list
    foreach i $l {
	set $i ""
    }
    set def_file [open $file r]
    gets $def_file line
    while {![eof $def_file]} {
        foreach i $l {
            if {[lindex $line 1] == $i} {
#		set $i [lindex $line 2]
               eval $line
	    }
	}
        gets $def_file line
    }
    close $def_file
}

# **********************************************************************
# Name:      save_and_exit 
# Input:     String String List
# Output:    -
# Function:  Calls write_defaults and closes window.
# **********************************************************************
proc save_and_exit { w f l } {
    write_defaults $f $l
    initialize_defaults
    destroy $w
}


# **********************************************************************
# Name:      write_defaults
# Input:     String List
# Output:    -
# Function:  Writes values of the entries to the .syndocdefaults file
# **********************************************************************
proc  write_defaults { file l } {

    global CONTROL INDEX FILTER
    global SYNDOCHOME
    global SYNDOCWORK STRUCTURE IGRAMMAR IPROGRAM
    global LGRAMMAR LPROGRAM DOCUMENT TGRAMMAR TPROGRAM OUTPUT 
    global PRINTER PRINTCMD EDITORCMD PREVPROG PREVFILE

    set def_file [open $file w]
    puts $def_file "# .syndocdefaults"

# In loop for each global variable are made own line in the file.
# For example: set SYNDOCWORK "~/mysyndoc"
    foreach i $l {
	set line "set "            ; #set
	append line "$i \""        ; #set SYNDOCWORK "
	set var "\$$i"
	append line [expr $var]    ; #set SYNDOCWORK "~/mysyndoc
	append line "\""           ; #set SYNDOCWORK "~/mysyndoc"

        puts $def_file $line 
    }
    close $def_file
}


