#************************************************************************
# syndocfiltertree.tcl
# 
# Function:   Creates a top-level window and loads a filter 
#	      (copy of syndoctree.tcl)
# Date:       10.5.1996
# Author:     Paula Leinonen
#	      Toni Ronkko
# Note!
# - This file is mostly copied from syndicinput.tcl, slightly modified by TR.
#**************************************************************************

#**************************************************************************
# Global variables - declarations in file  syndocglobals.tcl  
#**************************************************************************

#    set wi .win2;		# Name of the window 
    set filterhelp_file filter.hlp; # Name of the helpfile 

    set ask_ftitle "Add simple filter to the compound filter"

#    set rulewin .rulewin; # window for selecting rule 
#    set e .editor;        # text-widget


# *******************************************************************
# *******************************************************************
# ******************** M A I N   W I N D O W ************************
# *******************************************************************
# *******************************************************************  

# ************************************************************************
# Name:       make_filtertree
# Input:      String (filter tree in ascii form) 
#	      String (title of the window in ascii form)
# Output:     -
# Function:   Creates text-widget for the filter in the tree-mode.
# ************************************************************************
proc make_filtertree {filterChar title} {
     global wi 
     global wf2
     global filterhelp_file
     global SYNDOCHOME			;# Path for syndoc bitmap files
     					;# (syndoc_arrowup, syndoc_arrowdown)
     global filtText			;# Parse tree in the text widget 
     					;# (in ASCII)
     global rowNum			;# Number of the current row in the 
					;# text of the text widget
    set wi .winf
    set wf2 wf2


    set filtText $filterChar
    set rowNum 1
    catch {destroy $wi}
    toplevel $wi
    dpos $wi
    wm title $wi $title
    wm iconname $wi $title

    set wf iframe2
    frame $wi.$wf2 -borderwidth 1m
    pack $wi.$wf2 -side top -fill both -expand yes ;# -anchor nw -expand yes 

# Button-frames:
    frame $wi.$wf2.b1 -borderwidth 2 -relief sunken 
    frame $wi.$wf2.b2 -borderwidth 2 -relief sunken
    frame $wi.$wf2.b3 -borderwidth 2 -relief sunken
    frame $wi.$wf2.b4 -borderwidth 2 -relief sunken
    frame $wi.$wf2.b6 -borderwidth 2 -relief sunken
    frame $wi.$wf2.b2.l
    frame $wi.$wf2.b2.r
    frame $wi.$wf2.b3.l
    frame $wi.$wf2.b3.r
#    frame $wi.$wf2.b4.l
    frame $wi.$wf2.b4.r
    frame $wi.$wf2.b6.l
    frame $wi.$wf2.b6.r

#Up-arrow-button:
    button $wi.$wf2.b1.arrowup -bitmap @$SYNDOCHOME/syndoc_arrowup\
	       -command "p filter_up"
#Down-arrow-button:
    button $wi.$wf2.b1.arrowdown -bitmap @$SYNDOCHOME/syndoc_arrowdown\
	       -command "p filter_down"

#Zoom-button:
    button $wi.$wf2.b2.l.zoom -text Zoom \
	       -command "p filter_zoom" \
	       -width 8 
#Unzoom-button:
    button $wi.$wf2.b2.l.unzoom -text "Unzoom" \
	       -command "p filter_unzoom" \
	       -width 8

#Left-button(Parent):
    button $wi.$wf2.b3.l.left -text Parent -command "p filter_parent" -width 8 
#Child-button:
    button $wi.$wf2.b3.l.child -text Child -command "p filter_child" -width 8

#Up-button(Previous):
    button $wi.$wf2.b3.r.up -text Previous -command "p filter_left" -width 8
#Down-button (Next):
    button $wi.$wf2.b3.r.down -text Next -command "p filter_right" -width 8

#Next-button:
    button $wi.$wf2.b4.r.constr -text "Constraints" \
        -command "p filter_constraints
                  $wi.$wf2.b4.r.constr configure -state disabled" \
        -width 8 
#Delete-button:
    button $wi.$wf2.b4.r.annot -text "Annotations" \
        -command "p filter_annotations" \
        -width 8 

#Add filter
    button $wi.$wf2.b2.r.add -text "Filter Add" \
        -command "p filter_addfilter" \
        -width 8 
#Delete filt
    button $wi.$wf2.b2.r.del -text "Filter Delete" \
        -command "p filter_delfilter" \
        -width 8 

#Help-button:
    button $wi.$wf2.b6.l.help -text Help \
	    -command "open_help $filterhelp_file $wi" \
            -width 8
#Save-button:
    button $wi.$wf2.b6.l.save -text Save -command "p filter_save" \
            -width 8
#Exit-button:
    button $wi.$wf2.b6.r.exit -text Exit -command "p filter_save_exit
					destroy $wi" \
            -width 8
#Cancel-button:
    button $wi.$wf2.b6.r.cancel -text Cancel \
        -command "p filter_cancel; destroy $wi" -width 8



    frame $wi.$wf2.tf -borderwidth 5
    text $wi.$wf2.tf.t -yscrollcommand "$wi.$wf2.tf.sy set" \
	              -xscrollcommand "$wi.$wf2.tf.sx set" \
		      -wrap none -state normal
    scrollbar $wi.$wf2.tf.sy -command "$wi.$wf2.tf.t yview"
    scrollbar $wi.$wf2.tf.sx -orient horizontal -command "$wi.$wf2.tf.t xview"           

#    pack $wi.$wf2 -fill x -side bottom;                  # kehys (iframe)  
    pack $wi.$wf2.tf -fill both -expand yes ;#-anchor nw -side top -expand yes;# kehys tekstikentalle
    pack $wi.$wf2.tf.sy -side right -fill y;      # scrollbar
    pack $wi.$wf2.tf.sx -side bottom -fill x;     # scrollbar
    pack $wi.$wf2.tf.t -fill both -side top -expand yes;    # teksti (message)

# Pack for buttons:

    pack $wi.$wf2.b1 -side left -padx 4 -pady 4 -anchor s ;#-expand yes
    pack $wi.$wf2.b3 -side left -padx 4 -pady 4 -anchor s ;#-expand yes
    pack $wi.$wf2.b2 -side left -padx 4 -pady 4 -anchor s ;#-expand yes
    pack $wi.$wf2.b4 -side left -padx 4 -pady 4 -anchor s ;#-expand yes
    pack $wi.$wf2.b6 -side left -padx 4 -pady 4 -anchor s ;#-expand yes

    pack $wi.$wf2.b6.l -side left 
    pack $wi.$wf2.b6.r -side right

#    pack $wi.$wf2.b4.l -side left 
    pack $wi.$wf2.b4.r -side right
    pack $wi.$wf2.b4.r.constr $wi.$wf2.b4.r.annot -side top -expand yes 

    pack $wi.$wf2.b2.l -side left 
    pack $wi.$wf2.b2.r -side right

    pack $wi.$wf2.b3.l -side left 
    pack $wi.$wf2.b3.r -side right

    pack $wi.$wf2.b1.arrowup $wi.$wf2.b1.arrowdown -side top -expand yes 
    pack $wi.$wf2.b2.l.zoom $wi.$wf2.b2.l.unzoom -side top -expand yes 
    pack $wi.$wf2.b2.r.add $wi.$wf2.b2.r.del \
	  -side top -expand yes 

    pack $wi.$wf2.b3.l.left $wi.$wf2.b3.l.child -side top -expand yes 
    pack $wi.$wf2.b3.r.up $wi.$wf2.b3.r.down -side top -expand yes 

    pack $wi.$wf2.b6.l.save $wi.$wf2.b6.l.help -side top -expand yes
    pack $wi.$wf2.b6.r.cancel $wi.$wf2.b6.r.exit -side top -expand yes 
# Inserts new text
    insert_filter $filterChar 1 1 0 1 1 0 $rowNum

    focus $wi
#    grab $wi

}; #end of procedure make_tree


# ************************************************************************
# Name:       insert_filter
# Input:      String (parse tree to be inserted (ascii))
#             Integer (zoom button, 1 = enabled, 0 = disabled)
#             Integer ("constraints" button, 1 = enabled, 0 = disabled)
#             Integer ("annot" button, 1 = enabled, 0 = disabled)
#             Integer (current line)
# Output:     -
# Function:   Tree from ASCII to char. Inserts new tree to the text-
#             widget. Configure buttons to disabled or normal.
# 	      Buttons:
# 		1   zoom 
#		8   unzoom
#		10  delf
#		11  const
#		12  anno 
#		14  addf
# ************************************************************************
proc insert_filter { tree but1 but8 but10 but11 but12 but14 rows } {
   global wi 
   global wf2;	#Name of the window and the frame
   global filtText;	# Text in the text widget (ascii)
   global rowNum;	# Number of the current row in the text 
			# of the text widget

  #DBG

  
    set filtText $tree
    set rowNum $rows
   $wi.$wf2.tf.t configure -state normal
# Clear old text 
   $wi.$wf2.tf.t delete 1.0 end

# Insert new text
   # Muutetaan ascii-merkit tekstiksi
   set t [listint $tree]
   $wi.$wf2.tf.t insert end $t
#   foreach i $tree {
#       $wi.$wf2.tf.t insert end [format "%c" $i] 
#   }
# Shows the current row in the text widget
   $wi.$wf2.tf.t see $rowNum.0
   $wi.$wf2.tf.t configure -state disabled 


   if {$but1 == 0} then {
       $wi.$wf2.b2.l.zoom configure -state disabled
   } else {
       $wi.$wf2.b2.l.zoom configure -state normal
   }
   if {$but8 == 0} then {
       $wi.$wf2.b2.l.unzoom configure -state disabled
   } else {
       $wi.$wf2.b2.l.unzoom configure -state normal
   }
   if {$but10 == 0} then {
       $wi.$wf2.b2.r.del configure -state disabled
   } else {
       $wi.$wf2.b2.r.del configure -state normal
   }

   if {$but11 == 0} then {
       $wi.$wf2.b4.r.constr configure -state disabled
   } else {
       $wi.$wf2.b4.r.constr configure -state normal
   }
   if {$but12 == 0} then {
       $wi.$wf2.b4.r.annot configure -state disabled
   } else {
       $wi.$wf2.b4.r.annot configure -state normal
   }

   if {$but14 == 0} then {
       $wi.$wf2.b2.r.add configure -state disabled
   } else {
       $wi.$wf2.b2.r.add configure -state normal
   }
      
  #DBG

    
}

# *******************************************************************
# *******************************************************************
# ******************** C O N S T R A I N T S ************************
# *******************************************************************
# *******************************************************************  

# ************************************************************************
# Name:       ask_constraints
# Input:      title (string)
# Output:     -
# Function:   
# ************************************************************************

proc ask_constraints { title contain position quantity} {
  global ftypes
  global ftype_file
  global struct_file
  global wi
  global wf2
  global ready666
  global global_title
  global global_contain			;# User's contain constraint
  global global_position		;# User's postition constraint
  global global_quantity		;# User's quantity constraint
  global o_quantity 

  set global_title $title

  set global_contain [listint $contain]
  set global_position [listint $position]
  set global_quantity [listint $quantity]
  
  set o_quantity $global_quantity

  create_consw "Constraint for $title"

  .consw.apply configure -command \
      {
	p done([intlist $global_contain],[intlist $global_position],[intlist $global_quantity])
	grab release .consw
	grab $wi
	destroy .consw
      }

  .consw.cancel configure -command \
      {
        p done([intlist cancel],[intlist cancel],[intlist cancel])
	grab release .consw
	grab $wi
	destroy .consw
      }

  #
  # Ask quantity constraint
  #  

  frame .consw.b.g_frame
  label .consw.b.g_frame.lab -width 22 -text "Quantity constraint: " -anchor w

  entry .consw.b.g_frame.entr -relief sunken \
  	-textvariable global_quantity -width 30   
  button .consw.b.g_frame.sel -width 8 -text "" -relief flat -state disabled
  pack .consw.b.g_frame.sel -side right
  pack .consw.b.g_frame.lab -side left
  pack .consw.b.g_frame.entr -side right -fill x -expand yes -anchor w

  
  pack .consw.b -side top -fill x -anchor n
  pack .consw.b.d_frame -side top -fill x -anchor nw
  pack .consw.b.f_frame -side top -fill x -anchor w
  pack .consw.b.g_frame -side top -fill x -anchor w
  pack .consw.apply .consw.help .consw.cancel -side left -expand yes -anchor n


  tkwait visibility .consw
  grab release $wi
  grab .consw
  focus .consw
  $wi.$wf2.b4.r.constr configure -state normal 
  tkwait window .consw
}

# ************************************************************************
# Name:       ask_constraints1
# Input:      title (normal string)
#	      contain 
# Output:     -
# Function:   
# ************************************************************************

proc ask_constraints1 { title contain position} {
  global ftypes
  global ftype_file
  global struct_file
  global wi
  global wf2
  global ready666
  global global_contain
  global global_position
  global global_title
  
  set global_title $title

  if {[winfo exists .consw] == 1} {
      tkwait window .consw 
  }

  set global_contain [listint $contain]
  set global_position [listint $position]
  
  create_consw "Constraint for $title"

  .consw.apply configure -command \
      {
	p done([intlist $global_contain],[intlist $global_position])
	grab release .consw
	grab $wi
	destroy .consw
      }

  .consw.cancel configure -command \
      {
        # Return "cancel"
        p done([intlist cancel],[intlist cancel])
	grab release .consw
	grab $wi
	destroy .consw
      }


  pack .consw.b -side top -fill x
  pack .consw.b.d_frame -side top -fill x -anchor nw
  pack .consw.b.f_frame -side top -fill x -anchor w 
  pack .consw.apply .consw.help .consw.cancel -side left -expand yes -anchor n

  tkwait visibility .consw 
  grab release $wi
  grab .consw
  focus .consw
  $wi.$wf2.b4.r.constr configure -state normal 
  tkwait window .consw
}



# ************************************************************************
# Name:       create_consw
# Input:      -
# Output:     -
# Function:   Create window with common elements for asking constraints.
# ************************************************************************
proc create_consw { title } {
  global filterhelp_file
  global wi
  global ready666
  global global_title
  global global_contain
  global global_position
  global global_quantity
  catch { destroy .consw}
  global ftypes
  global ftype_file
  global struct_file
  set ready666 0

  #
  # Make window
  #
  if {[winfo exists .consw] == 1} {
    tkwait window .consw
  }

  toplevel .consw
  dpos .consw

  wm title .consw $title
  wm iconname .consw $title
  frame .consw.b -borderwidth 10
 
  #
  # ask contain constraint
  #
  frame .consw.b.d_frame
  label .consw.b.d_frame.lab -width 22 -text "Containment constraint: " -anchor w

  entry .consw.b.d_frame.entr -relief sunken -textvariable global_contain -width 30   
  button .consw.b.d_frame.sel -text Select \
            -width 8  \
	    -command \
	    { 
	      #
	      # Comment line below and uncomment lines below it to make
	      # operational. 
	      # NOTE!
	      # - tcl_searchNonterminals does not currently work.
	      # - Prolog does not seem to get commands posted here. To correct
	      #   the problem I suggest that you should pass the list of
	      #   matching nonterminals as a parameter to this function.
	      # - Be sure not to modify ftypes variable here!
	      #
              tk_dialog .dlg "Function is not implemented." \
  "When implemented this window will show a list of valid text types." {} -1 OK
#	      puts "Entering prolog tcl_searchNonTerminals."
#	      set ready666 0
#	      p tcl_searchNonterminals
#  		tkwait variable ready666
#	      puts "prolog tcl_searchNonTerminals has been done."	      
#	      set global_contain "$global_contain [ create_browser_list $ftypes $global_contain]"
#	      moveto_end .consw.b.d_frame.entr
	    }

  pack .consw.b.d_frame.sel -side right
  pack .consw.b.d_frame.lab -side left
  pack .consw.b.d_frame.entr -side left -fill x -expand yes -anchor w
  
  #
  # Ask position constraint
  #  

 
  frame .consw.b.f_frame
  label .consw.b.f_frame.lab -width 22 -text "Position constraint: " -anchor w

  entry .consw.b.f_frame.entr -relief sunken -textvariable global_position -width 30
  button .consw.b.f_frame.sel -width 8 -text "" -relief flat -state disabled
  pack .consw.b.f_frame.sel -side right
  pack .consw.b.f_frame.lab -side left
  pack .consw.b.f_frame.entr -side left -fill x -expand yes -anchor w



  #
  # Make buttons
  #
  button .consw.apply -width 8 -text Apply
  
  button .consw.help -text Help \
  	-command "open_help $filterhelp_file .afiltw" -width 8
    
  button .consw.cancel -text Cancel -width 8

}

# *******************************************************************
# *******************************************************************
# ******************** M I S C **************************************
# *******************************************************************
# *******************************************************************  


# ************************************************************************
# Name:       notreachable nonterminals 
# Input:      string (title of window)
#	      list of strings (list of not reachables)
# Output:     -
# Function:   show err msg.
# ************************************************************************

proc nonterminals { title list} {

  set t "For text type: [join [listint $title]]"
  set s "Text type(s):\n\n[join [listint $list] \n]\n\nnot valid!"
   tk_dialog .d $t $s {} -1 OK
   p ok
 }

# *******************************************************************
# *******************************************************************
# ******************** A D D   F I L T E R **************************
# *******************************************************************
# *******************************************************************  

# ************************************************************************
# Name:       ask_filter
# Input:      string (filter file)
#	      string (filter type)
# Output:     -
# Function:   Ask filter to add or filter type.
# ************************************************************************
proc ask_filter { ffile ftype} {  
  global filterhelp_file		;# helpfile
  global wi				;# Window that currently has a grab 
  global ask_ftitle			;# Caption
  global filts				;# Default path to filters
  global m_ffile			;# User's filterfile
  global ftypes				;# List of all nonterminals
  global ftype_file			;# User's nonterminal

  set ftype_file $ftype
  catch { destroy .afiltw}
  set m_ffile $ffile

  #
  # Make window
  #
  toplevel .afiltw
  dpos .afiltw

  wm title .afiltw $ask_ftitle
  wm iconname .afiltw $ask_ftitle
  frame .afiltw.b -borderwidth 10
  
  #
  # Make ordinary fields
  #
  frame .afiltw.b.d_frame 
  label .afiltw.b.d_frame.lab -width 20 -text " Filter file: " -anchor w

  entry .afiltw.b.d_frame.entr -relief sunken -textvariable m_ffile -width 30   
  button .afiltw.b.d_frame.sel -text Select \
            -width 8  -command \
	    { 
	      set m_ffile [create_browser $filts $m_ffile]
	      moveto_end .afiltw.b.d_frame.entr
	    }
  pack .afiltw.b.d_frame.sel -side right
  pack .afiltw.b.d_frame.lab -side left
  pack .afiltw.b.d_frame.entr -side left -fill x -expand yes -anchor w
  
  
  
  frame .afiltw.b.e_frame
  label .afiltw.b.e_frame.lab -width 20 -text "or" -anchor w
  pack .afiltw.b.e_frame.lab -side left
  
  
  
  frame .afiltw.b.f_frame
  label .afiltw.b.f_frame.lab -width 20 -text " Filter type: " -anchor w

  entry .afiltw.b.f_frame.entr -relief sunken -textvariable ftype_file -width 30   
  button .afiltw.b.f_frame.sel -text Select \
            -width 8  -command \
	    { 
	      # Note!
	      # - List of all nonterminals is currently stored in ftypes 
	      #   variable. Variable is set once when opening or creating a
	      #   new filter. If you are creating a subset of that list,
	      #   you should not overwrite ftypes variable.
	      # - Prolog does not seem to get commands posted here. To correct
	      #   the problem I suggest that you should pass the list of
	      #   matching nonterminals as a parameter to this function.
	      #   (Currently there is no need to call prolog).
	      set ftype_file [ create_browser_list $ftypes $ftype_file]
	      moveto_end .afiltw.b.f_frame.entr
	    }
  pack .afiltw.b.f_frame.sel -side right
  pack .afiltw.b.f_frame.lab -side left
  pack .afiltw.b.f_frame.entr -side left -fill x -anchor w -expand yes
  
  #
  # Make buttons
  #
  button .afiltw.go -width 8 -text Apply -command \
      {
	p done('$m_ffile','$ftype_file')
	grab release .afiltw
	grab $wi
	destroy .afiltw
      }

  button .afiltw.help -text Help -command "open_help $filterhelp_file .afiltw" -width 8
    
  button .afiltw.done -text Cancel -width 8 -command \
      {
        #TR230896: cancel palauttaa "cancel" ja ""
	p done('cancel','*')
	grab release .afiltw
	grab $wi
	destroy .afiltw
      }

  #
  # Show window
  #
  pack .afiltw.b -side top -anchor n -fill x
  pack .afiltw.b.d_frame -side top -anchor nw -fill x

  pack .afiltw.b.e_frame -side top -anchor w -fill x

  pack .afiltw.b.f_frame -side top -anchor w -fill x

  pack .afiltw.go .afiltw.help .afiltw.done -side left -expand yes -anchor n
  grab release $wi
  grab .afiltw
  focus .afiltw.b.d_frame.lab  
  #
  # Wait until user selects something.
  #
  tkwait window .afiltw
}


# *******************************************************************
# *******************************************************************
# ******************** A N N O T A T I O N **************************
# *******************************************************************
# *******************************************************************  

# ************************************************************************
# Name:       ask_annotation
# Input:      list of numbers (title of window)
#             list of numbers (old annotations)
# Output:     (annotation, prolog only!)
# Function:   Modify (possibly new) annotation. This function is called
#		from prolog and it should return legal annotation to prolog
#		in every possible case.
#	      Note!
#	      - Be sure to return a grab before exit.
# ************************************************************************
proc ask_annotation { title annot} {
  global filterhelp_file 
  global wi				;# Grab holder
  global annote				;# User's annotation
  global orig_annot			;# Original annotation (for cancel)

  grab release $wi
  
  catch { destroy .annotw}
  toplevel .annotw
  dpos .annotw

  set title [listint $title]
  
  wm title .annotw $title
  wm iconname .annotw $title
  frame .annotw.b -borderwidth 10
    
  #
  # Make text fields (1)
  #
  set annote [listint $annot]
  set orig_annot $annote

  frame .annotw.b.d_frame
  label .annotw.b.d_frame.lab -width 20 -text "Annotation: " -anchor w

  entry .annotw.b.d_frame.entr -relief sunken -textvariable annote -width 30

  pack .annotw.b.d_frame.lab -side left
  pack .annotw.b.d_frame.entr -side right -anchor w -fill x -expand yes
  
  #
  # Make buttons
  #
  button .annotw.go -width 8 -text Apply -command \
      {
	p done([intlist $annote])
	grab release .annot
	grab $wi
	destroy .annotw
      }

  button .annotw.help -text Help -command "open_help $filterhelp_file .annotw" -width 8

  button .annotw.done -text Cancel -width 8 -command \
      {
        p done([intlist $orig_annot])
	grab release .annot
	grab $wi
	destroy .annotw
      }


  #
  # All done, show window
  #
  pack .annotw.b -side top -anchor n -fill x
  pack .annotw.b.d_frame -side top -fill x -anchor n
  pack .annotw.go .annotw.help .annotw.done -side left -expand yes -anchor n
  grab .annotw
  focus .annotw
  tkwait window .annotw
}


# *******************************************************************
# *******************************************************************
# ******************** G E N E R A L ********************************
# *******************************************************************
# *******************************************************************  
#
# Few general purpose routines. Currently not much used.
#

# ************************************************************************
# Name:       intlist
# Input:      string 
# Output:     list of integers
# Function:   Convert ascii string into a list of integers.
#	      Examples:
#		abba => [97,98,98,97]
# ************************************************************************
proc intlist { str} {
  set list {}

  while { $str != ""} {
    scan $str %c t
    lappend list $t
    set str [string range $str 1 [string length $str]]
  }
  
  return \[[join $list ,]\]
}
# ************************************************************************
# Name:       listint
# Input:      list of integers 
# Output:     string
# Function:   Convert a list of integers into ascii string.
#  	      Examples:
#		{33 65 65 33} => !AA!
# ************************************************************************

proc listint { list} { 
  set t {}
  foreach i $list {
      append t [format "%c" $i]
  }
  return $t
}


