# *********************************************************************
# syndocgrammars.tcl 
#
# Function:   Creates top-level windows for SYNDOC programs grammars-  
#             functions. Creates browser for files in directory. 
# Date:       9.11.1995
# Author:     Paula Leinonen
# *********************************************************************
#
#**************************************************************************
# Global variables - declarations in file  syndocglobals.tcl  
#**************************************************************************

# *******************************************************************
# Name:     edit_structure
# Input:    String (help file)
# Output:   -
# Function: Creates top-level window for asking input grammar file
# *******************************************************************
proc edit_structure helpfile {
    global browse 
    global struct 
    global w 
    global struct_file
    global id;		# Process id, when editor is started

    set id 0
    set wi iframe
    catch {destroy $w}
    toplevel $w
    dpos $w
    wm title $w "Create/Edit structure grammar"
    wm iconname $w "Create/Edit structure grammar"

    frame $w.$wi -bd 1m
	frame $w.$wi.i -bd 2m
	label $w.$wi.i.label_gram -width 24 -text "Grammar file name" \
				  -anchor w 
	entry $w.$wi.i.entry_gram -relief sunken \
                        	  -textvariable struct_file 
	button $w.$wi.i.browse -text $browse \
	    -command { set struct_file [create_browser $struct $struct_file]
                       moveto_end $w.iframe.i.entry_gram }

    pack $w.$wi -fill x
    pack $w.$wi.i.label_gram -side left 
    pack $w.$wi.i -fill x
    pack $w.$wi.i.entry_gram $w.$wi.i.browse -side left 

    button $w.go -text Go \
      -command {$w configure -cursor watch 
		set file [def_dir $struct_file $struct]
                set struct_file $file
                moveto_end $w.iframe.i.entry_gram
		if {[string length $file] > 0} {
		    create_edit_file $file
                $w configure -cursor arrow 
                }
               }\
      -width 8

    button $w.done -text Done -command {close_window $w} -width 8
    button $w.help -text Help -command "open_help $helpfile $w" -width 8
    pack $w.go $w.help $w.done -side left -expand yes 

    moveto_end $w.iframe.i.entry_gram
    grab $w
}

# *********************************************************************
# Name:     create_edit_file
# Input:    String (new file to be edited)
# Output:   -
# Function: Executes unix editor command without checking if file is
#           a regular file (for new files).
# *********************************************************************
proc create_edit_file {file} {
    global id;		# Process id, when editor is started
    global env;         # Array of environment variables
    global EDITORCMD;   # Default editor

    if {[file isfile $file] == 0} {
	 if {[file isdirectory [file dirname $file]]} { 
	     if {[string compare $EDITORCMD ""] != 0} {
	         set id [eval exec $EDITORCMD $file &]
       	     } elseif [info exists env(EDITOR)] {
	         set id [eval exec $env(EDITOR) $file &]
             } else {
                 set id [exec vi $file &]
             }
         } else {
	     tk_dialog .d {} "Default path \"$file\" isn't a regular directory!" {} -1 OK
	 }
    } else {
	edit_file $file
    }
}

# *********************************************************************
# Name:     create_inputgram
# Input:    String (help file)
# Output:   -
# Function: Creates top-level window for asking structure grammar file 
#           and input grammar file
# *********************************************************************
proc create_inputgram helpfile {
    global browse struct grams
    global w
    global gram_file struct_file
    global id;		# Process id, when editor is started

    set id 0
    set wi iframe
    catch {destroy $w}
    toplevel $w
    dpos $w
    wm title $w "Create input grammar" 
    wm iconname $w "Create input grammar" 

    frame $w.$wi -bd 1m
	frame $w.$wi.i -bd 2m
	label $w.$wi.i.label1 -width 24 -text "Structure grammar file" \
				  -anchor w 
	entry $w.$wi.i.entry1 -relief sunken \
                	      -textvariable struct_file
	button $w.$wi.i.browse -text $browse \
	    -command { set struct_file [create_browser $struct $struct_file]
                       moveto_end $w.iframe.i.entry1 }

    frame $w.$wi.s -bd 2m
	label $w.$wi.s.label2 -width 24 -text "Input grammar file "\
				  -anchor w 
	entry $w.$wi.s.entry2 -relief sunken \
	    -textvariable gram_file
	button $w.$wi.s.browse -text $browse -command { 
		 set gram_file [create_browser $grams $gram_file]
                       moveto_end $w.iframe.s.entry2 }

    pack $w.$wi -fill x
    pack $w.$wi.i.label1 -side left
    pack $w.$wi.i -fill x
    pack $w.$wi.i.entry1 $w.$wi.i.browse -side left
    pack $w.$wi.s.label2 -side left 
    pack $w.$wi.s -fill x
    pack $w.$wi.s.entry2 $w.$wi.s.browse -side left 

    button $w.go -text Go -command {
		   make_new_gram $struct_file $gram_file $grams
                   moveto_end $w.iframe.s.entry2} \
			  -width 8
    button $w.done -text Done -command {close_window $w} -width 8
    button $w.help -text Help -command "open_help $helpfile $w" -width 8
    pack $w.go $w.help $w.done -side left -expand yes 

    moveto_end $w.$wi.i.entry1
    moveto_end $w.$wi.s.entry2
    grab set $w
}


# *******************************************************************
# Name:     edit_inputgrammar 
# Input:    String (help file)
# Output:   -
# Function: Creates top-level window for asking input grammar file
#           and calls edit_file procedure
# *******************************************************************
proc edit_inputgrammar helpfile {
    global browse grams 
    global w 
    global gram_file
    global id;		# Process id, when editor is started

    set id 0
    set wi iframe
    catch {destroy $w}
    toplevel $w
    dpos $w
    wm title $w "Create/Edit structure grammar"
    wm iconname $w "Create/Edit structure grammar"

    frame $w.$wi -bd 1m
	frame $w.$wi.i -bd 2m
	label $w.$wi.i.label_gram -width 24 -text "Grammar file name" \
				  -anchor w
	entry $w.$wi.i.entry_gram -relief sunken \
	    -textvariable gram_file 
	button $w.$wi.i.browse -text $browse \
	    -command { set gram_file [create_browser $grams $gram_file]
                       moveto_end $w.iframe.i.entry_gram }

    pack $w.$wi -fill x
    pack $w.$wi.i.label_gram -side left 
    pack $w.$wi.i -fill x
    pack $w.$wi.i.entry_gram $w.$wi.i.browse -side left 

    button $w.go -text Go \
      -command {edit_file $gram_file} -width 8

    button $w.done -text Done -command {close_window $w} -width 8
    button $w.help -text Help -command "open_help $helpfile $w" -width 8
    pack $w.go $w.help $w.done -side left -expand yes 

    moveto_end $w.iframe.i.entry_gram
    grab $w
}
   

# *********************************************************************
# Name:     create_layoutgram
# Input:    String (help file)
# Output:   -
# Function: Creates top-level window for asking input grammar file and
#            layout grammar file
# *********************************************************************
proc create_layoutgram helpfile {
    global browse struct layouts 
    global w
    global struct_file layoutgram_file
    global id;		# Process id, when editor is started

    set id 0
    set wi iframe
    catch {destroy $w}
    toplevel $w
    dpos $w
    wm title $w "Create layout grammar" 
    wm iconname $w "Create layout grammar" 

    frame $w.$wi -bd 1m
	frame $w.$wi.i -bd 2m
	label $w.$wi.i.label1 -width 24 -text "Structure grammar file" \
			      -anchor w
	entry $w.$wi.i.entry1 -relief sunken \
	    -textvariable struct_file 
	button $w.$wi.i.browse -text $browse \
	    -command { set struct_file [create_browser $struct $struct_file]
                       moveto_end $w.iframe.i.entry1 }

    frame $w.$wi.s -bd 2m
	label $w.$wi.s.label2 -width 24 -text "Layout grammar file " \
			      -anchor w 
	entry $w.$wi.s.entry2 -relief sunken \
	    -textvariable layoutgram_file
	button $w.$wi.s.browse -text $browse -command { 
		 set layoutgram_file [create_browser $layouts $layoutgram_file]
                       moveto_end $w.iframe.s.entry2 }

    pack $w.$wi -fill x
    pack $w.$wi.i.label1 -side left
    pack $w.$wi.i -fill x
    pack $w.$wi.i.entry1 $w.$wi.i.browse -side left
    pack $w.$wi.s.label2 -side left 
    pack $w.$wi.s -fill x
    pack $w.$wi.s.entry2 $w.$wi.s.browse -side left 

    button $w.go -text Go -command { check_name $layoutgram_file 
                                    moveto_end $w.iframe.s.entry2} \
                           -width 8
    button $w.done -text Done -command {close_window $w} -width 8
    button $w.help -text Help -command "open_help $helpfile $w" -width 8
    pack $w.go $w.help $w.done -side left -expand yes 

    moveto_end $w.iframe.i.entry1
    moveto_end $w.iframe.s.entry2
    grab set $w
}

# *******************************************************************
# Name:     check_name 
# Input:    String (layout grammar file)
# Output:   -
# Function: Checks if the name of the new layout grammar file is in
#           the form name_type.extension (e.g. article_html.layoutgram) 
# *******************************************************************
proc check_name {lf} {
    global layoutgram_file struct_file layouts

    set type  [string trimright $lf [file extension $layoutgram_file]]
    if {[string first \_ $type] < 0} {
        set type ""
    }
    set type [string range $type [expr [string first \_ $type] + 1] end]

    if {[string length $type] > 0} {

# Is given structure grammar a regular file
         set f1 $struct_file
         set f2 $layoutgram_file
         set path $layouts
         set f1 [file dirname $f1]/[file tail $f1] 
         if [file isfile $f1] {
# If no path in the name of the directory, sets default path 
           set f2 [def_dir $f2 $path]
           if {[string length $f2] > 0} {
	      set layoutgram_file $f2
              set file [open $f2 w]
# Name of the structure grammar file to the new grammar file
              set line '
              append line [file tail $f1]
              append line '.
              puts $file $line

              close $file
# Contents of the structure grammar file to the new grammar file
              exec cat $f1 >> $f2
              edit_file $f2
	  }
      } else {
	   tk_dialog .d {} "\"$f1\" isn't a regular file!" {} -1 OK
      }
    } else {
        tk_dialog .d {} "Give layout file name in a form: grammar_type.extension. E.g. article_html.layoutgram " {} -1 OK
    }

}


# *******************************************************************
# Name:     edit_layoutgram 
# Input:    String (help file)
# Output:   -
# Function: Creates top-level window for asking layout grammar file
#           and calls edit_file procedure
# *******************************************************************
proc edit_layoutgram helpfile {
    global browse layouts 
    global w
    global layoutgram_file
    global id;		# Process id, when editor is started

    set id 0
    set wi iframe
    catch {destroy $w}
    toplevel $w
    dpos $w
    wm title $w "Edit layout grammar"
    wm iconname $w "Edit layout grammar"

    frame $w.$wi -bd 1m
	frame $w.$wi.i -bd 2m
	label $w.$wi.i.label_gram -width 24 -text "Layout grammar file name"\
				  -anchor w
	entry $w.$wi.i.entry_gram -relief sunken \
	    -textvariable layoutgram_file 
	button $w.$wi.i.browse -text $browse \
	 -command {
	      set layoutgram_file [create_browser $layouts $layoutgram_file]
              moveto_end $w.iframe.i.entry_gram }

    pack $w.$wi -fill x
    pack $w.$wi.i.label_gram -side left 
    pack $w.$wi.i -fill x
    pack $w.$wi.i.entry_gram $w.$wi.i.browse -side left 

    button $w.go -text Go \
      -command {edit_file $layoutgram_file} -width 8

    button $w.done -text Done -command {close_window $w} -width 8
    button $w.help -text Help -command "open_help $helpfile $w" -width 8
    pack $w.go $w.help $w.done -side left -expand yes 

    moveto_end $w.iframe.i.entry_gram
    grab $w
}


# *********************************************************************
# Name:     make_new_gram
# Input:    String (structure grammar file)
#           String (new grammar file)
#           String (default path of the new file type)
# Output:   -
# Function: Creates new grammar file with structure grammar file
#           name in the beginning. 
# *********************************************************************
proc make_new_gram {f1 f2 path} { 
    global gram_file 

# Is given structure grammar a regular file
     set f1 [file dirname $f1]/[file tail $f1] 
     if [file isfile $f1] {
# If no path in the name of the directory, sets default path 
        set f2 [def_dir $f2 $path]
        if {[string length $f2] > 0} {
	    set gram_file $f2
            set file [open $f2 w]
# Name of the structure grammar file to the new grammar file
            set line '
            append line [file tail $f1]
            append line '.
            puts $file $line

            close $file
# Contents of the structure grammar file to the new grammar file
            exec cat $f1 >> $f2
            edit_file $f2
	}
    } else {
	 tk_dialog .d {} "\"$f1\" isn't a regular file!" {} -1 OK
    }
}














