/*----------------------------------------------------------------------------
 * syndocinput.pl
 *  User can write a new document or modify old one.
 *----------------------------------------------------------------------------
*/
/*--------------------------------------------------------------------------
 *  input_generate
 *--------------------------------------------------------------------------
*/
%Pred:input_generate(+Filename,+Filename)
input_generate(InputGram,Prog) :-
	    generateInput(InputGram,Prog),
	    call_tcl(prolog_done),
	    !.

/************************************************
 *
 *    generate new input program
 *
*************************************************
*/

%Pred:generateInput
generateInput(InputGram,InputProg):-
% Tcl-procedure give_name(filename) returns the name of the file
% without directorypath. Nimi kirjoitetaan input-ohjelman alkuun
% ilman polkua
        call_tcl(give_name(InputGram)), 
	tcl_in(call_prolog(done(Iname))),
	see(InputGram),
	read(StructureGram),
	read(Type),
	read(Text),
	tell(InputProg),!,
	writeq(Iname),write('.'),nl, %write grammar file without path
	writeq(StructureGram),write('.'),nl,
	writeq(Type),write('.'),nl,
	writeq(Text),write('.'),nl,
	repeat,
        ruleInput(RuleChar),
        process_input(Type,RuleChar,InputProg),
        seen,told.

%Pred:process_input
process_input(_,[end_of_file],_):-!.
process_input(_,[empty_line],_):-!,fail.
process_input(Type,RuleChar,InputProg):-
        word(RuleChar,NonTerm,_),
        append([39|RuleChar],[39],RuleChar1),
        name(Rule,RuleChar1),
        Production =..[rule,Type,NonTerm,Rule],
	writeterm(InputProg,Production),!,fail.



%Pred:word(+CharacterList,-Token,-RestCharacterList)
% Erotetaan sana (seuraavaan space-merkkiin asti) ASCII-merkkijonosta.
% Muuttujan A arvona on sana atomisessa muodossa.
% Muuttujan P1 arvona on loppuosa merkkijonosta.
%
% Sana sisaltaa isoja tai pienia kirjamia, alaviivan, ja ?-merkin.
% Palautetaan pienina kirjaimina    
word([C|P],A,P1):-
	letter(C),rest_atom(P,Cs,[P2|P1],letter_or_),
        question_mark(P2),
        append([C|Cs],[P2],C3),
	uc_to_lc(C3,L),name(A,L).
% Sana sisaltaa isoja tai pienia kirjamia, alaviivan, tahden tai
% plus-merkin.               
word([C|P],A,P1):-
	letter(C),rest_atom(P,Cs,P1,letter_or_star_or_plus),
	uc_to_lc([C|Cs],L),name(A,L).
% Sana sisaltaa yksinkertaisten lainausmerkkien valissa mita 
% tahansa merkkeja.
word([C|P],A,P1):-
	single_quote(C),!,rest_atom(P,Cs,[_|P1],any_but_single_quote),
        append([C|Cs],[C],L1),name(A,L1).
% Sana on  ->.
word([C,C2|Cs],'->',Cs):-
	hyphen(C),greater_than(C2),!.
% Sana sisaltaa vasemman sulun, mita tahansa merkkeja, oikean sulun,
%  ja *-merkin tai +-merkin. * ja + kasitellaan erilailla, +:lla
% palautetaan atominen nonterminaalinimi ja lisataan yhdistelma
% atominen nimi ja +
word([C|P],A,P1):-
	left_parenthesis(C),!,rest_atom(P,Cs,[P2,P3|P4],any_but_right_parenthesis),
	right_parenthesis(P2),
	(single_star(P3),
	append(Cs,[P3],P5),name(A,P5),
	same(P4,P1)
	;
	single_plus(P3),
	name(A,Cs),
	append(Cs,[P3],P5),
	append(P5,P4,P1)).
% Sana sisaltaa vasemman hakasulun, mita tahansa merkkeja,
% oikean hakasulun.
word([C|P],A,P1):-
	left_bracket(C),!,rest_atom(P,Cs,[P2|P1],any_but_right_bracket),
       	right_bracket(P2),
	append([C],Cs,P5),
	append(P5,[P2],P6),
	name(A,P6).
% Sana sisaltaa vasemman hakasulun sijasta ? ja mita tahansa merkkeja,
% (retrieval-merkinta optional-alkiolle
word([C|P],A,P1):-
	question_mark(C),!,rest_atom(P,Cs,P1,letter_or_),
	append([C],Cs,P5),
	name(A,P5).
% Sana sisaltaa vain lainausmerkit.
word([C|P],'empty',P):-double_quote(C),!.
% Merkkijonon ensimmainen merkki on jokin muu kuin kirjain, lainausmerkki,
% viiva, vasen sulku, vasen hakasulku.
word([_|P1],W,P2):-word(P1,W,P2).
% Sana on tyhja.
word([],'',[]) :- !.



/*--------------------------------------------------------------------------
 *  document_new
 *   Write a new document.
 *--------------------------------------------------------------------------
*/


%Pred:document_new
document_new(Doc,InputProg) :-
          setmarginals,
	    % Talletetaan dokumenttitiedoston nimi predikaatin docFile arvoksi.
	    % Tietoa tarvitaan  dokumentin tallennuksen yhteydessa.
	    assert(docFile_i(Doc)),
         % Tcl-procedure give_name(filename) returns the name of the file
         % without directorypath. Nimi kirjoitetaan filtterin alkuun
         % ilman polkua
        call_tcl(give_name(Doc)), 
	tcl_in(call_prolog(done(DocName))),
            % tehdaan tyhja tiedosto Cancel toimintaa varten
	    name(Doc,DocChar),
            append(DocChar,".tmp",TmpChar),
            name(Tmp,TmpChar),
            tell(Tmp),
            told,
            %konsultoidaan input-ohjelman predikaatit tietokantaan
            % rule(Type,Left, predikaateiksi
            % predikaatti tallentaa tietokantaan nonTerm_i(NonTerm) 
            % predikaattiin
            % dokumentin tyypin ja smallestGramEl_i(Text) merkkijonon
            % valikkeen nimen
 	    consultInputProg_new(InputProg),
	    nonTerm_i(NonTerm),
            smallestGramEl_i(Text),!,
            name(DocName,DocNameChar),
            name(NonTerm,NonTermChar),
            name('Document: ',Title0),
            name('   Document type: ',Title1),
            append(Title0,DocNameChar,Title00),
            append(Title00,Title1,Title2),
            append(Title2,NonTermChar,Title3),
            append([34|Title3],[34],TitleChar),
            name(Title,TitleChar),!,
            same(Tree,NonTerm),
	    % Tulostetaan dokumentin kasittely-ikkunaan dokumentin tyyppi. 
            assert(nonTerm(NonTerm)),
            assert(smallestGramEl(Text)),
	    displayCurrentPlace([[[],Tree,[]],[[[],'document',[]]]],
	                        TreeChar,_), %SelectionChar
            retractall(nonTerm(NonTerm)),
            retractall(smallestGramEl(Text)), 
   	    assert(currentTree([[[],Tree,[]],[[[],'document',[]]]])),
	    call_tcl(make_tree(TreeChar,Title)),
            !.
	    
%Pred:consultInputProg_new
consultInputProg_new(File) :-
	    % Luetaan inputohjelmatiedostosta File 
            % rakennekielioppitiedoston nimi, dokumentin tyyppi ja 
            % tekstityyppi.
            % assertoidaan tietokantaan koska tarvitaan ikkunan otsikossa
            % ja dokumentin tallennuksen yhteydessa
	    see(File),
            read(_),%InputGram
            read(StructureGram),
	    % rakannekieliopin nimi talletetaan predikaatin
            % structureGram_i arvoksi
            retractall(structureGram_i(_)),
            assert(structureGram_i(StructureGram)),
	    read(Type),
	    read(Text),!,
	    % Dokumentin tyyppi talletetaan predikaatin nonTerm_i arvoksi
	    % ja tekstityyppi predikaatin smallestGramEl_i.
            retractall(nonterm_i(_)),
            retractall(smallestGramEl_i(_)),
	    assert(nonTerm_i(Type)),
	    assert(smallestGramEl_i(Text)),
	    % Tuhotaan vanhat predikaatit tietokannasta.
            retractall(rule(Type,_,_)),
	    % Luetaan uudet predikaatti tietokantaan.
            read(Pred),
            consultInputProg(Pred,File),
            seen.

           
% Luetaan predikaatit rule(Type,NonTerm,Rule) 
% tietokantaan.
% Type on dokumentin tyyppi.
% NonTerm on kielioppisaannon vasemman puoleisin termi.
% Rule on kielioppisaanto.
%Pred:consultInputProg
consultInputProg(end_of_file,_):-!.
consultInputProg(Pred, File) :-
	 % Talletetaan kielioppisaanto tietokantaan.
	 assert(Pred),
	% Luetaan predikaatti tiedostosta.
	read(Pred1),
	 consultInputProg(Pred1,File).


/*--------------------------------------------------------------------------
 *  document_old
 *--------------------------------------------------------------------------
*/

%Pred:document_old
document_old(Doc,InputProg) :-
	setmarginals,
	    name(Doc,DocChar),
	    % Talletetaan dokumentin nimi predikaatin docFile arvoksi.
	    % Tietoa tarvitaan  dokumentin tallennuksen yhteydessa
	    assert(docFile_i(Doc)),
         % Tcl-procedure give_name(filename) returns the name of the file
         % without directorypath. Nimi kirjoitetaan filtterin alkuun
         % ilman polkua
        call_tcl(give_name(Doc)), 
	tcl_in(call_prolog(done(DocName))),
            % talletetaan tiedoston tieto Cancel - toimintaa varten
            append("cp ",DocChar,CmdChar1),
            append(CmdChar1,[32|DocChar],CmdChar2),
            append(CmdChar2,".tmp",CmdChar),
            name(Cmd,CmdChar),
	    call_tcl(exec(Cmd)),
            % konsultoidaan InputProg- ohjelma ja assertoidaan tietokantaan
            % rakennekielioppitiedoston nimi (structureGram_i), 
            % dokumentin tyyppi (nonTerm_i) ja
            % merkkijonovalikkeen nimi (smallestGramEl_i)
            consultInputProg_old(InputProg),
            structureGram_i(StructureGram1),
	    % Avataan dokumenttitiedosto.
	    see(Doc),
	    % Luetaan rakennekielioppitiedoston nimi, 
            %  dokumentin tyyppi ja tekstityyppi.
	    read(StructureGram),!,
            check_grams(StructureGram,StructureGram1),
            %dokumentissa ja ohjelmassa sama rakennekielioppi
	    read(NonTerm), 
            assert(nonTerm_i(NonTerm)),
	    read(Text), %Text
            assert(smallestGramEl_i(Text)),
	    % Luetaan dokumenttitiedostosta jasennyspuu.
	    read(Tree),
	    seen,!,
	    % Muokataan jasennyspuu naytolle tulostettavaan muotoon.
            name(DocName,DocNameChar),
            name(NonTerm,NonTermChar),
            name('Document: ',Title0),
            name('   Document type: ',Title1),
            append(Title0,DocNameChar,Title00),
            append(Title00,Title1,Title2),
            append(Title2,NonTermChar,Title3),
            append([34|Title3],[34],TitleChar),
            name(Title,TitleChar),!,
            assert(nonTerm(NonTerm)),
            assert(smallestGramEl(Text)),
	    displayCurrentPlace([[[],Tree,[]],[[[],'document',[]]]],
	                        TreeChar,_), %SelectionChar
            retractall(nonTerm(NonTerm)),
            retractall(smallestGramEl(Text)), 
	    assert(currentTree([[[],Tree,[]],[[[],'document',[]]]])),
            call_tcl(make_tree(TreeChar,Title)),
            !.

%Pred:consultInputProg_old
consultInputProg_old(File) :-
	    % Luetaan inputohjelmatiedostosta File dokumentin tyyppi
            % ja tekstityyppi.
	    see(File),
            read(_),%InputGram
            read(StructureGram),
	    retractall(structureGram_i(_)),
            assert(structureGram_i(StructureGram)),
	    read(Type),
	    read(Text),!,
	    % Tuhotaan vanhat input predikaatit tietokannasta.
             retractall(rule(Type,_,_)),
	    % Luetaan uudet predikaatit tietokantaan.
	    read(Pred),
            consultInputProg(Pred,File),
            seen.


/*
 *-----------------------------------------------------------------------------
 *    tree
 *     
 *-----------------------------------------------------------------------------
 * - Tulostetaan puu naytolle.
 * - Toiminnoilla expand, unexpand, next, delete muokataan jasennyspuuta.
 * - Toiminnoilla child, parent,left,right, down, up liikutaan jasennyspuussa.
 * - Toiminnolla save talletetaan puu.
 * - Toiminnolla save_exit talletetaan puu ja poistutaan edelliseen valikkoon.
 * - Toiminnolla cancel poistutaan edelliseen valikkoon ilman tallennusta.
 * - Predikaatin currentTree arvona on sen hetkinen jasennuspuu.
*/

	
% Kehitetaan uusia solmuja puuhun tai muokataan solmujen sisaltoa.
%Pred:tree_expand
tree_expand:-
    currentTree(CurrentPlace),
    nonTerm_i(NonTerm),
    smallestGramEl_i(Text),
    asserta(nonTerm(NonTerm)),
    asserta(smallestGramEl(Text)),
    % Suoritetaan operaatio down.
    exeCmd(down,CurrentPlace,NewCurrentPlace),
    % Muokataan jasennyspuu naytolle tulostettavaan muotoon.
    displayCurrentPlace(NewCurrentPlace,TreeChar,SelectionChar),
    countLines(TreeChar, 1, N), N1 is N - 1,
    % Tutkitaan, onko valitun solmun lopussa tahti- tai plusmerkki eli
    % onko toiminto next sallittu.
    (findStarPlus(SelectionChar,_),
	     B1 is 0, B5 is 1, B6 is 0;
    % Tutkitaan, onko valittu solmu osa listaa eli onko solmun vasen
    % tai oikea sisar sama. Sisar voi olla myos listan loppumerkki.
    % Eli tutkitaan, onko toiminto next ja delete sallittuja.
     checkListSisters(NewCurrentPlace,LrChar,RrChar),
          (findStar(RrChar,_),
	    B1 is 1, B5 is 1, B6 is 1;
	   findPlus(RrChar,_),
	    (same(LrChar,SelectionChar),
	     B1 is 1, B5 is 1, B6 is 1;
	     B1 is 1, B5 is 1, B6 is 0);
	     B1 is 1, B5 is 1, B6 is 1);
       B1 is 1, B5 is 0, B6 is 0),
    retractall(nonTerm(NonTerm)),
    retractall(smallestGramEl(Text)),
    retract(currentTree(CurrentPlace)),
    assert(currentTree(NewCurrentPlace)),
    % Tulostetaan jasennyspuu naytolle.
    call_tcl(insert_text(TreeChar, B1, B5, B6, N1)),!.


% Siirrytaan puussa lapsi-solmuun.
%Pred:tree_lower
tree_lower:-
    currentTree(CurrentPlace),
    nonTerm_i(NonTerm),
    smallestGramEl_i(Text),
    asserta(nonTerm(NonTerm)),
    asserta(smallestGramEl(Text)),
    % Suoritetaan operaatio low.
    exeCmd(low,CurrentPlace,NewCurrentPlace),
    % Muokataan jasennyspuu naytolle tulostettavaan muotoon.
    displayCurrentPlace(NewCurrentPlace,TreeChar,SelectionChar),
    countLines(TreeChar,1,N),N1 is N - 1,
    % Tutkitaan, onko valitun solmun lopussa tahti- tai plusmerkki eli
    % onko toiminto next sallittu.
    (findStarPlus(SelectionChar,_),
          B1 is 0, B5 is 1, B6 is 0;
    % Tutkitaan, onko valittu solmu osa listaa eli onko solmun vasen
    % tai oikea sisar sama. Sisar voi olla myos listan loppumerkki.
    % Eli tutkitaan, onko toiminto next ja delete sallittuja.
     checkListSisters(NewCurrentPlace,LrChar,RrChar),
          (findStar(RrChar,_),
	   B1  is 1, B5  is 1, B6 is 1;
           findPlus(RrChar,_),
	    (same(LrChar,SelectionChar),
	     B1 is 1, B5 is 1, B6 is 1;
             B1 is 1, B5 is 1, B6 is 0);
	   B1 is 1, B5 is 1, B6 is 1);
      B5 is 0, B1 is 1, B6 is 0),
    retractall(nonTerm(NonTerm)),
    retractall(smallestGramEl(Text)),
    retract(currentTree(CurrentPlace)),
    assert(currentTree(NewCurrentPlace)),
    % Tulostetaan jasennyspuu naytolle.
    call_tcl(insert_text(TreeChar, B1, B5, B6, N1)),
    !.


% Liikutaan ylemmalle tasolle, isa-solmuun (Left-painike)
%Pred:tree_upper
tree_upper:-
    currentTree(CurrentPlace),
    nonTerm_i(NonTerm),
    smallestGramEl_i(Text),
    asserta(nonTerm(NonTerm)),
    asserta(smallestGramEl(Text)),
    % Suoritetaan toiminto up.
    exeCmd(up,CurrentPlace,NewCurrentPlace),
    displayCurrentPlace(NewCurrentPlace,TreeChar,SelectionChar),
    countLines(TreeChar, 1, N), N1 is N - 1,
    (findStarPlus(SelectionChar,_),
          B1 is 0, B5 is 1, B6 is 0;
     checkListSisters(NewCurrentPlace,LrChar,RrChar),
          (findStar(RrChar,_),
          B1 is 1, B5 is 1, B6 is 1;
	   findPlus(RrChar,_),
	    (same(LrChar,SelectionChar),
	     B1 is 1, B5 is 1, B6 is 1;
	     B1 is 1, B5 is 1, B6 is 0);
	    B1 is 1, B5 is 1, B6 is 1);
	    B1 is 1, B5 is 0, B6 is 0),
    retractall(nonTerm(NonTerm)),
    retractall(smallestGramEl(Text)),
    retract(currentTree(CurrentPlace)),
    assert(currentTree(NewCurrentPlace)),
    % Tulostetaan jasennyspuu naytolle.
    call_tcl(insert_text(TreeChar, B1, B5, B6, N1)),!.

% Liikutaan puussa solmun oikeaan sisareen.
%Pred:tree_down
tree_down:-
    %retract(opened_window(W,T,B1,B5,B6)),
    currentTree(CurrentPlace),
    nonTerm_i(NonTerm),
    smallestGramEl_i(Text),
    asserta(nonTerm(NonTerm)),
    asserta(smallestGramEl(Text)),
    % Suoritetaan toiminto right.
    exeCmd(right,CurrentPlace,NewCurrentPlace),
    displayCurrentPlace(NewCurrentPlace,TreeChar,SelectionChar),
    countLines(TreeChar, 1, N), N1 is N - 1,
    (findStarPlus(SelectionChar,_),
	  B1 is 0, B5 is 1, B6 is 0;
     checkListSisters(NewCurrentPlace,LrChar,RrChar),
          (findStar(RrChar,_),
	   B1 is 1, B5 is 1, B6 is 1;
	   findPlus(RrChar,_),
	    (same(LrChar,SelectionChar),
	     B1 is 1, B5 is 1, B6 is 1;
	     B1 is 1, B5 is 1, B6 is 0);
	   B1 is 1, B5 is 1, B6 is 1);
      B5 is 0, B1 is 1, B6 is 0),
    retractall(nonTerm(NonTerm)),
    retractall(smallestGramEl(Text)),
    retract(currentTree(CurrentPlace)),
    assert(currentTree(NewCurrentPlace)),
    % Tulostetaan jasennyspuu naytolle.
    call_tcl(insert_text(TreeChar, B1, B5, B6, N1)),!.

% Liikutaan puussa solmun vasempaan sisareen.
%Pred:tree_up
tree_up:-
    %retract(opened_window(W,T,B1,B5,B6)),
    currentTree(CurrentPlace),
    nonTerm_i(NonTerm),
    smallestGramEl_i(Text),
    asserta(nonTerm(NonTerm)),
    asserta(smallestGramEl(Text)),
    % Suoritetaan toiminto left.
    exeCmd(left,CurrentPlace,NewCurrentPlace),
    displayCurrentPlace(NewCurrentPlace,TreeChar,SelectionChar),
    countLines(TreeChar, 1, N), N1 is N - 1,
    (findStarPlus(SelectionChar,_),
	  B1 is 0, B5 is 1, B6 is 0;
     checkListSisters(NewCurrentPlace,LrChar,RrChar),
          (findStar(RrChar,_),
	   B1 is 1, B5 is 1, B6 is 1;
	   findPlus(RrChar,_),
	    (same(LrChar,SelectionChar),
	     B1 is 1, B5 is 1, B6 is 1;
	     B1 is 1, B5 is 1, B6 is 0);
	   B1 is 1, B5 is 1, B6 is 1);
      B5 is 0, B1 is 1, B6 is 0),
    retractall(nonTerm(NonTerm)),
    retractall(smallestGramEl(Text)),
    retract(currentTree(CurrentPlace)),
    assert(currentTree(NewCurrentPlace)),
    % Tulostetaan jasennyspuu naytolle.
    call_tcl(insert_text(TreeChar, B1, B5, B6, N1)),!.

% Liikutaan naytossa alempaan elementtiin.
%Pred:tree_downscreen
tree_downscreen:-
    currentTree(CurrentPlace),
    nonTerm_i(NonTerm),
    smallestGramEl_i(Text),
    asserta(nonTerm(NonTerm)),
    asserta(smallestGramEl(Text)),
    % Suoritetaan toiminto downscreen.
    exeCmd(downscreen,CurrentPlace,NewCurrentPlace),
    displayCurrentPlace(NewCurrentPlace,TreeChar,SelectionChar),
    countLines(TreeChar,1,N),N1 is N - 1,
    (findStarPlus(SelectionChar,_),
          B1 is 0, B5 is 1, B6 is 0;
     checkListSisters(NewCurrentPlace,LrChar,RrChar),
          (findStar(RrChar,_),
	   B1 is 1, B5 is 1, B6 is 1;
	   findPlus(RrChar,_),
	    (same(LrChar,SelectionChar),
	     B1 is 1, B5 is 1, B6 is 1;
	     B1 is 1, B5 is 1, B6 is 0);
	   B1 is 1, B5 is 1, B6 is 1);
      B5 is 0, B1 is 1, B6 is 0),
    retractall(nonTerm(NonTerm)),
    retractall(smallestGramEl(Text)),
    retract(currentTree(CurrentPlace)),
    assert(currentTree(NewCurrentPlace)),	
    % Tulostetaan jasennyspuu naytolle.
    call_tcl(insert_text(TreeChar, B1, B5, B6, N1)),!.
 
% Liikutaan naytossa ylempaan elementtiin.
%Pred:tree_upscreen
tree_upscreen:-
    currentTree(CurrentPlace),
    nonTerm_i(NonTerm),
    smallestGramEl_i(Text),
    asserta(nonTerm(NonTerm)),
    asserta(smallestGramEl(Text)),
    % Suoritetaan toiminto upscreen
    exeCmd(upscreen,CurrentPlace,NewCurrentPlace),
    displayCurrentPlace(NewCurrentPlace,TreeChar,SelectionChar),
    countLines(TreeChar,1,N),N1 is N - 1,
    (findStarPlus(SelectionChar,_),
          B1 is 0, B5 is 1, B6 is 0;
     checkListSisters(NewCurrentPlace,LrChar,RrChar),
          (findStar(RrChar,_),
	   B1 is 1, B5 is 1, B6 is 1;
	   findPlus(RrChar,_),
	    (same(LrChar,SelectionChar),
	     B1 is 1, B5 is 1, B6 is 1;
	     B1 is 1, B5 is 1, B6 is 0);
	   B1 is 1, B5 is 1, B6 is 1);
      B5 is 0, B1 is 1, B6 is 0),
    retractall(nonTerm(NonTerm)),
    retractall(smallestGramEl(Text)),
    retract(currentTree(CurrentPlace)),
    assert(currentTree(NewCurrentPlace)),
    % Tulostetaan jasennyspuu naytolle.
    call_tcl(insert_text(TreeChar, B1, B5, B6, N1)),!.



% Kehitetaan solmusta uusi samanlainen solmu.
%Pred:tree_next
tree_next:-
    currentTree(CurrentPlace),
    nonTerm_i(NonTerm),
    smallestGramEl_i(Text),
    asserta(nonTerm(NonTerm)),
    asserta(smallestGramEl(Text)),
    % Suoritetaan toiminto next.
    exeCmd(next,CurrentPlace,NewCurrentPlace),
    displayCurrentPlace(NewCurrentPlace,TreeChar,SelectionChar),
    countLines(TreeChar, 1, N), N1 is N - 1,
    (findStarPlus(SelectionChar,_),
	  B1 is 0, B5 is 1, B6 is 0;
     checkListSisters(NewCurrentPlace,LrChar,RrChar),
          (findStar(RrChar,_),
	   B1 is 1, B5 is 1, B6 is 1;
	   findPlus(RrChar,_),
	    (same(LrChar,SelectionChar),
	     B1 is 1, B5 is 1, B6 is 1;
	     B1 is 1, B5 is 1, B6 is 0);
	   B1 is 1, B5 is 1, B6 is 1);
      B5 is 0, B1 is 1, B6 is 0),
    retractall(nonTerm(NonTerm)),
    retractall(smallestGramEl(Text)),
    retract(currentTree(CurrentPlace)),
    assert(currentTree(NewCurrentPlace)),
    % Tulostetaan jasennyspuu naytolle.
    call_tcl(insert_text(TreeChar, B1, B5, B6, N1)),!.
    

 % Poistetaan valittu solmu.
 % Solmuja on mahdollista poistaa vain listasta eli voidaan poistaa
 % sellaisia solmuja, jotka on kehitetty tahti- tai plusrakenteesta.
 % Solmua, jonka lopussa on tahti- tai plusmerkki, listan loppumerkki,
        % ei voi poistaa.
 % Listalta   a
 %            a*
 % voi poistaa a:n.
 % Listalta   a
 %            a+
 % ei voi poistaa a:ta.
 % Valinnaisen rakenteen voi aina poistaa.
%Pred:tree_del
tree_del:-
    currentTree(CurrentPlace),
    nonTerm_i(NonTerm),
    smallestGramEl_i(Text),
    asserta(nonTerm(NonTerm)),
    asserta(smallestGramEl(Text)),
    % Suoritetaan toiminto del.
    exeCmd(del,CurrentPlace,NewCurrentPlace),
    displayCurrentPlace(NewCurrentPlace,TreeChar,SelectionChar),
    countLines(TreeChar, 1, N), N1 is N - 1,
    (findStarPlus(SelectionChar,_),
	  B1 is 0, B5 is 1, B6 is 0;
     checkListSisters(NewCurrentPlace,LrChar,RrChar),
          (findStar(RrChar,_),
	   B1 is 1, B5 is 1, B6 is 1;
	   findPlus(RrChar,_),
	    (same(LrChar,SelectionChar),
	     B1 is 1, B5 is 1, B6 is 1;
	     B1 is 1, B5 is 1, B6 is 0);
	   B1 is 1, B5 is 1, B6 is 1);
      B1 is 1, B5 is 0, B6 is 0),
    retractall(nonTerm(NonTerm)),
    retractall(smallestGramEl(Text)),
    retract(currentTree(CurrentPlace)),
    assert(currentTree(NewCurrentPlace)),
    % Tulostetaan jasennyspuu naytolle.
    call_tcl(insert_text(TreeChar, B1, B5, B6, N1)),!.

% Poistetaan kasiteltavana olevan solmun lapset.

%Pred:tree_unexpand
tree_unexpand:-
    currentTree(CurrentPlace),
    nonTerm_i(NonTerm),
    smallestGramEl_i(Text),
    asserta(nonTerm(NonTerm)),
    asserta(smallestGramEl(Text)),
    % Suoritetaan toiminto unexpand.
%    exeCmd(unexpand,CurrentPlace,NewCurrentPlace),
    exeCmd(unexpandcurrent,CurrentPlace,NewCurrentPlace),
    displayCurrentPlace(NewCurrentPlace,TreeChar,SelectionChar),
    countLines(TreeChar, 1, N), N1 is N - 1,
    (findStarPlus(SelectionChar,_),
          B1 is 0, B5 is 1, B6 is 0;
     checkListSisters(NewCurrentPlace,LrChar,RrChar),
          (findStar(RrChar,_),
           B1 is 1, B5 is 1, B6 is 1;
	   findPlus(RrChar,_),
	    (same(LrChar,SelectionChar),
             B1 is 1, B5 is 1, B6 is 1;
             B1 is 1, B5 is 1, B6 is 0);
           B1 is 1, B5 is 1, B6 is 1);
      B1 is 1, B5 is 0, B6 is 0),
    retractall(nonTerm(NonTerm)),
    retractall(smallestGramEl(Text)),
    retract(currentTree(CurrentPlace)),
    assert(currentTree(NewCurrentPlace)),
    % Tulostetaan jasennyspuu naytolle.
    call_tcl(insert_text(TreeChar, B1, B5, B6, N1)),!.



% Talletetaan dokumentti tiedostoon File.
%Pred:tree_save
tree_save:-
    currentTree(CurrentPlace),
    % Jarjestetaan jasennyspuu talletettavaan muotoon.
    rewindToRoot(CurrentPlace,[[_,NewTree,_],_]),
    % Luetaan dokumenttitiedoston nimi.
    docFile_i(File),
    % Luetaan rakenne kielioppitiedoston nimi.
    structureGram_i(StructureGram),
    % Luetaan tekstityypin nimi.
    smallestGramEl_i(Text),
    % Luetaan dokumentin tyypin nimi.
    nonTerm_i(NonTerm),
    % Talletetaan tiedoston alkuun kielioppitiedoston nimi,
    % dokumentin tyyppi, tekstityyppi ja dokumentti.
    tell(File),
    writeq(StructureGram),write('.'),nl,
    writeq(NonTerm),write('.'),nl,
    writeq(Text),write('.'),nl,
    writeq(NewTree),write('.'),nl,
    % Suljetaan tiedosto.
    told,
    !.

% Jarjestetaan jasennyspuu talletettavaan muotoon.
%Pred:rewindToRoot
rewindToRoot(CurrentPlace,AtRoot) :-
        (same(CurrentPlace,[[[],Tree,[]],[[[],'document',[]]]]),!,
	 same([[[],Tree,[]],[[[],'document',[]]]],AtRoot);
	 up(CurrentPlace,NewCurrentPlace),!,
	 rewindToRoot(NewCurrentPlace,AtRoot)).



	% Talletetaan dokumentti tiedostoon File ja exit.
%Pred:tree_exit
tree_exit:-
    % Luetaan dokumenttitiedoston nimi.
    docFile_i(File),
    % tuhotaan File.tmp tiedosto 
    name(File,FileChar),
    append("rm ",FileChar,RmChar1),
    append(RmChar1,".tmp",RmChar),
    name(Rm,RmChar),
    call_tcl(exec(Rm)),
    retractall(currentTree(_)),
    cleanup_doc,
    !.

% Palataan edelliseen valikkoon tallettamatta dokumenttiin tehtyja muutoksia.
%Pred:tree_cancel
tree_cancel:-
    retractall(currentTree(_)),
    %Kopioidaan File.tmp tiedostossa oleva alkuperainen
    %dokumentti entiselle nimelleen
    docFile_i(File),
    name(File,FileChar),
    append("cp ",FileChar,CmdChar1),
    append(CmdChar1,".tmp ",CmdChar2),
    append(CmdChar2,FileChar,CmdChar),
    name(Cmd,CmdChar),
    call_tcl(exec(Cmd)),
    % tuhotaan File.tmp tiedosto
    append("rm ",FileChar,RmChar1),
    append(RmChar1,".tmp",RmChar),
    name(Rm,RmChar),
    call_tcl(exec(Rm)),
    cleanup_doc,
    !.


%Pred:clean_doc_old
cleanup_doc :-
	retractall(docFile_i(_)),
	retractall(structureGram_i(_)),
        retractall(nonTerm_i(_)),
        retractall(smallestGramEl_i(_)),
        retractall(rule(_,_,_)).



/*********** 
*
* commands for tree operations 
*
*
***************/

% Liikutaan puussa solmun isa-solmuun.
% Ollaan  juuri solmussa jolla ei ole isaa.
%Pred:up
up([[[],R,[]],[[[],'document',[]]]],[[[],R,[]],[[[],'document',[]]]]):-
       call_tcl(open_msg("Already in the root ! ")),
       tcl_in(call_prolog(ok)),
       !.
% Ollaan solmussa jolla on isa.
up([[Tl,Tc,Tr],[[LS,R,RS]|Context]],[[LS,T,RS],Context]):-
        reverse(Tl,TlRev),
	checkislist([Tc|Tr],Ts),
	append(TlRev,Ts,Tsub),
	checklist(Tsub,Tsub1),
	T=..[R|Tsub1].


% Kaytetaan predikaatissa up.
% Muodostetaan samaan *- ja +-rakenteeseen kuuluvista solmuista lista.
% Nain muodostetaan sisakkaisia listarakenteita, kun liikutaan ylemmalle
% tasolle puussa.
%Pred:checklist
checklist([],[]):-!.
checklist([X|RestOfTsub],Tsub1):-
	X =..[Root|_],
	name(Root,S1),append(S1,[42],S2),name(Root1,S2),
        append(S1,[43],S22),name(Root2,S22),
	((member(Root1,RestOfTsub),same(Root1,Root3);
          member(Root2,RestOfTsub),same(Root2,Root3)),
	 assert(restoflist(RestOfTsub)),
	 makeList(RestOfTsub,Root3,Xs),
	 restoflist(TsubRest),
	 checklist(TsubRest,Tsub2),
	 append([[X|Xs]],Tsub2,Tsub1)
	;
	 checklist(RestOfTsub,Tsub2),
	 append([X],Tsub2,Tsub1)
	;
	 true).

%Pred:makelist
makeList([X|Xs],Root1,List):-
	(same(X,Root1),same([],List1),
	retractall(restoflist(_)),assert(restoflist(Xs))
	;
	 makeList(Xs,Root1,List1)),
	 append([X],List1,List).

% Jos X on lista, poistetaan sisakkaiset listat.
% Muuten Y saa arvokseen X.
%Pred:checkislist
checkislist(X,Y):-
	      (is_list(X),flatten(X,Y);
	       same(X,Y)).



% Kehitetaan solmusta uusi samanlainen solmu.
% Uusi solmu syntyy vanhan solmun ylapuolelle.
% Vain listarakenteen solmuista voidaan kehittaa uusia solmuja.
% Ollaan solmussa jolla ei ole vasempia sisaruksia
%Pred:next
next([[[],T2,Tright],Context],[[[],T3,[T2|Tright]],Context]):-
	T2=..[Root|_],
	(name(Root,T2Char),
        findStarPlus(T2Char,T4),!,
	name(T3,T4)
	;
	same(Root,T3)).
% Ollaan solmussa, jolla on vasempia sisaruksia.
next([[[T1|Tleft],T2,Tright],Context],[[[T1|Tleft],T3,[T2|Tright]],Context]):-
	T2=..[Root|_],
        (name(Root,T2Char),
	 findStarPlus(T2Char,T4),!,
	 name(T3,T4);
	 same(Root,T3)).

% solmun poisto, otetaan ensimmainen 
% oikea sisarpuu kursorilla osoitettavaksi puuksi
% Input-osassa:
% vain listan alkioita voi poistaa, ei listan loppumerkkia joka on oikea sisar
% Retrieval-osassa:
% poistetaan filttereita juurisolmu on *, ei saisi poistaa viimeista
% filtteria, joten oikeat sisaret ei saa olla tyhja
% solmulla ei ole vasempia eika oikeita sisaruksia
%Pred:del
%del([[[],_,[]],[[T1Left,T1Tree,[T11|T1Right]]|Context]],
%               [[[T1Tree|T1Left],T11,T1Right],Context]):- !.
% solmulla ei ole vasempia sisaria, 
del([[[],_,[T1|Tright]],Context],[[[],T1,Tright],Context]):- !.
% Solmulla ei ole oikeita sisaruksia:
%del([[[T1|Tleft],_,[]],Context],[[Tleft,T1,[]],Context]):-!.
% Solmulla on oikeita ha vasempia sisaruksia. 
del([[Tleft,_,[T1|Tright]],Context],[[Tleft,T1,Tright],Context]):-!.

%solmun ja sen vasemman ja oikean sisaren poisto
%Pred:unexpand
unexpand([[_,_,_],[T1|Context]],[T1,Context]):-!.

% Solmulla ei ole sisaria.
%Pred:noRightSisters
noRightSisters([[_,_,[]],_]).

% Jos solmulla ei ole sisaria siirrytaan ylemmalle tasolle eli isa-solmuun.
 % Siirtymista lopetetaan, kun ollaan solmussa jolla on oikeita 
% sisaria tai tullaan
% jasennyspuun juureen.
%Pred:moreupRight
moreupRight([Tree,[[[],'document',[]]]],[Tree,[[[],'document',[]]]]):-!.
moreupRight(NewCurrentPlace,NewCurrentPlace1):-
	(noRightSisters(NewCurrentPlace),
	 (up(NewCurrentPlace,NewCurrentPlace2),
 	  moreupRight(NewCurrentPlace2,NewCurrentPlace1)
          ;
 	  same(NewCurrentPlace,NewCurrentPlace1)
          )
         ;
 	 same(NewCurrentPlace,NewCurrentPlace1)
         ).
 
 
%Pred:rightFirstSister
rightFirstSister([[Tleft,T,[]],Context],
            [[Tleft,T,[]],Context]).
rightFirstSister([[Tleft,T,[T1|Tright]],Context],
           [[[T|Tleft],T1,Tright],Context]).
     
     

% solmulla on vasempia sisaruksia
% Liikutaan naytossa alempaan elementtiin.
% jos ollaan juuressa ja juuri atominen, ei menna mihinkaan
%Pred:downscreen
downscreen([[[],R,[]],[[[],'document',[]]]],
                   [[[],R,[]],[[[],'document',[]]]]):-
 atom(R).
%Jos ollaan tarpeeksi oikealla eli viimeisessa lapsessa
% ja se on atominen niin siirrytaan  oikeita sisaruksia omistavan
% esi-isan oikeaan sisareen
%jos ei sisarta, niin  ei menna mihinkaan
downscreen([[Tleft,T,[]],Context],
                       NewCurrentPlace):-    
    atom(T),
    moreupRight([[Tleft,T,[]],Context],NewCurrentPlace1),
        rightFirstSister(NewCurrentPlace1,NewCurrentPlace2),      
       (same(NewCurrentPlace2,NewCurrentPlace1),
       same([[Tleft,T,[]],Context],NewCurrentPlace)
        ;
        same(NewCurrentPlace2,NewCurrentPlace)).

% ja se ei ole atominen niin siirrytaan sen ensimmaiseen lapseen    
% jos lapsi on teksityyppia downlow palauttaa saman solmun ja
% jolloin pitaa siirtya sisaruksia omistavan esi-isan 
% oikeaan sisareen, muussa tapauksessa downlow palauttaa
% 1. lapsen
downscreen([[Tleft,T1,[]],Context],NewCurrentPlace):-
      downlow([[Tleft,T1,[]],Context],NewCurrentPlace1),
      (same([[Tleft,T1,[]],Context],NewCurrentPlace1),
      moreupRight(NewCurrentPlace1,NewCurrentPlace2),
      right(NewCurrentPlace2,NewCurrentPlace)
      ;
      same(NewCurrentPlace1,NewCurrentPlace)).
% jos ollaan keskella sisaruksia
% ja solmu on atominen, niin siirrytaan seuraavaan sisareen
downscreen([[Tleft,T1,[T2|Tright]],Context],
                   [[[T1|Tleft],T2,Tright],Context]):-
      atom(T1).
% ja solmu ei ole atominen siirrytaan ensimmaiseen lapseen
% jos lapsi on tekstityyppia niin pitaa siirtya solmun seuraavaa
% sisarukseen
downscreen([[Tleft,T1,Tright],Context],
                   NewCurrentPlace):-
      downlow([[Tleft,T1,Tright],Context],NewCurrentPlace1),
      (same([[Tleft,T1,Tright],Context],NewCurrentPlace1),
      right(NewCurrentPlace1,NewCurrentPlace)
      ;
      same(NewCurrentPlace1,NewCurrentPlace)).
 
% Liikutaan naytossa ylempaan elementtiin.
% jos ollaan juuressa, ei menna mihinkaan
%Pred:upscreen
upscreen([[[],R,[]],[[[],'document',[]]]],
                    [[[],R,[]],[[[],'document',[]]]]).
% jos ollaan jo tarpeeksi vasemmalla, ensimmaisessa lapsessa
 % siirrytaan isaan, joka on naytossa ensimmaisen lapsen 
 % ylapuolella 
upscreen([[[],T1,Tright],Context],NewCurrentPlace):-
        up([[[],T1,Tright],Context],NewCurrentPlace).
% jos ollaan muussa lapsessa, sen ylapuolelle on joko atominen
 % vasen sisar 
upscreen([[[T1|Tleft],T2,Tright],Context],
      [[Tleft,T1,[T2|Tright]],Context]):-
       atom(T1).
 % tai vasemman sisaren puun  ensimmainen lehti  esijarjestyksessa
upscreen([[[T1|Tleft],T2,Tright],Context],NewCurrentPlace):-
       left([[[T1|Tleft],T2,Tright],Context],NewCurrentPlace1),
       previousInSequence(NewCurrentPlace1,NewCurrentPlace).

%Pred:previousInSequence
previousInSequence(NewCurrentPlace1,NewCurrentPlace):-
       (uplow(NewCurrentPlace1,NewCurrentPlace2),
         (same(NewCurrentPlace1,NewCurrentPlace2),
           same(NewCurrentPlace2,NewCurrentPlace)
          ;
          rightSister(NewCurrentPlace2,NewCurrentPlace3),
            previousInSequence(NewCurrentPlace3,NewCurrentPlace))
        ; same(NewCurrentPlace1,NewCurrentPlace)).


 % siirrytaan puussa alaspain.
 % puussa Tree on rakenne valmiina, puussa vain yksi alipuu
 % tutkitaan onko se tekstityyppipuu, jolloin ei tehda mitaan
 % muussa tapauksessa siirrytaan puun juureen, jos se 
 % lista latistetaan, jolloin siirrytaan 1. listan alkioon
%Pred:downlow
downlow([[LS,Tree,RS],Context],[[LS1,Tree1,RS1],Context1]):-
        Tree =..[Root,T],
	T =.. [First,_],!,
 	smallestGramEl(Element),
 	(same(First,Element),!,
 	 same(Tree,Tree1),
 	 same(Context,Context1),same(LS,LS1),same(RS,RS1)
          ;
 	  notempty(T),
       	 (is_list(T),flatten(T,[Tree1|RS1]),same([],LS1);
 	  same(T,Tree1),same([],LS1),same([],RS1)),
 	 same([[LS,Root,RS]|Context],Context1)).
 
% puussa Tree on kaksi tai useampia alipuita, jolloin siirrytaan 1. alipuun
% juureen.
% jos se on lista, latistetaan ja siirrytaan 1. lista-alkioon
% muussa tapauksessa siita tulee uusi puu, tutkitaan
% onko oikeat sisaret lista (jos ei otetaan sellaisenaan
% muuten  poistetaan ylimaaraiset listasulut)
downlow([[LS,Tree,RS],Context],[[[],T,Tright],[[LS,Root,RS]|Context]]):-
	Tree=..[Root,T1|Tright1],
 	 (is_list(T1),flatten([T1|Tright1],[T|Tright])
          ;
 	   same(T1,T),
              (is_list(Tright1),flatten(Tright1,Tright)
              ;
 	      same(Tright1,[Tright])
              )
           ).
          
downlow([[LS,Tree,RS],Context],[[[],T,Tright],[[LS,Root,RS]|Context]]):-
 	 Tree=..[Root|T1],
           notempty(T1),
  	 (is_list(T1),flatten(T1,[T|Tright])
          ;
 	  same(T1,T),same(Tright,[])
         ).
 
 


 
  % siirrytaan puussa alaspain.
  % puussa Tree on rakenne valmiina, puussa vain yksi alipuu
  % tutkitaan onko se tekstityyppipuu, jolloin ei tehda mitaan
  % muussa tapauksessa siirrytaan puun juureen, jos se 
  % lista latistetaan, jolloin siirrytaan 1. listan alkioon
%Pred:uplow
uplow([[LS,Tree,RS],Context],[[LS1,Tree1,RS1],Context1]):-
         Tree =..[Root,T],
 	T =.. [First,_],!,
 	smallestGramEl(Element),
 	(same(First,Element),!,
 	 same(Tree,Tree1),
 	 same(Context,Context1),same(LS,LS1),same(RS,RS1)
          ;
 	  notempty(T),
       	 (is_list(T),flatten(T,[Tree1|RS1]),same([],LS1);
 	  same(T,Tree1),same([],LS1),same([],RS1)),
 	 same([[LS,Root,RS]|Context],Context1)).
 
 % puussa Tree on kaksi tai useampia alipuita, jolloin siirrytaan 1. alipuun
 % juureen.
 % jos se on lista, latistetaan ja siirrytaan 1. lista-alkioon
 % muussa tapauksessa siita tulee uusi puu, tutkitaan
 % onko oikeat sisaret lista (jos ei otetaan sellaisenaan
 % muuten  poistetaan ylimaaraiset listasulut)
uplow([[LS,Tree,RS],Context],[[[],T,Tright],[[LS,Root,RS]|Context]]):-
 	(Tree=..[Root,T1|Tright1],
 	 (is_list(T1),flatten([T1|Tright1],[T|Tright]);
 	  same(T1,T),(is_list(Tright1),flatten(Tright1,Tright);
 		      same(Tright1,[Tright])))
          ;
 	 Tree=..[Root|T1],
           notempty(T1),
  	 (is_list(T1),flatten(T1,[T|Tright]);
 	  same(T1,T),same(Tright,[]))).
 
 
%Pred:rightSister
rightSister([[Tleft,T,[]],Context],[[Tleft,T,[]],Context]).
rightSister(CurrentPlace,NewCurrentPlace):-
        right(CurrentPlace,NewCurrentPlace1),
        rightSister(NewCurrentPlace1,NewCurrentPlace).
 


% solmun lasten poisto
%Pred:unexpandcurrent
unexpandcurrent([[R,T,L],Context],[[R,T1,L],Context]):-
           (atomic(T),same(T,T1);
            T=..[Root|_],
            same(Root,T1)).

% Liikutaan puussa solmun oikeaan sisareen.
%Pred:right
right([[Tleft,T1,[]],Context],[[Tleft,T1,[]],Context]):-
    call_tcl(open_msg("Already in the last sister!")),
    tcl_in(call_prolog(ok)),
    !.
% solmulla on oikeita sisaruksia
right([[Tleft,T1,[T2|Tright]],Context],
      [[[T1|Tleft],T2,Tright],Context]).

% Liikutaan puussa solmun vasempaan sisareen.
% Ollaan vasemmaisessa lapsessa
%Pred:left
left([[[],Ti,Tright],Context],[[[],Ti,Tright],Context]):-
         call_tcl(open_msg("Already in the first sister!")),
         tcl_in(call_prolog(ok)),
	 !.

% Solmulla on vasempia sisaruksia.
left([[[T1|Tleft],T2,Tright],Context],
     [[Tleft,T1,[T2|Tright]],Context]).


% Kehitetaan atomia.
% Haetaan tietokannasta atomia vastaava kielioppisaanto.
% Kielioppisaannosta riippuen joko editoidaan solmun sisaltoa tai
% solmuun liitetaan alipuu.
%Pred:down
down([[LS,Root,RS],Context],[[[],T,Tright],[[LS,Root,RS]|Context]]):-
	atom(Root),
        name(Root,S1),
	% Valinnaisen solmun ymparilta poistetaan hakasulut.
	(findBracket(S1,S2),name(Root1,S2);
	 name(Root1,S1)),
	% Tietokannasta haetaan oikea kielioppisaanto.
 	!,getSelectedRule(Root1),!,
 	% Predikaatin  selectedRule arvoksi on talletettu 
        % valittu kielioppisaanto.
 	selectedRule(SelectedRule),
 	% Kirjoitetaan kielioppisaanto muotoon paa -> vartalo.
 	genSubtrees(Root1,SelectedRule,[First|Tright]),
 	retract(selectedRule(SelectedRule)),
 	smallestGramEl(Element),
 	(same(First,Element),!,
 	name(Root1,RootChar),
 	% Avataan editor-ikkuna.
	call_tcl(open_editor(RootChar)),
	tcl_in(call_prolog(done(NewTextList))),
 	retractall(close_editor(_)),
 	retractall(editor(_)),
 	% Predikaattiin talletetaan tieto siita, onko sisaltoa editoitu.
 	assert(editor(yes)),
	editor_done(NewTextList),
	%assert(close_editor(yes)),
 	% Editoitu merkkijon predikaatiss newtext.
 	newtext(NewTextChar),
 	% Puuhun ei talleteta tyhjia alkioita.
 	(empty(NewTextChar),
 	 same(T,First);
 	 name(NewText,NewTextChar),
 	 T=..[First,NewText]),
         retract(newtext(NewTextChar));
 	same(T,First)).
 
 
% Siirrytaan puussa ainoaan lapseen.
% puussa Tree on rakenne valmiina, puussa vain yksi alipuu
% tutkitaan onko se tekstityyppipuu, jolloin avataan editori
% muussa tapauksessa siirrytaan puun juureen, jos se 
% lista latistetaan, jolloin siirrytaan 1. listan alkioon
%Pred:low
low([[LS,Tree,RS],Context],[[LS1,Tree1,RS1],Context1]):-
        Tree =..[Root,T],
	T =.. [First,Text],
 	smallestGramEl(Element),
 	(same(First,Element),!,
 	 name(Root,RootChar),name(Text,TextChar),
 	 retractall(close_editor(_)),
 	 retractall(editor(_)),
 	 % Predikaattiin talletetaan tieto siita, onko editor-ikkuna suljettu.
 	 assert(close_editor(yes)),
 	 % Predikaattiin talletetaan tieto siita, onko sisaltoa editoitu.
 	 assert(editor(yes)),
 	 % Avataan editori-ikkuna.
	 call_tcl(open_texteditor(RootChar, TextChar)),
	 tcl_in(call_prolog(done(NewTextList))),
	 editor_done(NewTextList),
 	 %assert(close_editor(yes)),
 	 % Editoitu merkkijono predikaatissa newtext.
 	 newtext(Text1Char),
 	 % Puuhun ei talleteta tyhjia alkioita.
 	 (empty(Text1Char),
 	  Tree1=..[Root];
 	  name(Text1,Text1Char),
 	  T1 =..[First,Text1],
 	  Tree1=..[Root,T1]),
 	 retract(newtext(Text1Char)),
 	 same(Context,Context1),same(LS,LS1),same(RS,RS1)
         ;
 	 % Poistetaan sisakkaiset listarakenteet.
 	 (is_list(T),flatten(T,[Tree1|RS1]),same([],LS1);
 	 same(T,Tree1),same([],LS1),same([],RS1)),
 	 same([[LS,Root,RS]|Context],Context1)).
 
% siirrytaan ensimmaiseen lapseen,
% puussa Tree on rakenne valmiina, jolloin siirrytaan 1. alipuun
% juureen.
% jos se on lista, latistetaan ja siirrytaan 1. lista-alkioon
% muussa tapauksessa siita tulee uusi puu, tutkitaan
% onko oikeat sisaret lista (jos ei otetaan sellaisenaan
% muuten  poistetaan ylimaaraiset listasulut)
low([[LS,Tree,RS],Context],[[[],T,Tright],[[LS,Root,RS]|Context]]):-
 	(Tree=..[Root,T1,Tright1],
 	 (is_list(T1),flatten([T1|Tright1],[T|Tright]);
 	  same(T1,T),(is_list(Tright1),same(Tright1,Tright);
 		      same(Tright1,[Tright])))
 	 ;
 	 Tree=..[Root,T1],
 	 (is_list(T1),flatten(T1,[T|Tright]);
 	  same(T1,T),same(Tright,[]))
          ;
 	 Tree=..[Root|T1],
 	 notempty(T1),
 	  (is_list(T1),flatten(T1,[T|Tright]);
 	   same(T1,T),same(Tright,[]))).
 
 

/*
 *----------------------------------------------------------------------------
 *   read structure from the grammar
 *---------------------------------------------------------------------------
 * - Kielioppisaannot talletetaan tietokantaan predikaatin rule arvoksi:
 *   rule(Type,NonTerm,Rule).
 * - Etsitaan solmua vastaava(t) kielioppisaanto/saannot tietokannasta.
 * - Jos kielioppisaantoja loytyy enemman kuin yksi kayttaja valitsee niista haluamansa.
*/

%Pred:getSelectedRule
getSelectedRule(Root):-
	retractall(selectedRule(_)),
	% Haetaan kielioppisaannot tietokannasta.
        searchRules(Root,ListOfRulesForRoot),
	% Lasketaan valittujen kielioppisaantojen lukumaara.
	length(ListOfRulesForRoot,NumberOfRules),
	% Jos kielioppisaantoja loytyi enemman kuin yksi, kayttaja valitsee
	% haluamansa.
        (NumberOfRules > 1,
	      rules_to_char(ListOfRulesForRoot,RulesChar,_), %Wi
	      % Avataan kielioppisaantojen valintaikkuna.
	      name(Root,RootChar),
              call_tcl(open_rulelist(RootChar,RulesChar)),
	      tcl_in(call_prolog(selected(Rule))),
	      %talletetaan valittusaanto predikaattiin selectedRule
	      select_rule(Rule)
              ;
	% Kielioppisaantoja on vain yksi.
	NumberOfRules = 1,
	      oneRule(ListOfRulesForRoot);
	% Solmua vastaavia kielioppisaantoja ei loytynyt yhtaan kappaletta.
	NumberOfRules < 1,
              call_tcl(open_msg("No rule for this element.")),
              tcl_in(call_prolog(ok)),
	      !,fail),
	!.


% Etsitaan solmua vastaavat kielioppisaannot tietokannasta.
%Pred:searchRules
searchRules(Root, List) :-
	nonTerm(DocType),
	findall(R, (C =..[rule,DocType,Root,R],call(C)), List).

% Lasketaan pisimman kielioppisaannon pituus. Sen avulla maarataan kielioppi-
% saantojen valintaikkunan leveys.
%Pred:rules_to_char
rules_to_char([], [], 0) :- !.
rules_to_char([TextH | TextT], [RCharH | RCharT], MaxLen) :-
        name(TextH, RCharH),                       /* ei ole lista */
        length(RCharH,ThisLen),
	rules_to_char( TextT, RCharT, Longest),
	(ThisLen > Longest, MaxLen is ThisLen;
	 MaxLen is Longest),
	 !. 


% Kielioppisaanto kirjoitetaan muotoon paa->vartalo. 
%Pred:genSubtrees
genSubtrees(Root,Rule,[FirstSubtree|RestSubtrees]):-
	name(Rule,RuleChar),
	word(RuleChar,Word,RestRuleChar),
	same(Root,Word),
	word(RestRuleChar,Arrow,RestRuleChar1),
	same(Arrow,'->'),  
	word(RestRuleChar1,FirstSubtree,RestRuleChar2),
	restrule(_,RestRuleChar2,RestSubtrees).

%Pred:restrule
restrule(_,[],[]):-!.
restrule(_,P,[W1|Ws]):- word(P,W1,P1),!,restrule(W1,P1,Ws).

%Pred:select_rule
select_rule(Rule):- assert(selectedRule(Rule)).



% Valittu kielioppisaanto talletetaan predikaatin selectedRule arvoksi.
%Pred:oneRule
oneRule([Rule]) :-  assert(selectedRule(Rule)).


/*--------------------------------------------------------------------------
 *  editor
 *   User can edit the content of the node.
 *--------------------------------------------------------------------------
 *  - Clear - tyhjentaa editor-ikkunan,
 *  - Done - tallettaa kirjoitetun tekstin ja lopettaa editoinnin,
 *  - Cancel - palutetaan alkuperainen teksti ja lopetetaan editointi.
*/

% Talletetaan kirjoitettu teksti muuttujan NewTextListChar arvoksi.
% Assertoidaan editoitu teksti predikaatin newtext arvoksi ja
% suljetaan editori-ikkuna.
%Pred:editor_done
editor_done(NewTextList):-
    retractall(newtext(_)),
    name(NewTextList, NewTextListChar), 
    assert(newtext(NewTextListChar)),!,
    % Ilmoitus, etta editor-ikkunasta halutaan poistua.
    retractall(close_editor(_)),
    assert(close_editor(yes)).

% Hylataan kayttajan editointi ja suljetaan editori-ikkuna.
%Pred:editor_cancel
editor_cancel:-
   % Ilmoitus, etta editor-ikkunasta halutaan poistua.
    retractall(close_editor(_)),
    assert(close_editor(yes)).










