# *******************************************************************
# syndocinput.tcl
#
# Function:   Creates top-level windows for SYNDOC-programs input-
#             functions.
# Date:       20.11.1995  
# Author:     Paula Leinonen                   
# **********************************************************************

#**************************************************************************
# Global variables - declarations in file  syndocglobals.tcl  
#**************************************************************************

# *********************************************************************
# Name:       new_document
# Input:      String (helpfile)
# Output:     -
# Function:   Creates top-level window for asking input program file
#             and new document file name.
# *********************************************************************

proc new_document helpfile {
    global win w
    global browse grams docs 
    global doc_file gram_file 

    set wi iframe
    catch {destroy $w}
    toplevel $w
    dpos $w
    wm title $w "Write new document"
    wm iconname $w "Write new document"

    frame $w.$wi -bd 1m
        frame $w.$wi.i -bd 2m
        label $w.$wi.i.label_doc -width 24 -text "New document file " \
				 -anchor w
        entry $w.$wi.i.entry_doc -relief sunken \
	    -textvariable doc_file 
        button $w.$wi.i.browse -text $browse \
	    -command { set doc_file [ create_browser $docs $doc_file]
		       moveto_end $w.iframe.i.entry_doc }

        frame $w.$wi.s -bd 2m
        label $w.$wi.s.label_gram -width 24 -text "Input grammar file " \
				  -anchor w
        entry $w.$wi.s.entry_gram -relief sunken \
	    -textvariable gram_file 
        button $w.$wi.s.browse -text $browse \
	    -command { set gram_file [ create_browser $grams $gram_file]
		       moveto_end $w.iframe.s.entry_gram }
    
    pack $w.$wi -fill x
    pack $w.$wi.s.label_gram -side left
    pack $w.$wi.s -fill x
    pack $w.$wi.s.entry_gram $w.$wi.s.browse -side left
    pack $w.$wi.i.label_doc -side left 
    pack $w.$wi.i -fill x
    pack $w.$wi.i.entry_doc $w.$wi.i.browse -side left 
     


    button $w.go -text Go \
      -command {new_doc $doc_file $gram_file} -width 8

    button $w.done -text Done -command {grab release $w
					destroy $w} -width 8

    button $w.help -text Help -command "open_help $helpfile $w" -width 8
    pack $w.go $w.help $w.done -side left -expand yes 

    moveto_end $w.iframe.i.entry_doc
    moveto_end $w.iframe.s.entry_gram
    grab set $w

}

# *********************************************************************
# Name:       new_doc
# Input:      String (document file)
#	      String (input grammar file) 
# Output:     -
# Function:   Creates program from given inputgrammar-file and calls
#             prolog-predicate document_new.
# *********************************************************************
proc new_doc {t1 t2} {
    global w docs doc_file iprog_file
    global ready;	# tk uses variable ready to wait 

    grab release $w
    
# Moves ~ to real path 
    set t2 [file dirname $t2]/[file tail $t2]
# Is grammar file a regular file
    $w configure -cursor watch
    if [file isfile $t2] {

# Creates new inputprogram file
	set tmp [file tail $t2].pl
# ready is a variable, which get value from prolog until generation has 
# been done.
	set ready 0 
        generate_i $t2 $tmp 
        tkwait variable ready
        set t1 [def_dir $t1 $docs]
        
	if {[string length $t1] > 0} {
		set doc_file $t1
		# TR270896: Scroll entry because it's contents were
		# changed.
                moveto_end $w.iframe.i.entry_doc
        
        
	        p document_new('$t1','$iprog_file')
	}
    } else {
	tk_dialog .d {} "\"$t2\" isn't a regular file!" {} -1 OK
	prolog_done
    }
    $w configure -cursor arrow
	
}


# *********************************************************************
# Name:       generate_i
# Input:      String (name of the input grammar file)
#	      String (name of the input program file) 
# Output:     -
# Function:   Calls prolog-predicate input_generate with arguments     
#             input grammar file and input program file.
# *********************************************************************
proc generate_i {t1 t2} {
    global iprogs iprog_file
    global ready
    global w 

# Moves ~ to real path
    set t1 [file dirname $t1]/[file tail $t1]
# Is grammar file a regular file
    if [file isfile $t1] {
        set t2 [def_dir $t2 $iprogs]
	if {[string length $t2] > 0} {
	    set iprog_file $t2
            p input_generate('$t1','$t2')
	} else {
	    prolog_done
        }
    } else {
	tk_dialog .d {} "\"$t1\" isn't a regular file!" {} -1 OK
	prolog_done
    }
}




# *********************************************************************
# Name:       old_document
# Input:      String (help file) 
# Output:     -
# Function:   Creates top-level window for asking input program file
#             and new document file name.
# *********************************************************************
proc old_document helpfile {
    global win w
    global browse docs grams 
    global doc_file gram_file

    set wi iframe
    catch {destroy $w}
    toplevel $w
    dpos $w
    wm title $w "Modify old document"
    wm iconname $w "Modify old document"

    frame $w.$wi -bd 1m
        frame $w.$wi.i -bd 2m
        label $w.$wi.i.label_doc -width 24 -text "Document file " \
				 -anchor w
        entry $w.$wi.i.entry_doc -relief sunken \
	    -textvariable doc_file 
        button $w.$wi.i.browse -text $browse \
	    -command { set doc_file [ create_browser $docs $doc_file]
		       moveto_end $w.iframe.i.entry_doc }

        frame $w.$wi.s -bd 2m
        label $w.$wi.s.label_gram -width 24 -text "Input grammar file " \
				  -anchor w
        entry $w.$wi.s.entry_gram -relief sunken \
	    -textvariable gram_file 
        button $w.$wi.s.browse -text $browse \
	    -command { set gram_file [ create_browser $grams $gram_file]
		       moveto_end $w.iframe.s.entry_gram }
    
    pack $w.$wi -fill x
    pack $w.$wi.s.label_gram -side left
    pack $w.$wi.s -fill x
    pack $w.$wi.s.entry_gram $w.$wi.s.browse -side left
    pack $w.$wi.i.label_doc -side left 
    pack $w.$wi.i -fill x
    pack $w.$wi.i.entry_doc $w.$wi.i.browse -side left 
     
    button $w.go -text Go \
      -command {old_doc $doc_file $gram_file} -width 8

    button $w.done -text Done -command {destroy $w} -width 8

    button $w.help -text Help -command "open_help $helpfile $w" -width 8
    pack $w.go $w.help $w.done -side left -expand yes 

    moveto_end $w.iframe.i.entry_doc
    moveto_end $w.iframe.s.entry_gram
    grab $w
}

#
# kutsutaan prolog-predikaattia document_old (syndoc_input.pl)
# Valitetaan ruudulla annettu tiedostonimi:
# dokumenttitiedosto (.doc)
#
# *********************************************************************
# Name:       old_doc
# Input:      String (name of a document file)
#             String (name of an input grammar file)
# Output:     -
# Function:   Calls prolog-predicate document_old with arguments     
#             document file and input program file.
# *********************************************************************
proc old_doc {t1 t2} {
    global w iprogs iprog_file
    global ready;	# tk waits variable

    grab release $w

    $w configure -cursor watch

# Moves ~ to real path in grammar file name 
    set t2 [file dirname $t2]/[file tail $t2]
# Uses input program file 
    set tmp [def_dir [file tail $t2].pl $iprogs]

    if {![file exists $tmp]} {
# Creates new input program file
        set ready 0
	generate_i $t2 [file tail $t2].pl
	tkwait variable ready
    } else {
	set iprog_file $tmp
    }

    set t1 [file dirname $t1]/[file tail $t1]
    if [file isfile $t1] {
        exec cp $t1 $t1.tmp 
        p document_old('$t1','$iprog_file')
    } else {
        tk_dialog .d {} "\"$t1\" isn't a regular file!" {} -1 OK
    }
	
    $w configure -cursor arrow
}










