#!/usr/local/tclX/7.4b3/bin/tclsh
#
#########################################################################
#
# Name:         syndocmakeindex.tcl
# Input:        SYNDOC document, index control database
# Output:       indexed document
# Function:     This program generates index tables from a SYNDOC
#               document. Tables are written to standard output.
# Date:         7.12.1995
# Author:       Jouni Mykkanen
#  
#########################################################################

#########################################################################
# Name:         Initialise
# Function:     Open files and  sets variables.
#########################################################################

 if $argc==0 { 
   puts "Too few arguments. "
   puts "At least document is required. "
   exit
 }
 if {[lindex $argv 0]=="-"} { 
   set file stdin 
 } else { 
    set file [open [lindex $argv 0] r]};   # Open the given document file
 if $argc==2 {
   if {[lindex $argv 1]!=""} {
     set ICDB [open [lindex $argv 1] r];   # Open the given control file
     set IcdbON T  
   } else { set IcdbON F }
 }

 gets $file gram ;                         # Get the grammar
 set gram [string trim $gram '.];          # Remove extra characters


# Global variables

  set N 1;                      # Index variable
  set FirstTime T;              # For backtracking control
  set TextOn F;                 # For index variable controlling
  set Stack {};                 # For keeping track of elements
#  set IndexTable(dummy) "";    # word indexis
#  set ElementTable(dummy) "";  # Elements index pairs
  set NoIndex {};               # Words that are not indexed

# Other variable(s)

 set separator "(,.' ";        # For splitting data

 gets $file root;        # Read start point
 gets $file data;        # Read last non terminal
 set data [string tolower [string trimright $data .]]    
                         # Remove . and lowercase string 

# Read the index control database, if used

 if {$argc==2 && $IcdbON=="T"} {
   while {[gets $ICDB word]>0} { lappend NoIndex $word }
 }


#########################################################################
# Name:         pre_process_list
# Input:        String
# Output:       String
# Function:     Removes all question marks from a given string.
#########################################################################

proc pre_process_list {text} {
   if [regsub -all {'\?*,} $text *, text] {}
   if [regsub -all {]'\(} $text {(} text] {}
   return $text 
}


#########################################################################
# Name:         AddWord
# Input:        String
# Output:       String
# Function:     This procedure add a new index to the index table. 
#               The parameter object specify the word. If the object 
#               does not exist, it is added to table with index.
#########################################################################
 
proc AddWord {object} {

    global N
    global IndexTable
    global NoIndex
    global IcdbON

   set index -1

# We index word only if it is not banned
   if [lsearch -exact $NoIndex $object]==-1 {
      lappend IndexTable($object) $N
      incr N
   }
}


#########################################################################
# Name:         process_list
# Input:        list, string
# Output:       -
# Function:     This procedure process a given list, indexing words 
#               and elements. The given string is used to recognise
#               words.
#########################################################################

proc process_list {list data} {

# Used global variables

    global N
    global TextOn
    global FirstTime
    global Stack
    global IndexTable
    global ElementTable
   

# Main loop

   foreach object $list { 
  
# Do we have non empty object or is it needed object?
      if {[string length $object]>0 && [regexp {[+\*]} $object]==0} {
        if {[regexp {\[} $object ]==0 || [regexp {\]} $object]==0} {
        
# Lowercase and trim the object
        set object [string tolower $object]    
        set object [string trim $object \[\]?]

# Is the object the character string nonterminal? 
    if {$TextOn=="F" && $object==$data} { 
        set TextOn T             
        set FirstTime T         

# else, is there any ending mark?
    } elseif [regexp {\)} $object] { 
      if {$TextOn=="T"} {set TextOn F}

  # Determine the begin index of backtrack character
      set begin [string first ")" $object]

  # if there is a word, store it to the IndexTable
        if {$begin>0} {
          AddWord [string range $object 0 [expr $begin-1]]
        }

  # Check every end character in object        
        for {set i [expr [string length $object]]} \
                 {$i>=$begin} {incr i -1} {
          if {[string index $object $i]==")"} {
            if {$FirstTime=="T"} {

  # The first ")" one we skip 
              set FirstTime F
            } else { 

  # The following ")" we use to decide the end of current element
  # from a top of stack
               set tlist [lindex $Stack 0]
               set element [lindex $tlist 0]

               lappend ElementTable($element) \
                      "[lindex $tlist 1],[expr $N-1]"
               set Stack [lreplace $Stack 0 0]
            }
         }
      }
# else, is it the text object?         
      } elseif {$TextOn=="T"} {
         AddWord $object 
     
# else, is it the element?
 # Eliminate extra elements 
      } elseif {![regexp {[\*|]} $object]} {
      
 # Push stack, if object is non empty
          if {$object!=""} {
            set Stack "{$object $N} $Stack"
          }    
      }    
     }    
    }
  };         # end foreach
}


##################################################################
# Name:         Main
# Output:       Matching result
# Function:     Split and process every line from document.
#               Tables are written to a file.               
##################################################################


while {[gets $file text]>=0} { 
  set text [pre_process_list $text]
  process_list [split $text $separator] $data
}

if {[lindex $Stack 0]!=""} {
   puts "Backtracking failure!  Stack: $Stack"
} else {
   puts $gram
   puts "1,$N"
   puts ""
   foreach name [array names IndexTable] {
     puts "$name $IndexTable($name)"
   }
   puts ""
   foreach name [array names ElementTable] {
   puts "$name $ElementTable($name)"
   }
}


##################################################################
# Name:         Clean up
# Function:     Closes open files.
##################################################################

if {$argc==2 && $IcdbON=="T"} { close $ICDB }
close $file

