#!/usr/local/tclX/7.4b3/bin/tclsh
#  /appl/majakka/tcl8_01a/bin/tclsh8.0     
# 
#########################################################################
# Name:         syndoc_makematch.tcl
# Input:        Indexed document, filter
# Output:       Matching result, MATCH or NO_MATCH
# Function:     The program checks filter matches from indexed document.
# Date:         11.3.1996
# Author:       Jouni Mykkanen
#########################################################################


#########################################################################
# Name:         Initialise
# Input:        String
# Output:       String
# Function:     Open files.
#########################################################################


 if {$argc<2} {
   puts "Usage: syndoc_makematch.tcl \[index table file\] \[filter file\]"
   exit
 }
 if {[lindex $argv 0]=="-"} { 
    set table_file stdin 
 } else { 
    set table_file [open [lindex $argv 0] r] }
 if {[lindex $argv 1]=="-"} { 
    set profile_file stdin 
 } else { 
    set profile_file [open [lindex $argv 1] r] 
 }


  set DBMin 1
  set DBMax 1000

  set result {}
  set AnnotationTable(DUMMY) "ttemp";  # if we don't have annotations..


#########################################################################
# Name:         ReadCFilter
# Input:        String
# Output:       List
# Function:     Reads compound filter from a given file, puts it in 
#               to a list and returns the it.
#########################################################################

proc ReadCFilter {file} {

  foreach i {gram database_root data filter_root} {
    gets $file i
  }
  while {[gets $file line]>0} { # Do nothing }
  while {[eof $file]==0} {
    set filter ""
    while {[gets $file line]>0} {append filter $line}
    lappend profile $filter 
  }
  return $profile
}


#########################################################################
# Name:         ReadIndexTables
# Input:        String
# Function:     Reads index table and element table from a given file. 
#               Tables are put on global variables.
#########################################################################

proc ReadIndexTables {file} {

    global ElementTable
    global IndexTable
    global DBMin
    global DBMax
    set Word T    

  gets $file grammar
  gets $file database_range
  gets $file dummy;             # Read the first empty line

  set database_range [split $database_range ,]
  set DBMin [lindex $database_range 0]
  set DBMax [lindex $database_range 1]

  while {[gets $file line]>=0} {
    if [string length $line]>0 {
      if {$Word=="T"} { 
        set w [lindex $line 0]
        set line [lreplace $line 0 0]
        foreach i $line { lappend IndexTable($w) $i }
      } else { 
        set e [lindex $line 0]
        set line [lreplace $line 0 0]
        foreach i $line { lappend ElementTable($e) [split $i ,] } 
      }
    } else { set Word F }
  }
}


#########################################################################
# Name:         GetElement
# Input:        String
# Output:       String
# Function:     Isolates the element from a given string and returns it.
#########################################################################

proc GetElement {string} {
 
  set string [string trimleft $string \[']
  set mark [string first "\{" $string]
  set element [string range $string 0 [incr mark -1]]
  set element [string trim $element \]?*+]
  return $element
}


#########################################################################
# Name:         CetConstraint
# Input:        String
# Output:       String
# Function:     Isolates the constraints from a given string and 
#               returns it.
#########################################################################

proc GetConstraint {string} {

  set constraint ""
  set begin [string first "\{" $string]
  set end [string first ";" $string]
  incr begin
  incr end -1
  if $begin<$end {
    set constraint [string range $string $begin $end]
  }
  return $constraint 
}


#########################################################################
# Name:         GetPositionConstraint
# Input:        String
# Output:       String
# Function:     Removes all question marks from a given string. Isolates
#               the position from a given string and returns it.
#########################################################################

proc GetPositionConstraint {string} {

  set position ""
  set begin [string first ";" $string]
  incr begin
  set end [string first "\}" $string]
  set string [string range $string $begin $end]

  set end [string first ";" $string]
 
  if 0<$end {
    incr end -1
    set position [string range $string 0 $end]
  }  
  return $position
}


#########################################################################
# Name:         Set_Positions
# Input:        string and two position variables.
# Output:       variables c_start and c_end.
# Function:     Removes all question marks from a given string. Isolates
#               the position from a given string and updates other vars.
#########################################################################

proc Set_Positions {object c_start c_end} {

      upvar $c_start A
      upvar $c_end B

  set Position_Constraint [GetPositionConstraint $object]
  set end [string first "." $Position_Constraint]
  if {$end!=-1} {
     incr end -1
     set A [string range $Position_Constraint 0 $end]
     incr end 3
     set B [string range $Position_Constraint $end end]
  } else {
     set A $Position_Constraint
     set B $Position_Constraint
  }
}


#########################################################################
# Name:         GetQuantity
# Input:        String
# Output:       String
# Function:     Isolates the quantity from a given string and returns it. 
#########################################################################

proc GetQuantity {string} {

  set begin [string first ";" $string]
  incr begin

  set end [string first "\}" $string]
  incr end -1

  set string [ string range $string $begin $end] 
  set begin [string first ";" $string]
  incr begin

  set string [string range $string $begin end]
  set string [string trimleft $string "<>="]
  return $string
}


#########################################################################
# Name:         GetQuantityOperand
# Input:        String
# Output:       String
# Function:     Isolates the quantity operand from a given string and 
#               returns it. 
#########################################################################

proc GetQuantityOperand {string} {

  set begin [string first ";" $string]
  incr begin

  set end [string first "\}" $string]
  incr end -1

  set string [ string range $string $begin $end] 
  set begin [string first ";" $string]
  incr begin

  set string [string range $string $begin end]
  if [regexp ">" $string] { 
    if [regexp "=" $string] {  return ">=" 
    } else {                   return ">" }
  } elseif [regexp "<" $string] { 
    if [regexp "=" $string] {  return "<="
    } else {                   return "<" }
  } else { return NONE }
}

 
#########################################################################
# Name:         Check_Word
# Input:        integer, integer, string, string.
# Output:       boolean
# Function:     Is the given word in correct position in the Table?
#########################################################################

proc  Check_Word {Start End operand word} {

    global IndexTable

  foreach index $IndexTable($word) {
    switch $operand { 
    "&" { 
          if {$index>=$Start && $index<=$End} { 
            return T
          }
        }
    "|" { 
          if {$index>=$Start && $index<=$End} {
          return T
          }
        }
    " " {
          if {$index>=$Start && $index<=$End} {
            if {$index==$Start} { 
            return "$index T"
            } else { return "$index T" }
          }
        }
    "NONE" {
          if {$index>=$Start && $index<=$End} {
            return T 
          }
        }
    }
  }
  return F
}


#########################################################################
# Name:         Check_Element
# Input:        integer, integer, string, string
# Output:       boolean
# Function:     Is the given element in correct position in the Table?
#########################################################################

proc  Check_Element {Start End operand element} {

    global ElementTable

  foreach range $ElementTable($element) {
    switch $operand { 
    "&" { 
          if {[lindex $range 0]>=$Start && [lindex $range 1]<=$End} { 
            return T
          }
        }
    "|" { 
          if {[lindex $range 0]>=$Start && [lindex $range 1]<=$End} {
            return T
          }
        }
    " " {
          if {[lindex $range 0]>=$Start && [lindex $range 1]<=$End} {
            if {[lindex $range 0]==$Start} {  
             return "[lindex $range 1] T"
            } else { return "[lindex $range 1] T" }
          }
        }
    "NONE" {
          if {[lindex $range 0]>=$Start && [lindex $range 1]<=$End} {
            return T
          }
        }
    }
  }
  return F
}


#########################################################################
# Name:         Check_Annotation
# Input:        integer, integer, string, string
# Output:       boolean
# Function:     Is the annotation between correct range in the 
#               annotation table?
#########################################################################

proc  Check_Annotation {Start End operand element} {

    global AnnotationTable

  foreach range $AnnotationTable($element) {
    switch $operand { 
    "&" { 
          if {[lindex $range 0]>=$Start && [lindex $range 1]<=$End} { 
           return T
          }
        }
    "|" { 
          if {[lindex $range 0]>=$Start && [lindex $range 1]<=$End} {
            return T
          }
        }
    " " {
          if {[lindex $range 0]>=$Start && [lindex $range 1]<=$End} {
            if {[lindex $range 0]==$Start} {  
              return "[lindex $range 1] T"
            } else { return "[lindex $range 1] F" }
          }
        }
    "NONE" {
          if {[lindex $range 0]>=$Start && [lindex $range 1]<=$End} {
            return T
          }
        }
    }
  }
  return F
}


#########################################################################
# Name:         GetOperand
# Input:        string, string
# Output:       String
# Function:     Finds out the used operand and returns it. If not 
#               found, "NONE" is returned.
#########################################################################

proc GetOperand {operands Constraint} {

  foreach op $operands {
    set o_index [string first $op $Constraint]
    if $o_index>0  { return $op } 
  }
  return "NONE"
}


#########################################################################
# Name:         CheckConstraint
# Input:        String, integer, integer, string, string, string
# Output:       boolean
# Function:     Is Constraints OK? Returns result.
#########################################################################

proc CheckConstraint {object Start End } {

      global Words
      global Annotations
      global Elements

  set Operands "| &  { }" 
  set Constraint [GetConstraint $object]
  set Constraint [string trimleft $Constraint =]
  set Result F
  
  if {$Constraint!=""} {

# Find out the used operand.
    set operand [GetOperand $Operands $Constraint] 
    if {$operand!="NONE"} {
      set Constraint [split $Constraint $operand]
    }
    set C_Start $Start

# Check needed constraints. 
    foreach string $Constraint {

  # Do we want negation of the object? 
      if {[string index $string 0]=="!"} { set Not T
      } else {                             set Not F }

      set string [string trim $string " \"!"]
      set string [string tolower $string]

  # Are we dealing with words? 
      if { [lindex $object 0]!="\"" && 
       [lsearch -exact $Words $string]>=0 } {
    set Result [Check_Word $C_Start $End $operand $string]

  # or annotations?
      } elseif [lsearch -exact $Annotations $string]>=0 {
    set Result [Check_Annotation $C_Start $End $operand $string]
    
  # or elements?
      } elseif [lsearch -exact $Elements $string]>=0 {
    set Result [Check_Element $C_Start $End $operand $string]

  # or with aliens?
      } elseif {$operand=="&" || $operand==" "} { return F }

  # Get new start point, if exists.
      if {[llength $Result]==2} { 
        set C_Start [lindex $Result 0]
        set Result [lreplace $Result 0 0]
      }

  # Correct answer, if we want negation.
      if {$Not=="T"} {
        if {$Result=="T"} { set Result F
        } else { set Result T }
      } 

  # Make decision? 
      if {$Result=="T"} {
        if {$operand=="|"} { 
          return T
        } elseif {$operand==" "} { incr C_Start }
      } elseif {$operand=="&" || $operand==" "} { return F } 
    }; # foreach

# We don't have constraint at all. 
  } else { set Result T }
  return $Result
}


#########################################################################
# Name:         GetFilterRoots
# Input:        String, integer, integer, string
# Output:       List or string
# Function:     All filter roots between given start and end index 
#               position are retrieved from element table and returned. 
#               If not found, empty string is returned.
#########################################################################

proc GetFilterRoots {string start end Elements} {

    global ElementTable

  set filter_root [GetElement $string]
  if [lsearch -exact $Elements $filter_root]>=0 {
    foreach k $ElementTable($filter_root) {
      if [lindex $k 0]>=$start { 
        if [lindex $k 1]<=$end { lappend ranges $k }
      }
    }
  } else return ""
  return "$filter_root $ranges" 
}


#########################################################################
# Name:         Get_Annotation
# Input:        String
# Output:       String
# Function:     The name for annotation is separted from a given object 
#               and it is returned. If not exists, the empty string is 
#               returned.
#########################################################################

proc Get_Annotation {object} {

  set mark [string first "\}" $object]
  incr mark 2
  if {[string index $object $mark]!="\}"} {
    set anno [string range $object $mark [string length $object]]
    set mark [string first "\}" $anno]
    incr mark -1
    set anno [string range $anno 0 $mark]
    return $anno
  } else return ""
}


#########################################################################
# Name:         Update_Ranges
# Input:        pair of numbers, stacked number pairs, integer, integer,
#               string, boolean, boolean, list of number pairs.
# Output:       pair of numbers, stacked number pairs, integer, integer.
# Function:     Handles range backtracking and updates ranges. 
#########################################################################

proc Update_Ranges { CRanges RStack S E \
                        object No_Element \
                        Sequence Sequence_Ranges } {

   upvar $CRanges CurrentRanges
   upvar $RStack RangeStack
   upvar $S Start
   upvar $E End

 # Range backtracking
 if [regexp {\)} $object] { 
    for {set x [string length $object]} {
        $x>=[string first ")" $object]} {incr x -1} {
      if {[string index $object $x]==")"} {

 # Update current ranges when backtracking
         set CurrentRanges [lindex $RangeStack 0]
         set RangeStack [lreplace $RangeStack 0 0]
       }
     }
  }
 
 # Range updating
 if {$No_Element=="F"} { 
    if {$Sequence=="T"} { 
      set Start [lindex [lindex $Sequence_Ranges end] 1] 
      incr Start
    } else { set Start [expr $End+1] }
    set End [lindex [lindex $RangeStack 0] 1]
 }
 
}


#########################################################################
# Name:         Check_Constraints
# Input:        Boolean, list of number pairs, string, integer, integer.
# Output:       Boolean.
# Function:     Handles constraint checking.
#########################################################################

proc Check_Constraints {  Sequence \
                          Sequence_Ranges \
                          object \
                          Start End } {

     set Round 0
     set Q_Counter 0
    
  Set_Positions $object P_Start P_End

  if {$Sequence=="T"} {
     set Quantity [GetQuantity $object]
     set Operand [GetQuantityOperand $object]
     set How_many [llength $Sequence_Ranges]
 
     foreach range $Sequence_Ranges {
        incr Round
        set Result [ CheckConstraint $object [lindex $range 0] \
                                     [lindex $range 1] ]
        if {$Result=="T"} { 
          incr Q_Counter
          if {$P_Start!=""} {
             if {$P_Start<0} {
                if { [expr $How_many +1 + $P_Start] <= $Round &&
                     [expr $How_many +1 + $P_End]   >= $Round } {
                   break; # ok
                } else { return F }
             } elseif {$P_Start<=$Round && $P_End>=$Round }  { 
                break; # ok 
             } else { return F } 
          }
        } elseif {$Quantity=="ALL"} { return F 
        } elseif {$P_End>0 && $Round>=$P_End} { return F }
     }

     if {$Quantity==""} { return T }
     switch $Operand { 
       "NONE" { if {$Quantity==$Q_Counter} { 
                   return T
                } else { return F }
              }
        ">"   { if {$Q_Counter>$Quantity} {
                   return T
                } else { return F } 
              }
        ">="  { if {$Q_Counter>=$Quantity} {
                   return T
                } else { return F }
              }
         "<"   { if {$Q_Counter<$Quantity} {
                    return T
                 } else { return F } 
               }
         "<="  { if {$Q_Counter<=$Quantity} {
                    return T
                 } else { return F }
               }
     }  

  } else { return [ CheckConstraint $object $Start $End ] }
}


#########################################################################
# Name:         Find_Ranges
# Input:        List of number pairs, integer, integer, boolean,
#               string, boolean.
# Output:       List of number pairs, integer, integer, boolean.
# Function:     Get sequence of ranges and initialises variables 
#               Structure, Start and End.                           
#########################################################################

proc Find_Ranges { Seq_Ranges E_Start E_End Struct \
                   element Sequence } {

    global ElementTable

    upvar $Seq_Ranges Sequence_Ranges
    upvar $E_Start Start 
    upvar $E_End End
    upvar $Struct Structure

    set S $Start
    set E $End

  foreach k $ElementTable($element) {
    if [lindex $k 0]==$S {
       if [lindex $k 1]<=$E {  
          lappend Sequence_Ranges "$S [lindex $k 1]"
          set Structure T
          if {$Sequence=="F"} { break }
          set S [lindex $k 1]
          incr S
       }
    }
  } 
  set End [ lindex [lindex $Sequence_Ranges 0] 1]
  set Start [lindex [lindex $Sequence_Ranges 0] 0] 
}


#########################################################################
# Name:         Mark_Annotation
# Input:        stacked pairs (string and number), table, string.
# Output:       stacked pairs, table.
# Function:     Make annotation to temporary tables and stack. 
#               (If annotation exists.)            
#########################################################################

proc Mark_Annotation { a_stack tmp_a object CurrentRanges Start} {

  upvar $a_stack anno_stack
  upvar $tmp_a tmp_anno
        
  set annotation [Get_Annotation $object]
  if {$annotation!=""} { 
     lappend tmp_anno($annotation) $CurrentRanges
     set anno_stack "{$annotation $Start} $anno_stack"
   } else { set anno_stack "NO_ANNOTATION $anno_stack" }
}


#########################################################################
# Name:         process_objects
# Input:        string, stacked pair of numbers, stacked pairs, table,
#               boolean, boolean, pair of number, integer, integer,
#               boolean.
# Output:       stacked pair of numbers, stacked pairs, boolean, boolean,
#               pair of number, integer, integer.
# Function:     Process every separate object from a given line.  
#               Variables are updated during the process.             
#########################################################################

proc process_objects { line \
                       RStack \
                       astack \
                       tmp_a \
                       Struct \
                       Con_OK \
                       CRanges \
                       S E }  { 

         global ElementTable
         global Elements         

         upvar $astack anno_stack
         upvar $tmp_a tmp_anno
         upvar $RStack RangeStack
         upvar $CRanges CurrentRanges
         upvar $Con_OK Constraint_OK 
         upvar $S Start 
         upvar $E End   
         upvar $Struct Structure

         set No_Element F


 if [regexp , $line]>=0 { set line [split $line ,] } 

 foreach object $line {
        set Structure F
        set CurrentRanges "$Start $End"
        set element [GetElement $object]

  # Check the structure.
  # Is the element in table? 
  # If so we have to update the current ranges.

        if [lsearch -exact $Elements $element]>=0 {
          if [regexp {\+} $object] { 
                   set Sequence T 
          } elseif [regexp {\*} $object] {
                   set Sequence T
          } else { set Sequence F } 
          set Sequence_Ranges {}  
          Find_Ranges Sequence_Ranges Start End Structure \
                      $element $Sequence           

          Mark_Annotation anno_stack tmp_anno $object \
                          $CurrentRanges $Start
             
  # In the case of conditional element.
          if [regexp {'\[} $object] { 
            set Structure T 
            set No_Element F    
          } else {set No_Element F}
 
  # Break out if structure fails
          if {$Structure=="F"} { return }
 
  # Check the constrains, break out if fails
          set Constraint_OK [\
                Check_Constraints $Sequence \
                                  $Sequence_Ranges \
                                  $object \
                                  $Start $End ]
          if {$Constraint_OK=="F"} { return }
          Update_Ranges CurrentRanges RangeStack Start End \
                        $object $No_Element \
                        $Sequence $Sequence_Ranges  
 
        } else { continue; # There are not such element }

   }; # End foreach


 # Update ranges, if structure and constraints are all right.

   if {$Structure=="T" && $Constraint_OK=="T"} {
      set Start [lindex $CurrentRanges 0]
      set End [lindex $CurrentRanges 1] 
   }

}

#########################################################################
# Name:         match
# Input:        List, integer, integer.
# Output:       List.
# Function:     This procedure check's the structure and constraints of 
#               given filter from a element table and index table. It 
#               returns a result vector.     
#########################################################################

proc match {filter start_i end_i} {

    global ElementTable
    global IndexTable
    global AnnotationTable
    global Words
    global Elements
    global Annotations

  set tmp_anno(DUMMY) "dummy"
  set Elements [array names ElementTable]
  set Words [array names IndexTable]

# Filter roots has to be retrieved
  
  set FilterRoots [GetFilterRoots [lindex $filter 0] \
                   $start_i $end_i $Elements]
  if {$FilterRoots==""} { 
    return "No filter roots in elements" 
  } else { 
    set Result [lreplace $FilterRoots 0 0] 
  }
  set Filter_Root [lindex $FilterRoots 0]
  set FilterRoots [lreplace $FilterRoots 0 0]

# Every filter root has to be checked

  foreach Root $FilterRoots { 
    set Start [lindex $Root 0]
    set End [lindex $Root 1]
    set CurrentRanges "$Start $End"
    set RangeStack {}
    set anno_stack {}
    set Structure F
    set Constraint_OK F
    set Annotations [array names AnnotationTable]

    if {$filter==""} { 
      set Structure T 
      set Constraint_OK T
    }

    foreach line $filter { 
      set RangeStack "{$CurrentRanges} $RangeStack" 

      process_objects $line RangeStack anno_stack tmp_anno \
                      Structure Constraint_OK \
                      CurrentRanges Start End  
    }

  # Pop stack
    set RangeStack [lreplace $RangeStack 0 0]

  # Make decision about match in current round
    if {$Structure=="T" && $Constraint_OK=="T"} { 
      if {$RangeStack==""} { 
        lappend Result MATCH
        set array_names [array names tmp_anno]
        foreach aname $array_names {
          if {$aname=="NO_ANNOTATION"} {continue}
          lappend AnnotationTable($aname) $tmp_anno($aname)
        }
      } else {   
         lappend Result NO_MATCH 
      }
    } else {  
       lappend Result NO_MATCH }    
  } 
  return $Result
}


########################################################################
# Name:         Main
# Function:     Process every single filter from a compound filter and
#               writes the result in to a file.
########################################################################

  set profile [ReadCFilter $profile_file]

  ReadIndexTables $table_file

  set filter [lindex $profile 0]
  set profile [lreplace $profile 0 0]

  while {$filter!=""} {
    set filter [split $filter (]
    set result "[match $filter $DBMin $DBMax] $result"
    set filter [lindex $profile 0]
    if {$profile != ""} {
        set profile [lreplace $profile 0 0]
    }
  }

  puts [lrange $result end end]

########################################################################
# Name:         Clean up
# Function:     Closes open files.
########################################################################
 
 close $table_file
 close $profile_file

