/*
*----------------------------------------------------------------------------
* syndoc_output.pl
*---------------------------------------------------------------------------
*/

/*--------------------------------------------------------------------------
 *  layout_generate
 *--------------------------------------------------------------------------
*/

%Pred:layout_generate(+File,+File)
layout_generate(Lay,Prog):-
	    generateLayout(Lay,Prog),
	    clean_lay_gen,
	    % Tk-process is waiting variable ready. 
            % Procedure prolog_done changes value of variable ready.
            call_tcl(prolog_done).

%Pred:clean_lay_gen
clean_lay_gen :-
	retractall(outputGram(_)),
	retractall(outputProg(_)),
	retractall(nonTerm(_)),
	retractall(smallestGramEl(_)).


/*--------------------------------------------------------------------------
 *  layout_program_format
 *--------------------------------------------------------------------------
*/

%Pred:layout_program_format
layout_program_format(Doc,Out,Prog) :-
	    see(Doc),
            % rakenne kielioppi
	    read(Gram),
            % dokumentin tyyppi
	    read(NonTerm),
            % dokumentin tekstityyppi
	    read(_),
            % dokumenttipuu
	    read(Tree),
	    seen,!,
	    see(Prog),
            % input grammar
	    read(_),
            % rakennekielioppi
	    read(PrGram),!,
            % tarkistetaan etta kieliopit samat
            check_grams(Gram,PrGram),
            % dokumentin tyyppi
	    read(_),
            % tekstityyppi
	    read(_),
	    seen,
	    oconsult(Prog),
	    Goal =..[NonTerm,Tree,_,_],
	    tell(Out),!,
	    call(Goal),
	    told,
            name(Out,OutChar),
            append("Document has been marked up!",[10],IChar1),
            append(IChar1,"The output file is ",IChar2),
            append(IChar2,OutChar,InfoChar),      
	    call_tcl(open_info(InfoChar)),
	    tcl_in(call_prolog(ok)),
%	    call_tcl(make_preview(Out)),
            !.




/*--------------------------------------------------------------------------
 *  layout_program_preview
 *--------------------------------------------------------------------------
*/

%Pred:layout_program_preview
layout_program_preview(Doc,Out,Prog) :-
	    see(Doc),
            % rakenne kielioppi
	    read(Gram),
            % dokumentin tyyppi
	    read(NonTerm),
            % dokumentin tekstityyppi
	    read(_),
            % dokumenttipuu
	    read(Tree),
	    seen,!,
	    see(Prog),
            % input grammar
	    read(_),
            % rakennekielioppi
	    read(PrGram),!,
            % tarkistetaan etta kieliopit samat
            check_grams(Gram,PrGram),
            % dokumentin tyyppi
	    read(_),
            % tekstityyppi
	    read(_),
	    seen,
	    oconsult(Prog),
	    Goal =..[NonTerm,Tree,_,_],
	    tell(Out),!,
	    call(Goal),
	    told,
            name(Out,OutChar),
            append("Document has been marked up!",[10],IChar1),
            append(IChar1,"The output file is ",IChar2),
            append(IChar2,OutChar,InfoChar),      
	    call_tcl(open_info(InfoChar)),
	    tcl_in(call_prolog(ok)),
	    call_tcl(make_preview(Out)),!.




/*--------------------------------------------------------------------------
 *  layout_program_print
 *--------------------------------------------------------------------------
*/

%Pred:layout_program_print
layout_program_print(Doc,Out,Prog) :-
	    see(Doc),
            % rakennekielioppi
	    read(Gram),
            % tyyppi
	    read(NonTerm),
            % sisaltotyyppi
	    read(_),
            % dokumentti
	    read(Tree),
	    seen,!,
	    see(Prog),
            % input kielippi
	    read(_),
            % rakennekielioppi
	    read(PrGram),!,
            check_grams(Gram,PrGram),
            % tyyppi 
	    read(_),
            % sisaltotyyppi
	    read(_),
	    seen,
	    oconsult(Prog),
	    Goal =..[NonTerm,Tree,_,_],
	    tell(Out),!,
	    call(Goal),
	    told,
            name(Out,OutChar),
            append("Document has been marked up!",[10],IChar1),
            append(IChar1,"The output file is ",IChar2),
            append(IChar2,OutChar,InfoChar),      
	    call_tcl(open_info(InfoChar)),
	    tcl_in(call_prolog(ok)),
	    call_tcl(make_print(Out)),!.





/************************************************
 *
 *    generate new layout program
 *
*************************************************
*/
%Pred:generateLayout(+File,+File)
generateLayout(LayoutFile,OutputProg):-
	tell(user),
      % Tcl-procedure give_name(filename) returns the name of the file
      % without directorypath.
        call_tcl(give_name(LayoutFile)), 
	tcl_in(call_prolog(done(Lname))),
	see(LayoutFile),
        % rakenne kielioppi
	read(StructureGram),
        % dokuentin tyyppi
	read(Type),
        % dokumentin tekstityyppi
	read(Text),
	tell(OutputProg),!,
	writeq(Lname),write('.'),nl,
	writeq(StructureGram),write('.'),nl,
	writeq(Type),write('.'),nl,
	writeq(Text),write('.'),nl,
	repeat,
        termInput(Termlist),
	process(Termlist,OutputProg),
	careCharString1(Text,Production),
	writeterm(OutputProg,Production),
        seen,told.


% Muodostaa nonterminaalin text arvosta produktiosaannon 
% text(text(A))-->{write(A)}.
%Pred:careCharString1(+CharacterStringType,-Production)
careCharString1(Text,Production) :-
	Prod =..[Text,Ws],
	Head =..[Text,Prod],
	Body =..['write',Ws],
	Production = (Head --> {Body}).
%	numbervars(Production,0,_).


%Pred:writeterm(+File,+Term)
% numbervars muuntaa muuttujat muotoa _135  aakkosmuotoon
writeterm(_,[]):-!.
writeterm(File,Term):-tell(File),
                       numbervars(Term,0,_),write(Term),write('.'),nl.

%Pred:write_term1
% numbervars muuntaa muuttujat muotoa _135  aakkosmuotoon
write_term1(_,[]):-!.
write_term1(File,Term):-tell(File),numbervars(Term,0,_),write(Term),write('.'),nl.


%Pred:write_term2
%write_term2(_,[]):-!.
%write_term2(File,Term):-tell(File),write(Term),write('.'),nl.



%Pred:termInput
termInput(Termlist):-
	ruleInput(RuleChar),!,
	formTermlist(RuleChar,Termlist).

%Pred:formTermList
formTermlist(RuleChar,[Word|Textlist]):-
	analyse_word(RuleChar,Word,RestRule),!,
	restwords(Word,RestRule,Textlist).

%Pred:restwords
restwords(_,[],[]):-!.
restwords(empty_line,_,[]):-!.
restwords(end_of_file,_,[]):-!.
restwords(_,Rule,[Word1|Textlist]):-analyse_word(Rule,Word1,Rest),!,
        restwords(Word1,Rest,Textlist).

%Pred:analyse_word
analyse_word([empty_line],empty_line, []):-!.
analyse_word([end_of_file],end_of_file,[]):-!.
analyse_word([C1|P],nonterminallist(A),P1):-
	left_parenthesis(C1),!,rest_atom(P,Cs,[C2,C3,C4|P2],letter_or_),
	right_parenthesis(C2),!,
	(single_star(C3);single_plus(C3)),
	(single_quote(C4),
	rest_atom(P2,Css,[C5|P1],any_but_single_quote),
        single_quote(C5),
	uc_to_lc(Cs,Cs1),
	append(Cs1,[108,105,115,116,C3,C4|Css],L1),
	append(L1,[C5],L2),
	name(A,L2);
	lbrace(C4),
	rest_atom(P2,Css,[C5|P1],any_but_rbrace),
        rbrace(C5),
	uc_to_lc(Cs,Cs1),
	append(Cs1,[108,105,115,116,C3,C4|Css],L1),
	append(L1,[C5],L2),
	name(A,L2)).
analyse_word([C1|P],optional_nonterminal(A),P1):-
	left_bracket(C1),!,rest_atom(P,Cs,[C2|P1],any_but_right_bracket),
	right_bracket(C2),!,
	uc_to_lc(Cs,Cs1),
	name(A,Cs1).
analyse_word([C|P],nonterminal(A),P1):-
	letter(C),!,rest_atom(P,Cs,P1,letter_or_),
	uc_to_lc([C|Cs],NT),name(A,NT).
analyse_word([C|P],syntactic_terminal(A),P1):-
	single_quote(C),!,rest_atom(P,Cs,[_|P1],any_but_single_quote),
        append([C|Cs],[C],L1),name(A,L1).
analyse_word([C,C1|Cs],'-->',Cs):-
        hyphen(C),greater_than(C1),!.
analyse_word([_|P1],W,P2):-analyse_word(P1,W,P2).


%Pred:process
process([end_of_file],_):-!.
process([empty_line],_):-!,fail.
process(Termlist,OutputProg):-!,
	grammar_rule(Production,Termlist,[]),!,
	write_atoms(OutputProg),!,
	write_lists(OutputProg),!,
	write_optionals(OutputProg),!,
	write_changes(OutputProg),!,
	writeterm(OutputProg,Production),!,fail.

%Pred:write_lists
write_lists(OutputProg):-(retract(empty_list_production(ProdStar)),
			  writeterm(OutputProg,ProdStar),
			  (retract(one_list_production(Prod)),
			   writeterm(OutputProg,Prod),
			   retract(list_production(Prod1)),
			   writeterm(OutputProg,Prod1);
			   retract(list_production(Prod1)),
			   writeterm(OutputProg,Prod1))),!,
			 write_lists(OutputProg).
write_lists(_).

%Pred:write_atoms
write_atoms(OutputProg) :-retract(atom_production(Prod)),
			 writeterm(OutputProg,Prod),
			 write_atoms(OutputProg).
write_atoms(_).

%Pred:write_optionals
write_optionals(OutputProg) :-retract(optional_layout_production(Prod)),
			 writeterm(OutputProg,Prod),
			 write_optionals(OutputProg).
write_optionals(_).

%Pred:write_changes
write_changes(OutputProg) :-retract(change_production(Prod)),
			 writeterm(OutputProg,Prod),
			 write_changes(OutputProg).
write_changes(_).

%Pred:grammar_rule
grammar_rule(Production) --> [nonterminal(Functor)],['-->'],
        pr_body(Vars,Body),
        {append([Functor],Vars,Vars1),
	Argument =.. Vars1,	
        Head =.. [Functor,Argument],
	Production = (Head --> Body),
	He =..[Functor,Functor],
	Prod = (He --> !),
	(retract(atom_production(P)),
	      (same(P,Prod),asserta(atom_production(P));
		asserta(atom_production(P)),
				 asserta(atom_production(Prod)));
			 asserta(atom_production(Prod))),!}.

%Pred:pr_body
pr_body(Vars,Terms) -->
	pr_body_item(I_Var,I_term),pr_body(I_Vars,I_terms),
	{(same(I_Var,[]),same(I_Vars,Vars);
	 append(I_Var,I_Vars,Vars)),
	 Terms = (I_term,I_terms)}.
pr_body(Var,Term) --> pr_body_item(Var,Term).


%Pred:pr_body_item
pr_body_item([],Term) --> [nonterminal(nl)],
	               {Term1 =..[nl],!,
		        Term = ({Term1})}.

pr_body_item([X],Term) --> [nonterminal(F)],
                       {Term =.. [F,X]}.

pr_body_item([X],Term) --> [nonterminallist(F)],
		       {!,name(F,FChar),
		        separator(FChar,FlChar,FfChar,C,SepChar),
			name(Fl,FlChar),name(Ff,FfChar),name(Sep,SepChar),
			append([39|FfChar],[C,39],FstarChar),name(Fstar,FstarChar),
			Term =.. [Fl,X],
			(lit(SepChar),Se=..[write,Sep];com(Se,SepChar)),
% list*
			(single_star(C),
			      He=..[Fl,[A,Fstar]],
			      Bo=..[Ff,A],
			      Prod = (He --> Bo,!),
			      HeStar=..[Fl,Fstar],
			    %  BoStar=..[write,Fstar],
			      ProdStar = (HeStar --> !),
			      HeStar1=..[Fl,[Fstar]],
			      ProdStar1 = (HeStar1 --> !),
          		      He1=..[Fl,[D|Ds]],
			      Bo3=..[Ff,D],
			      Bo4=..[Fl,Ds],
			      Prod1 = (He1 --> Bo3, {Se}, Bo4),
			(retract(atom_production(PS)),
			        (same(PS,ProdStar),asserta(atom_production(PS));
				 asserta(atom_production(PS)),
				 asserta(atom_production(ProdStar)));
                                 asserta(atom_production(ProdStar))),!,
			(retract(empty_list_production(PS1)),
				(same(PS1,ProdStar1),asserta(empty_list_production(PS1));
				 asserta(empty_list_production(PS1)),
				 asserta(empty_list_production(ProdStar1)));
				 asserta(empty_list_production(ProdStar1))),!,
			(retract(list_production(P1)),
			        (same(P1,Prod1),asserta(list_production(P1));
				 asserta(list_production(P1)),
				 asserta(list_production(Prod1)));
				 asserta(list_production(Prod1))),!;				
% list+	
			 single_plus(C),
			      He=..[Fl,[B,B1,Fstar]],
			      Bo1=..[Ff,B],
			      Bo2=..[Ff,B1],
			      Prod = (He --> Bo1, {Se}, Bo2),
			      HeStar1=..[Fl,[E,Fstar]],
			      BoStar1 =.. [Ff,E],
			      ProdStar1 = (HeStar1 -->BoStar1,!),
			      He1=..[Fl,[D|Ds]],
			      Bo3=..[Ff,D],
			      Bo4=..[Fl,Ds],
			      Prod1 = (He1 --> Bo3, {Se}, Bo4),
			(retract(one_list_production(P)),
			        (same(P,Prod),asserta(one_list_production(P));
				 asserta(one_list_production(P)),
				 asserta(one_list_production(Prod)));
				 asserta(one_list_production(Prod))),!,
			(retract(empty_list_production(PS1)),
				(same(PS1,ProdStar1),asserta(empty_list_production(PS1));
				 asserta(empty_list_production(PS1)),
				 asserta(empty_list_production(ProdStar1)));
				 asserta(empty_list_production(ProdStar1))),!,
			(retract(list_production(P1)),
			        (same(P1,Prod1),asserta(list_production(P1));
				 asserta(list_production(P1)),
				 asserta(list_production(Prod1)));
				 asserta(list_production(Prod1))),!)}.

pr_body_item([],Term) --> [syntactic_terminal(Const)],
		{Term1 =.. [write,Const],
		 Term = ({Term1})}.

pr_body_item([X],Term) --> [optional_nonterminal(F)],
                       {name(F,FChar),analyse_optional(FChar,FChar1),
		        name(F1,FChar1),
		        append([39,91|FChar1],[93,39],FaChar),
		        name(Fa,FaChar),
			append(FChar1,[111,112,116,105,111,110,97,108],FnChar),
			name(Fn,FnChar),
                        Term =.. [Fn,X],
			Arg1 =.. [Fa,A],He1 =.. [F1,Arg1],
			Arg2 =.. [F1,A],Bo1 =.. [F1,Arg2],
			Pro = (He1 --> Bo1),!,
			(retract(change_production(P1)),
			        (same(P1,Pro),asserta(change_production(P1));
				 asserta(change_production(P1)),
				 asserta(change_production(Pro)))
				;
				 asserta(change_production(Pro))),
			!}.
		        

%Pred:separator
separator([C|Cs],FlChar,FfChar,C1,SepChar) :-letter(C),
	rest_atom(Cs,FlsChar,[C1|SepChar],letter_or_),
	same([C|FlsChar],FlChar),
	append(FfChar,[108,105,115,116],FlChar).

%Pred:lit
lit([C|_]):-single_quote(C).

%Pred:com
com(Se,[C|Cs]):-lbrace(C),rest_atom(Cs,SeChar,[C2|_],any_but_rbrace),
	rbrace(C2),name(Se,SeChar).

%Pred:analyse_optional
analyse_optional(FChar,FChar1):-%lbracketsAway(FChar,FOptChar),
	formTermlist(FChar,Termlist),
	optional_layout_production(F,Termlist,[]),
	name(F,FChar1).

%Pred:optional_layout_production
optional_layout_production(F) --> opt_body(F,Var,Body),
	{name(F,FChar),append([39,91|FChar],[93,39],FaChar),
	 name(Fa,FaChar),
	 append(FChar,[111,112,116,105,111,110,97,108],FoChar),
	 name(Fo,FoChar),
	 Head =.. [Fo,Var],
	 Production = (Head --> Body),
         He =.. [Fo,Fa],
	 Prod = (He --> !),
	(retract(optional_layout_production(P1)),
	   (same(P1,Production),asserta(optional_layout_production(P1));
	    asserta(optional_layout_production(P1)),
	    asserta(optional_layout_production(Production)),
	    asserta(optional_layout_production(Prod)));
	    asserta(optional_layout_production(Production)),
	    asserta(optional_layout_production(Prod))),!}.

%Pred:opt_body
opt_body(F,Vars,Terms) --> opt_body_item(F1,I_Var,I_Term),
	                    opt_body(F2,I_Vars,I_Terms),
			    {(var(I_Var), Vars = I_Var,
			     Terms = (I_Term,I_Terms);
			     Vars = I_Vars,
			     Terms = (I_Term,I_Terms)),
			     (atom(F1),F = F1; F = F2)
			    }.
opt_body(F,Var,Term) --> opt_body_item(F,Var,Term).

opt_body_item(_,[],Term) --> [nonterminal(nl)],
	                      {Term1 =.. [nl],!,
			       Term = ({Term1})}.
opt_body_item(F,X,Term) --> [nonterminal(F)],
	                      {Term =.. [F,X]}. % F= '[date]'
opt_body_item(_,[],Term) --> [syntactic_terminal(Const)],
	                       {Term1 =.. [write,Const],
			        Term = ({Term1})}.












