# *******************************************************************
# syndocoutput.tcl 
#
# Function:   Creates top-level windows for SYNDOC programs output-
#             functions.
# Date:       9.11.1995
# Author:     Paula Leinonen
# *******************************************************************

#**************************************************************************
# Global variables - declarations in file  syndocglobals.tcl  
#**************************************************************************


# *******************************************************************
# Name:      format_document 
# Input:     String (help file)
# Output:    -
# Function:  Creates top-level window for asking layout grammar file,
#            document file.
# *******************************************************************
proc format_document helpfile {
    global win w
    global browse progs layouts texs
    global tex_file prog_file layoutgram_file

    set wi iframe
    catch {destroy $w}
    toplevel $w
    dpos $w
    wm title $w "Markup document"
    wm iconname $w "Markup document"

    frame $w.$wi -bd 1m
        frame $w.$wi.i -bd 2m
        label $w.$wi.i.label_doc -width 24 -text "Document file " \
				 -anchor w
        entry $w.$wi.i.entry_doc -relief sunken \
	    -textvariable doc_file 
        button $w.$wi.i.browse -text $browse \
	    -command { set doc_file [ create_browser $docs $doc_file]
                       moveto_end $w.iframe.i.entry_doc }

#        frame $w.$wi.s -bd 2m
#        label $w.$wi.s.label_tex -width 24 -text "Preview program " \
#				 -anchor w
#        entry $w.$wi.s.entry_tex -relief sunken \
#	    -textvariable prev_prog
#        button $w.$wi.s.browse -text $browse \
#	    -command { set tex_file [ create_browser $texs $tex_file]
#                       moveto_end $w.iframe.s.entry_tex }

        frame $w.$wi.k -bd 2m
        label $w.$wi.k.label_prog -width 24 -text "Layout grammar file " \
				  -anchor w
        entry $w.$wi.k.entry_prog -relief sunken \
	    -textvariable layoutgram_file 
        button $w.$wi.k.browse -text $browse \
	    -command { 
	    set layoutgram_file [ create_browser $layouts $layoutgram_file]
                       moveto_end $w.iframe.k.entry_prog }
    
    pack $w.$wi -fill x
    pack $w.$wi.k.label_prog -side left 
    pack $w.$wi.k -fill x
    pack $w.$wi.k.entry_prog $w.$wi.k.browse -side left 
     
    pack $w.$wi -fill x
    pack $w.$wi.i.label_doc -side left 
    pack $w.$wi.i -fill x
    pack $w.$wi.i.entry_doc $w.$wi.i.browse -side left 
     
#    pack $w.$wi -fill x
#    pack $w.$wi.s.label_tex -side left 
#    pack $w.$wi.s -fill x
#    pack $w.$wi.s.entry_tex -side left 
#    $w.$wi.s.browse -side left 
     
    button $w.go -text Go \
      -command {layout_format $doc_file $layoutgram_file} -width 8

    button $w.done -text Done -command {destroy $w} -width 8

    button $w.help -text Help -command "open_help $helpfile $w" -width 8
    pack $w.go $w.help $w.done -side left -expand yes 

    moveto_end $w.iframe.k.entry_prog
    moveto_end $w.iframe.i.entry_doc
#    moveto_end $w.iframe.s.entry_tex
    grab $w
}

# *******************************************************************
# Name:      layout_format
# Input:     String (document file)
#	     String (layout grammar file)
# Output:    -
# Function:  Creates layout program file by calling procedure 
#            generate. Calls prolog-predicate
#            layout_program, which creates document layout.
# *******************************************************************
proc layout_format {t1 t3} {
    global texs tex_file prog_file dir 
    global layoutgram_file doc_file
# value is changed in proc prolog_done called by prolog
    global ready w

# Name for the new output file (variable t2):
# New extension for the file (e.g. html or tex) is stored to variable type:
    set t6 [file tail $t3]
    set type [string range $t6 [expr [string first \_ $t6] + 1] [expr [string first \. $t6] - 1] ]
#this is corrected above
#    set type [string range $t3 [expr [string first \_ $layoutgram_file] + 1] [expr [string first \. $layoutgram_file] - 1]]
 
# Start of the new file name is stored to variable sname 
    set t5 [file tail $t1]
    set sname [string range $t5   [expr 0] [expr [string first \. $t5] - 1]]
    set t2 $sname.$type

    set t3 [file dirname $t3]/[file tail $t3]
    if [file isfile $t3] {

        set tmp [file tail $t3].pl
	set ready 0
        $w configure -cursor watch 
# procedure "generate" creates layout program file
	generate $t3 $tmp 
	tkwait variable ready

        set t1 [file dirname $t1]/[file tail $t1]
        if [file isfile $t1] {
            set t2 [def_dir $t2 $texs]
	    if {[string length $t2] > 0} {
		set tex_file $t2
                p layout_program_format('$t1','$t2','$prog_file')
            } else {
                prolog_done
	    }
        } else {
   	    tk_dialog .d {} "\"$t1\" isn't a regular file!" {} -1 OK
            prolog_done
        }
    } else {
	tk_dialog .d {} "\"$t3\" isn't a regular file!" {} -1 OK
    }
}
# *******************************************************************
# Name:      preview_document 
# Input:     String (help file)
# Output:    -
# Function:  Creates top-level window for asking layout grammar file,
#            document file and output file.
# *******************************************************************
proc preview_document helpfile {
    global win w
    global browse progs layouts texs
    global tex_file prog_file layoutgram_file

    set wi iframe
    catch {destroy $w}
    toplevel $w
    dpos $w
    wm title $w "Preview document"
    wm iconname $w "Preview document"

    frame $w.$wi -bd 1m
        frame $w.$wi.i -bd 2m
        label $w.$wi.i.label_doc -width 24 -text "Document file " \
				 -anchor w
        entry $w.$wi.i.entry_doc -relief sunken \
	    -textvariable doc_file 
        button $w.$wi.i.browse -text $browse \
	    -command { set doc_file [ create_browser $docs $doc_file]
                       moveto_end $w.iframe.i.entry_doc }

        frame $w.$wi.s -bd 2m
        label $w.$wi.s.label_tex -width 24 -text "Preview program " \
				 -anchor w
        entry $w.$wi.s.entry_tex -relief sunken \
	    -textvariable prev_prog
#        button $w.$wi.s.browse -text $browse \
#	    -command { set tex_file [ create_browser $texs $tex_file]
#                       moveto_end $w.iframe.s.entry_tex }

        frame $w.$wi.k -bd 2m
        label $w.$wi.k.label_prog -width 24 -text "Layout grammar file " \
				  -anchor w
        entry $w.$wi.k.entry_prog -relief sunken \
	    -textvariable layoutgram_file 
        button $w.$wi.k.browse -text $browse \
	    -command { 
	    set layoutgram_file [ create_browser $layouts $layoutgram_file]
                       moveto_end $w.iframe.k.entry_prog }
    
    pack $w.$wi -fill x
    pack $w.$wi.k.label_prog -side left 
    pack $w.$wi.k -fill x
    pack $w.$wi.k.entry_prog $w.$wi.k.browse -side left 
     
    pack $w.$wi -fill x
    pack $w.$wi.i.label_doc -side left 
    pack $w.$wi.i -fill x
    pack $w.$wi.i.entry_doc $w.$wi.i.browse -side left 
     
    pack $w.$wi -fill x
    pack $w.$wi.s.label_tex -side left 
    pack $w.$wi.s -fill x
    pack $w.$wi.s.entry_tex -side left 
#    $w.$wi.s.browse -side left 
     
    button $w.go -text Go \
      -command {layout_layout_preview $doc_file $layoutgram_file} -width 8

    button $w.done -text Done -command {destroy $w} -width 8

    button $w.help -text Help -command "open_help $helpfile $w" -width 8
    pack $w.go $w.help $w.done -side left -expand yes 

    moveto_end $w.iframe.k.entry_prog
    moveto_end $w.iframe.i.entry_doc
    moveto_end $w.iframe.s.entry_tex
    grab $w
}

# *******************************************************************
# Name:      layout_layout_preview
# Input:     String (document file)
#	     String (layout grammar file)
# Output:    -
# Function:  Creates layout program file by calling procedure 
#            generate. Calls prolog-predicate
#            layout_program, which creates document layout and calls
#            tcl/tk-procedure make_preview
# *******************************************************************
proc layout_layout_preview {t1 t3} {
    global texs tex_file prog_file dir 
    global layoutgram_file doc_file
# value is changed in proc prolog_done called by prolog
    global ready w

# Name for the new output file (variable t2):
# New extension for the file (e.g. html or tex) is stored to variable type:
    set t6 [file tail $t3]
    set type [string range $t6 [expr [string first \_ $t6] + 1] [expr [string first \. $t6] - 1] ]
# this is corrected above
#    set type [string range $t3 [expr [string first \_ $layoutgram_file] + 1] [expr [string first \. $layoutgram_file] - 1]]
 
# Start of the new file name is stored to variable sname 
    set t5 [file tail $t1]
    set sname [string range $t5   [expr 0] [expr [string first \. $t5] - 1]]
    set t2 $sname.$type

    set t3 [file dirname $t3]/[file tail $t3]
    if [file isfile $t3] {

        set tmp [file tail $t3].pl
	set ready 0
        $w configure -cursor watch 
# prcedure "generate" creates layout program file
	generate $t3 $tmp 
	tkwait variable ready

        set t1 [file dirname $t1]/[file tail $t1]
        if [file isfile $t1] {
            set t2 [def_dir $t2 $texs]
	    if {[string length $t2] > 0} {
		set tex_file $t2
                p layout_program_preview('$t1','$t2','$prog_file')
            } else {
                prolog_done
	    }
        } else {
   	    tk_dialog .d {} "\"$t1\" isn't a regular file!" {} -1 OK
            prolog_done
        }
    } else {
	tk_dialog .d {} "\"$t3\" isn't a regular file!" {} -1 OK
    }
}

# *******************************************************************
# Name:      generate
# Input:     String (layout grammar file)
#            String (layout program file)
# Output:    -
# Function:  Calls prolog-predicate layout_generate.
# *******************************************************************
proc generate {t1 t2} {
    global progs prog_file

    set t1 [file dirname $t1]/[file tail $t1]
    if [file isfile $t1] {
	set t2 [def_dir $t2 $progs]
	if {[string length $t2] > 0} {
            set prog_file $t2
            p layout_generate('$t1','$t2')
        }
    } else {
	tk_dialog .d {} "\"$t1\" isn't a regular file!" {} -1 OK
    }
}

# *******************************************************************
# Name:      make_preview
# Input:     String (File to be previewed)
# Output:    -
# Function:  Preview file using program given in global variable
#            prev_prog. tex-files are previewd in procedure
#            preview_latex. Procedure make_preview is called from
#            Prolog predicate layout_program in file syndocoutput.pl.
# *******************************************************************
proc make_preview { file } {
    global prev_prog w
    global lFile

# If no preview program defined, sets prev_prog to "xmosaic"
    if {[string length $prev_prog] == 0} {
	set prev_prog "xmosaic"
    }

    set lFile [file dirname $file]/[file tail $file] 
    if [file isfile $lFile] {
	if {[file extension $lFile] != ".tex"} {
	    exec $prev_prog $lFile &
	} else {
	    preview_latex
	}
    } else {
	tk_dialog .d {} "\"$file\" isn't a regular file!" {} -1 OK
    }
}

# *******************************************************************
# Name:      preview_latex
# Input:     String (File to be previewed)
# Output:    -
# Function:  Creates the toplevel window for selecting Latex, Bibtex
#            and Preview functions.
# *******************************************************************
proc preview_latex {  } {
    global w wp
    global lpid;	# Processid of the latex-process
    global lFile;	# tex-file

    set lpid 0
    set wp .pwin
    catch {destroy $wp}
    toplevel $wp
    wm geometry $wp +220+150
    wm title $wp "Preview [file tail $lFile]"
    wm iconname $wp "Preview"

    foreach i { f1 f2 f3 f4 } {
	    frame $wp.$i -bd 2
	    label $wp.$i.label -width 45
	    pack $wp.$i.label -side left
    }

    pack  $wp.f1 $wp.f2 $wp.f3 $wp.f4
    $wp.f1.label config -text $lFile -anchor w 
    button $wp.f1.btn -text Latex -width 6\
	      -command {$wp.f2.btn config -state normal
			$wp.f3.btn config -state normal
			set lpid [exec latex $lFile &]}
    button $wp.f2.btn -text Bibtex -width 6\
		      -state disabled \
		      -command "make_bibtex $lFile"
    button $wp.f3.btn -text Preview -width 6\
		      -state disabled \
		      -command "make_tex_preview $lFile"
    button $wp.f4.btn -text Done -width 6 \
		      -command "destroy $wp" 
    foreach i { f1 f2 f3 f4 } {
	    pack $wp.$i.btn -side right
    }
}

# *******************************************************************
# Name:      make_bibtex
# Input:     String (File)
# Output:    -
# Function:  Executes bibtex.
# *******************************************************************
proc make_bibtex {lf} {
    global wp;		# Latex preview window 
    global prev_prog;   # Preview program (e.g xmosaic, netscape..)
    global lpid;	# Pid of the latex-process

    set errCode 0
    set auxFile [file tail [file rootname $lf]]
    if {$lpid != 0} { 
	catch {set errCode [exec ps $lpid]}
	if {$errCode != 0} { 
	    $wp.f2.label config -text \
			 "$auxFile.aux not ready, try again!" \
				-anchor w
        } else {
	    $wp.f2.label config -text \
	    "Executing bibtex with [file dirname $auxFile]/[file tail $auxFile].aux"\
				-anchor w
	    exec bibtex $auxFile &
	}    
					
    }
}

# *******************************************************************
# Name:      make_tex_preview
# Input:     String (File)
# Output:    -
# Function:  Executes preview for the dvi-file, if the latex-command
#            is executed.
# *******************************************************************
proc make_tex_preview {lf} {
    global wp;		# Latex preview window 
    global prev_prog;   # Preview program (e.g xmosaic, netscape..)
    global lpid;	# Pid of the latex-process

    set errCode 0
    set dviFile [file tail [file rootname $lf]].dvi 
    if {$lpid != 0} { 
	catch {set errCode [exec ps $lpid]}
	if {$errCode != 0} { 
	    $wp.f3.label config -text \
			 "$dviFile not ready, try again!" \
				-anchor w
        } else {
	    $wp.f3.label config -text \
	    "Executing preview with [file dirname $dviFile]/[file tail $dviFile]" \
				-anchor w
            exec $prev_prog $dviFile &
	}    
					
    }
}




# *******************************************************************
# Name:      print1_document 
# Input:     String (help file)
# Output:    -
# Function:  Creates top-level window for asking layout grammar file,
#            document file and output file.
# *******************************************************************
proc print1_document helpfile {
    global win w
    global browse progs layouts texs
    global tex_file prog_file layoutgram_file

    global printer print_cmd 
    global p_cmd

    set wi iframe
    catch {destroy $w}
    toplevel $w
    dpos $w
    wm title $w "Print document"
    wm iconname $w "Print document"
    set p_cmd $print_cmd
    append p_cmd $printer

#    set prev_prog $print_cmd
#    append prev_prog $printer

    frame $w.$wi -bd 1m
        frame $w.$wi.i -bd 2m
        label $w.$wi.i.label_doc -width 24 -text "Document file " \
				 -anchor w
        entry $w.$wi.i.entry_doc -relief sunken \
	    -textvariable doc_file 
        button $w.$wi.i.browse -text $browse \
	    -command { set doc_file [ create_browser $docs $doc_file]
                       moveto_end $w.iframe.i.entry_doc }

        frame $w.$wi.k -bd 2m
        label $w.$wi.k.label_prog -width 24 -text "Layout grammar file " \
				  -anchor w
        entry $w.$wi.k.entry_prog -relief sunken \
	    -textvariable layoutgram_file 
        button $w.$wi.k.browse -text $browse \
	    -command { 
	    set layoutgram_file [ create_browser $layouts $layoutgram_file]
                       moveto_end $w.iframe.k.entry_prog }

        frame $w.$wi.c -bd 2m
        label $w.$wi.c.label1 -width 24 -text "Print command " \
			      -anchor w
        entry $w.$wi.c.entry1 -relief sunken \
	    -textvariable p_cmd
    
    pack $w.$wi -fill x
    pack $w.$wi.k.label_prog -side left 
    pack $w.$wi.k -fill x
    pack $w.$wi.k.entry_prog $w.$wi.k.browse -side left 
     
    pack $w.$wi -fill x
    pack $w.$wi.i.label_doc -side left 
    pack $w.$wi.i -fill x
    pack $w.$wi.i.entry_doc $w.$wi.i.browse -side left 
     
    pack $w.$wi -fill x
    pack $w.$wi.c.label1 -side left
    pack $w.$wi.c -fill x
    pack $w.$wi.c.entry1 -side left
#    $w.$wi.s.browse -side left 
     
    button $w.go -text Go \
      -command {layout_layout_print $doc_file $layoutgram_file} -width 8

    button $w.done -text Done -command {destroy $w} -width 8

    button $w.help -text Help -command "open_help $helpfile $w" -width 8
    pack $w.go $w.help $w.done -side left -expand yes 

    moveto_end $w.iframe.k.entry_prog
    moveto_end $w.iframe.i.entry_doc
    moveto_end $w.iframe.c.entry1
    grab $w
}


# *******************************************************************
# Name:      layout_layout_print 
# Input:     String (document file)
#	     String (layout grammar file)
# Output:    -
# Function:  Creates layout program file by calling procedure 
#            generate. Calls prolog-predicate
#            layout_program_print, which creates document layout and calls
#            tcl/tk-procedure make_print
# *******************************************************************
proc layout_layout_print {t1 t3} {
    global texs tex_file prog_file dir 
    global layoutgram_file doc_file
# value is changed in proc prolog_done called by prolog
    global ready w

# Name for the new output file (variable t2):
# New extension for the file (e.g. html or tex) is stored to variable type:
    set t6 [file tail $t3]
    set type [string range $t6 [expr [string first \_ $t6] + 1] [expr [string first \. $t6] - 1] ]
# this is corrected above
#    set type [string range $t3 [expr [string first \_ $layoutgram_file] + 1] [expr [string first \. $layoutgram_file] - 1]]
 
# Start of the new file name is stored to variable sname 
    set t5 [file tail $t1]
    set sname [string range $t5   [expr 0] [expr [string first \. $t5] - 1]]
    set t2 $sname.$type

    set t3 [file dirname $t3]/[file tail $t3]
    if [file isfile $t3] {

        set tmp [file tail $t3].pl
	set ready 0
        $w configure -cursor watch 
# prcedure "generate" creates layout program file
	generate $t3 $tmp 
	tkwait variable ready

        set t1 [file dirname $t1]/[file tail $t1]
        if [file isfile $t1] {
            set t2 [def_dir $t2 $texs]
	    if {[string length $t2] > 0} {
		set tex_file $t2
                p layout_program_print('$t1','$t2','$prog_file')
            } else {
                prolog_done
	    }
        } else {
   	    tk_dialog .d {} "\"$t1\" isn't a regular file!" {} -1 OK
            prolog_done
        }
    } else {
	tk_dialog .d {} "\"$t3\" isn't a regular file!" {} -1 OK
    }
}


# *******************************************************************
# Name:      make_print
# Input:     String (File to be previewed)
# Output:    -
# Function:  Print file using the printer command in global variable
#            p_cmd. 
# *******************************************************************
proc make_print { print_file } {
   
   global p_cmd

    set print_file [file dirname $print_file]/[file tail $print_file] 
    if [file isfile $print_file] {
	eval exec $p_cmd $print_file
    } else {
	tk_dialog .d {} "\"$print_file\" isn't a regular file! " {} -1 OK
    }

}









