# *********************************************************************
# syndocretrieval.tcl 
#
# Function:   Creates top-level windows for SYNDOC programs retrieval-
#             functions.
# Date:       24.11.1995, 30.7.1996
# Author:     Paula Leinonen
#	      Jouni Mykkanen
#	      Toni Ronkko
# NOTE!
# - When future implemantations of tk support raising of a window (with raise)
#   you can uncomment raise commands from this file. DO NOT REMOVE RAISE
#   commands, they might be useful.
# - Search for words "Missing helpfile" or "hlp" to find out places in where
#   you should name the helpfile used.
# - Global variable browse (that contains text "Select") is not correctly
#   used in this module.
# - There are some possiblities of error when user has multiple windows
#   opened and he/she modifies the structure grammar. In that case some
#   procedures fail to accomplish their tasks without error message.
#
# *********************************************************************

#**************************************************************************
# Global variables - declarations in file  syndocglobals.tcl  
#**************************************************************************

    # Default values for list of nonterminals.
    set ftype_file *			;# Default nonterminal
    set ftypes *			;# Default list of nonterminals

    #
    # Window captions, also note that some of these are used in retrieval
    # menu.
    #
    set index_window_name 	"Index documents"
    set match_window_name 	"Compare filter to documents"
    set matched_window_name 	"Result of comparison"
    set oldfilt_window_name 	"Modify old filter"
    set newfilt_window_name 	"Create new filter"
    set ret_window_name 	"Document retrieval"

    #
    # Window names
    #
    set rw .rw				;# Main retrieval window
    set index_window .idxw		;# Index documents
    set match_window .matw
    set matched_window .matcwx		;# Preview docs
    set oldfilt_window .oldfilt		;# Open old filter
    set newfilt_window .newfilt
    set gou .generic_output		;# Prefix to ouput type windows

    #
    # Others
    #
    set output_form 1			;# Default output type
    set TMPHTML tmp.html		;# Filename of temporary file used
    					;#  in preview.


    #
    # These could be added to system defaults so user could modify them.
    #
    set LATEX_CMD  latex		;# Command used to create .dvi
    set BIBTEX_CMD bibtex


# *******************************************************************
# Name:     retrieval
# Input:    String
# Output:   -
# Function: Creates top-level window for asking structure grammar file
# 	    NOTE!
#	    - This procedure was done by Paula. Slightly modified by TR.
# *******************************************************************
proc retrieval helpfile {
    global browse 			;# "Select"
    global struct 			;# Def. path for struct grammars
    global struct_file			;# User's struct grammar
    global rw
    global index_window_name		;# Names to show on menu
    global match_window_name		;#
    global ret_window_name		;# Window's caption

    catch {destroy $rw}
    toplevel $rw
    dpos $rw
    wm title $rw    $ret_window_name
    wm iconname $rw $ret_window_name
    focus $rw

    #TR220896: add gram/ to *.gram if this is the first time.    
    # Here, unlike in other syndoc subsystems, user will be shown the directory
    # AT FIRST (others append default directory afterwards, so the user might 
    # not know to where did the file go).
    #
    set struct_file [def_dir2 $struct $struct_file]
      
# Menu:
   frame $rw.rmenu -relief raised -borderwidth 1
   pack $rw.rmenu -side top -fill x

# Index documents$rw.rmenu:
    menubutton $rw.rmenu.index -text "Indexing" \
     -menu $rw.rmenu.index.m \
     -state disabled -underline 0
    menu $rw.rmenu.index.m
# Indexing:
   $rw.rmenu.index.m add command -label $index_window_name \
      -command "index_documents $helpfile" -underline 0

# Define filter:
   menubutton $rw.rmenu.filter -text "Defining Filters" \
     -menu $rw.rmenu.filter.m -state disabled -underline 0
   menu $rw.rmenu.filter.m
   
   # TR230896: struct_file can be modified after selecting new filter from menu,
   # so grammar must be read just before needing it.
   # TR270896: Missing helpfile below (not my fault!)
   #
   $rw.rmenu.filter.m add command -label "Create New Filter" -underline 0 \
      -command {new_filter h.hlp} 

   $rw.rmenu.filter.m add command -label "Open Filter" \
       -command "old_filter h.help" \
       -underline 0

# Search Documents 
    menubutton $rw.rmenu.search -text "Filtering"  \
	-menu $rw.rmenu.search.m \
        -underline 0
	#TR220896
	#-state disabled
    menu $rw.rmenu.search.m
   $rw.rmenu.search.m add command -label $match_window_name \
	  -command  "match_filter help.hlp "\
          -underline 0

   pack $rw.rmenu.index $rw.rmenu.filter \
        $rw.rmenu.search -side left 
	##-anchor n -expand yes 

# Entry for asking grammar file
    frame $rw.iframe -bd 1m
	frame $rw.iframe.i -bd 2m
	label $rw.iframe.i.label_gram -width 24 \
		-text "Document structure file: "
	entry $rw.iframe.i.entry_gram -relief sunken \
	    -textvariable struct_file 
        bind $rw.iframe.i.entry_gram <Any-KeyRelease> {
				  check_file $rw.rmenu }
				  
	#TR220896: enable/disable buttons when variable struct_file is 
	# modified
	trace variable struct_file w "trace_struct_file $rw.rmenu"

	button $rw.iframe.i.browse -text $browse \
	    -command { set struct_file [create_browser $struct $struct_file]
	               moveto_end $rw.iframe.i.entry_gram 
		       check_file $rw.rmenu }

    pack $rw.iframe -fill x -anchor n
    pack $rw.iframe.i.label_gram -side left 
    pack $rw.iframe.i -fill x -anchor n

    pack $rw.iframe.i.browse -side right
    pack $rw.iframe.i.entry_gram  -side left -anchor w -fill x -expand yes 



# Buttons done and help
    button $rw.done -text Done -width 16 -command \
    { destroy $rw; trace vdelete struct_file w "trace_struct_file $rw.rmenu"} 
    button $rw.help -text Help -command "open_help $helpfile $rw" -width 16
    pack $rw.help $rw.done -side left -expand yes -anchor n

    moveto_end $rw.iframe.i.entry_gram
    check_file $rw.rmenu

}

# *********************************************************************
# Name:       trace_struct_file
# Input:      menu
#	      name1 name2 op (See trace variable for more information)
# Output:     -
# Function:   Enable/disable menu every time variable struct_file is modified.
#	      This function is hooked to variable struct_file with
#	      "trace variable" command. The purpose of this function is to
#	      prevent user from entering areas where invalid structure grammar
#	      would result in error. (Although those places make some
#	      error checkings by themselves). Remember that this function 
#	      might be entered when there are no windows on screen.
# *********************************************************************
proc trace_struct_file { m name1 name2 op} {
  global rw
  global filt_file

  # Check and enable/disable retrieval main window
  check_file $m

  if {[winfo exists $rw] != 0} {
    moveto_end $rw.iframe.i.entry_gram
  } 
 
  # Check and enable/disable new filter window
  CheckNewFiltCreation $filt_file  
  
  # Open old filter
  CheckOldFilter $filt_file
  
  # Preview button in matched_window
  PreviewEnable

}

# *********************************************************************
# Name:       IsLegalGrammar 
# Input:      String (filename to grammar)
# Output:     0 no, non 0 yes
# Function:   Check if a grammar is legal. This function is currently the
#	      only function in tcl that checks it, and is called widely
#	      inside syndocretrieval.tcl. In case that this function is 
#	      used in other subsystems, move this function to syndocsystem.tcl.
#	      NOTE!
#	      - Currently only the existence is checked. Grammar may be
#	        broken and thus unusable or invalid for some tasks.
#	        Use check_grams in prolog to check is it legal for given
#	        purpose.
# *********************************************************************
proc IsLegalGrammar { struct_file } {
  set ok 0
  
  set sf [fullpath $struct_file]

  if {[file exist $sf] != 0} {set ok 1}

  return $ok
}

# *********************************************************************
# Name:       check_file
# Input:      String
# Output:     0 functions disabled, non 0 enabled
# Function:   Checks if the name of the structure grammar file 
#             in entry-widget is an existing file (says Paula).  
# *********************************************************************
proc check_file { m } {
    global struct_file
    global rw

    set ok [IsLegalGrammar $struct_file]
    
    # Note!
    # - Main window might not exist when this function is called. So
    #   enable/disable only when window exist.
    if {[winfo exists $rw] != 0} {

      if {$ok != 0} {
	$m.index configure -state normal
	$m.filter configure -state normal
	#
	#TR220896: search does not seem to care about struct_file
	# If you want to search documents without selecting struct grammar,
	# comment line below AND $m.search... 5 lines below.
	#
	$m.search configure -state normal
      } else {
	$m.index configure -state disabled 
	$m.filter configure -state disabled 
	$m.search configure -state disabled 
      }
    }
    
    return ok
}


# *******************************************************************
# Name:     get_grammar_with_fullpath
# Input:    string_grammarfile
# Output:   -
# Function: Read first line from file (strip ' and .)
#	    Modified by TR from Jouni's work.
# *******************************************************************
proc get_grammar_with_fullpath {file} {
    set fp [open $file r]
    gets $fp grammar
    close $fp
    set grammar [string trim $grammar '.]
    return $grammar

}


# *******************************************************************
# *******************************************************************
# ******************** I N D E X I N G ******************************
# *******************************************************************
# *******************************************************************


# *******************************************************************
# Name:     index_documents
# Input:    helpfile
# Output:   -
# Function: User-interface for filtering documents.
# 	    NOTE!
#		Indexing is done only once (per session at most) so
#		values entered by user are not remembered.
# *******************************************************************

proc index_documents helpfile {
    global index_window			;# Window to create
    global index_window_name		;# Caption
    global documents_selected  		;# User's documents
    global icdb_file 			;# User's controlfile
    global icdb_mask			;# Mask and path to controlfiles
    global control_path			;# Default path for controls
    global docs 			;# Default path for documents
    global doc_file			;# Currently selected document
    global doc_mask			;# Mask and path to documents

    #TR220896: if window exists we can assume that might indexing is in 
    # progress. Thus we do not destroy the old window. If we would continue
    # from this point and destroy the window, user would be allowed to make
    # things that would otherwise be forbidden.
    #
    if {[winfo exists $index_window] == 1} {
#      raise $index_window
      focus $index_window
      return
    }


    set doc_mask [disppath [file dirname $docs]/*[file extension $doc_file]]

    #
    # NOTE!
    # - We are not using user-given contorlfile as a mask. This way we need
    #   not know did a user specify a mask or a file. Both ways work ok.
    # - Only bad thing is that this fixes control mask to *.ctrl
    #
    set icdb_file [disppath $control_path]
    set icdb_mask [disppath [file dirname $control_path]/*.ctrl]
     
    catch {destroy $index_window}
    toplevel $index_window
    wm geometry $index_window +50+250
    wm title $index_window $index_window_name
    wm iconname $index_window $index_window_name
    focus $index_window

    frame $index_window.sepa1 -height 10

    #
    # For selecting files for indexing
    #
    frame $index_window.d_frame
    label $index_window.d_frame.lab -width 20 -text "Documents files: " \
        -anchor w

    entry $index_window.d_frame.entr -relief sunken \
                        	  -textvariable documents_selected -width 30
    bind $index_window.d_frame.entr <Any-KeyRelease> {
	      check_go_state $documents_selected $icdb_file
	    }
    button $index_window.d_frame.sel -text Select \
            -width 8  -command \
	    { 
	      set documents_selected [create_browser $doc_mask $documents_selected]
	      moveto_end $index_window.d_frame.entr
	      check_go_state $documents_selected $icdb_file
	    }
    pack $index_window.d_frame.lab -side left
    pack $index_window.d_frame.sel -side right
    pack $index_window.d_frame.entr -side left -fill x -anchor w -expand yes

    #
    # For selecting control file
    #
    frame $index_window.c_frame
    label $index_window.c_frame.lab -width 20 -text "Index control file: " \
          -anchor w

    entry $index_window.c_frame.entr -relief sunken \
                        	  -textvariable icdb_file -width 30
    bind $index_window.c_frame.entr <Any-KeyRelease> {
	      check_go_state $documents_selected $icdb_file
	    }
    button $index_window.c_frame.sel -text Select \
	    -width 8 -command \
	    { 
	      set icdb_file [create_browser $icdb_mask [disppath $icdb_file]]
	      moveto_end $index_window.c_frame.entr
	      check_go_state $documents_selected $icdb_file
	    }

 
    pack $index_window.c_frame.lab -side left
    pack $index_window.c_frame.sel -side right
    pack $index_window.c_frame.entr -side left -fill x -anchor w -expand yes

    frame $index_window.sepa2 -height 16
        
    #
    # buttons
    #    
    button $index_window.go -text Apply \
	    -width 8 -state disabled \
            -command \
	    {
	      set control_path [disppath $icdb_file]
	      set a [makeindex $documents_selected $icdb_file $index_window]
	      if {$a != 0} {
	        # Index was generated OK
  	        destroy $index_window
	      } else {
	        # Index was NOT generated
                wm title $index_window "$index_window_name - indexing failed."
                wm iconname $index_window "$index_window_name - indexing failed."
		update
	      }
	    }
    button $index_window.help -text Help \
            -command "open_help $helpfile $index_window" -width 8
    button $index_window.cancel -text Done \
            -command {
	      set control_path [disppath $icdb_file]
	      destroy $index_window
	    } -width 8

    #
    # Show window
    #
    check_go_state $documents_selected $icdb_file
    moveto_end $index_window.c_frame.entr
    moveto_end $index_window.d_frame.entr
    
    pack $index_window.sepa1 -side top -anchor n
    pack $index_window.d_frame -side top  -fill x
    pack $index_window.c_frame -side top  -fill x
    pack $index_window.sepa2 -side top -anchor n

    pack $index_window.go -side left -expand yes -anchor n
    pack $index_window.help -side left -expand yes -anchor n
    pack $index_window.cancel -side left -expand yes -anchor n
}


# *******************************************************************
# Name:     check_go_state
# Input:    string (document, document mask OR list of documents)
#	    string (controlfile)
# Output:   -
# Function: Enable/disable go button (indexing window)
#	    NOTE!
#	    - Button is enabled when control points to a single file and
#	      document points to one or many documents.
#	    - [file exist] cannot be used with mask parameter.
# *******************************************************************
proc check_go_state {mask ctrl} {
    global index_window


    if {$mask == ""} { 
    	set no_go_on 1
    } else {
        set no_go_on [catch { glob -- [fullpath $mask] } t]
    }
    
    if {$ctrl != ""} {
        set ctrl [fullpath $ctrl]
        catch {
        if { [file exists $ctrl] == 0} {
            set no_go_on 1
        } else {
          if {[file isfile $ctrl] == 0} {
            set no_go_on 1
           }
        }
      } t
    
      if {$t != ""} { set no_go_on 1}
    } else {
      set no_go_on 1
    }
			 
    if {$no_go_on == 0} { 
      $index_window.go config -state normal
    } else {
      $index_window.go config -state disabled
    }
}


# *******************************************************************
# Name:     makeindex
# Input:    documents_selected 
#           control_file
#	    windowpath (for displaying caption)
# Output:   0 failure, non 0 OK
# Function: Executes syndocmakeindex.tcl to index documents
#	    NOTE!
#	    - This function creates index for:
#		A) one file, file=~/syndoc/xxx.doc
#		B) masked files, file=~/syndoc/*.doc
#		C) multiple files, file="~/syndoc/x1.doc x2.doc /*.doc"
#	    - Modified from Jouni's work by TR.
# *******************************************************************
proc makeindex {file control index_window} {
    global struct_file 
# 12.6.97 EK added the name of the index path
    global index_path

    global index_window_name
    global SYNDOCHOME;      # Directory for Syndoc program files
    
    if {[IsLegalGrammar $struct_file] == 0} {
      # Grammar is illegal, should we inform user?
      return 0
    }

    $index_window configure -cursor watch 
    update
    
    set control [fullpath $control]

    set grammar [file tail $struct_file]

    set status "OK"
    set files [glob -nocomplain -- [fullpath $file]]

    set cnt 1    
    set files_cnt [llength $files]
    foreach i $files {
        set ots "$index_window_name - indexing... ($cnt/$files_cnt)"
        wm title $index_window "$ots"
        wm iconname $index_window "$ots"
	update
	
	set cnt [expr $cnt+1]

    	set tgrammar [get_grammar_with_fullpath $i]
	if [ string match $tgrammar $grammar] {
# path of the index file is not path of the document file
# path should be index_path
# 	    set ip [file dirname $i]
 	    set ip [file dirname $index_path]
	    set table_file [fullpath $ip/[file tail $i].index]
	    	set status "$i -> $table_file"
      	    	exec $SYNDOCHOME/syndocmakeindex.tcl $i $control > $table_file
	}
    }

    wm title $index_window "$index_window_name"
    wm iconname $index_window "$index_window_name"
    $index_window configure -cursor arrow
    return 1
}


# *******************************************************************
# *******************************************************************
# ********* F I L T E R  D E F I N I T I O N  ***********************
# *******************************************************************
# *******************************************************************

# *********************************************************************
# Name:       new_filter
# Input:      String (help file)
# Output:     -
# Function:   Creates top-level window for asking structure grammar file
#             and new filter file name.
#	      NOTE!
#	      - Original by Paula, Modified by TR. (Used fullpath, removed
#		dependency to $w, commented buttons, labels and entries
#		corresponding to struct_file)
# *********************************************************************

proc new_filter helpfile {
    global ready666			;# Prolog signals using this when done
    global browse 			;# "Select"
    global struct 
    global filts 
    global ftypes			;# List of nonterminals
    global ftype_file 			;# User's nonterminal
    global struct_file 
    global filt_file 			;# User's filter
    global filts 
    global filter_mask
    global newfilt_window 		;# Window to create
    global newfilt_window_name 		;# Caption
    global oldfilt_window		;# destroy this window

    #TR220896
    set filt_file [def_dir2 $filts $filt_file]

    catch {destroy $oldfilt_window}
    catch {destroy $newfilt_window}
    toplevel $newfilt_window

    wm geometry $newfilt_window +200+250
    wm title $newfilt_window $newfilt_window_name
    wm iconname $newfilt_window $newfilt_window_name
    focus $newfilt_window

    set filter_mask [disppath [file dirname $filts]/*[file extension $filt_file]]

    frame $newfilt_window.iframe -bd 1m

        frame $newfilt_window.iframe.s -bd 1m
        label $newfilt_window.iframe.s.label_gram -width 24 \
		-text "Filter file: " -anchor w
        entry $newfilt_window.iframe.s.entry_gram -relief sunken \
	    -textvariable filt_file 
        button $newfilt_window.iframe.s.browse -width 8 -text $browse \
	    -command { set filt_file [ create_browser $filter_mask $filt_file]
    			CheckNewFiltCreation $filt_file
		       moveto_end $newfilt_window.iframe.s.entry_gram } 
        bind $newfilt_window.iframe.s.entry_gram <Any-KeyRelease> {
		    	CheckNewFiltCreation $filt_file }
    
        frame $newfilt_window.iframe.k -bd 1m
        label $newfilt_window.iframe.k.label_gram -width 24 \
		-text "Filter type: " -anchor w
        entry $newfilt_window.iframe.k.entry_gram -relief sunken \
	    -textvariable ftype_file 
        button $newfilt_window.iframe.k.browse -width 8 -text $browse \
	    -command { 
	       set ready666 0
	       #TR230896: read grammar when it is sure that user cannot
	       # modify it anymore.
	       p readGrammar_new('$struct_file')
               p tcl_searchNonterminals
	       tkwait variable ready666
	       set ftype_file [ create_browser_list $ftypes $ftype_file]
	       moveto_end $newfilt_window.iframe.s.entry_gram }
   
    pack $newfilt_window.iframe -fill x -anchor n
    
    pack $newfilt_window.iframe.s.label_gram -side left
    pack $newfilt_window.iframe.s -fill x -anchor n
    pack $newfilt_window.iframe.s.entry_gram \
    	-side left -fill x -expand yes -anchor w
    pack $newfilt_window.iframe.s.browse -side right
    
    pack $newfilt_window.iframe.k.label_gram -side left
    pack $newfilt_window.iframe.k -fill x -anchor n
    pack $newfilt_window.iframe.k.entry_gram \
    	-side left -fill x -expand yes -anchor w
    pack $newfilt_window.iframe.k.browse -side right
     


    button $newfilt_window.go -text Go -state disabled \
      -command {  
                 #270896: If user types a short name, we must expand it
		 # so it looks like selected.
		 set filt_file [fullpath $filt_file]
		
	         p readGrammar_new('$struct_file')
                 p tcl_searchNonterminals
	         tkwait variable ready666
                 set a [make_new_filter $struct_file $filt_file $ftype_file]
                 if {$a == 0} {
		   #failure
		 } else {
                   destroy $newfilt_window
		 }
	} -width 8

    button $newfilt_window.done -text Done -command {
					destroy $newfilt_window} -width 8

    button $newfilt_window.help -text Help \
         -command "open_help $helpfile $newfilt_window" -width 8
    pack $newfilt_window.go \
         $newfilt_window.help \
	 $newfilt_window.done -side left -expand yes -anchor n

    moveto_end $newfilt_window.iframe.s.entry_gram
    moveto_end $newfilt_window.iframe.k.entry_gram
    CheckNewFiltCreation $filt_file

}

# *******************************************************************
# Name:       CheckNewFiltCreation 
# Input:      String (filter file)
# Output:     0 Can create, 1 cannot
# Function:   Check if given filter can be created.
#	      NOTE!
#	      - This function does not cover all possible errors. If disk space
#	        does not allow file creation or user has no rights to write a
#	        directory this function will give wrong information.
# *******************************************************************
proc CheckNewFiltCreation {filt_file} {
  global newfilt_window			;# Window that contains browse and go
  global struct_file
  
  set ok 0		;# 0 = enable Go
  set gram 0		;# 0 = enable select
  
  set filt_file [fullpath $filt_file]
  
  # check directory containing file
  if { [file isdirectory [file dirname $filt_file]] == 0} {
    # was not directory, so we cannot create file there
    set ok 1
  } else {
    if { [llength [glob -nocomplain -- $filt_file]] > 1} {
      # File matches to several files, like *.doc
      set ok 1
    } else {
      if { ([file exist $filt_file] != 0) && ([file writable $filt_file] == 0) } {
        # File exists but is not writable
        set ok 1
      }
    }
  }
  
  # If struct_file does not point to a legal file, we cannot create a filter
  if {[IsLegalGrammar $struct_file] == 0} { 
    set ok 1
    set gram 1
  }
   
  if {[winfo exists $newfilt_window] != 0} {
    if {$ok == 0} {
      $newfilt_window.go configure -state normal
    } else {
      $newfilt_window.go configure -state disabled
    }
    
    # If grammar is illegal, we cannot select a nonterminal from it.
    if {$gram == 0} {
      $newfilt_window.iframe.k.browse configure -state normal
    } else {
      $newfilt_window.iframe.k.browse configure -state disabled
    }
  }

  return $ok
}

# *******************************************************************
# Name:       make_new_filter
# Input:      String (document structure, MUST EXIST)
#	      String (name of new filter)
#	      String (filter type, NOT A FILENAME!)
# Output:     0 failure, 1 OK
# Function:   If filenames are ok, call prolog-predicate profile_new.
#	      Paula, modified by TR.
# *******************************************************************
proc make_new_filter {t1 t2 t3} {

    set t2 [fullpath $t2]
    set t1 [fullpath $t1]

    if {[IsLegalGrammar $t1] == 0} {
      # Grammar is illegal, should we inform user?
      return 0
    }

    if [file isfile $t1] {
          p filter_new('$t1','$t2','$t3')
    } else {
        # Note!
	# - This error message should never be shown. Currently user cannot
	#   start creating a new filter when structure is not a file.
        tk_dialog .d {} "\"$t1\" isn't a regular file!" {} -1 OK
    }
    return 1
}


# *********************************************************************
# Name:       old_filter
# Input:      String (help file) 
# Output:     -
# Function:   Creates top-level window for asking filter file.
#	      Note!
#	      - Original by Paula, Modified by TR (Removed dependency to $w,
# 		Commented struct_file (see new_filter)).
# *********************************************************************
proc old_filter helpfile {
    global browse			;# "Select"
    global filts 			;# Default path to filters
    global filt_file			;# User's filter
    global struct_file
    global filter_mask			;# Path to filters
    global oldfilt_window		;# Window to create
    global oldfilt_window_name		;# Caption
    global newfilt_window		;# Destroy this window

    #TR220896
    set filt_file [def_dir2 $filts $filt_file]


    catch {destroy $oldfilt_window}
    catch {destroy $newfilt_window}
    toplevel $oldfilt_window
    wm geometry $oldfilt_window +200+250
    wm title $oldfilt_window $oldfilt_window_name
    wm iconname $oldfilt_window $oldfilt_window_name
    focus $oldfilt_window

    set filter_mask [disppath [file dirname $filts]/*[file extension $filt_file]]

    frame $oldfilt_window.iframe -bd 1m
        frame $oldfilt_window.iframe.i -bd 1m
        label $oldfilt_window.iframe.i.label_doc -width 24 \
		-text "Filter file: "  -anchor w
        entry $oldfilt_window.iframe.i.entry_doc -relief sunken \
	    -textvariable filt_file 
        bind $oldfilt_window.iframe.i.entry_doc <Any-KeyRelease> {
    			CheckOldFilter $filt_file }
        button $oldfilt_window.iframe.i.browse -width 8 -text $browse \
	    -command {  set filt_file [ create_browser $filter_mask $filt_file]
    			CheckOldFilter $filt_file
		        moveto_end $oldfilt_window.iframe.i.entry_doc }


    pack $oldfilt_window.iframe -fill x -anchor n

    pack $oldfilt_window.iframe.i.label_doc -side left 
    pack $oldfilt_window.iframe.i -fill x -anchor n
    pack $oldfilt_window.iframe.i.entry_doc \
    	-side left -fill x -anchor w -expand yes
    pack $oldfilt_window.iframe.i.browse -side right
     
    button $oldfilt_window.go -text Go \
      -command { 
                #TR230896
	        p readGrammar_new('$struct_file')
                p tcl_searchNonterminals
	        tkwait variable ready666
                set a [make_old_filter $filt_file $struct_file]
                if {$a != 0} {
		  # OK
                  destroy $oldfilt_window
		}
	       } -width 8

    button $oldfilt_window.done -text Done -command {
                                        destroy $oldfilt_window} -width 8

    button $oldfilt_window.help -text Help \
    	-command "open_help $helpfile $oldfilt_window" -width 8
    pack $oldfilt_window.go \
         $oldfilt_window.help \
	 $oldfilt_window.done -side left -expand yes -anchor n

    moveto_end $oldfilt_window.iframe.i.entry_doc
    CheckOldFilter $filt_file


}

# *******************************************************************
# Name:       CheckOldFilter
# Input:      String (filter file)
# Output:     0 exists, 1 does not
# Function:   Check if given filter can be found. Function is used
#	      to enable/disable go button in old filter window.
#	      Button will be enabled when filter file exists and document
#	      structure is legal.
# *******************************************************************
proc CheckOldFilter {filt_file} {
  global oldfilt_window			;# Window that contains go button
  global struct_file
  set ok 0
  
  set filt_file [fullpath $filt_file]
  
  # check directory containing file
  if { [file exist $filt_file] == 0 } {
    #does not exist
    set ok 1
  }

  # If struct_file does not point to a legal file, we cannot open a filter
  if {[IsLegalGrammar $struct_file] == 0} { 
    set ok 1
  }

  # NOTE!
  # - This function will be called also when the window does not exist.
  #
  if {[winfo exists $oldfilt_window] != 0} {
    # Window exists, enable/disable 
    if {$ok == 0} {
      $oldfilt_window.go configure -state normal
    } else {
      $oldfilt_window.go configure -state disabled
    }
  }

  return $ok
}


# *********************************************************************
# Name:       make_old_filter
# Input:      String (name of a filter file)
#             String (name of a structure grammar file)
# Output:     0 failure, 1 OK
# Function:   Calls prolog-predicate filter_old with arguments     
#             filter file and structure grammar file. Modified from Paula's
#	      original work by TR (added fullpath).
# *********************************************************************
proc make_old_filter {t1 t2} {

    set t2 [fullpath $t2]
    set t1 [fullpath $t1]

    if {[IsLegalGrammar $t2] == 0} {
      # Grammar is illegal, should we inform user?
      return 0
    }

    if [file isfile $t1] {
        exec cp $t1 $t1.tmp 
        p filter_old('$t1','$t2')
    } else {
        # Note!
	# - This error message should never be shown. Currently user cannot
	#   select go-button if filter is not a file.
        tk_dialog .d {} "\"$t1\" isn't a regular file!" {} -1 OK
    }	
    return 1
}



# *******************************************************************
# *******************************************************************
# ******************** F I L T E R I N G  ***************************
# *******************************************************************
# *******************************************************************



# *******************************************************************
# ****************** C O M P A R I N G  F I L T E R  ****************
# *******************************************************************


# *******************************************************************
# Name:     match_filter
# Input:    String (helpfile)
# Output:   -
# Function: Creates top-level window for asking filter file.
# *******************************************************************
proc match_filter helpfile {

    global match_window			;# Window to create
    global match_window_name		;# Caption
    global filter_mask			;# Path and mask to filters
    global filts			;# Default path to filters
    global filt_file			;# User's filter
    global index_file			;# Default indexed filename
    global index_path                   ;# default path of idexed files
    global documentidxs_selected	;# User's document indices
    global doc_mask 			;# Path and mask to documents
    global index_mask                   ;# path and mask to indeces
    global docs				;# Default path to documents
    global documents_selected		;# Documents selected in indexing

    # If window exists, raise it above others and leave
    # Note!
    # - We shall not destroy the window. Matching might be in progress.
    if {[winfo exists $match_window] == 1} {
#      raise $match_window
      focus $match_window
      return
    }

    #TR220896
    set filt_file [def_dir2 $filts $filt_file]

    catch {destroy $match_window}
    toplevel $match_window
    wm geometry $match_window +400+250
    wm title $match_window $match_window_name
    wm iconname $match_window $match_window_name
    focus $match_window

    # NOTE!
    # - Since create_browser cannot not currently handle paths around
    #   $dir, we create browsers in static directories ($filts, $docs).
    #   If browser is fixed some day, we could take these masks from previously
    #   selected filename.
    #
    set filter_mask [disppath [file dirname $filts]/*[file extension $filt_file]]
    set doc_mask [disppath [file dirname $docs]/*[file extension $index_file]]

    set index_mask [disppath [file dirname $index_path]/*[file extension $index_file]]

    # Note!
    # - This code gets document names from indexing. If user has selected a
    #   document elsewhere in syndoc, it WILL NOT be default when it comes
    #   to matching.
    #
    set documentidxs_selected ""
    foreach i $documents_selected {
# name of indexed documents are shown to the user (12.6.1997 EK)
#	set i [fullpath [file dirname $i]/[file tail $i].index]
	set i [fullpath [file dirname $index_path]/[file tail $i].index]
	set documentidxs_selected "$documentidxs_selected$i "
    }
    set documentidxs_selected [string trimright $documentidxs_selected]


    #
    # Leave some space between caption and first component
    #
    frame $match_window.sepa1 -height 10

    #
    # Select filter
    #
    frame $match_window.c_frame
    label $match_window.c_frame.lab -width 20 -text "Filter file: " \
          -anchor w

    entry $match_window.c_frame.entr -relief sunken \
                        	  -textvariable filt_file -width 30
    bind $match_window.c_frame.entr <Any-KeyRelease> {
	      check_filt_state $documentidxs_selected $filt_file
	    }
    button $match_window.c_frame.sel -text Select \
	    -width 8 -command \
	    { 
	      set filt_file [create_browser $filter_mask $filt_file]
	      moveto_end $match_window.c_frame.entr
	      check_filt_state $documentidxs_selected $filt_file
	    }

 
    pack $match_window.c_frame.lab -side left
    pack $match_window.c_frame.sel -side right
    pack $match_window.c_frame.entr -side left -fill x -anchor w -expand yes

    #
    # For selecting documents
    #
    frame $match_window.d_frame
    label $match_window.d_frame.lab -width 20 -text "Documents files: " \
        -anchor w

    entry $match_window.d_frame.entr -relief sunken \
                        	  -textvariable documentidxs_selected -width 30
    bind $match_window.d_frame.entr <Any-KeyRelease> {
	      check_filt_state $documentidxs_selected $filt_file
	    }
    button $match_window.d_frame.sel -text Select \
            -width 8  -command \
	    { 
	      set documentidxs_selected [create_browser $index_mask $documentidxs_selected]
	      moveto_end $match_window.d_frame.entr
	      check_filt_state $documentidxs_selected $filt_file
	    }
    pack $match_window.d_frame.lab -side left
    pack $match_window.d_frame.sel -side right
    pack $match_window.d_frame.entr -side left -fill x -anchor w -expand yes



    frame $match_window.sepa2 -height 10

    #
    # buttons
    #    
    button $match_window.go -text Apply \
	    -width 8 -state disabled \
            -command \
	    {

	      set documents_matched [ma_control [fullpath $documentidxs_selected] [fullpath $filt_file] $match_window]
	      if {[llength $documents_matched] == 0} {
	        tk_dialog .d $match_window_name "No matching documents." {} 0 "OK"
	      } else {
	        #
		# NOTE!
		# - Window is not destroyed so user can try again if 
		#   filtering matched with too many documents. Window is
		#
	        #destroy $match_window
		#
		# NOTE!
		# - Missing helpfile below.
		#
	        DisplayDocuments "hlp" $documents_matched
	      }
	    }
    button $match_window.help -text Help \
            -command "open_help $helpfile $match_window" -width 8
    button $match_window.cancel -text Done \
            -command {
	      destroy $match_window
	    } -width 8

    #
    # Show
    #
    check_filt_state $documentidxs_selected $filt_file
    moveto_end $match_window.c_frame.entr
    moveto_end $match_window.d_frame.entr

    pack $match_window.sepa1 -side top -anchor n
    pack $match_window.c_frame -side top -fill x -anchor n
    pack $match_window.d_frame -side top -fill x -anchor n
    pack $match_window.sepa2 -side top -anchor n

    pack $match_window.go -side left -expand yes -anchor n
    pack $match_window.help -side left -expand yes -anchor n
    pack $match_window.cancel -side left -expand yes -anchor n

}


# *******************************************************************
# Name:     check_filt_state
# Input:    string string
# Output:   -
# Function: Enable/disable go button (match window). The button will be 
#	    disabled when index does not correspond to single a file or when 
#	    the document mask does not match a file or files.
# *******************************************************************
proc check_filt_state {documents filter} {
    global match_window			;# Window that contains go button

    if { $documents == ""} {
    	set no_go_on 1
    } else {
	set no_go_on [catch { glob -- [fullpath $documents] } t]
    }
     
    if {$filter == ""} {
    	set no_go_on 1
    } else {
        set filter [fullpath $filter]


        # ? Is that catch really necessary ?
        catch {
          if { [file exists $filter] == 0} {
            set no_go_on 1
          } else {
            if {[file isfile $filter] == 0} {
              set no_go_on 1
             }
          }
      	} t
    
      	if {$t != ""} { set no_go_on 1}
    
    	if { [llength $filter] > 1 } {
    	  set no_go_on 1
    	}
    }
			 
    if {$no_go_on == 0} { 
      $match_window.go config -state normal
    } else {
      $match_window.go config -state disabled
    }
}


#########################################################################
# Name:         match_with_fullpath
# Input:        name of table file
#		name of filter file
# Output:	string containing "MATCH" in case of match
# Function:     executes syndocmakematch.tcl to match a filter to a
#		indexed file
#		Note!
#		- Modified from Jouni's work by TR.
#########################################################################
proc match_with_fullpath {table_file filter_file} {
   global SYNDOCHOME;      # Directory for Syndoc program files

   return [exec $SYNDOCHOME/syndocmakematch.tcl $table_file $filter_file]
}


#########################################################################
# Name:         ma_control
# Input:        file (string, Multiple filenames/masks to search)
#               filter (full path to filter)
#		windowpath (to show caption)
# Returns:      LIST{} of matching documents
# Function:     Matches all table files in current path against
#               the filter.               
#		NOTE!
#		- filter_file argument must be handled with fullpath.
#		- Modified from Jouni's work by TR.
#	
#########################################################################
proc ma_control { file filter_file match_window} {
    global match_window_name		;# Caption to modify
    global index_path                   ;# Default path to indices
    global docs                         ;# Default path to documents 

    $match_window configure -cursor watch
    update

    set grammar [get_grammar_with_fullpath $filter_file]
 
    set files [glob -nocomplain -- $file]
    
    set cnt 1
    set files_cnt [llength $files]
    set mdoc {}
    foreach i $files { 
        set ots "$match_window_name - comparing... ($cnt/$files_cnt)"
        wm title $match_window "$ots"
        wm iconname $match_window "$ots"
	update
	set cnt [expr $cnt+1]

        if [string match $grammar [get_grammar_with_fullpath $i]] {
# in the window is the name of doc, matching should be
# made comparing index-files in index directory
 	    set ip [file dirname $index_path]
	    set table_file [fullpath $ip/[file tail $i]]
#      	    if [string match MATCH [match_with_fullpath $i $filter_file]]   
    if [string match MATCH [match_with_fullpath $table_file $filter_file]] {
		lappend mdoc "[fullpath [file dirname $docs]/[file rootname [file tail $i]]]"
#		lappend mdoc "[fullpath [file dirname $i]/[file rootname [file tail $i]]]"
    }
}
}
    wm title $match_window "$match_window_name"
    wm iconname $match_window "$match_window_name"
    $match_window configure -cursor arrow
    update

    return $mdoc
}


# *******************************************************************
# **************** D I S P L A Y   D O C U M E N T S ****************
# *******************************************************************

# *******************************************************************
# Name:     DisplayDocuments
# Input:    helpfile (string)
#           documents (LIST of documents to preview)
# Output:   -
# Function: Show matched documents. This is the windows that lets user
#           select the type of output.
# 	    Bugs:
#           - When user selects "no output", currently opened output type 
#	      windows are left open. When user opens this window next time,
#	      the previously opened windows are still open and thus
#	      preview button will be disabled until all ouput type windows
#	      have been closed.
#	    - This function might be handy in other subsystems also.
#	      All you need to do is to insert a select button so user could
#	      modify list of previously selected docs or select new ones.
# *******************************************************************

proc DisplayDocuments {helpfile docs} {
    global matched_window		;# Window to create
    global matched_window_name		;# Caption
    global output_form			;# Position of red block in menu 

    # If window exists, raise it above others and leave
    if {[winfo exists $matched_window] == 1} {
#      raise $matched_window
      focus $matched_window
      return
    }

    catch {destroy $matched_window}
    toplevel $matched_window
    wm geometry $matched_window +400+400
    wm title $matched_window $matched_window_name
    wm iconname $matched_window $matched_window_name
    focus $matched_window

    # 
    # Menu
    #
    frame $matched_window.rmenu -relief raised -borderwidth 1

    # Define Output form:
    menubutton $matched_window.rmenu.output -text "Define Output Form" \
	-menu $matched_window.rmenu.output.m -underline 0
    menu $matched_window.rmenu.output.m
    $matched_window.rmenu.output.m add radiobutton -label "Default" \
	 -command "generic_output $helpfile 1" \
	 -underline 0 -variable output_form -value 1
    $matched_window.rmenu.output.m add radiobutton -label "Input Form" \
	 -command "generic_output $helpfile 2" \
	 -underline 8 -variable output_form -value 2
    $matched_window.rmenu.output.m add radiobutton -label "Formatted" \
	  -command  "generic_output $helpfile 3"\
          -underline 0 -variable output_form -value 3
    $matched_window.rmenu.output.m add radiobutton \
    	  -label "Transformed and Formatted" \
	  -command  "generic_output $helpfile 4"\
          -underline 0 -variable output_form -value 4
    #
    # ! INSERT MENU ENTRIES FOR NEW OUTPUT TYPES HERE !
    # Use lines above as templates.
    #

    pack $matched_window.rmenu -side top -fill x
    pack $matched_window.rmenu.output -side left

    #
    # Space between menu and frame
    #
    frame $matched_window.sepa1 -height 10

    
    frame $matched_window.fr
    label $matched_window.fr.txt -anchor w -width 40 \
    	-text "\n  Found [llength $docs] documents.\n" 
        
    pack  $matched_window.fr.txt
        
    #
    # Space between frame and buttons
    #
    frame $matched_window.sepa2 -height 10

    #
    # buttons
    #    
    # Note!
    # - Since preview can take a long time to finish, we must disable the
    #   button for a while. Also note that we MUST NOT destroy and create 
    #   this window while generating documents. That is taken care of in
    #   start of this procedure.
    # - Command must be in quoted string. Braces will not do since docs is
    #   not a global variable.
    #
    button $matched_window.go -text "Preview" \
	    -width 8 -state disabled\
            -command \
"$matched_window.go configure -state disabled;PreviewDocuments [list $docs];$matched_window.go configure -state normal"

    button $matched_window.help -text Help\
            -command "open_help $helpfile $matched_window" -width 8
    button $matched_window.cancel -text "Done"\
            -command {
	      destroy $matched_window
	    } -width 8

    #
    # Show components
    #
    pack $matched_window.sepa1 -side top -anchor n
    pack $matched_window.fr -side top -anchor n
    pack $matched_window.sepa2 -side top -anchor n
    
    pack $matched_window.go -side left -expand yes -anchor n
    pack $matched_window.help -side left -expand yes -anchor n
    pack $matched_window.cancel -side left -expand yes -anchor n
    
    PreviewEnable
}

# *******************************************************************
# Name:       PreviewEnable
# Input:      -
# Output:     -
# Function:   Check if preview is allowed.
#	      NOTE!
#	      - Preview button is disabled when:
#		A) structure grammar is illegal
#		B) At least one of the output type windows is open.
#	      - If preview is disable because of reason A, user might not
#		known the exact reason. Structure grammar is displayed in
#		different window, and user might not realize the connection
#		between those windows (main retrieval and matched windows).
#	      - This function will be called also when matched_window DOES NOT
#		exist.
# *******************************************************************
proc PreviewEnable {} {
  global struct_file 
  global matched_window 		;# Window that has go button
  global gou				;# Prefix to output type windows
  
  set ok 0		;# 0 = enable Go
  
  # Check every possible output type window. Currently only 3 and 4 are
  # possible.
  if {[winfo exists ${gou}1] != 0} {
    set ok 1
  } 
  if {($ok == 0) && ([winfo exists ${gou}2] != 0)} {
    set ok 1
  } 
  if {($ok == 0) && ([winfo exists ${gou}3] != 0)} {
    set ok 1
  } 
  if {($ok == 0) && ([winfo exists ${gou}4] != 0)} {
    set ok 1
  }
  # 
  # ! INSERT CHECKINGS FOR NEW OUTPUT TYPES HERE !
  # Use lines above as a template.
  #
  
  # If struct_file does not point to a legal file, we cannot allow preview
  if {($ok == 0) && ([IsLegalGrammar $struct_file] == 0)} { 
    set ok 1
  }
   
  if {[winfo exists $matched_window] != 0} {
    if {$ok == 0} {
      $matched_window.go configure -state normal
    } else {
      $matched_window.go configure -state disabled
    }
 
  }

  return $ok
}



# *******************************************************************
# Name:     PreviewDocuments
# Input:    Documents (LIST{})
# Output:   -
# Function: Create necessary programs(pl) so documents can be generated
#	    and viewed.
#           NOTE!
#	    - This function uses global variable output_form to decide
#	      which type of ouput is wanted. In order to preview
#             documents you MUST LET THE USER SELECT THE TYPE OF OUTPUT.
#	    - This procedure is tightly coupled to "define output type"
#	      menu. If you want to use this function, please include
#	      "define output menu" from DisplayDocuments to your
#	      window.
# *******************************************************************
proc PreviewDocuments { docl} {
  global TMPHTML 			;# Name of file to create and display
  global tulos 				;# Result of prolog predicates
  global struct_file 

  global w				;# Wait for window named $w (type 2)
  global doc_file			;# Pass name of document to Paula's
  					;#  old_document using this (type 2).

  global ready				;# Prolog signals using this when done

  global transform_struct_file		;# Struct gram of transformed doc
  global transform_NewType   
  global transform_NewText  
  
  global output_form 			;# Type of ouput wanted (by user)
  global prevs				;# Default path to preview files

  global prev_prog			;# User's preview program
 
  global progs				;# Def. path to layout programs (pl)
  global trprogs 			;# Def. path to transform programs

  global generic_param_t_frame$output_form	;# User's transformprog
  global generic_mask_t_frame$output_form	;# mask 
  global generic_param_l_frame$output_form	;# User's layout gram
  global generic_mask_l_frame$output_form	;# mask


  #
  # ! MODIFY LINE BELOW TO MACTH NEWLY CREATED OUTPUT TYPES !
  #     
  if {$output_form>4 || $output_form<1} { return}


  set htmlout [fullpath [file dirname $prevs]/$TMPHTML]
  switch $output_form {
  1 {
      #
      # View as plain text files
      #

      set FP [initHTML $htmlout [llength $docl]]

      foreach doc $docl {
	writeHTML $FP [disppath $doc] $doc
      }
      doneHTML $FP
      RunPreviewCmd $htmlout
    }
  2 {
      #
      # View as input form
      #

      foreach doc $docl {
	
	# Pass document name to Paula's function. If user decides to view
	# document he/she must select input gram.
	set doc_file $doc
	# Missing helpfile below.
	old_document "hlp"
	
	# Wait until window for opening document is closed.
	# Note!
	# - If syndocinput.tcl is modified to close the window when
	#   document is opened this code will not work.
	tkwait window $w
      }
    }
  3 {
      #
      # View as formatted
      #
      #TR220896: this can be very lenghty operation.
      set window [focus]
      if {$window != "none"} { $window configure -cursor watch }
      update
 
      set out [fullpath [file dirname $progs]/[file tail $generic_param_l_frame3].pl]
      puts "Generating program ($out) for layoutgrammar ($generic_param_l_frame3)"
      generate $generic_param_l_frame3 $out
      tkwait variable ready
      
      # TR210896
      set paate [StripOutputFileType $generic_param_l_frame3]

      set FP [initHTML $htmlout [llength $docl]]
      puts "Generating documents. ($htmlout)"
      
      set tul [ConsultLayoutProg $struct_file $out]
      if {$tul == 0} {

        foreach doc $docl {
          update
          set tul 0
	
          set docout [fullpath [file dirname $prevs]/[file tail [file rootname $doc]].$paate]

          set tul [LayoutDocument $doc $docout]
	  if {$tul == 0} {
            if {[file extension $docout] == ".tex"} {
	      # 
	      # LayoutDocument creates .dvi for each .tex document. Now that
	      # we know it, we can modify the name of our output file.
	      # NOTE!
	      # - if ($tul != 0) the .dvi WAS NOT created.
  	      #
              set docout [file rootname $docout].dvi
            }

	    writeHTML $FP [file tail $docout] $docout
	  } else {
	    writeHTMLline $FP "$doc - failed to generate (reason $tul)."
	    puts "Failed to generate $docout"
	  }
	}
      } else {
        # Consult failed
	writeHTMLline $FP "Wrong stucture grammar, cannot generate documents."
	puts "Failed to consult $out"
      }
      doneHTML $FP
      prolog_done
      RunPreviewCmd $htmlout
      if {$window != "none"} { $window configure -cursor arrow}
    }
  4 {
      #
      # Transform and view as formatted
      #
      #
      #TR220896: this can be very lenghty operation.
      set window [focus]
      if {$window != "none"} { $window configure -cursor watch}
      update

      # Generate transform program. Currently code below is commented out
      # because we do not know which method should we use: STDS or transducer.
      # NOTE!
      # - If you add a method for reasoning which method are we using uncomment
      #   code below AND replace $generic_param_t_frame4 with $out1 in 
      #   ConsultTransformProg call. Of course you must generate necessary
      #   programs. Use line with transform_generate_sdts as a template.
      # - DO NOT REMOVE COMMENTED LINES BELOW!
      #
      #set out1 [fullpath [file dirname $trprogs]/[file tail $generic_param_t_frame4].pl]
      #puts "Generating program ($out1) for transformgrammar ($generic_param_t_frame4)"
      #p "transform_generate_sdts( '$generic_param_t_frame4', '$out1')"


      set out [fullpath [file dirname $progs]/[file tail $generic_param_l_frame4].pl]
            
      set paate [StripOutputFileType $generic_param_l_frame4]

      set FP [initHTML $htmlout [llength $docl]]
      set tul [ConsultTransformProg $struct_file $generic_param_t_frame4]
      # NOTE!
      # - ConsultTransformProg sets (in case of success) following variables:
      #   transform_struct_file  = Struct grammar of transformed document
      #   transform_NewType      = Document type
      #   transform_NewText      = merkkijonovalike
      
      if {$tul == 0} {
        puts "Generating program ($out) for layoutgrammar ($generic_param_l_frame4)"
        generate $generic_param_l_frame4 $out
        tkwait variable ready

        set tul [ConsultLayoutProg $transform_struct_file $out]
	
      }
      if {$tul == 0} {
        foreach doc $docl {
          set tul 0
	  update

          set docout  [fullpath [file dirname $prevs]/[file tail [file rootname $doc]].$paate]

          # Create name of transformed document.
	  # Note!
	  # - Name of transformed document MUST NOT be the same as 
	  #   nontransformed. Remember that not all users have transformed
	  #   documents in a different directory, and we do not want to 
	  #   overwrite those very special documents.
	  #
	  set docnimi [file tail [file rootname $doc]]_
	  set gramnimi [file rootname [file tail $transform_struct_file]]
	  set docpaate [file extension $doc]
	  
          set docout2 [fullpath [file dirname $prevs]/$docnimi$gramnimi$docpaate]

          puts "Transforming Document: $docout2"
	  set tulos 0
          p "tcl_transformDocument('$doc','$docout2','$transform_struct_file','$transform_NewType','$transform_NewText')"
          tkwait variable ready
	  # tcl_transformDocument sets variable tulos to 1 in case of failure.

	  if {$tulos == 0} {
            set tul [LayoutDocument $docout2 $docout]
            if {[file extension $docout] == ".tex"} {
             set docout [file rootname $docout].dvi
            }

  	    if {$tul == 0} {
	      # NOTE!
	      # - Name of transformed document (docout2) is shown is HTML,
	      #   not the original.
	      #
	      writeHTML $FP [file tail $docout2] $docout
	    } else {
	      writeHTMLline $FP "$docout - failed to generate (reason $tul)."
	      puts "Failed to generate: $docout2"
	    }
	  } else {
	    writeHTMLline $FP "$docout2 - failed to transform."
	    puts "Failed to transform: $docout2"
	  }
	}
      } else {
        # Consult of either transform or layout failed. Note that structure 
	# grammar is not the only reason of failure. Still, it is the most
	# common one.
	# 
	writeHTMLline $FP "Wrong stucture grammar, cannot generate documents."
	puts "Failed to consult $out"
      }
      doneHTML $FP
      prolog_done
      RunPreviewCmd $htmlout
      if {$window != "none"} { $window configure -cursor arrow}
    }
  }
  
}


# *******************************************************************
# Name:     LayoutDocument
# Input:    Existing document file (fullpath filename, string)
#           File to create (fullpath, string)
# Output:   0 = OK, not 0=error
#	    Errorcodes:
#	    1  Prolog fails
#	    2  tcl_layoutDocument did not generate outpufile (.tex),
#	       though prolog claimed to generate.
#	    3  .aux was not generated by bibtex
#	    4  Second latex fails
#	    5  Third latex fails
#	    6  .dvi was not generated by the first latex command
#	    
# Function: Call prolog to layout given document. Tex documents (any file with
#           .tex suffix) are converted to dvi by this function. Caller must know
#	    that when it want's to create a.tex, it gets both
#	    a.tex AND a.dvi.
#	    NOTE!
#	    - tex file is created in multiple passes: latex+bibtex+latex+latex.
#	      Conversion is run to the point when first command gives and error.
#	      For example when .tex does not contain any bibtex references
#	      it will give out an error and processing will stop (if $err 
#	      lines uncommented).
#	    - There is a risk of latex or bibtex commands to become interactive
#	      (wait for user input). Since output is redirected away and the 
#	      user cannot see it, program seems to hang. Only way to stop
#	      a program is to end it with CTRL+C, too bad that syndoc is
#	      ended at the same time. Problem could be partially solved
#	      be running latex and bibtex in background (like in Output).
#	      There would be still a risk of becoming interactive, but our
#	      program could operate in any case (though it would wait for
#	      command to finish).
#	    - If you want to see latex and bibtex error, you should
#	      uncomment "puts $t" lines below (4).
#	    - Uncomment lines that has $err (and related lines) if you want
#	      to have some error checking. Processing would be stopped at
#	      first error.
#	    - You MUST set back the directory before exiting.
#	    - DO NOT REMOVE COMMENTED LINES BELOW!
#	    - This procedure leaves MANY scrap files on tex output.
# *******************************************************************
proc LayoutDocument { document outputfile} {
  global tulos ready 
  global prevs
  global LATEX_CMD 
  global BIBTEX_CMD

  set tulos 0
  puts "Generating Document: $outputfile"
  p "tcl_layoutDocument('$document','$outputfile')"
  tkwait variable ready

  if {([file extension $outputfile] == ".tex") && ($tulos == 0)} {
    puts "Assuming latex document."

    #
    # To Paula: Latex and bibtex commands will create their files to directory
    # where they were started. Using cd we move to a different directory and
    # start commands from there. When commands have been done the previous
    # directory must be set back. Current work directory is obtained by
    # pwd command.
    #
    set hak [pwd]
    set d [file dirname [fullpath $prevs]]
    cd $d

    # NOTE!
    # - Since old files are not deleted before layout, there is a risk of
    #   showing old documents. If outputfile was not generated but layot
    #   claimed to (or generated to a different name or to a different 
    #   directory) AND there is an old file, code below will assume that
    #   it was generated.
    if {[file exists $outputfile] == 0} {
      # Latex fill fuck up when file is not found
      cd $hak
      return 2
    }
    
    # Create .dvi from .tex
    set err [catch {exec $LATEX_CMD $outputfile} t]
    puts "$t"
    if {$err == 0} {
    
      # Process .aux
      if {[file exists [file rootname $outputfile].aux] == 0} {
        # .aux MUST exist
        cd $hak
        return 3
      }      
      set err [catch {exec $BIBTEX_CMD [file rootname $outputfile]} t]
      puts "$t"
#      if {$err == 0} {

        if {[file exists $outputfile] == 0} {
          cd $hak
          return 4
        }
        set err [catch {exec $LATEX_CMD $outputfile} t]
        puts "$t"
#	if {$err == 0} {

          if {[file exists $outputfile] == 0} {
            cd $hak
            return 5
          }
          set err [catch {exec $LATEX_CMD $outputfile} t]
          puts "$t"
#	}
#      }
    } else {
      # Assume that .dvi was not created when latex fails at first.
      set tulos 6
    }

    cd $hak

  }

  return $tulos
}

# *******************************************************************
# ****************** O U T P U T   F O R M **************************
# *******************************************************************
#
# Procedures below contain all the code used in define output form menu.
# Since those four output types have so much in common, procedures below
# are designed to handle every possible type. There is not a separate
# window for each type. Procedures are labeled as "generic" because
# they generate variable names on the fly (In standard approach you should
# use arrays).
#

# *******************************************************************
# Name:     generic_output
# Input:    helpfile type_of_output
# Output:   -
# Function: Userinterface asking misc. output forms.
#           NOTE!
#           - This function must set output_form to a legal value if
#             user canceled the operation.
#	    - Currently only types 3 and 4 have window. Other types (1 and 2)
#	      do not need one. Thus only types 3 and 4 must have generic
#	      procedures.
# *******************************************************************
proc generic_output { helpfile toup} {
  global output_form			;# Currently selected type of ouput
  global gou				;# Window to create (prefix)
  global of

  global generic_param_t_frame$toup	;# transform
  global generic_mask_t_frame$toup	;# mask

  global generic_param_l_frame$toup	;# layout
  global generic_mask_l_frame$toup	;# mask

  global prev_prog
     
  #
  # ! MODIFY LINES BELOW TO MATCH NEWLY CREATED OUTPUT TYPES !
  # 1) take care of invalid types
  # 2) If type has no window, modify code so we will not create one.
  # 3) Destroy previously opened windows.
  #
  if {$toup>4 || $toup<1} { return}
  
  # Type 1 (default) and 2 (input) does not need a window to ask for
  # parameters.
  if {($toup == 1) || ($toup == 2)} "set output_form $toup;return"

  # Get rid of previous windows. This function might be re-entered by the
  # user selecting from output type menu without closing the previous window.
  #
  catch {destroy ${gou}1}
  catch {destroy ${gou}2}
  catch {destroy ${gou}3}
  catch {destroy ${gou}4}

  set of $toup
  toplevel $gou$of
  PreviewEnable

  #
  # Leave some space between menu and first component
  #
  frame $gou$of.sepa1 -height 10
  pack $gou$of.sepa1 -side top -anchor n

  ## 2
  frame $gou$of.t_frame
  label $gou$of.t_frame.lab -width 20 -text "Transform program: " \
          -anchor w

  entry $gou$of.t_frame.entr -relief sunken \
            -textvariable generic_param_t_frame$of -width 30
  bind $gou$of.t_frame.entr <Any-KeyRelease> \
  	{ generic_check_state$of $gou$of $of }
  button $gou$of.t_frame.sel -text Select \
	    -width 8 -command \
	    {
	      set generic_param_t_frame$of [create_browser \
	      	[expr \$generic_mask_t_frame$of] [expr \$generic_param_t_frame$of]]
	      moveto_end $gou$of.t_frame.entr
	      generic_check_state$of $gou$of $of
	    }

  pack $gou$of.t_frame.lab -side left
  pack $gou$of.t_frame.sel -side right
  pack $gou$of.t_frame.entr -side left -expand yes -anchor w -fill x
  moveto_end $gou$of.t_frame.entr

  ## 3
  frame $gou$of.l_frame
  label $gou$of.l_frame.lab -width 20 -text "Layout grammar: " \
          -anchor w

  entry $gou$of.l_frame.entr -relief sunken \
            -textvariable generic_param_l_frame$of -width 30
  bind $gou$of.l_frame.entr <Any-KeyRelease> \
  	{ generic_check_state$of $gou$of $of }
  button $gou$of.l_frame.sel -text Select \
	    -width 8 -command \
	    {
	      set generic_param_l_frame$of [create_browser \
	      	[expr \$generic_mask_l_frame$of] [expr \$generic_param_l_frame$of]]
	      moveto_end $gou$of.l_frame.entr
	      generic_check_state$of $gou$of $of
	    }

  pack $gou$of.l_frame.lab -side left
  pack $gou$of.l_frame.sel -side right
  pack $gou$of.l_frame.entr -side left -expand yes -anchor w -fill x
  moveto_end $gou$of.l_frame.entr

  ## 4
  # Ask for preview command. Command entered here is common to whole syndoc
  # system.
  #
  frame $gou$of.o_frame
  label $gou$of.o_frame.lab -width 20 -text "Preview command: " \
          -anchor w

  # TR210896: generic_param_o_frame$of => prev_prog
  entry $gou$of.o_frame.entr -relief sunken \
            -textvariable prev_prog -width 30

  bind $gou$of.o_frame.entr <Any-KeyRelease> \
  	{ generic_check_state$of $gou$of $of }
  button $gou$of.o_frame.sel -width 8 -text "" -relief flat -state disabled

  pack $gou$of.o_frame.lab -side left
  pack $gou$of.o_frame.sel -side right
  pack $gou$of.o_frame.entr -side left -expand yes -anchor w -fill x
  moveto_end $gou$of.o_frame.entr

  #
  # ! INSERT ELEMENTS OF NEW OUTPUT TYPES WITH WINDOW HERE !
  # Use blocks labeled as 2, 3 or 4 above as templates.
  #   

  #
  # buttons
  #    
  button $gou$of.go -text Apply \
	    -width 8 -state disabled \
            -command \
	    "set output_form $of; generic_go$of $gou$of $of; destroy $gou$of ; PreviewEnable"
  button $gou$of.help -text Help \
          -command "open_help $helpfile $gou$of" -width 8
  button $gou$of.cancel -text Done \
            -command "set output_form 1;destroy $gou$of;PreviewEnable" -width 8


  #
  # Type specific options
  #
  switch $of {
  #
  # ! ADD HERE DEFINITIONS OF ADDITIONAL OUTPUT TYPES !
  # (the part that is not common to other types)
  # Use blocks below as templates.
  #
  3 {
      set ots "Formatted"

      set generic_param_l_frame$of [def_dir2 [expr \$generic_mask_l_frame$of] [expr \$generic_param_l_frame$of]]
      pack $gou$of.l_frame -side top -fill x
      pack $gou$of.o_frame -side top -fill x
    }
  4 {
      set ots "Transformed and Formatted"

      set generic_param_t_frame$of [def_dir2 [expr \$generic_mask_t_frame$of] [expr \$generic_param_t_frame$of]]
      set generic_param_l_frame$of [def_dir2 [expr \$generic_mask_l_frame$of] [expr \$generic_param_l_frame$of]]
      pack $gou$of.t_frame -side top -fill x
      pack $gou$of.l_frame -side top -fill x
      pack $gou$of.o_frame -side top -fill x
    }
  }

  frame $gou$of.sepa2 -height 10
  pack $gou$of.sepa2 -side top
  
  pack $gou$of.go -side left -expand yes -anchor n
  pack $gou$of.help -side left -expand yes -anchor n
  pack $gou$of.cancel -side left -expand yes -anchor n

  generic_check_state$of $gou$of $of

  wm geometry $gou$of +100+250
  wm title $gou$of $ots
  wm iconname $gou$of $ots
  focus $gou$of
}

#
# Generic commands. Every possible command (_go and check_state) should be
# defined for those generic types that have a window (currently 3 and 4).
# NOTE!
# - DO NOT REMOVE EMPTY FUNCTIONS BELOW! They are used from generic_output
#   function. To find out all places where functions below are used
#   do not use "grep generic_check_state3 *". Use plain
#   "grep generic_check_state *".
#

# *******************************************************************
# Name:     generic_check_state3 
# Input:    ikkuna toup
# Output:   -
# Function: Used to check output_form3
# *******************************************************************

proc generic_check_state3 { ikkuna toup} {
  global of
  global generic_param_l_frame3
  global gou
  
  if {[file exists [fullpath $generic_param_l_frame3]]} {
    $gou$of.go configure -state normal
  } else {
    $gou$of.go configure -state disabled
  }
}
# *******************************************************************
# Name:     generic_go3 
# Input:    ikkuna toup
# Output:   -
# Function: Apply output_form3
# *******************************************************************
proc generic_go3 { ikkuna toup} {
}


# *******************************************************************
# Name:     generic_check_state4 
# Input:    ikkuna toup
# Output:   -
# Function: Used to check output_form4
# *******************************************************************
proc generic_check_state4 { ikkuna toup} {
  global of
  global generic_param_t_frame4
  global generic_param_l_frame4
  global gou
  
  if {[file exists [fullpath $generic_param_t_frame4]] &&\
      [file exists [fullpath $generic_param_l_frame4]]} {
    $gou$of.go configure -state normal
  } else {
    $gou$of.go configure -state disabled
  }

}
# *******************************************************************
# Name:     generic_go4 
# Input:    ikkuna toup
# Output:   -
# Function: Apply output_form4
# *******************************************************************
proc generic_go4 { ikkuna toup} {
}


# ************************* H T M L *********************************
#
# TR210896: Procedures below this headline are used in creating the TMP.HTML
# document. The document constains header and list of files that matched
# the search. IF interface to HTML is expanded or used by multiple subsystems,
# this code should be separated from syndocretrieval.tcl.
#

# *******************************************************************
# Name:     initHTML
# Input:    string (output HTML document)
#           count (nummber of matching docs)
# Output:   file handle
# Function: Open specified HTML document for write, write headers
# *******************************************************************
proc initHTML { outname count} {
  set FP [open $outname w]
  puts $FP "<HTML><HEAD>"
  puts $FP "<TITLE>Documents</TITLE></HEAD>"
  puts $FP "<BODY><H2>Filter matches the following documents ($count):</H2>"

  return $FP
}

# *******************************************************************
# Name:     writeHTML
# Input:    file handle (of HTML)
#           string (visible document name)
#           string (hidden document name, fullpath)
# Output:   -
# Function: Write footers, close file
# *******************************************************************
proc writeHTML { FP name doc} {
  puts $FP "<A HREF=\"$doc\">$name</A><BR>"
}

# *******************************************************************
# Name:     writeHTMLline
# Input:    file handle (of HTML)
#           string (print as a line to a file)
# Output:   -
# Function: Print line to a file.
# *******************************************************************
proc writeHTMLline { FP line} {
  puts $FP "$line<BR>"
}


# *******************************************************************
# Name:     doneHTML
# Input:    file handle (of HTML)
# Output:   -
# Function: Write footers, close file
# *******************************************************************
proc doneHTML { FP} {
  puts $FP "</BODY>"
  puts $FP "</HTML>"
  close $FP
}

# *******************************************************************
# Name:     RunPreviewCmd $htmlout
# Input:    string (possibly list in "", cmdline parameters of viewer)
# Output:   -
# Function: Run viewer
# *******************************************************************
proc RunPreviewCmd { htmlout } {
  global prev_prog
  
  puts "Running: $prev_prog $htmlout &"
  exec $prev_prog $htmlout &
}


#####################################################################
# 
# GENERAL PURPOSE ROUTINES (usable in this system as well as in others)
#
#
# *******************************************************************
# Name:     def_dir2
# Input:    string (default directory, like "grammar/*.gram")
#	    string ([directory/]filename, like "~/a" or "a")
# Output:   string (file OR defdir/file in disppath format. Like "grammar/a")
# Function: If filename does not contain path, append default path to it.
# 	    This function work much like def_dir, but the main differences
#	    are: the directory is not checked, simplier, can handle default
#	    directories anywhere (def_dir want's them to be under $dir).
#	    NOTE!
#	    - If file contains path, the result will be exactly the
#	      same file, not in disppath format. 
#	    - If file begins with a dot, default path will be not appended.
#
#	    Examples:
#		defdir                          file            result
#		==============================================================
#		doc/*.doc                       reko.doc      doc/reko.doc  
#		gram/*.trgram                   tmp.filt      gram/tmp.filt
#		/home/syndoc/                   *.tr          /home/syndoc/*.tr
#		/home/opisk/tronkko/x.doc       reko.doc      ~/reko.doc  
#		/appl/wrk/Syndoc/a/*.pl         reko.doc      a/reko.doc   
#		a/*.doc                          ~/reko.doc   ~/reko.doc   
#		/syndoc/                        /a/a.doc      /a/a.doc   
#		/syndoc/                        ./a.doc       ./a.doc     
#		/syndoc/                        .doc          .doc        
#
# *******************************************************************
proc def_dir2 {defdir file} {

  if {([file dirname $file] == ".") && ([string range $file 0 0] != ".") } {
    set file [disppath [file dirname $defdir]/$file]
  }
  return $file
}
	   

# *******************************************************************
# Name:     fullpath
# Input:    list of strings (like '~/syndoc/*.tcl')
# Output:   list of strings (like '/home/opisk/tronkko/syndoc/*.tcl')
# Function: Make path(s) full (longest possible form).
#		Resulting path(s) will not contain ~ or . and will start
#		from the root (/). Function can be used to strip special
#		characters from filename before passing it to exec.
#	    NOTE!
#	    - If $dir starts with . this routine will not work ok
#	    - Filename can be converted back and worth using functions
#	      fullpath and disppath.
#	    - This function can be used with non-existent files, like *.doc
#
#	    Examples ($dir = /syndoc):
#		article.doc	=> /syndoc/article.doc
#		~/article.doc	=> $HOME/article.doc
#		doc/article.doc => /syndoc/doc/article.doc
# *******************************************************************
proc fullpath file {
  global env				;# Environmental variables
  global dir				;# Default path to syndocwork

  set a ""
  foreach f $file {
    regsub -all -- {^\./} $f $dir/ f
    regsub -all -- {^[^\./~]} $f $dir/[string range $f 0 0] f
    #
    # TR210896: pistell alkavat tiedostot ovat $dir hakemistossa
    #
    regsub -all -- {^\.[^/]} $f $dir/[string range $f 0 1] f
    regsub -all -- ^~/ $f $env(HOME)/ f
    regsub -all -- {/\./} $f / f
    regsub -all -- {//} $f / f
    while {[regsub -all -- {/[^/]*/\.\./} $f / f] != 0} {}
    
    set a "$a$f "
  }
  return [string trimright $a]
}

# *******************************************************************
# Name:     disppath
# Input:    list of strings (like 'syndoc/*.tcl')
# Output:   list of strings (like '~/syndoc/*.tcl')
# Function: Make path(s) displayable (shortest possible form).
#	    Function is used to shorten the path so that user can more
#	    easily see it.
#
#	    Examples ($dir = /syndoc):
#		/syndoc/article.doc	=> article.doc
#		$HOME/article.doc	=> ~/article.doc
#		/syndoc/doc/article.doc	=> doc/article.doc
# *******************************************************************
proc disppath file {
  global env
  global dir				;# Default path to syndocwork
  
  set HOME $env(HOME)/
  set homel [expr [string length $HOME]-1]
  set dirl [expr [string length $dir]]
  
  set a ""
  foreach f $file {
    regsub -all -- {^\./} $f $dir/ f
    regsub -all -- {/\./} $f / f
    regsub -all -- {//} $f / f
    if { [string range $f 0 $homel] == $HOME} {
      set f ~/[string range $f [expr $homel+1] end]
    }
    if { [string range $f 0 $dirl] == "$dir/" } {
      set f [string range $f [expr $dirl+1] end]
    }
    while {[regsub -all -- {/[^/]*/\.\./} $f / f] != 0} {}
    set a "$a$f "
  }
  return [string trimright $a]
}


#####################################################################
# 
# GENERAL PURPOSE ROUTINES (usable to syndoc subsystems)
#
# Currently no other subsystem uses functions below. If you decide to use
# one of these, move the function to syndocsystem.tcl.
#

# *********************************************************************
# Name:       StripOutputFileType
# Input:      String (layoutgrammar file: for example article_HTML.layoutgram)
# Output:     string (for example HTML)
# Function:   Returns part of a string starting from last _ (not including) 
#	      and ending to a first dot (not including). If input string does
# 	      not contain '_', "doc" will be returned.
#
#	      Note!
#	      - I suggest that other implementators should also use function
#		to prevent possible errors. If this function is used in other
#		subsystems move this to syndocsystem.tcl. 
#	      - The procedure here is simpliest I can make, if you think
#	        that you have a simplier one, think again!
#
#	      Examples:
#		article_filt_tex.layoutgram	=> tex
#		table.layoutgram		=> doc
#		article_HTML.layoutgram		=> HTML
# *********************************************************************
proc StripOutputFileType gram {
  # Remove extension
  set i [string first . $gram]
  if {$i == -1} {
    # No dot in filename
    set type $gram
  } else {
    # Leave out first dot and everything after that
    set type [string range $gram 0 [expr $i-1]]
  }
  
  set i [string last \_ $type]
  if {$i == -1} {
    # Error: string does not contain _, return "doc"
    set type "doc"
  } else {
    # String contains _
    set type [string range $type [expr $i + 1] end]
  }

  return $type	  
}

#
# Some functions for tcl -> prolog interface
#

# *******************************************************************
# Name:     ConsultLayoutProg
# Input:    string (document structure)
#	    string (fullpath to layout program)
# Output:   0 OK, non 0 error
# Function: Consult layout program, use once before LayoutDocument
# *******************************************************************

proc ConsultLayoutProg { struct_file pl} {
  global tulos				;# Errorcode from prolog
  
  set tulos 0
  p tcl_consultLayoutProg('$struct_file','$pl')
  tkwait variable ready
  return $tulos

}
# *******************************************************************
# Name:     ConsultTransformProg
# Input:    string (transformed document structure)
#	    string (fullpath to layout program)
# Output:   0 OK, non 0 error
# Function: Consult layout program, use once before LayoutDocument
# *******************************************************************

proc ConsultTransformProg { struct_file pl} {
  global tulos				;# Errorcode from prolog
  
  set tulos 0
  p tcl_consultTransformProg('$struct_file','$pl')
  tkwait variable ready
  return $tulos
}

#
# Some functions for prolog -> tcl interface.
#


# *********************************************************************
# Name:       set_tcl_variable
# Input:      variablename
#	      value
# Output:     -
# Function:   This function is called from prolog to set a global tcl variable
#	      Variables that are set using this procedure:
#	      transform_struct_file: used in ConsultTransformProg to
#	        return the name of a structure grammar that a given transform
#	        grammar is for. 
#	      transform_NewType: Document type
#	      transform_NewText: ?
# *********************************************************************
proc set_tcl_variable { var value} {
  global $var				;# Prolog says what to set
  
  set $var $value

}

# *********************************************************************
# Name:       give_list
# Input:      String
# Output:     -
# Function:   Sets value of list for ftypes
#	      This procedure is called from prolog to pass a list of
#	      nonterminals to tcl. Called from prolog tcl_searchNonterminals.
# *********************************************************************
proc give_list { list} {
  global ftypes				;# List of nonterminals
  global ready666			;# signal caller (tcl) that prolog
  					;#  is ready.

  set ftypes $list
  
  set ready666 1
}

# *******************************************************************
# Name:     prolog_fail
# Input:    -
# Output:   -
# Function: Called by prolog to indicate falure. Will set tcl variable 
#	    named tulos.
# *******************************************************************

proc prolog_fail {t} {
  global tulos				;# Errorcode to set

  # Note!
  # - Return value must be set BEFORE prolog_done. 
  set tulos $t
  
  # Note!
  # - Currently calling prolog_done is a bit risky. Prolog predicates
  #   call prolog_done by themselves, so we should not call it here.
  #prolog_done
}


