/*----------------------------------------------------------------------------
 * syndocsystem.pl
 * 
 * Contains coomon predicates which may be part of the used compiler.
 *----------------------------------------------------------------------------
*/

%Pred:same(?Term1,?Term2).
% is true if Term1 is same as Term2.
same(C,C).

%Pred:empty(+List).
% is true if List is empty. List has to be proper list.
empty([]).

%Pred:noempty(+List).
% is true if List is not empty. List has to be proper list.
notempty(List):-List\==[].

%Pred:flatten(+List1,?List2).
% is true if List2 is the same list as List1
% from which lists of  lists are removed. List1 must be a proper list.
% Kaytetaan predikaatissa down.
% Listasta poistetaan sisakkaiset listat.
% Sisakkaiset listat puretaan, kun liikutaan puussa juuresta alipuuhun.
flatten(X,[X]):-var(X).
flatten([],[]):- !.
flatten([X|Xs],Ys):-
	flatten(X,Ys1),flatten(Xs,Ys2),append(Ys1,Ys2,Ys).
flatten(X,[X]).

%Pred:reverse(?List, ?Reversed)
% is true when Reversed is has the same element as List but in a 
% reversed order. List must be a proper list.
reverse([],[]).
reverse([X|Xs],R):-
   reverse(Xs,T),
   append(T,[X],R).
%   app(T,[X],R).

%app(?List,?List,?List)
app([],Ys,Ys).
app([A|Xs],Ys,[A|Zs]):-
   app(Xs,Ys,Zs).

%Pred:is_list(+List)
%  is true when List is a proper list.
is_list(X) :- var(X), !, fail.
is_list([]).
is_list([_|Tail]) :- is_list(Tail).


%Pred:count(+List,-Number)
% Lasketaan listan alkioiden  lukumaaran.
count(List,N):-!,count(List,0,N).
count([],N,N):-!.
count([_|Rs],N1,N):-N2 is N1 + 1,!,count(Rs,N2,N).




