# **********************************************************************
# syndocsystem.tcl 
#
# Function:   Contains procedures which are valled from several other
#             procedures and which execute system-type  actions
#             Procedures copied from several other files
# Date:       19.5.1996
# Author:     Eila Kuikka
# **********************************************************************


# *********************************************************************
# Name:     close_window       
# Input:    String (window to be closed)
# Output:   -
# Function: Closes window if there is no editor open.                  
# *********************************************************************
proc close_window cw {
    global id;		# Process id, when editor is started

    set errCode 0
    if {$id != 0} { 
        catch {set errCode [exec ps $id]}
        if {$errCode != 0} { 
          	tk_dialog .d {} "Close editor !" {} -1 OK
        } else {
	    destroy $cw
        }
    } else {
       	destroy $cw
    }
}

# ******************************************************************
# Name:     create_browser
# Input:    String (directory and extension of files to be listed)
#           String (old text in the calling entry widget)
#
# Output:   String (selected file)
# Function: Lists all files in directory $dir. Double-click chooses
#           clicked file to the calling field ($entry)
# ******************************************************************
proc create_browser { files oldtext } {
 
  global win 
  global toField dir
  global directory
  global showid

  set directory "$dir/$files"
  set showid 0

  set toField "" 
  if {[string length $oldtext] == 0} {
      set oldtext * 
  }
  toplevel $win
  wm title $win "Select filename"
  dpos $win

  button $win.close -text "Close" -command " set toField \"$oldtext\" "
  button $win.show -text "Show" -command "show_file $win"
  pack $win.close -side bottom -fill x
  pack $win.show -side bottom -fill x
   
  label $win.dlabel -text "Directory:"
  entry $win.dentry -width 30 -textvariable directory
  bind $win.dentry <Return> "load_dir $win"
  pack $win.dlabel $win.dentry -side top -anchor w

  frame $win.lf
  pack $win.lf -fill x
  scrollbar $win.lf.scroll -command "$win.lf.list yview"
  pack $win.lf.scroll -side right -fill y
  listbox $win.lf.list -width 30 -yscroll "$win.lf.scroll set" -relief raised \
	-setgrid yes
  bind $win.lf.list <Double-Button-1> { load_selected $win}
  pack $win.lf.list -side left -fill both -expand yes
  wm minsize $win 5 5

  tkwait visibility $win 
  grab $win  
  focus $win.lf.list
  load_dir $win

  tkwait variable toField
  exit_show
  destroy $win
  return $toField

}

# ******************************************************************
# Name:     create_browser_list
# Input:    List (List of items in ascii codes, like 65 65 13)
#           String (old text in the calling entry widget)
#
# Output:   String (selected item)
# Function: Browse-a-list
# ******************************************************************
proc create_browser_list { list oldtext} {
 
  global win 
  global toField
  global showid

  set list2 {}
  foreach i $list {
      append list2 [format "%c" $i]
  }

  set showid 0

  set toField "" 
  toplevel $win
  wm title $win "Select type"
  dpos $win

  button $win.close -text "Close" -command "set toField $oldtext"
  pack $win.close -side bottom -fill x -anchor s
   
  frame $win.lf
  pack $win.lf -fill both -expand yes
  scrollbar $win.lf.scroll -command "$win.lf.list yview"
  pack $win.lf.scroll -side right -fill y
  listbox $win.lf.list -width 30 -yscroll "$win.lf.scroll set" -relief raised \
	-setgrid yes
  bind $win.lf.list <Double-Button-1> { set toField [selection get]}
  pack $win.lf.list -side left -fill both -anchor nw -expand yes
  wm minsize $win 5 5
  
  foreach i $list2 {
    $win.lf.list insert end $i
  }
  tkwait visibility $win 
  grab $win  
  focus $win.lf.list

  tkwait variable toField
#  exit_show
  destroy $win
  return $toField

}

# *********************************************************************
# Name:     def_dir       
# Input:    String (new file)
#           String (default directory for the file)
# Output:   String (file name with path)
# Function: If not given directory for the result file, sets
#           default directory.
# *********************************************************************
proc def_dir {newFile defDir} {
    global dir

# Initialize return value, sets ~ to real path
    set dir1 [file dirname $newFile]

if {[string compare $dir1 .] == 0} {
    set file [file tail $newFile] 
    } else {
	set file [file dirname $newFile]/[file tail $newFile] 
    }

# if not any directorypath in filename 
    if {[string compare [file dirname $newFile] .] == 0 } {
	set file "$dir/[file dirname $defDir]/[file tail $file]"
	set file [file dirname $file]/[file tail $file]
    }

# Is given or default directory a regular directory
	set d [file dirname $file]
# Is there path like default directory
	if {[file isdirectory $d] == 0} {
	     tk_dialog .d {} "\"$d\" isn't a regular directory!" {} -1 OK
	     set file ""
	}
    return $file
}


# ************************************************************************
# Name:       dpos 
# Input:      String
# Output:     -
# Function:   Positions the window to the screen.                      
# ************************************************************************
proc dpos w {
    wm geometry $w +0+0
}


# *****************************************************************
# Name:     edit_file
# Input:    String (file to be edited)
# Output:   -
# Function: Open editor (from $EDITORCMD, $EDITOR or vi) with file
# *****************************************************************
proc edit_file {file} { 
    global EDITORCMD;	# Text editor from .syndocdefaults or
			# from .syndocsysdefaults
    global env;		# Array of environment variables 
    global id;		# Pid of editor process 

    if [file isfile $file] {
	if {[string compare $EDITORCMD ""] != 0} {
	      set id [eval exec $EDITORCMD $file &]
	} elseif [info exists env(EDITOR)] {
	      set id [eval exec $env(EDITOR) $file &]
	} else {
	      set id [exec vi $file &]
	}
     } else {
	 tk_dialog .d {} "\"$file\" isn't a regular file!" {} -1 OK
     }
}



# *******************************************************************
# Name:      exec_cmd 
# Input:     String (unix command)
#            String (file in the commands) 
# Output:    -
# Function:  Executes command given in parameter cmd. Called form
#            prolog.
# *******************************************************************
proc exec_cmd { cmd file } {
    eval exec $cmd $file &
    p ok 

}


# *****************************************************************
# Name:     exit_show 
# Input:    -
# Output:   -
# Function: Close browser window.
# *****************************************************************
proc exit_show {} {
    global showid;	# pid of less-process
    if {$showid != 0} { 
        catch {exec kill -9 $showid}
#	exec clear &
	set showid 0
    }
}


# **********************************************************************
# Name:        give_name 
# Input:       String (file name with path)
# Output:      String (file name without path, returned by calling
#              prolog predicate done/1)
# Function:    Returns filename without path to prolog.
# **********************************************************************
proc give_name { file } {
    set fileName [file tail $file]
    p done('$fileName')
}


# *****************************************************************
# Name:     load_dir
# Input:    String (toplevel window)
# Output:   -
# Function: Loads files in given directory
# *****************************************************************
proc load_dir w {
    global directory;		# files to be listed
    $w.lf.list delete 0 end
    if [file isdirectory $directory] {
	set directory $directory/*
    }
    foreach i [lsort [glob -nocomplain $directory]] {
	set directory [file dirname $i]
	if [file isdirectory $i] {
            $w.lf.list insert end [file tail $i]/ 
        } else {
            $w.lf.list insert end [file tail $i]
	}
    }
}



# *****************************************************************
# Name:     load_selected
# Input:    String (window with listbox)
# Output:   -
# Function: Checks if selected atom in list is directory or file. 
#           If file: sets file to field in window. Else loads 
#           selected directory.
# *****************************************************************
proc load_selected {w} {
    global toField directory

# Up in directory hierarcy not in use
#    if {[string compare [selection get] ..] == 0} {
#	set selected [file dirname $directory]
#    } else {
    set selected $directory/[selection get]
#    }

    if [file isdirectory $selected] {
	 set directory $selected
	 load_dir $w
    } else {
	 if [file isfile $selected] {
	     set toField $selected
         }
    }
}



# ******************************************************************
# Name:     make_command  
# Input:    -
# Output:   - 
# Function: Executes command in global variable s_file.
# ******************************************************************
proc make_command {} {
    global s_file

    if {$s_file != ""} {
        eval exec $s_file &
        set s_file ""
    }
}



# *****************************************************************
# Name:     make_copy
# Input:    String (source file)
#           String (target file)
# Output:   -
# Function: Copies file given in first argument to file given in
#           second argument
# *****************************************************************
proc make_copy {file1 file2} { 
    set file1 [file dirname $file1]/[file tail $file1]
    if [file isfile $file1] {
         set file2 [file dirname $file2]/[file tail $file2]
         if [file isdirectory [file dirname $file2]] {
             set errCode [exec cp $file1 $file2]
	 } else {
	     tk_dialog .d {} "\"$file2\" isn't a regular directory!" {} -1 OK
         }
     } else {
	 tk_dialog .d {} "\"$file1\" isn't a regular file!" {} -1 OK
     }
}


# *********************************************************************
# Name:     moveto_end
# Input:    String (entry widget)
# Output:   -
# Function: Moves view in the entry widget to the end of the string. 
# *********************************************************************
proc moveto_end {field} {
    $field xview moveto 0.9
}



    
# ************************************************************************
# Name:       open_msg 
# Input:      String (message text)
# Output:     -
# Function:   Moves ASCII-messages from prolog to chars. Calls tk_dialog
#             with message.
# ************************************************************************
proc open_msg {msg} {
    foreach i $msg {
        append errmsg [format "%c" $i]
    }
    tk_dialog .d {Error} $errmsg {} -1 OK
    p ok
}
    
# ************************************************************************
# Name:       open_info 
# Input:      String (message text)
# Output:     -
# Function:   Moves ASCII-messages from prolog to chars. Calls tk_dialog
#             with message.
# ************************************************************************
proc open_info {msg} {
    foreach i $msg {
        append errmsg [format "%c" $i]
    }
    tk_dialog .d {Information} $errmsg {} -1 OK
    p ok
}


# ***************************************************************
# Name:      program_consulted
# Input:     String (message text)
# Output:    -
# Function:  Calls dialog-box with message for user. After user
#            has answered OK for the message, calls prolog to
#            continue.
# **************************************************************
proc program_consulted { msg_text } {
    open_msg $msg_text 
    p consulted
}


# *********************************************************************
# Name:       prolog_done
# Input:      -
# Output:     -
# Function:   Sets ready to 1. Variable ready is set to 0 if tk is
#             waiting some prolog-predicate to run. Moves cursor
#             from watch to arrow. Procedure is called when prolog-
#             predicate is done or some error is occured. 
# *********************************************************************
proc prolog_done {} {
    global ready;	# tk uses variable ready to wait 
    global w;		# waiting window

    set ready 1
    #TR220896: Cannot be sure that $w is an existing window.
    if {[winfo exist $w] != 0} {
      $w configure -cursor arrow
    }
}



# *****************************************************************
# Name:     show_file 
# Input:    String
# Output:   -
# Function: Executes less with active file in listbox
# *****************************************************************
proc show_file w {
    global showid;		# pid of emacs (less)-process
    global directory;		# files to be listed

# If earlier executed less process is running, it is killed
    exit_show

    set file $directory/[$w.lf.list get active]
    if [file isfile $file] {
#	exec clear &
#	set showid [eval exec less $file &]
	set showid [eval exec emacs $file &]
     } else {
	 tk_dialog .d {} "\"$file\" isn't a regular file!" {} -1 OK
    }
}



# *********************************************************************
# Name:     system_command 
# Input:    String (help file)
# Output:   -
# Function: Creates top-level window for asking system_command. 
# *********************************************************************
proc system_command helpfile {
    global browse all               
    global w                    
    global s_file

    set s_file "" 

    set wi iframe
    catch {destroy $w}
    toplevel $w
    dpos $w
    wm title $w "System command" 
    wm iconname $w "System command" 

    frame $w.$wi -bd 1m
	frame $w.$wi.i -bd 2m
	label $w.$wi.i.label1 -width 24 -text "System command " \
			      -anchor w 
	entry $w.$wi.i.entry1 -relief sunken -width 60\
	    -textvariable s_file 
        bind $w.$wi.i.entry1 <Return> { make_command } 
	button $w.$wi.i.browse -text $browse \
	    -command { set file [create_browser $all $s_file]
		       if [file isfile $file] {
			   set s_file "$s_file $file "}
                       moveto_end $w.iframe.i.entry1 }

    pack $w.$wi -fill x
    pack $w.$wi.i.label1 -side top -anchor w
    pack $w.$wi.i -fill x
    pack $w.$wi.i.entry1 $w.$wi.i.browse -side left -expand yes 

    button $w.go -text Go \
       -command { make_command } \
       -width 8
    button $w.done -text Done -command { destroy $w } -width 8
    button $w.help -text Help -command "open_help $helpfile $w" -width 8
    pack $w.go $w.help $w.done -side left -expand yes 

    moveto_end $w.iframe.i.entry1
    grab set $w
}








