/* 
*----------------------------------------------------------------------------
* syndoc_transform.pl
*----------------------------------------------------------------------------
*/

/*
 *----------------------------------------------------------------------------
 *    transform_compare_sdts
 *----------------------------------------------------------------------------
*/


%Pred:transform_compare_sdts(+Grammar,+Grammar,+Grammar)
transform_compare_sdts(Old,New,Trans):-
	    compare_grammars(Old,New,Trans),
	    !.


clean_up_compare:-
       retractall(restNewRules(_)),
       retractall(ruleNew(_,_,_,_)),
       retractall(ruleOld(_,_,_,_)).



/*********************************************************
*
*   compare_grammars
*
********************************************************
*/

%Pred:compare_grammars(+Grammar,+Grammar,+Grammar)
compare_grammars(Old,New,Trans) :-
	% Varmistetaan, etta tyypit ja tekstit ovat samat.
	checkTypeAndTextAndWrite(Old,New,Trans),
	% Tuhotaan vanhat kielioppisaannot tietokannasta.
	retractall(ruleOld(_,_,_,_)),
	see(Old),
	read_and_assertGrammarRules(Old,old,[],_),   
	% Tuhotaan uudet kielioppisaannot tietokannasta.
	retractall(ruleNew(_,_,_,_)),
	seen,	
	see(New),
	read_and_assertGrammarRules(New,new,[],NewRules),
	seen,
        retractall(restNewRules(_)),
        assert(restNewRules(NewRules)),
	%Haetaan tietokannasta old-saanto
        retract(ruleOld(LeftOld,RightOld,ListofRightOld,RuleO)),
	compare_and_write(LeftOld,RightOld,ListofRightOld,RuleO,Trans),
	told,
	call_tcl(open_info("Comparing complete!")),
	tcl_in(call_prolog(ok)).

%Pred:checkTypeAndTextAndWrite(+File,+File,+File)
% Predikaatti tarkastaa ovatko tiedostojen tyyppi ja tekstitieto samoja
checkTypeAndTextAndWrite(Old,New,Trans):-
	tell(user),
	% Tcl-precedure give_name returns the name of the file without
	% directorypath.
	call_tcl(give_name(Old)),
	tcl_in(call_prolog(done(Oname))),
	call_tcl(give_name(New)),
	tcl_in(call_prolog(done(Nname))),
	see(Old),
	% Luetaan Old-kieliopin tyyppi ja tekstityyppi.
	read(OldType),
	read(OldText),
	see(New),
	% Luetaan New-kieliopin tyyppi ja tekstityyppi.
 	read(NewType),
	read(NewText),!,
	% Verrataan Oldin ja Newin tyyppeja keskenaan.
        (same(OldType,NewType);
           seen, see(Old),seen,
           call_tcl(open_msg("Structure grammars not comparable!")),
           tcl_in(call_prolog(ok))),
	% Verrataan Oldin ja Newin tekstityyppeja keskenaan.
        (same(OldText,NewText);
           seen, see(Old),seen,
           call_tcl(open_msg("Structure grammars not comparable!")),
           tcl_in(call_prolog(ok))),
	tell(Trans),
	% Kirjoitetaan Trans-kielioppiin Old- ja New-kielioppien 
        % nimet.
	writeq(Oname),write('.'),nl,
	writeq(Nname),write('.'),nl,!,	
        % Kirjoitetaan tyyppi ja tekstityyppi Transiin.       
        writeq(OldType),write('.'),nl,
        writeq(OldText),write('.'),nl.
 

%Pred:read_and_assertGrammarRules(File,Atom,+Rules,-Rules)
% Luetaan kielioppisaannot (Atom = new tai old)  predikaatin 
% ruleNew(LeftSide,RightSide,ListofElementOfRightSide,Rule) arvoksi
% tai ruleOld(LeftSide,RightSide,ListofElementOfRightSide,Rule) arvoksi
% tietokantaan.
% RightSide on kielioppisaannon oikea puoli.
% LeftSide on kielioppisaannon vasemman puoleisin termi.
% ListofElementOfRightSide on lista, joka sisaltaa kasittelemattoman 
% osan saannosta.
read_and_assertGrammarRules(File,NewOrOld,Rules1,Rules) :-
	% Luetaan kielioppisaanto tiedostosta.
	ruleInput(RuleChar),
	% Tarkistetaan, onko kaikki saannot jo luettu.
	(same(RuleChar,[end_of_file]),
         %lista on syntynyt vaarassa jarjestyksessa 
         reverse(Rules1,Rules)
         ;
         (same(RuleChar,[empty_line]),
             same(Rules1,Rules2),
	     % Jatketaan seuraavasta saannosta rekursiivisesti.
	     read_and_assertGrammarRules(File,NewOrOld,Rules2,Rules)
          ;
          % on jokin saanto
          % muutetaan atomiksi
	  name(Rule,RuleChar),	
	  % Luetaan vasen puoli saannosta.
	  word_c(RuleChar,LeftSide,RestRuleChar1),   
	  % Luetaan nuoli saannosta.
	  % varmistetaan, etta se oli nuoli.
	  word_c(RestRuleChar1,Arrow,[_|RightSideChar]),
	  same(Arrow,'->'),
          % loppu on oikea puoli
	  name(RightSide,RightSideChar),
	  % Luetaan saannon jaljella oleva osa ListofElementsOfRightSide:en.
	  restrule_c(_,RightSideChar,ListofElementsOfRightSide),
	  % Talletetaan kielioppisaanto tietokantaan.
	  % Joko Newiin
	  (same(NewOrOld,new),
	    assert(ruleNew(LeftSide,RightSide,
                           ListofElementsOfRightSide,Rule)),
            % uuden kieliopin saannoista tehdaan listaa
            append([Rule],Rules1,Rules2)
            ;
	    % tai Oldiin.
	    same(NewOrOld,old),
	    assert(ruleOld(LeftSide,RightSide,
                           ListofElementsOfRightSide,Rule)),
            % vanhan kieliopin saannoista ei tehda listaa
            same(Rules1,Rules2)
	   ),
	    % Jatketaan seuraavasta saannosta rekursiivisesti
 	    read_and_assertGrammarRules(File,NewOrOld,Rules2,Rules)
          )
         ).

%Pred:compare_and_write(+LeftSide,+RightSide,+RightElements,+Rule,+NewRules,
% +Trans)
% Predikaatti vertailee vanhaa ja uutta  saantoa toisiinsa ja
% kirjoittaa muunnossaannot tiedostoon Trans
compare_and_write(LeftOld,RightOld,ListOfRightOld,RuleO,Trans):-
	findCorrespondingRule(LeftOld,ListOfRightOld,RuleO,
                              LeftNew,RightNew,Changes),
	tell(Trans),
	writeTransRule(LeftOld,LeftNew,RightOld,RightNew,Changes),
	%Haetaan tietokannasta seuraava old-saanto
        (retract(ruleOld(LO,RO,LRO,RuleOld)),
	compare_and_write(LO,RO,LRO,RuleOld,Trans)
	;
        % ei enaa old-saantoja, kasitellaan loput new-sannot
	%Haetaan tietokannasta new-saanto
	findAndWriteNewRule(Trans)
	).


%Pred:findCorrespondingRule(+LOld,+ElementO,+RuleOld,
%                                -LNew,-RNew,-Changes)
% Predikaatti suorittaa varsinaisen Oldin ja Newin saantojen vertailun.
% Etsii ensin vastaavat vasemmat puolet ja sitten vertaa oikeita puolia.
findCorrespondingRule(LeftOld,ListOfRightOld,RuleO,
                                  LeftNew,RightNew,Changes):-

  	  %haetaan kaikki new-saannot, joissa sama nonterminaali
  	  (findall(ListOfRightNew,
                  (C=..[ruleNew,LeftOld,_,ListOfRightNew,_],
                  call(C)), List)),
          % muuttujassa List on lista, jossa loydettyjen saantojen
          % oikeat puolet elementtilistana, sama maara kuin loydettyja
          % saantoja
 	  % Lasketaan saantojen lkm
	  length(List,NumberOfRules),
	  %jos saantoja > 1 
	  (NumberOfRules>1,
             findCorrFromSeveral(LeftOld,ListOfRightOld,RuleO,
                                  LeftNew,RightNew,List,Changes)
	    ;	  
	  %if saantoja = 1     
	  NumberOfRules = 1,
             findCorrFromOne(LeftOld,ListOfRightOld,RuleO,
                                  LeftNew,RightNew,List,Changes)
	  ;
	  %jos  saantoja=0 
	  NumberOfRules=0,
             findCorrFromOthers(ListOfRightOld,RuleO,
                                  LeftNew,RightNew,Changes)
           ).

%Pred:findCorrFromSeveral(+LOld,+ElementO,+RuleOld,
%                                -LNew,-RNew,+List,-Changes)
findCorrFromSeveral(LeftOld,ListOfRightOld,RuleO,
                    LeftNew,RightNew,Lists,Changes):-
            % vanhalla ja uudella saannolla samat vasemmat puolet 
	    same(LeftOld,LeftNew),
            % tutkii oikeiden puolien loydettyjen joukosta vertaamalla
            % vanhan saannon oikeaa puolta  ListOfRightOld ja loydettyja
            % oikeita puolia Lists, palauttaa ListOfRightNew
	    (compareRules(ListOfRightOld,Lists,ListOfRightNew),
	    % ei nonterminaalien nimien muutoksia
            same(Changes,[]),
	    %tuhotaan valittu uuden kieliopin saanto tietokannasta
            % ja haetaan RightNew ja RuleN arvot
            retract(ruleNew(LeftNew,RightNew,ListOfRightNew,RuleN)),
            retract(restNewRules(NewRules)),
            deleteRn(NewRules,RuleN,NewRules1),
            retractall(restNewRules(_)),
            assert(restNewRules(NewRules1))
	    ;
            % samat nonterminaalit sisaltavaa oikeaa puolta ei ole, 
            % kysytaan kayttajalta
	      askingRule(RuleO,Lists,RightNew),
	      retract(ruleNew(LeftNew,RightNew,ListOfRightNew,RuleN)),
              retract(restNewRules(NewRules)),
              deleteRn(NewRules,RuleN,NewRules1),
              retractall(restNewRules(_)),
              assert(restNewRules(NewRules1)),
              % tutkitaan  miten nonterminaalit vastaavat toisiaan
              % ja generoidaan nimimuutosten lista
              askingRightSide(ListOfRightOld,RuleO,ListOfRightNew,RuleN,[],
                                           ChangesChar),
              (same(ChangesChar,[]),
                same(Changes,[]);
               name(Changes,ChangesChar))
	    ).
	 	
 
%Pred:compareRules(+ListOfRightOld,+ListOfListsOfRightNew,-ListOfRightNew)
% Vertaa saantoja. Loytyyko niista tsmlleen samoja saantoja tai
% samoja saantoja eri jrjestyksess 
compareRules(ListOfRightOld,ListsRN,ListOfRightNew):-
	%Tutkii onko vanha oikea puoli uusien oikeiden puolien listassa
	(member(ListOfRightOld,ListsRN),
        same(ListOfRightOld,ListOfRightNew)
	;
        % tutkii onko vanha oikeapuoli jonkin uuden permutaatio
        % muutetaan ensin oikea puoli  sellaisiksi ettei siina ole
        % metasymboleja
        findSPBRight(ListOfRightOld,[],ListOfRightOld1),
	checkPermutation(ListOfRightOld1,ListsRN,ListOfRightNew)
	).                      				


%Pred:findSPBRight(+ListWithMetaSymbols,+List,-ListWithoutMetaSymbols)
%Poistaa RN:sta sulkeet ja plussan tai tahden
%palauttaa samassa jarjestyksessa
findSPBRight([],RN,RN1):-!,reverse(RN,RN1).
findSPBRight([R|RN],RN1,RN2):-
        (name(R,RChar),
	 findStarOrPlusOrBracket(RChar,R1Char),
          name(R1,R1Char),!,
	  findSPBRight(RN,[R1|RN1],RN2)
	;
	  findSPBRight(RN,[R|RN1],RN2)
	).


%Pred:findStarOrPlusOrBracket
findStarOrPlusOrBracket(RChar,R1Char):-
        %Poistaa merkkijonon lopusta + tai * merkin ja 
	(findStarPlus_c(RChar,R1Char)
	;
        % jonon ymparilta hakasulkeet
        % tai valinnaisuuden ilmaisevat kysymysmerkin alussa tai lopussa
        % predikaatti tiedostossa syndoc.pl
	  findBracket(RChar,R1Char)
	).

%Tutkii onko jonossa plus- tai tahtimerkki ja palauttaa jonon ilman sulkeita
%Pred:findStarPlus_c
findStarPlus_c([C1,C|P],[C|Cs]):- 
	left_parenthesis(C1),!,
	letter(C),rest_atom(P,Cs,P1,letter_or_),
        (member(42,P1);member(43,P1)).

%Pred:checkPermutation(ListOfRightOld,+ListOfListsRightNew,ListOfRightNew)
% Vertaa onko vanhan saannon  oikea puoli (ilman metasymboleja)
% johonkin loydettyjen saantojen oikeista puolista 
% samat elementit mutta eri jarjestyksessa
checkPermutation(_,[],_):-
	fail.
checkPermutation(ListOfRightOld1,[L|Ls],ListOfRightNew):-
	%poistetaan metasymbolit oikealta puolelta
        findSPBRight(L,[],L1),
        (
        checkRight(ListOfRightOld1,L1),
        same(L,ListOfRightNew)
	;
	checkPermutation(ListOfRightOld1,Ls,ListOfRightNew)
	).
 
%Pred:checkRight(+List,+List)
% Predikaatti tarkastaa onko saantojen oikeissa puolissa samat
% nonterminaalit (ei sisalla enaa  metasymboleja),
% mutta elementit eri jarjestyksessa
checkRight([],_).
checkRight([R|Rs],L):-
	member(R,L),
	checkRight(Rs,L).

%Pred:deleteRn(+ListOfAtoms,+Atom,-ListOfAtomsNew)
%Poistaa listasta ListOfAtoms atomin Atom ja palauttaa jaljelle jaavan 
% listan ListOfAtomsNew:ssa samassa jarjestyksessa
deleteRn([],_,[]):-!.
deleteRn([Atom1|RN],Atom,RRN):-
	same(Atom1,Atom),
	  same(RN,RRN).
deleteRn([Atom1|RN],Atom,[Atom1|RRN]):-
	  deleteRn(RN,Atom,RRN).
	  	
%Pred:askingRule(+RuleOld,+ListOfRightsNew,-RightNew)
% Predikaatti kysyy kayttajalta mika oikean puolen 
% vaihtoehdoista vastaavat toisiaan
askingRule(RuleO,ListOfRightsNew,RightNew):-
        name(RuleO,RuleOChar),
        length(RuleOChar,Wi),
        append("Which right side is for: ",RuleOChar,Heading),
	rightsToChar(ListOfRightsNew,ListOfRightsNewAtom),
        Width is 40 + Wi,
	 % Avataan kielioppisaantojen valintaikkuna.
	    tell(user),
	    call_tcl(open_rules(Heading,ListOfRightsNewAtom,Width)),
	    tcl_in(call_prolog(selected(RightNew))).

%Pred:askingRightSide(+ListOfRightOld,+RuleO,+ListOfRightNew,+RuleN,
%                                +Changes1Char,-ChangesChar)
%Predikaatti muodostaa oikeiden puolien ehdot, automaattisesti tai
%kysymalla kayttajalta, jos  nimissa on eroja. Avaa ikkunan 
%rule_selection
askingRightSide([],_,_,_,ChangesChar,ChangesChar).
askingRightSide([R|RO],OldRule,RN,NewRule,Changes1Char,ChangesChar):-	  
        member(R,RN),   
          % sama nonterminaali on oikealla puolella, poistetaan eika
          % tule nimimuutoksien listaan uutta elementtia
	  deleteRn(RN,R,RN1),
	  same(Changes1Char,Changes2Char),
	  askingRightSide(RO,OldRule,RN1,NewRule,
                                   Changes2Char,ChangesChar).
askingRightSide([R|RO],OldRule,RN,NewRule,Changes1Char,ChangesChar):-	  
          % nonterminaalia ei ole oikeanpuolen listassa
          % tehdaan otsikko ikkunaan jolla kysytaan mika jaljella olevista
          % nonterminaaleista on saman niminen
          name(R,RChar),
	  append("New name for element: ", RChar ,Heading),
	  name(OldRule,OldRuleChar),
	  name(NewRule,NewRuleChar),
          % muutetaan oikea lista  ascii-merkkijonoksi, pisin saanto maaraa
          % ikkunan leveyden
  	  rules_to_char(RN,RNChar,Wi),
	  Width is Wi + 40,
	    tell(user),
	    call_tcl(rule_selection(Heading,OldRuleChar,NewRuleChar,
                                    RN,RNChar,Width)),
	    tcl_in(call_prolog(done(Part))),
            %palautetaan valittu nonterminaalinimi ilman 
            % metasymboleja tai cancel
            (same(Part,cancel),
	     % ei valittu mitaan, joten ei nimimuutoksia
	     same(Changes1Char,Changes2Char),
             same(RN,RN1)
	     ; 	
             %valittiin joku nonterminaali Part, joka vastaa kasiteltavaa
             % nonterminaalia, tehdaan  uusi muutosehto muuttujaan 
             % Changes2Char
             formChanges(RChar,RO,Part,Changes1Char,Changes2Char),
           %poistetaan valittu nonterminaali oikealta puolelta 
	    deleteRn(RN,Part,RN1)
            ),
            askingRightSide(RO,OldRule,RN1,NewRule,
                                          Changes2Char,ChangesChar).

formChanges(RChar,RO,Part,Changes1Char,Changes2Char):-
             name(Part,PartChar),
	     append(RChar," = ",C3),
	     append(C3,PartChar,C5),
	     (RO=[],
             %jos nonterminaalit loppu
	       (same(Changes1Char,[]),
               %jos ei lisaksi yhtaan ehtoa, kirjoitetaan ilman pilkkua, 
               %muuten pilkku eteen
                 same(C5,C4);
                 append(" , ", C5,C4)
                )
	     ;
             %jos nonterminaalit ei loppu 
               (same(Changes1Char,[]),
               %eika yhtaan ehtoa, 
               %kirjoitetaan ilman pilkkua
               same(C5,C4)
	      ; 
              % jos ehtoja ja viela nonterminaaleja, 
              % kirjoitetaan pilkku eteen
               append(" , ",C5, C4))
	     ),  
             % lisataan vanhoihin ehtoihin
             append(Changes1Char,C4,Changes2Char). 

	    	  
%Pred:findCorrFromOne(+LOld,+ElementO,+RuleOld,
%                                -LNew,-RNew,+List,-Changes)
% loydettiin vain yksi oikeapuoli samalla vanhalle puolelle
findCorrFromOne(LeftOld,ListOfRightOld,RuleO,
                             LeftNew,RightNew,[ListOfRightNew],Changes):-
            % vanhalla ja uudella saannolla samat vasemmat puolet 
	      same(LeftOld,LeftNew),
	    % tutkii ainoaa loydettya oikeaa puolta vertaamalla
            % vanhan saannon oikeaa puolta  ListOfRightOld ja loydettya
            % oikeaa puolta ListOfRightNew, 
             (
                (same(ListOfRightOld,ListOfRightNew)
               ;
               findSPBRight(ListOfRightOld,[],ListOfRightOld1),
               findSPBRight(ListOfRightNew,[],ListOfRightNew1),
               checkRight(ListOfRightOld1,ListOfRightNew1)
               ),
  	    % ei nonterminaalien nimien muutoksia
	      same(Changes,[]),
	      %tuhotaan valittu saanto uudesta kieliopista 	
              % ja haetaan RightNew arvo
	      retract(ruleNew(LeftNew,RightNew,ListOfRightNew,RuleN)),
              retract(restNewRules(NewRules)),
              deleteRn(NewRules,RuleN,NewRules1),
              retractall(restNewRules(_)),
              assert(restNewRules(NewRules1))
	    ;
             % saannot eivat sisalla samoja nonterminaaleja
	      %poistetaan valittu saanto uudesta kieliopista
              % ja haetaan RightNew arvo ja RuleN 
	      retract(ruleNew(LeftNew,RightNew,ListOfRightNew,RuleN)),
              retract(restNewRules(NewRules)),
              deleteRn(NewRules,RuleN,NewRules1),
              retractall(restNewRules(_)),
              assert(restNewRules(NewRules1)), 
              % tutkitaan  miten nonterminaalit vastaavat toisiaan
              % ja generoidaan nimimuutosten lista
              % poistetaan metasymbolit sita ennen koska tuloslistoissa
              % nonterminaalit ilman metasymboleja
               findSPBRight(ListOfRightOld,[],ListOfRightOld2),
               findSPBRight(ListOfRightNew,[],ListOfRightNew2),
              askingRightSide(ListOfRightOld2,RuleO,
                                 ListOfRightNew2,RuleN,
                                     [],ChangesChar),
	        (same(ChangesChar,[]),
                  same(Changes,[])
                 ;
                  name(Changes,ChangesChar)) 
	      ).

%Pred:findCorrOthers(+ElementO,+RuleOld,
%                                -LNew,-RNew,-Changes)
findCorrFromOthers(ListOfRightOld,RuleO,
                                  LeftNew,RightNew,Changes):-
            %vanhalle saannolle ei loydy yhteen vastaavaa saantoa
            % haetaan lista kaikista uuden kieliopin jaljella olevista
            % saannoista
            restNewRules(ListNewRules),
           %muutetaan saannot merkkilistaksi ja tehdaan otsikot
            % vastaavan saannon valitsemiseksi
	    rules_to_char(ListNewRules,RLChar,Wi),
	    name(RuleO,RuleOChar),
	    append("Corresponding rule for: ",RuleOChar,Heading),
	    Width is Wi + 40,
	    tell(user),
	    call_tcl(open_rules(Heading, RLChar, Width)),
	    tcl_in(call_prolog(selected(Part))),
	    (Part=cancel,
              % uudessa kieliopissa ei ole saantoa
	      same(LeftNew,[]),same(RightNew,[]),
              same(Changes,[])
	    ;
	      % poistetaan saanto tietokannasta ja haetaan arvot
              % LeftNew, RightNew ja ListOfRightNew
	      retract(ruleNew(LeftNew,RightNew,ListOfRightNew,Part)),
              retract(restNewRules(NewRules)),
              deleteRn(NewRules,Part,NewRules1),
              retractall(restNewRules(_)),
              assert(restNewRules(NewRules1)),
	    % tutkii  loydettya oikeaa puolta vertaamalla
            % vanhan saannon oikeaa puolta  ListOfRightOld ja loydettya
            % oikeaa puolta ListOfRightNew
             (
              (same(ListOfRightOld,ListOfRightNew)
                ;
                findSPBRight(ListOfRightOld,[],ListOfRightOld1),
                findSPBRight(ListOfRightNew,[],ListOfRightNew1),
        	checkRight(ListOfRightOld1,ListOfRightNew1)
                ),
        	% ei nonterminaalien nimien muutoksia
	        same(Changes,[])
               ;
              % tutkitaan miten valittujen saantojen oikeiden puolien
              % nonterminaalit vastaavat toisiaan
              % ja generoidaan nimimuutosten lista
              % poistetaan metasymbolit sita ennen koska tuloslistoissa
              % nonterminaalit ilman metasymboleja
	      findSPBRight(ListOfRightOld,[],ListOfRightOld2),
	      findSPBRight(ListOfRightNew,[],ListOfRightNew2),
              askingRightSide(ListOfRightOld2,RuleO,
                                 ListOfRightNew2,Part,
                                     [],ChangesChar),
	        (same(ChangesChar,[]),
                   same(Changes,[])
                  ;
                  name(Changes,ChangesChar))
              )
             ).



%Pred:rightsToChar(+ListOfLists,-CharList)
% Muuttaa New-saantojen  oikeat puolet ascii-listoiksi
rightsToChar([],[]):-!.
rightsToChar([R|Rrights],[RC|RightsChar]):-
	listToChar_c(R,[],RC,0,_),
	rightsToChar(Rrights,RightsChar).


%Pred:listToChar_c(+List,+CharList,-CharList,+Width0,-Width)
% Muuttaa listan ascii-listaksi
listToChar_c([],RChar,RChar,FirstLen,FirstLen):-!.
listToChar_c([TextH | TextT],RChar,RCharNew, FirstLen,MaxLen) :-
	name(TextH,TextHChar),
	(TextT=[],
	   same(TextHChar,TextChar1)
	   ;
           append(TextHChar," ",TextChar1)),
	append(RChar,TextChar1,RChar1),
        length(TextChar1,ThisLen),
	SecondLen is FirstLen+ThisLen,
	listToChar_c(TextT,RChar1,RCharNew,SecondLen,MaxLen),!,
	!. 



%Pred:writeTransRule(+Atom,+Atom,+Atom,+Atom,+Atom)
% Predikaatti suorittaa muutossaannon kirjoittamisen transiin
writeTransRule(LeftOld,LeftNew,RightOld,RightNew,Changes):-
	(same(LeftNew,[]) % ei saantoa uudessa kieliopissa
                          % ei kirjoiteta mitaan
        ;
	write(LeftOld),write(' ; '),
	write(LeftNew),write(' -> '),
	write(RightOld),write(' ; '),
        write(RightNew),
	(
        same(Changes,[]) % ei muutoksia nonterminaalinimissa
	;  
	  write(' { '),
	  write(Changes),
	  write(' }')
	),
	write('.'),
	nl).



%Pred:findAndWriteNewRule(+File)
% Predikaatti etsii jas kirjoittaa jaljella olevat New-saannot Transiin ellei
% Old-saantoja enaa ole. 
findAndWriteNewRule(Trans):-
	%Haetaan tietokannasta new-saanto
        (retract(ruleNew(LeftNew,RightNew,_,_)),
	tell(Trans),
    	write(LeftNew),write(' ; '),write(LeftNew),
	write(' -> '),
	write('""'),write(' ; '),write(RightNew),
	write('.'),
	nl,		     	
	findAndWriteNewRule(Trans)
	;
	true).

%Pred:restrule_c
restrule_c(_,[],[]):-!.
restrule_c(_,P,[W1|Ws]):- word_c(P,W1,P1),!,restrule_c(W1,P1,Ws).


%Pred:word_c
% Tutkitaan merkkijonon sisaltoa transform Compare-osassa.
% Muuttujan A arvona on se osa merkkijonosta, joka on kayty lapi.
% Muuttujan P1 arvona on loppuosa merkkijonosta.
%
% Sana sisaltaa isoja tai pienia kirjamia, alaviivan, ja ?-merkin.
% Palautetaan pienina kirjaimina    
word_c([C|P],A,P1):-
	letter(C),rest_atom(P,Cs,[P2|P1],letter_or_),
        question_mark(P2),
        append([C|Cs],[P2],C3),
	uc_to_lc(C3,L),name(A,L).
% Sana sisaltaa isoja tai pienia kirjamia, alaviivan, tahden, 
% plus-merkin.               
word_c([C|P],A,P1):-
	letter(C),
        rest_atom(P,Cs,P1,letter_or_star_or_plus),
	uc_to_lc([C|Cs],L),name(A,L).
% Sana on  ->.
word_c([C,C2|Cs],'->',Cs):-
	hyphen(C),greater_than(C2),!.
% Sana sisaltaa vasemman sulun, mita tahansa merkkeja, oikean sulun,
% *-merkin tai +-merkin. * ja + kasitellaan samalla tavalla 
word_c([C|P],A,P1):-
	 left_parenthesis(C),!,
         rest_atom(P,Cs,[P2,P3|P4],any_but_right_parenthesis),!,
	right_parenthesis(P2),
	(single_star(P3);single_plus(P3)),
	append([C|Cs],[P2,P3],P5),
        name(A,P5),
	same(P4,P1).
% Sana sisaltaa vasemman hakasulun, mita tahansa merkkeja,
% oikean hakasulun.
word_c([C|P],A,P1):-
	left_bracket(C),!,
        rest_atom(P,Cs,[P2|P1],any_but_right_bracket),!,
       	right_bracket(P2),
	append([C],Cs,P5),
	append(P5,[P2],P6),
	name(A,P6).
% Sana sisaltaa vasemman hakasulun sijasta ? ja mita tahansa merkkeja,
% (retrieval-merkinta optional-alkiolle
word_c([C|P],A,P1):-
	question_mark(C),!,
        rest_atom(P,Cs,P1,letter_or_),!,
	append([C],Cs,P5),
	name(A,P5).
% Sana sisaltaa vain lainausmerkit.
word_c([C|P],'empty',P):-double_quote(C),!.
% Sanan ensimmainen merkki on jokin muu kuin kirjain, lainausmerkki,
% viiva, vasen sulku, vasen hakasulku, kysymysmerkki, kaksoislainausmmerkki.
word_c([_|P1],W,P2):-word_c(P1,W,P2).
% Merkkijono on tyhja.
word_c([],'',[]) :- !.



/*
 *----------------------------------------------------------------------------
 *    transform_sdts
 *----------------------------------------------------------------------------
*/

%Pred:transform_sdts(+TransformGram,+OldDocFile,+NewDocFile)
transform_sdts(TransformGram, DocFile, NewDocFile, TransformProg):-	
            generateTransform(TransformGram,TransformProg),
            % luetaan tranformaatio-ohjelmasta
            see(TransformProg),
            % transform grammar file
	    read(_),
            % old skeleton grammar
	    read(OldGram),
            % new skeleton grammar
            read(NewGram),
            % typpi ja tekstityyppi
            read(TrType),
	    read(TrText),
	    seen,
            % luetaan dokumentti
	    see(DocFile),
            % skeleton grammar
	    read(Gram),!,
            % verrataan etta vanha rakennekielioppi ja 
            % dokumentin rakennekielioppi samat
	    check_grams(OldGram,Gram),
	    read(Type),
	    read(Text),
	    read(Tree),
	    seen,!,
            % konsultoidaan ohjelma Prolog-tietokantaan
            % kaantaminen samanlainen kuin Sicstus, 
            %pyyhkii entiset predikaatit
            oconsult(TransformProg),
            % tehdaan muunnos
	    Goal =..[Type,Tree,NewTree,_,_],
	    call(Goal),
            % kirjoitetaan tulos uuteen dokumenttitiedostoon
	    tell(NewDocFile),!,
	    writeq(NewGram),write('.'),nl,
	    writeq(Type),write('.'),nl,
	    writeq(Text),write('.'),nl,
	    writeq(NewTree),write('.'),nl,
	    told,
            call_tcl(open_info("Document has been transformed!")),
	    tcl_in(call_prolog(ok)),
	    !.

/*
 * ----------------------------------------------------------------------------
 *    generateTransform
 
 * ----------------------------------------------------------------------------
 */

%Pred:generateTransform(+File,+File)
generateTransform(SdtsFile,TransformProg):-
        	call_tcl(give_name(SdtsFile)),
	        tcl_in(call_prolog(done(SdtsFileName))),
	        see(SdtsFile),
                % vanha skeleton grammar
		read(OldGram),
                % uusi skeleton grammar
                read(NewGram),
                % tyyppi ja sisaltotyppi
		read(Type),
		read(Text),
                % kirjoitetaan transformaatio-ohjelman alkuun
		tell(TransformProg),!,
		writeq(SdtsFileName),write('.'),nl,
	        writeq(OldGram),write('.'),nl,
	        writeq(NewGram),write('.'),nl,
	        writeq(Type),write('.'),nl,
	        writeq(Text),write('.'),nl,
		repeat,
		termInput_tr(Termlist),
		process_tr(Termlist,TransformProg),!,
                   write_added_tr(TransformProg),
		careCharString2(Text,Production),
		writeterm(TransformProg,Production),
		seen,told.


%Pred:careCharString2(+CharStringType,-Production)
% Muodostaa CharStringType = text arvosta  Text  produktiosaannon 
% text(text(A),text(B))--> A=B. 
% numbervars muuntaa muuttujat muotoa _135  aakkosmuotoon
careCharString2(Text,Production) :-
	Prod =..[Text,Ws],
	Prod2 =..[Text,Wss],
	Head =..[Text,Prod,Prod2],
	Body = (Ws=Wss),
	Production = (Head --> {Body}).



%Pred:write_added_tr
write_added_tr(OutputProg):-
                             retract(added_nonterminal_production(_,P)),
                             writeterm(OutputProg,P),!,
                             write_added_tr(OutputProg).
write_added_tr(_).

%Pred:termInput_tr
termInput_tr(Termlist):-
	        ruleInput(RuleChar),!,
		formTermlist_tr(RuleChar,Termlist).

%Pred:formTermlist_tr
formTermlist_tr(RuleChar,[Word|Textlist]):-
	analyse_word_tr(RuleChar,Word,RestRule),!,
	restwords_tr(Word,RestRule,Textlist).

%Pred:restwords_tr
restwords_tr(_,[],[]):-!.
restwords_tr(empty_line,_,[]):-!.
restwords_tr(end_of_file,_,[]):-!.
%restwords(full_stop,_,[]):-!.
restwords_tr(_,Rule,[Word1|Textlist]):-analyse_word_tr(Rule,Word1,Rest),!,
        restwords_tr(Word1,Rest,Textlist).

%Pred:analyse_word_tr
analyse_word_tr([empty_line],empty_line,[]):-!.
analyse_word_tr([end_of_file],end_of_file,[]):-!.
analyse_word_tr([C|P],nonterminal(A),P1):-
% nonterminaali
	        letter(C),!,rest_atom(P,Cs,P1,letter_or_),
		uc_to_lc([C|Cs],NT),name(A,NT).
analyse_word_tr([C1|P],nonterminallist(A),P1):-
	        left_parenthesis(C1),!,rest_atom(P,Cs,[C2,C3|P1],
		                           letter_or_),
		right_parenthesis(C2),!,
		(single_star(C3);single_plus(C3)),!,
		uc_to_lc(Cs,Cs1),
		append(Cs1,[108,105,115,116,C3],L1),	
	        name(A,L1).
analyse_word_tr([C1|P],optional_nonterminal(A),P1):-
	        left_bracket(C1),!,rest_atom(P,Cs,[C2|P1],letter_or_),
		right_bracket(C2),!,
		uc_to_lc(Cs,Cs1),
		append([C1|Cs1],[C2],O),
		name(A,O).
analyse_word_tr([C|P],syntactic_terminal(A),P1):-
	single_quote(C),!,rest_atom(P,Cs,[_|P1],any_but_single_quote),
        append([C|Cs],[C],L1),name(A,L1).
analyse_word_tr([C,C1|Cs],'-->',Cs):-
	        hyphen(C),greater_than(C1),!.
analyse_word_tr([C|Cs],';',Cs):- semicolon(C),!.
analyse_word_tr([C|Cs],',',Cs):- comma(C),!.
analyse_word_tr([C|Cs],'}',Cs):- rbrace(C),!.
analyse_word_tr([C|Cs],'{',Cs):- lbrace(C),!.
analyse_word_tr([C|Cs],'=',Cs):- equal(C),!.
analyse_word_tr([C1,C2|Cs],empty_word,Cs):-double_quote(C1,C2),!.
analyse_word_tr([C|Cs],full_stop,Cs):-period(C),!.
analyse_word_tr([_|P1],W,P2):-analyse_word_tr(P1,W,P2).


%Pred:process_tr
process_tr([end_of_file],_):-!.
process_tr([empty_line],_):-!,fail.
process_tr(Termlist,TransformProg):-
	        grammar_rule_tr(Production,Termlist,[]),
		(Production = (empty),!,fail;
		 write_atoms(TransformProg),
		 write_lists_tr(TransformProg),
		 write_changes(TransformProg),
		 writeterm(TransformProg,Production),!,fail
	        ).


%Pred:write_lists_tr
write_lists_tr(OutputProg):-(retract(empty_list_production(Prod0)),
			      writeterm(OutputProg,Prod0),
			      (retract(one_list_production(Prod1)),
			       writeterm(OutputProg,Prod1),
			      retract(list_production(Prod)),
			       writeterm(OutputProg,Prod);
			      retract(list_production(Prod)),
			       writeterm(OutputProg,Prod))
			     ;
			      retract(one_list_production(Prod1)),
			       writeterm(OutputProg,Prod1),
			      retract(list_production(Prod)),
			       writeterm(OutputProg,Prod)
                             ),!, 
			 write_lists_tr(OutputProg).
write_lists_tr(_).

%Pred:grammar_rule_tr
grammar_rule_tr(Production) --> [nonterminal(Functor)],[';'],[nonterminal(Functor1)],['-->'],
	        body1(Vars1,Body1),[';'],body2(Vars2,Body2),
		['{'],rename(Names,Renames),['}'],
		{(Functor = Functor1,
		  (Body2 = [], Production = (empty);
		   (Body1 = [],Head =..[Functor,_,V],
		              Production = (Head --> {V=Functor});
%                    flatten1([Functor,Vars1],Arg1),
                    flatten([Functor,Vars1],Arg1),
		    Argument1 =.. Arg1,
%                    flatten1([Functor,Vars2],Arg2),
                    flatten([Functor,Vars2],Arg2),
                    Argument2 =.. Arg2,
%                    flatten1([Functor,Argument1,Argument2],Head1),
                    flatten([Functor,Argument1,Argument2],Head1),
		    Head =.. Head1,
%                    flatten1(Body2,Body22),flatten1(Body1,Body11),
                    flatten(Body2,Body22),flatten(Body1,Body11),
		    newbody(Body22,Body11,Names,Renames,Body),
		    Production = (Head --> Body)),
		    He =.. [Functor,Functor,Functor],
		    Prod = (He --> !));
%                  flatten1([Functor,Vars1],Arg1),
                  flatten([Functor,Vars1],Arg1),
		  Argument1 =.. Arg1,
                  append([Functor1],Vars2,Arg2),
		  Argument2 =.. Arg2,
%                  flatten1([Functor1,Argument1,Argument2],Head1),
                  flatten([Functor1,Argument1,Argument2],Head1),
		  Head =.. Head1,
%		  flatten1(Body2,Body22),flatten1(Body1,Body11),
		  flatten(Body2,Body22),flatten(Body1,Body11),
		  newbody(Body22,Body11,Names,Renames,Body),
	          Production = (Head --> Body),
		  He =..[Functor1,Functor,Functor1],
		  Prod = (He --> !)),
		  (retract(atom_production(P)),
		   (same(P,Prod),asserta(atom_production(P));
		       asserta(atom_production(P)),
		       asserta(atom_production(Prod)));
		   asserta(atom_production(Prod))),!}.

grammar_rule_tr(Production) --> [nonterminal(Functor)],[';'],[nonterminal(Functor1)],['-->'],
	        body1(Vars1,Body1),[';'],body2(Vars2,Body2),
		{(Functor = Functor1,
		  (Body2 = [], Production = (empty);
		   (Body1 = [],Head =..[Functor,_,V],
		              Production = (Head --> {V=Functor});
	            
%		    flatten1([Functor,Vars1],Arg1),
		    flatten([Functor,Vars1],Arg1),
	            Argument1 =.. Arg1,
%                    flatten1([Functor,Vars2],Arg2),
                    flatten([Functor,Vars2],Arg2),
                    Argument2 =.. Arg2,
%                    flatten1([Functor,Argument1,Argument2],Head1),
                    flatten([Functor,Argument1,Argument2],Head1),
		    Head =.. Head1,
%                    flatten1(Body2,Body22),flatten1(Body1,Body11),
                    flatten(Body2,Body22),flatten(Body1,Body11),
		    newbody(Body22,Body11,Body),
		    Production = (Head --> Body)),
		    He =.. [Functor,Functor,Functor],
		    Prod = (He --> !));
%		  flatten1([Functor,Vars1],Arg1),
		  flatten([Functor,Vars1],Arg1),
		  Argument1 =.. Arg1,
                  append([Functor1],Vars2,Arg2),
		  Argument2 =.. Arg2,
%                  flatten1([Functor1,Argument1,Argument2],Head1),
                  flatten([Functor1,Argument1,Argument2],Head1),
		  Head =.. Head1,
%		  flatten1(Body2,Body22),flatten1(Body1,Body11),
		  flatten(Body2,Body22),flatten(Body1,Body11),
		  newbody(Body22,Body11,Body),
	          Production = (Head --> Body),
		  He =..[Functor1,Functor,Functor1],
		  Prod = (He --> !)),
		  (retract(atom_production(P)),
		   (same(P,Prod),asserta(atom_production(P));
		       asserta(atom_production(P)),
		       asserta(atom_production(Prod)));
		   asserta(atom_production(Prod))),!}.

%Pred:rename
rename(Names,Renames) --> rename_item(I_Name,I_Rename),[','], rename(I_Names,I_Renames),
	                  {append(I_Name,I_Names,Names),
			   append(I_Rename,I_Renames,Renames)}.
rename(Name,Rename) --> rename_item(Name,Rename).


%Pred:rename_item
rename_item([Name],[Rename]) --> [nonterminal(Name)],['='],[nonterminal(Rename)].

%Pred:body1
body1(Vars,Terms) -->
	        body_item1(I_var,I_term),body1(I_vars,I_terms),
		{append(I_var,I_vars,Vars),
	        append(I_term,I_terms,Terms)}.
body1(I_var,I_term) --> body_item1(I_var,I_term).

%Pred:body_item1
body_item1([X],[Term]) --> [nonterminal(F)],
	                       {!,Term =.. [F,X]}.
body_item1([X],[Term]) --> [nonterminallist(F)],
                       {!,name(F,FChar),append([39|FChar],[39],FlChar),
                        name(Fl,FlChar),
			Term =.. [Fl,X]}.
body_item1([X],[Term]) --> [optional_nonterminal(F)],
	               {!,name(F,FChar),append([39|FChar],[39],FoChar),
		       name(Fo,FoChar),
		       Term =.. [Fo,X]}.
body_item1([Const],[]) --> [syntactic_terminal(Const)].
body_item1([],[]) --> [empty_word].

%Pred:body2
body2(Vars,Terms) -->
	        body_item2(I_var,I_term),body2(I_vars,I_terms),
		{append(I_var,I_vars,Vars),
	        append(I_term,I_terms,Terms)}.
body2(I_var,I_term) --> body_item2(I_var,I_term).

%Pred:body_item2
body_item2([X],[Term]) --> [nonterminal(F)],
	                       {!,Term =.. [F,X]}.
body_item2([X],[Term]) --> [nonterminallist(F)],
                       {!,name(F,FChar),
                        append([39|FChar],[39],FlChar),
                        name(Fl,FlChar),
                         Term =.. [Fl,X]}.
body_item2([X],[Term]) --> [optional_nonterminal(F)],
	               {!, name(F,FChar),append([39|FChar],[39],FoChar),
			name(Fo,FoChar),
			Term =.. [Fo,X]}.
body_item2([Const],[]) --> [syntactic_terminal(Const)].
body_item2([],[]) --> [empty_word].

%Pred:trans_item
trans_item(LHS,Vars,RHS) --> [nonterminal(LHS)],['-->'],rhs_body(Vars,RHS).

%Pred:rhs_body
rhs_body(Vars,RHS) --> rhs_item(Var,Item),rhs_body(I_Vars,I_Items),
	{append(Var,I_Vars,Vars),
         append(Item,I_Items,RHS)}.
rhs_body(Var,Item) --> rhs_item(Var,Item).

%Pred:rhs_item
rhs_item([X],[Item]) --> [nonterminal(F)],
	{!,Item =.. [F,X]}.


%Pred:newbody
newbody([Term],Body1,Names,Renames,Body) :- 
	define_args(Term,Body1,Names,Renames,_,_,Args),
	Term =..[Functor,_],
        append([Functor],Args,Body11),
        Body =.. Body11.
newbody([Term|Terms],Body1,Names,Renames,Body) :- 
	define_args(Term,Body1,Names,Renames,Ns,Rs,Args),
	Term =.. [Functor,_],
	append([Functor],Args,Term11),
	Term1 =.. Term11,!,
	newbody(Terms,Body1,Ns,Rs,Bodys),
	Body = (Term1,Bodys).
newbody([Term],Body1,Body) :-
	                    define_args(Term,Body1,Args),
	                    Term =..[Functor,_],
			    append([Functor],Args,Body11),
			    Body =.. Body11.
newbody([Term|Terms],Body1,Body) :- 
	define_args(Term,Body1,Args),
	Term =.. [Functor,_],
        append([Functor],Args,Term11),
	Term1 =.. Term11,
	newbody(Terms,Body1,Bodys),
	Body = (Term1,Bodys).


%Pred:define_args
define_args(Term,Body1,Names,Renames,Ns,Rs,Args) :- 
	Term =..[Functor2,Arg2],
	define_name(Functor2,FunctorName2),
	(search_name_pair(FunctorName2,Names,Renames,FunctorName1,[],[],Ns,Rs),!,
	     search_same_body(FunctorName1,Body1,Functor2,FunctorName2,Arg1),
	     Args = [Arg1,Arg2];
	 same(Names,Ns),same(Renames,Rs),!,
	     (search_same_body(FunctorName2,Body1,Functor2,FunctorName2,Arg1),
	        Args = [Arg1,Arg2];
	        Args = [_,Arg2]),!,
%		              P = (H --> {V=Functor})
                H =..[Functor2,_,H2],
                P = (H --> {H2=Functor2}),
		    (retract(added_nonterminal_production(Functor2,P0)),
                           asserta(added_nonterminal_production(Functor2,P0));
                             asserta(added_nonterminal_production(Functor2,P)))
                  ).
define_args(Term,Body1,Args) :- Term =..[Functor,Arg2],
         define_name(Functor,FunctorName),
	(search_same_body(FunctorName,Body1,Functor,FunctorName,Arg1),
	                    Args = [Arg1,Arg2];
	                  Args = [_,Arg2],!,
%		              P = (H --> {V=Functor})
                           H =..[Functor,_,H2],
                           P = (H --> {H2=Functor}),
		    (retract(added_nonterminal_production(Functor,P0)), 
                           asserta(added_nonterminal_production(Functor,P0));
                             asserta(added_nonterminal_production(Functor,P)))
                          ).

%Pred:search_name_pair
search_name_pair(_,[],[],_,Ns,Rs,Ns,Rs) :- !,fail.
search_name_pair(FunctorName,[N|Names],[R|Renames],N,Ns1,Rs1,Ns,Rs) :-
         (same(FunctorName,R),append(Ns1,Names,Ns),append(Rs1,Renames,Rs);
	    append(Ns1,[N],Ns11),append(Rs1,[R],Rs11),!,
	    search_name_pair(FunctorName,Names,Renames,_,Ns11,Rs11,_,_)).


%Pred:define_name
define_name(Functor,FunctorName):-
             name(Functor,FunctorChar),
             (append([39|FunctorNameChar],[108,105,115,116,_,39],FunctorChar);
              append([39,91|FunctorNameChar],[93,39],FunctorChar);
               same(FunctorChar,FunctorNameChar)),
             name(FunctorName,FunctorNameChar).


%Pred:define_listatom
define_listatom(FunctorName,C,FunctorAtom):-
             name(FunctorName,FunctorNameChar),
             append([39|FunctorNameChar],[C,39],FunctorAtomChar),
             name(FunctorAtom,FunctorAtomChar).  

%Pred:nonterminallist
nonterminallist(Functor,C) :- name(Functor,FunctorChar),!,
                            append([39|_],[108,105,115,116,C,39],FunctorChar).

%Pred:optional
optional(Functor) :- name(Functor,FunctorChar),!,
		     append([39,91|_],[93,39],FunctorChar).

%Pred:search_same_body
search_same_body(FunctorName,Term,Functor2,FunctorName2,Arg):- 
	Term =..[Functor1,Arg1],!,
        search_same_term(FunctorName,Functor1,Functor2,FunctorName2,Arg1,Arg).
search_same_body(FunctorName,[Term|Terms],Functor2,FunctorName2,Arg) :-
	Term =.. [Functor1,Arg1],!,
        (search_same_term(FunctorName,Functor1,Functor2,FunctorName2,Arg1,Arg);
        search_same_body(FunctorName,Terms,Functor2,FunctorName2,Arg)).


%Pred:search_same_term
search_same_term(FunctorName,Functor1,Functor2,FunctorName2,Arg1,Arg):-
	define_name(Functor1,FunctorName1),!,
% nonterminal = nonterminal
         (same(Functor1,FunctorName1),same(Functor2,FunctorName2),!,
          same(FunctorName,FunctorName1),Arg=Arg1
          ;
% nonterminallist? = nonterminallist?
          nonterminallist(Functor2,C2),nonterminallist(Functor1,C1),!,
             same(FunctorName,FunctorName1),Arg=Arg1,
             define_listatom(FunctorName1,C1,Atom1),
             define_listatom(FunctorName2,C2,Atom2),
                        (single_star(C2),
      			    He0=..[Functor2,[Atom1],[Atom2]],
			    Prod0 = (He0 --> !),
			    (retract(empty_list_production(P0)),
                             (same(P0,Prod0),asserta(empty_list_production(P0));
                              asserta(empty_list_production(P0)),
                              asserta(empty_list_production(Prod0)));
                             asserta(empty_list_production(Prod0))),!
                         ;
                         single_plus(C2),
                            (same(C1,C2),
                            He1=..[Functor2,[B,Atom1],[B1,Atom2]],
                            Bo1=..[FunctorName2,B,B1],
                            Prod1 = (He1 --> Bo1),
			    (retract(one_list_production(P1)),
                             (same(P1,Prod1),asserta(one_list_production(P1));
                              asserta(one_list_production(P1)),
                              asserta(one_list_production(Prod1)));
                             asserta(one_list_production(Prod1))),!
			    ;
                            He0=..[Functor2,[Atom1],[FunctorName2,Atom2]],
                            Prod0 = (He0 --> !),
                            (retract(empty_list_production(P0)),
                             (same(P0,Prod0),asserta(empty_list_production(P0));
                              asserta(empty_list_production(P0)),
                              asserta(empty_list_production(Prod0)));
                             asserta(empty_list_production(Prod0))),!,
                            He1=..[Functor2,[B,Atom1],[B1,Atom2]],
                            Bo1=..[FunctorName2,B,B1],
                            Prod1 = (He1 --> Bo1),
			    (retract(one_list_production(P1)),
                             (same(P1,Prod1),asserta(one_list_production(P1));
                              asserta(one_list_production(P1)),
                              asserta(one_list_production(Prod1)));
                             asserta(one_list_production(Prod1))),!
                             )
                         ),
			He=..[Functor2,[A|As],[A1|As1]],
			Bo=..[FunctorName2,A,A1],
			Bo2=..[Functor2,As,As1],
			Prod = (He --> Bo,Bo2),!,
                        (retract(list_production(P)),
                            (same(P,Prod),asserta(list_production(P));
                             asserta(list_production(P)),
                             asserta(list_production(Prod)));
			asserta(list_production(Prod)))
           ;
% optional = optional
           optional(Functor2),optional(Functor1),!, 
             same(FunctorName,FunctorName1),Arg=Arg1,
              ArgOpt1 =.. [Functor1,A1],
              ArgOpt2 =.. [Functor2,A2],
              He =.. [Functor2,ArgOpt1,ArgOpt2],
              ArgOpt3 =.. [FunctorName1,A1],
              ArgOpt4 =.. [FunctorName2,A2],
              Bo =.. [FunctorName2,ArgOpt3,ArgOpt4],
              Prod = (He --> Bo),!,
	      (retract(change_production(Pr)),
                           (same(Pr,Prod),     
				asserta(change_production(Prod));
                            asserta(change_production(Pr)),
                            asserta(change_production(Prod)));
                          asserta(change_production(Prod)))      
          ;
% nonterminal = nonterminallist?
          same(Functor1,FunctorName1),
          nonterminallist(Functor2,C2),!, 
          define_listatom(FunctorName2,C2,Atom2),
          same(FunctorName,FunctorName1),Arg=[Arg1,Atom2],!,
                        (single_star(C2),
      			    He0=..[Functor2,[Atom2],[Atom2]],
			    Prod0 = (He0 --> !),
	                    (retract(empty_list_production(P0)),
                  	     (same(P0,Prod0),asserta(empty_list_production(P0));
                              asserta(empty_list_production(P0)),
                              asserta(empty_list_production(Prod0)));
                             asserta(empty_list_production(Prod0))),!
                         ;
                         single_plus(C2),
                            He1=..[Functor2,[B,Atom2],[B1,Atom2]],
                            Bo1=..[FunctorName2,B,B1],
                            Prod1 = (He1 --> Bo1),
	                    (retract(one_list_production(P1)),
                  	     (same(P1,Prod1),asserta(one_list_production(P1));
                              asserta(one_list_production(P1)),
                              asserta(one_list_production(Prod1)));
                             asserta(one_list_production(Prod1))),!
                        ),
			He=..[Functor2,[A|As],[A1|As1]],
			Bo=..[FunctorName2,A,A1],
			Bo2=..[Functor2,As,As1],
			Prod = (He --> Bo,Bo2),!,
                        (retract(list_production(P)),
                            (same(P,Prod),asserta(list_production(P));
                             asserta(list_production(P)),
                             asserta(list_production(Prod)));
			asserta(list_production(Prod)))
           ;
% nonterminal = optional
           same(Functor1,FunctorName1),optional(Functor2),!,
           same(FunctorName,FunctorName1),Arg=Arg1,
             ArgOpt1 =.. [Functor1,A1],
              ArgOpt2 =.. [Functor2,A2],
              He =.. [Functor2,ArgOpt1,ArgOpt2],
              ArgOpt3 =.. [FunctorName1,A1],
              ArgOpt4 =.. [FunctorName2,A2],
              Bo =.. [FunctorName2,ArgOpt3,ArgOpt4],
              Prod = (He --> Bo),!,
	      (retract(change_production(Pr)),
                           (same(Pr,Prod),     
				asserta(change_production(Prod));
                            asserta(change_production(Pr)),
                            asserta(change_production(Prod)));
                          asserta(change_production(Prod)))  
           ;
% optional = nonterminal
           optional(Functor1),
           same(Functor2,FunctorName2),!,
            same(FunctorName,FunctorName1),Arg=Arg1,
  	      ArgOpt1 =.. [Functor1,A1],
              ArgOpt2 =.. [Functor2,A2],
              He =.. [Functor2,ArgOpt1,ArgOpt2],
              ArgOpt3 =.. [FunctorName1,A1],
              ArgOpt4 =.. [FunctorName2,A2],
              Bo =.. [FunctorName2,ArgOpt3,ArgOpt4],
              Prod = (He --> Bo),!,
	      (retract(change_production(Pr)),
                           (same(Pr,Prod),     
				asserta(change_production(Prod));
                            asserta(change_production(Pr)),
                            asserta(change_production(Prod)));
                          asserta(change_production(Prod)))
            ;            
% optional = nonterminallist?
            optional(Functor1),
            nonterminallist(Functor2,C2),!, 
            define_listatom(FunctorName2,C2,Atom2),
            same(FunctorName,FunctorName1),Arg=[Arg1,Atom2],
  	      ArgOpt1 =.. [Functor1,A1],
              ArgOpt2 =.. [Functor2,A2],
              HeC =.. [Functor2,ArgOpt1,ArgOpt2],
              ArgOpt3 =.. [FunctorName1,A1],
              ArgOpt4 =.. [FunctorName2,A2],
              BoC =.. [FunctorName2,ArgOpt3,ArgOpt4],
              ProdC = (HeC --> BoC),!,
	      (retract(change_production(Pr)),
                           (same(Pr,ProdC),     
				asserta(change_production(ProdC));
                            asserta(change_production(Pr)),
                            asserta(change_production(ProdC)));
                          asserta(change_production(ProdC))),
              (single_star(C2),
      			    He0=..[Functor2,[FunctorName1,Atom2],[Atom2]],
			    Prod0 = (He0 --> !),
			    (retract(empty_list_production(P0)),
                             (same(P0,Prod0),asserta(empty_list_production(P0));
                              asserta(empty_list_production(P0)),
                              asserta(empty_list_production(Prod0)));
                             asserta(empty_list_production(Prod0))),!
              ;
              single_plus(C2),
                            He=..[Functor2,[B],[B1,Atom2]],
                            Bo=..[FunctorName2,B,B1],
                            Prod1 = (He --> Bo),
	            	    (retract(one_list_production(P1)),
                             (same(P1,Prod1),asserta(one_list_production(P1));
                              asserta(one_list_production(P1)),
                              asserta(ona_list_production(Prod1)));
                             asserta(one_list_production(Prod1))),!
                ),
		He1=..[Functor2,[A|As],[A1|As1]],
		Bo1=..[FunctorName2,A,A1],
		Bo2=..[Functor2,As,As1],
		Prod = (He1 --> Bo1,Bo2),!,
                (retract(list_production(P)),
                 (same(P,Prod),asserta(list_production(P));
                      asserta(list_production(P)),
                      asserta(list_production(Prod)));
     	         asserta(list_production(Prod)))
                ).











