# *******************************************************************
# syndoctransform.tcl 
#
# Function:   Creates top-level windows for SYNDOC programs transform-
#             functions.
# Date:       9.11.1995
# Author:     Paula Leinonen
# *******************************************************************

#**************************************************************************
# Global variables - declarations in file  syndocglobals.tcl  
#**************************************************************************

# ******************************************************************
# Name:     sdts_modify
# Input:    String (help file)
# Output:   -
# Function: Creates top-level window for asking structure grammar
#           file and new structure grammar file
# ******************************************************************
proc sdts_modify helpfile {
    global win w browse struct
    global struct_file gram_file1 gram_file2
    global id;	# Pid of the editor process

    set id 0
    set gram_file1 $struct_file
    set gram_file2 $struct_file
    set wi iframe
    catch {destroy $w}
    toplevel $w
    dpos $w
    wm title $w "Create new structure grammar"
    wm iconname $w "Create new structure grammar"

    frame $w.$wi -bd 1m
	frame $w.$wi.i -bd 2m
	label $w.$wi.i.label_lay -width 24 \
				 -text "Old structure grammar file " \
				 -anchor w
	entry $w.$wi.i.entry_lay -relief sunken \
	    -textvariable gram_file1 
	button $w.$wi.i.browse -text $browse \
	  -command {
	       set gram_file1 [create_browser $struct $gram_file1]
	       moveto_end $w.iframe.i.entry_lay }

	frame $w.$wi.s -bd 2m
	label $w.$wi.s.label_prog -width 24 \
				  -text "New structure grammar file " \
				  -anchor w
	entry $w.$wi.s.entry_prog -relief sunken \
	    -textvariable gram_file2
	button $w.$wi.s.browse -text $browse \
	    -command { set gram_file2 [create_browser $struct $gram_file2]
	               moveto_end $w.iframe.s.entry_prog }
    
    pack $w.$wi -fill x
    pack $w.$wi.i.label_lay -side left
    pack $w.$wi.i -fill x
    pack $w.$wi.i.entry_lay $w.$wi.i.browse -side left
    pack $w.$wi.s.label_prog -side left 
    pack $w.$wi.s -fill x
    pack $w.$wi.s.entry_prog $w.$wi.s.browse -side left 
     
    button $w.go -text Go \
      -command { make_sdts_modify $gram_file1 $gram_file2 
                 moveto_end $w.iframe.s.entry_prog} -width 8

    button $w.done -text Done -command {close_window $w} -width 8


    button $w.help -text Help -command  "open_help $helpfile $w" -width 8
    pack $w.go $w.help $w.done -side left -expand yes 

    moveto_end $w.iframe.i.entry_lay
    moveto_end $w.iframe.s.entry_prog
    grab set $w
}

# *******************************************************************
# Name:      make_sdts_modify
# Input:     String (old structure grammar file)
#            String (new structure grammar file)
# Output:    -
# Function:  Checks filenames. Calls make_copy and edit_file.
# *******************************************************************
proc make_sdts_modify { t1 t2 } {
     global struct gram_file2

     set t1 [file dirname $t1]/[file tail $t1]
     if [file isfile $t1] {
	 set t2 [def_dir $t2 $struct]
	 if {[string length $t2] > 0} {
	     set gram_file2 $t2
	     make_copy $t1 $t2
	     edit_file $t2
         }
    } else {
	tk_dialog .d {} "\"$t1\" isn't a regular file!" {} -1 OK 
    }
}

# *******************************************************************
# Name:      sdtd_compare 
# Input:     String (help file)
# Output:    -
# Function:  Creates top-level window for asking structure grammar 
#            file, new structure grammar file and transformation
#            grammar file.
# *******************************************************************
proc sdts_compare helpfile {
    global win w
    global browse struct trgrams dir
    global trgram_file struct_file gram_file1 gram_file2

    set wi iframe
    catch {destroy $w}
    toplevel $w
    dpos $w
    wm title $w "Generate SDTS-transformation grammar "
    wm iconname $w "Generate SDTS-transformation grammar "

    frame $w.$wi -bd 1m
	frame $w.$wi.i -bd 2m
	label $w.$wi.i.label_doc -width 34 \
				 -text "Structure grammar file " \
				 -anchor w
	entry $w.$wi.i.entry_doc -relief sunken \
	    -textvariable gram_file1
	button $w.$wi.i.browse -text $browse \
	    -command { 
	     set gram_file1 [ create_browser $struct $gram_file1]
                       moveto_end $w.iframe.i.entry_doc }

	frame $w.$wi.s -bd 2m
	label $w.$wi.s.label_tex -width 34 \
				 -text "New structure grammar file " \
				 -anchor w
	entry $w.$wi.s.entry_tex -relief sunken \
	    -textvariable gram_file2 
	button $w.$wi.s.browse -text $browse \
	    -command { set gram_file2 [ create_browser $struct $gram_file2]
                       moveto_end $w.iframe.s.entry_tex }

	frame $w.$wi.k -bd 2m
	label $w.$wi.k.label_prog -width 34 \
				  -text "SDTS-Transformation grammar file " \
				  -anchor w
	entry $w.$wi.k.entry_prog -relief sunken \
	    -textvariable trgram_file 
	button $w.$wi.k.browse -text $browse \
	  -command { set trgram_file [ create_browser $trgrams $trgram_file]
                     moveto_end $w.iframe.k.entry_prog }
    
    pack $w.$wi -fill x
    pack $w.$wi.i.label_doc -side left 
    pack $w.$wi.i -fill x
    pack $w.$wi.i.entry_doc $w.$wi.i.browse -side left 
     
    pack $w.$wi -fill x
    pack $w.$wi.s.label_tex -side left 
    pack $w.$wi.s -fill x
    pack $w.$wi.s.entry_tex $w.$wi.s.browse -side left 
     
    pack $w.$wi -fill x
    pack $w.$wi.k.label_prog -side left 
    pack $w.$wi.k -fill x
    pack $w.$wi.k.entry_prog $w.$wi.k.browse -side left 
     
    button $w.go -text Go \
      -command {
       make_sdts_compare $gram_file1 $gram_file2 $trgram_file
       moveto_end $w.iframe.k.entry_prog } \
      -width 8

    button $w.done -text Done -command {p clean_up_compare
					destroy $w} -width 8

    button $w.help -text Help -command  "open_help $helpfile $w" -width 8
    pack $w.go $w.help $w.done -side left -expand yes 

# Default values for the entries
    if {[string length $gram_file1] == 0 } {
        set gram_file1 $struct_file }
    if {[string length $gram_file2] == 0 } {
        set gram_file2 $struct_file }
    moveto_end $w.iframe.i.entry_doc
    moveto_end $w.iframe.s.entry_tex
    moveto_end $w.iframe.k.entry_prog
    grab $w
}
# *******************************************************************
# Name:      make_sdts_compare 
# Input:     String (old structure grammar file)
#            String (new structure grammar file)
#            String (new grammar file for the transformation)
# Output:    -
# Function:  If given filenames are Ok, calls prolog-predicate 
#            transform_compare_sdts.
# *******************************************************************
proc make_sdts_compare {t1 t2 t3} {
    global trgrams trgram_file 

    set t1 [file dirname $t1]/[file tail $t1]
    if [file isfile $t1] {
        set t2 [file dirname $t2]/[file tail $t2]
        if [file isfile $t1] {
            set t3 [def_dir $t3 $trgrams]
	    if {[string length $t3] > 0} {
		 set trgram_file $t3
                 p transform_compare_sdts('$t1','$t2','$t3')
            }
        } else {
   	    tk_dialog .d {} "\"$t2\" isn't a regular file!" {} -1 OK
        }
    } else {
	tk_dialog .d {} "\"$t1\" isn't a regular file!" {} -1 OK
    }
}


# **********************************************************************
# Name:        rule_selection 
# Input:       String String Srting String Integer
# Output:      String (selected nonterm)
# Function:    Opens top-level window with list of nonterms. Returns
#              selected nonterm.
# **********************************************************************
proc rule_selection { headC oldrC newrC nonTerms RLChar winW} {
    global rulewin;		# Name of the selection window
    global selectedRule;	# Selected nonterm (return value)


    set rulewin .rulewin
    if {[winfo exists $rulewin] == 1} {
        tkwait window $rulewin
    }
    toplevel $rulewin
    foreach i $headC {
	append head [format "%c" $i]
    }
    foreach i $oldrC {
	append oldr [format "%c" $i]
    }
    foreach i $newrC {
	append newr [format "%c" $i]
    }
    wm title $rulewin $head 

    button $rulewin.close -text "No selection" -command "set selectedRule cancel"
    pack $rulewin.close -side bottom -fill x

    label $rulewin.l1 -width 20 -text "Old rule: " -anchor w
    label $rulewin.r1 -text $oldr -anchor w
    label $rulewin.l2 -width 20 -text "New rule: " -anchor w
    label $rulewin.r2 -text $newr -anchor w
    pack $rulewin.l1 $rulewin.r1 -side top -anchor w 
    pack $rulewin.l2 $rulewin.r2 -side top -anchor w

    scrollbar $rulewin.scroll -command "$rulewin.list yview"
    pack $rulewin.scroll -side right -fill y
    listbox $rulewin.list -yscroll "$rulewin.scroll set" -relief raised \
                                  -setgrid 1 -width $winW 
    pack $rulewin.list -side left -fill both -expand yes

    bind $rulewin.list <Double-Button-1> {set selectedRule [selection get]}

# Merkkijono muutetaan listaksi (koostuu "sanoista")
    set listOfRules $nonTerms
    split $listOfRules

    foreach i $listOfRules {
        $rulewin.list insert end $i
    }

   tkwait visibility $rulewin
   grab $rulewin  
   focus $rulewin

   tkwait variable selectedRule
  
   w_select_rule $selectedRule
}

# **********************************************************************
# Name:        w_select_rule
# Input:       String (selected nonterm) 
# Output:      -
# Function:    Returns selected nonterm to the prolog.
# **********************************************************************
proc w_select_rule { selectedR } {

    global rulewin;	# Name of the selection window

    destroy $rulewin

    p done('$selectedR')

}

# **********************************************************************
# Name:        open_rules
# Input:       String String
# Output:      String
# Function:    Opens top-level window with list of rules. Returns
#              selected rule to calling prolog-program.
# **********************************************************************
proc open_rules {root rulesC winWidth} {
    global rulewin;		# Name of the selection window
    global selectedRule;	# Selected rule

    set rulewin .rulewin

    if {[winfo exists $rulewin] == 1} {
        tkwait window $rulewin
    }
    toplevel $rulewin
    foreach i $root {
	append root1 [format "%c" $i]
    }
    wm title $rulewin $root1 

    button $rulewin.close -text "No selection" -command "set selectedRule cancel"
    pack $rulewin.close -side bottom -fill x

    scrollbar $rulewin.scroll -command "$rulewin.list yview"
    pack $rulewin.scroll -side right -fill y
    listbox $rulewin.list -yscroll "$rulewin.scroll set" -relief raised \
                                  -setgrid yes -width $winWidth
    pack $rulewin.list -side left -fill both -expand yes
    #wm minsize $rulewin 5 5

    bind $rulewin.list <Double-Button-1> {set selectedRule [selection get]}

    foreach i $rulesC {
	foreach j $i {
	     append rules [format "%c" $j]
        }
        $rulewin.list insert end [string trim $rules {}]
        set rules ""
    }
   set oldFocus [focus]

   tkwait visibility $rulewin
   grab $rulewin  
   focus $rulewin

   tkwait variable selectedRule
   
   c_select_rule $selectedRule $oldFocus
}

# **********************************************************************
# Name:        c_select_rule
# Input:       String (selected rule) 
# Output:      -
# Function:    Returns selected rule to the prolog.
# **********************************************************************
proc c_select_rule { selectedR oldF } {

    global rulewin w;	# Name of the selection window

    destroy $rulewin
    p selected('$selectedR')

}


# *******************************************************************
# Name:      sdtd_transform 
# Input:     String (help file)
# Output:    -
# Function:  Creates top-level window for asking transformation
#            program file, old document file and new document file
# *******************************************************************
proc sdts_transform helpfile {
     global win w
     global browse trgrams dir
     global trgram_file doc_file doc_file1 doc_file2

    set doc_file1 $doc_file
    set doc_file2 $doc_file
    set wi iframe
    catch {destroy $w}
    toplevel $w
    dpos $w
    wm title $w "Transform document using SDTS"
    wm iconname $w "Transform document using SDTS"

    frame $w.$wi -bd 1m
	frame $w.$wi.i -bd 2m
	label $w.$wi.i.label_doc -width 34 \
				 -text "SDTS-Transformation grammar file " \
				 -anchor w
	entry $w.$wi.i.entry_doc -relief sunken \
	    -textvariable trgram_file
	button $w.$wi.i.browse -text $browse \
	    -command { 
	     set trgram_file [ create_browser $trgrams $trgram_file]
             moveto_end $w.iframe.i.entry_doc }

	frame $w.$wi.s -bd 2m
	label $w.$wi.s.label_tex -width 34 \
				 -text "Old document file " \
				 -anchor w
	entry $w.$wi.s.entry_tex -relief sunken \
	    -textvariable doc_file1 
	button $w.$wi.s.browse -text $browse \
	    -command { set doc_file1 [ create_browser $docs $doc_file1]
                       moveto_end $w.iframe.s.entry_tex }

	frame $w.$wi.k -bd 2m
	label $w.$wi.k.label_gram -width 34 \
				  -text "New document file " \
				  -anchor w
	entry $w.$wi.k.entry_gram -relief sunken \
	    -textvariable doc_file2 
	button $w.$wi.k.browse -text $browse \
	  -command { set doc_file2 [ create_browser $docs $doc_file2]
                               moveto_end $w.iframe.k.entry_gram }

    pack $w.$wi -fill x
    pack $w.$wi.i.label_doc -side left 
    pack $w.$wi.i -fill x
    pack $w.$wi.i.entry_doc $w.$wi.i.browse -side left 

    pack $w.$wi -fill x
    pack $w.$wi.s.label_tex -side left 
    pack $w.$wi.s -fill x
    pack $w.$wi.s.entry_tex $w.$wi.s.browse -side left 
     
    pack $w.$wi -fill x
    pack $w.$wi.k.label_gram -side left 
    pack $w.$wi.k -fill x
    pack $w.$wi.k.entry_gram $w.$wi.k.browse -side left 
     
    button $w.go -text Go \
      -command {
       make_sdts_transform $trgram_file $doc_file1 $doc_file2
       moveto_end $w.iframe.k.entry_gram } \
      -width 8

    button $w.done -text Done -command {destroy $w} -width 8

    button $w.help -text Help -command  "open_help $helpfile $w" -width 8
    pack $w.go $w.help $w.done -side left -expand yes 

    moveto_end $w.iframe.i.entry_doc
    moveto_end $w.iframe.s.entry_tex
    moveto_end $w.iframe.k.entry_gram
    grab $w
}
# *******************************************************************
# Name:      make_sdts_transform 
#              Input:     String (transformation program file)
# Input:     String (transformation grammar file)
#            String (old document file)
#            String (new document file)
# Output:    -
# Function:  If given filenames are Ok, calls prolog-predicate 
#            transform_sdts.
# *******************************************************************
proc make_sdts_transform {t1 t2 t3} {
    global docs doc_file2 
    global trprogs trprog_file 

    set t1 [file dirname $t1]/[file tail $t1]
    if [file isfile $t1] {
        set t2 [file dirname $t2]/[file tail $t2]
        if [file isfile $t1] {
            set t3 [def_dir $t3 $docs]
# Name for the transformation program file (variable t4):
            set t4 [file tail $t1].pl
            set t4 [def_dir $t4 $trprogs]
	    if {[string length $t3] > 0} {
		 set doc_file2 $t3
                 p transform_sdts('$t1','$t2','$t3','$t4')
            }
        } else {
   	    tk_dialog .d {} "\"$t2\" isn't a regular file!" {} -1 OK
        }
    } else {
	tk_dialog .d {} "\"$t1\" isn't a regular file!" {} -1 OK
    }
}









