/* 
*----------------------------------------------------------------------------
* syndoctransform1.pl
*----------------------------------------------------------------------------
*/

/*
 *----------------------------------------------------------------------------
 *    transform_compare_transducer
 *----------------------------------------------------------------------------
*/

%Pred:transform_compare_transducer
transform_compare_transducer(Old, New, Trans) :-
	    compare_grammars_transducer(Old,New,Trans),            
	    !.


/*
************************************************************************
*
*  compare_grammars_transducer            
*
***********************************************************************
*/


%Pred:compare_grammars_transducer            
compare_grammars_transducer(_,_,_):-true.            



/*
 *----------------------------------------------------------------------------
 *    transform_transducer
 *----------------------------------------------------------------------------
*/

%Pred:transform_transducer(+TransformGram,+DocFile,+NewDocFile)
transform_transducer(TransformGram,DocFile,NewDocFile,TransformProg) :-
            generateTransform_transducer(TransformGram,TransformProg),
	    % luetaan transformaatio-ohjelman alusta tiedot,
	    see(TransformProg),
            % transformaatiokieliopin nimi
	    read(_),
            % vanha kielioppi
	    read(OldGram),
            % uusi kielioppi
            read(NewGram),
            % tyyppi ja tekstityyppi
            read(TrType),
	    read(TrText),
            % alukutila, jonka ymparilta poistetaan sulut
            read(Ini),
            Ini=..[initials,InitialStates],
	    seen,
            % luetaan dokumenttia
	    see(DocFile),
            % kielioppi
	    read(Gram),!,
	    check_grams(OldGram,Gram),
            % tyyppi ja tekstityyppi
	    read(Type),
	    read(Text),
            % dokumentti
	    read(Tree),
	    seen,!,
            % konsultoidaan ohjelma Prolog-tietokantaan
            % samanlainen kuin Sicstus Prolog, 
            %pyyhkii entiset predikaatit            
            oconsult(TransformProg),
            formAndRunGoal_tr(InitialStates,Tree,NewTree),
            % kirjoitetaan uuteen kielioppiin
	    tell(NewDocFile),!,
	    writeq(NewGram),write('.'),nl,
	    writeq(Type),write('.'),nl,
	    writeq(Text),write('.'),nl,
	    writeq(NewTree),write('.'),nl,
	    told,
            call_tcl(open_info("Document has been transformed!")),
	    tcl_in(call_prolog(ok)),
	    !.


%Pred:formAndRunGoal
formAndRunGoal_tr([],_,[]).
formAndRunGoal_tr([InSt|InSts],Tree,NewTree):-
	    Goal =..[InSt,Tree,NewTree],
	    (call(Goal);
             formAndRunGoal_tr(InSts,Tree,NewTree)).
        


/*
************************************************************************
*
*  generateTransform_transducer
*
***********************************************************************
*/

%Pred:generateTransform_transducer(+File,+File)
generateTransform_transducer(TransducerGramFile,TransformProg):-
        	call_tcl(give_name(TransducerGramFile)),
	        tcl_in(call_prolog(done(TransducerGramFileName))),
                % luetaan transformaatiokielioppia
	        see(TransducerGramFile),
                % vanha rakennekielioppi
 		read(OldGram),
                % uusi rakennekielioppi
                read(NewGram),
                % tyyppia ja tekstityyppi
		read(Type),
                read(Text),
                % tilat, assertoidaan Prolog tietokantaan
                read(States),
                assert(states(States)),
                % alkutila(t)
                read(InitialStates),
                Ini=..[initials,InitialStates],
                %kirjoitetaan ohjelmatiedostoon
		tell(TransformProg),!,
		writeq(TransducerGramFileName),write('.'),nl,
	        writeq(OldGram),write('.'),nl,
	        writeq(NewGram),write('.'),nl,
	        writeq(Type),write('.'),nl,
	        writeq(Text),write('.'),nl,
                write(Ini),write('.'),nl,
                %luetaan muunnoskielioppitiedostosta saanto kerallaan
		repeat,
		termInput_trd(Termlist),
		process_trd(Termlist,TransformProg),
		seen,told,
		cleanup_transform_tr.


%Pred:cleanup_transform_tr
cleanup_transform_tr:-
        retractall(states(_)).

%Pred:termInput_trd
termInput_trd(Termlist):-
	        ruleInput(RuleChar),!,
		formTermlist_trd(RuleChar,Termlist).

%Pred:formTermlist_trd
formTermlist_trd(RuleChar,[Word|Textlist]):-
	analyse_word_trd(RuleChar,Word,RestRule),!,
	restwords_trd(Word,RestRule,Textlist).

%Pred:restwords_trd
restwords_trd(_,[],[]):-!.
restwords_trd(empty_line,_,[]):-!.
restwords_trd(end_of_file,_,[]):-!.
%restwords(full_stop,_,[]):-!.
restwords_trd(_,Rule,[Word1|Textlist]):-analyse_word_trd(Rule,Word1,Rest),!,
        restwords_trd(Word1,Rest,Textlist).

%Pred:s_integer
s_integer(C):-C>47,C<59.

%Pred:letter_or_integer
letter_or_integer(C):-lc_letter(C);uc_letter(C);s_integer(C).

%Pred:letter_or_or_integer
letter_or_or_integer(C):-lc_letter(C);uc_letter(C);underscore(C);s_integer(C).

%Pred:analyse_word_trd
analyse_word_trd([empty_line],empty_line,[]):-!.
analyse_word_trd([end_of_file],end_of_file,[]):-!.
analyse_word_trd([C|P],state(A),P1):-
	        lc_letter(C),rest_atom(P,Cs,P1,letter_or_integer),
                append(_,[C1],Cs),
                s_integer(C1),
		name(A,[C|Cs]).
analyse_word_trd([C|P],nonterminal(A),P1):-
	        lc_letter(C),rest_atom(P,Cs,P1,letter_or_),
		uc_to_lc([C|Cs],NT),name(A,NT).
analyse_word_trd([C|P],variable(A),P1):-
	        uc_letter(C),rest_atom(P,Cs,P1,letter_or_or_integer),
		name(A,[C|Cs]).
analyse_word_trd([C1|P],variablelist(A),P1):-
	        left_parenthesis(C1),rest_atom(P,Cs,[C2,C3|P1],
		                           letter_or_or_integer),
                append(_,[C4],Cs),
		s_integer(C4),                
		right_parenthesis(C2),
		(single_star(C3);single_plus(C3)),
		append(Cs,[C3],L1),	
	        name(A,L1).
analyse_word_trd([C1|P],optionalvariable(A),P1):-
	        left_bracket(C1),rest_atom(P,Cs,[C3|P1],
                                                letter_or_or_integer),
                append(_,[C2],Cs),
		s_integer(C2),
                right_bracket(C3),
		append([C1|Cs],[C3],O),
		name(A,O).
analyse_word_trd([C1|P],listnonterminal(A),P1):-
	        left_parenthesis(C1),rest_atom(P,Cs,[C2,C3|P1],
		                           letter_or_),
		right_parenthesis(C2),
		(single_star(C3);single_plus(C3)),
		append([39|Cs],[C3,39],L1),	
	        name(A,L1).
analyse_word_trd([C1|P],optionalnonterminal(A),P1):-
	        left_bracket(C1),rest_atom(P,Cs,[C2|P1],
                                                letter_or_),
		right_bracket(C2),
%		append([39,C1|Cs],[C2,39],O),
		append([C1|Cs],[C2],O),
		name(A,O).
analyse_word_trd([C,C1|Cs],'-->',Cs):-
	        hyphen(C),greater_than(C1),!.
analyse_word_trd([C|Cs],',',Cs):- comma(C),!.
analyse_word_trd([C|Cs],')',Cs):- right_parenthesis(C),!.
analyse_word_trd([C|Cs],'(',Cs):- left_parenthesis(C),!.
analyse_word_trd([C1,C2|Cs],empty_word,Cs):-double_quote(C1,C2),!.
analyse_word_trd([C|Cs],full_stop,Cs):-period(C),!.
analyse_word_trd([_|P1],W,P2):-analyse_word_trd(P1,W,P2).



%Pred:process_trd
process_trd([end_of_file],_):-!.
process_trd([empty_line],_):-!,fail.
process_trd(Termlist,TransformProg):-
	        grammar_rule_trd(Production,Termlist,[]),
                 write_lists_trd(TransformProg),
                 write_optionals_trd(TransformProg),
		 writeterm(TransformProg,Production),!,
                fail.


%Pred:write_lists_trd
write_lists_trd(OutputProg):-retract(empty_list_production(Prod0)),
			      writeterm(OutputProg,Prod0),
			      retract(list_production(Prod)),
			       writeterm(OutputProg,Prod)
			     ,!, 
			 write_lists_trd(OutputProg).
write_lists_trd(_).

%Pred:write_optionals_trd
write_optionals_trd(OutputProg):-
			      retract(optional_production(Prod)),
			       writeterm(OutputProg,Prod),
                              retract(atomic_optional_production(Prod0)),
			       writeterm(OutputProg,Prod0)
			     ,!, 
			 write_optionals_trd(OutputProg).
write_optionals_trd(_).




%Pred:grammar_rule_trd
grammar_rule_trd(Production) --> [state(State)],['('],
                [nonterminal(Root1)],['('],
                trd_tree1(Subtrees1,_),[')'],[')'],['-->'],[nonterminal(Root2)],
                ['('],trd_tree2(Subtrees2,StatesVars),[')'],
		{append([Root1],Subtrees1,RootSubtrees1),
                 Pattern1 =.. RootSubtrees1,
                 append([Root2],Subtrees2,RootSubtrees2),
                 Pattern2 =.. RootSubtrees2,
                 Head =.. [State,Pattern1,Pattern2],
                 (same(StatesVars,[]),

                  same(Production,Head); 
%                 flatten1(StatesVars,StatesVars1),
                 flatten(StatesVars,StatesVars1),
                  body_trd(StatesVars1,Body),
                 Production = (Head :- Body))}.
 
%Pred:trd_tree1
trd_tree1(Subtrees,Vars) -->
	        tree_item1(I_subtree,I_var),[','],
                trd_tree1(I_subtrees,I_vars),
		{append(I_var,I_vars,Vars),
	        append(I_subtree,I_subtrees,Subtrees)}.
trd_tree1(Subtree,Var) --> tree_item1(Subtree,Var).

%Pred:tree_item1
tree_item1([Tree],[Vars]) --> [nonterminal(Root)],['('],
                              trd_tree1(Subtrees,Vars),[')'],
                          {append([Root],Subtrees,RootSubtrees),
                           Tree =.. RootSubtrees}.
tree_item1([Tree],[Vars]) --> [optionalnonterminal(Root)],['('],
                             trd_tree1(Subtrees,Vars),[')'],
                            {name(Root,RootChar),
                             append([39|RootChar],[39],Root1Char),
                             name(Root1,Root1Char), 
                             append([Root1],Subtrees,RootSubtrees),
                            Tree =.. RootSubtrees}.
tree_item1([Var1],[Var]) --> [optionalvariable(Var)],
                            {name(Var,VarChar),
                             append([91|Var1Char],[93],VarChar),
                             name(Var1,Var1Char)}.
tree_item1([Var],[Var]) --> [variable(Var)].
tree_item1([Tree],[VarList]) --> [variablelist(VarList)],
                       {!,name(VarList,ListChar),
                       (append(TreeChar,[42],ListChar);
                              append(TreeChar,[43],ListChar)),
                        name(Tree,TreeChar)}.
tree_item1([Leaf],[]) --> [nonterminal(Leaf)].
%tree_item1([Leaf],[]) --> [terminal(Leaf)].
 
%Pred:trd_tree2
trd_tree2(Subtrees,StatesVars) -->
	        tree_item2(I_subtree,I_state_var),[','],
                trd_tree2(I_subtrees,I_state_vars),
%		{flatten1(I_state_var,I_state_var1),
		{flatten(I_state_var,I_state_var1),
                 append(I_state_var1,I_state_vars,StatesVars),
	        append(I_subtree,I_subtrees,Subtrees)}.
trd_tree2(Subtree,StateVar) --> tree_item2(Subtree,StateVar).

%Pred:tree_item2
tree_item2([Tree],[StatesVars]) --> [nonterminal(Root)],['('],
                              trd_tree2(Subtrees,StatesVars),[')'],
                          {append([Root],Subtrees,RootSubtrees),
                           Tree =.. RootSubtrees}.
tree_item2([Tree],[StatesVars]) --> [optionalnonterminal(Root)],['('],
                      trd_tree2(Subtrees,StatesVars),[')'],
                     {name(Root,RootChar),
                             append([39|RootChar],[39],Root1Char),
                             name(Root1,Root1Char), 
                      append([Root1],Subtrees,RootSubtrees),
                      Tree =.. RootSubtrees}.
tree_item2([Var1],[StateVar]) --> [state(State)],['('],[variable(Var)],[')'],
                            {!,
                             StateVar =.. [State,Var,Var1]}.
tree_item2([Var1],[StateVarList]) --> [state(State)],['('],
                                         [variablelist(VarList)],[')'],
                       {!,
                         StateVarList =.. [State,VarList,Var1]}.
tree_item2([Var1],[StateVarList]) --> [state(State)],['('],
                                         [optionalvariable(Var)],[')'],
                       {!,
                         StateVarList =.. [State,Var,Var1]}.
tree_item2([Leaf],[]) --> [nonterminal(Leaf)].
tree_item2([Leaf],[]) --> [optionalnonterminal(Leaf)].
tree_item2([Leaf],[]) --> [listnonterminal(Leaf)].
tree_item2([Var],[]) --> [variable(Var)].
%tree_item2([Leaf],[]) --> [terminal(Leaf)].


%Pred:body_trd
body_trd([StateVar],Body) :- 
                  StateVar =..[State,V1,V2], 
                  name(V1,V1Char),
                 (% muuttujat listoja
                   (member(42,V1Char),append(V11Char,[42],V1Char),same(ListType,42),
                    name(V11,V11Char)
                   ;
                   member(43,V1Char),append(V11Char,[43],V1Char),same(ListType,43),
                   name(V11,V11Char)
                   ),
                   B1 =..[State,V11,V2],
                   Body =.. [list,B1],
                        defineListEnd(State,ListType,A0),
                        Arg0 =..[State,[_],[A0]],
			He0=..[list,Arg0],
			same(He0,Prod0),!,
                        (retract(empty_list_production(P0)),
                            (same(P0,Prod0),asserta(empty_list_production(P0));
                             asserta(empty_list_production(P0)),
                             asserta(empty_list_production(Prod0)));
			asserta(empty_list_production(Prod0))),
                         
			Arg =.. [State,[A|As],[A1|As1]],
			He=..[list,Arg],
                        Bo=..[State,A,A1],
                        Arg1 =..[State,As,As1],
			Bo2=..[list,Arg1],
			Prod = (He :- Bo,Bo2),!,
                        (retract(list_production(P)),
                            (same(P,Prod),asserta(list_production(P));
                             asserta(list_production(P)),
                             asserta(list_production(Prod)));
			asserta(list_production(Prod)))
                 ;
                 %muuttujat valinnaisia
                  member(91,V1Char), append([91|V11Char],[93],V1Char),
                   name(V11,V11Char),
                   B1 =..[State,V11,V2],
                   Body =..[opt,B1],
                        defineOptional(State,A0,A0plain),                
                        Arg0 =..[State,_,A0],
			He0=..[opt,Arg0],
			same(He0,Prod0),!,
                        (retract(atomic_optional_production(P0)),
                        (same(P0,Prod0),asserta(atomic_optional_production(P0));
                             asserta(atomic_optional_production(P0)),
                             asserta(atomic_optional_production(Prod0)));
			asserta(atomic_optional_production(Prod0))),
                        A =..[A0,X],A1=..[A0,X1], 
			Arg =.. [State,A,A1],
			He=..[opt,Arg],
                        As =.. [A0plain,X],As1 =.. [A0plain,X1], 
                        Bo =..[State,As,As1],
			Prod = (He :- Bo),!,
                        (retract(optional_production(P)),
                            (same(P,Prod),asserta(optional_production(P));
                             asserta(optional_production(P)),
                             asserta(optional_production(Prod)));
			asserta(optional_production(Prod)))
                 ;             
                   Body =..[State,V1,V2]).
body_trd([StateVar|SVs],Body) :-   
                  body_trd(SVs,Body1),
                  StateVar=..[State,V1,V2],
                  name(V1,V1Char),
                 (%muuttujat listoja
                  (member(42,V1Char),append(V11Char,[42],V1Char),same(ListType,42),
                    name(V11,V11Char)
                   ;
                   member(43,V1Char),append(V11Char,[43],V1Char),same(ListType,42),
                   name(V11,V11Char)
                   ),
                   Bi1 =..[State,V11,V2],
                   Body_item =.. [list,Bi1],
                        defineListEnd(State,ListType,A0),
                        Arg0 =..[State,[_],[A0]],
%                        Arg0 =..[State,[A0],[A0]],
			He0=..[list,Arg0],
			same(He0,Prod0),!,
                        (retract(empty_list_production(P0)),
                            (same(P0,Prod0),asserta(empty_list_production(P0));
                             asserta(empty_list_production(P0)),
                             asserta(empty_list_production(Prod0)));
			asserta(empty_list_production(Prod0))),
                         
			Arg =.. [State,[A|As],[A1|As1]],
			He=..[list,Arg],
                        Bo=..[State,A,A1],
                        Arg1 =..[State,As,As1],
			Bo2=..[list,Arg1],
			Prod = (He :- Bo,Bo2),!,
                        (retract(list_production(P)),
                            (same(P,Prod),asserta(list_production(P));
                             asserta(list_production(P)),
                             asserta(list_production(Prod)));
			asserta(list_production(Prod)))
                  ;
                 %muuttujat valinnaisia
                  member(91,V1Char), append([91|V11Char],[93],V1Char),
                   name(V11,V11Char),
                   Bi1 =..[State,V11,V2],
                   Body_item =..[opt,Bi1],
                        defineOptional(State,A0,A0plain),                
                        Arg0 =..[State,_,A0],
			He0=..[opt,Arg0],
			same(He0,Prod0),!,
                        (retract(atomic_optional_production(P0)),
                        (same(P0,Prod0),asserta(atomic_optional_production(P0));
                             asserta(atomic_optional_production(P0)),
                             asserta(atomic_optional_production(Prod0)));
			asserta(atomic_optional_production(Prod0))),
                        A =..[A0,X],A1=..[A0,X1], 
			Arg =.. [State,A,A1],
			He=..[opt,Arg],
                        As =.. [A0plain,X],As1 =.. [A0plain,X1], 
                        Bo =..[State,As,As1],
			Prod = (He :- Bo),!,
                        (retract(optional_production(P)),
                            (same(P,Prod),asserta(optional_production(P));
                             asserta(optional_production(P)),
                             asserta(optional_production(Prod)));
			asserta(optional_production(Prod)))
                 ;             
                  Body_item =.. [State,V1,V2]),
                  Body = (Body_item,Body1).

%Pred:defineListEnd
defineListEnd(State,ListType,End) :-
                  states(List),
                  searchNonterminalName(State,List,NonT),
                  name(NonT,NonTChar),
                  append([39|NonTChar],[ListType,39],NonT1Char),
                  name(End,NonT1Char).

%Pred:defineOptional
defineOptional(State,Opt,NonT) :-
                  states(List),
                  searchNonterminalName(State,List,NonT),
                  name(NonT,NonTChar),
                  append([39,91|NonTChar],[93,39],NonT1Char),
                  name(Opt,NonT1Char).

%Pred:searchNonterminalName
searchNonterminalName(State,[State,N],N):-!.
searchNonterminalName(State,[S,N|States],N1):-
                  (same(State,S),same(N,N1);
                  searchNonterminalName(State,States,N1)).






















