# *******************************************************************
# syndoctransform1.tcl 
#
# Function:   Creates top-level windows for SYNDOC programs transform
#             transducer-functions.
# Date:       9.11.1995
# Author:     Paula Leinonen
# *******************************************************************

#**************************************************************************
# Global variables - declarations in file  syndocglobals.tcl  
#**************************************************************************

# ******************************************************************
# Name:     transducer_modify
# Input:    String (help file)
# Output:   -
# Function: Creates top-level window for asking structure grammar
#           file and new structure grammar file
# ******************************************************************
proc transducer_modify helpfile {
    global win w browse struct
    global struct_file gram_file1 gram_file2
    global id

    set id 0
    set gram_file1 $struct_file
    set gram_file2 $struct_file
    set wi iframe
    catch {destroy $w}
    toplevel $w
    dpos $w
    wm title $w "Create new structure grammar"
    wm iconname $w "Create new structure grammar"

    frame $w.$wi -bd 1m
	frame $w.$wi.i -bd 2m
	label $w.$wi.i.label_lay -width 24 \
				 -text "Old structure grammar file " \
				 -anchor w
	entry $w.$wi.i.entry_lay -relief sunken \
	    -textvariable gram_file1 
	button $w.$wi.i.browse -text $browse \
	  -command {
	       set gram_file1 [create_browser $struct $gram_file1]
	       moveto_end $w.iframe.i.entry_lay }

	frame $w.$wi.s -bd 2m
	label $w.$wi.s.label_prog -width 24 \
				  -text "New structure grammar file " \
				  -anchor w
	entry $w.$wi.s.entry_prog -relief sunken \
	    -textvariable gram_file2
	button $w.$wi.s.browse -text $browse \
	    -command { set gram_file2 [create_browser $struct $gram_file2]
	               moveto_end $w.iframe.s.entry_prog }
    
    pack $w.$wi -fill x
    pack $w.$wi.i.label_lay -side left
    pack $w.$wi.i -fill x
    pack $w.$wi.i.entry_lay $w.$wi.i.browse -side left
    pack $w.$wi.s.label_prog -side left 
    pack $w.$wi.s -fill x
    pack $w.$wi.s.entry_prog $w.$wi.s.browse -side left 
     
    button $w.go -text Go \
      -command { make_transducer_modify $gram_file1 $gram_file2
                moveto_end $w.iframe.s.entry_prog } -width 8

    button $w.done -text Done -command {close_window $w} -width 8

    button $w.help -text Help -command  "open_help $helpfile $w" -width 8
    pack $w.go $w.help $w.done -side left -expand yes 

    moveto_end $w.iframe.i.entry_lay
    moveto_end $w.iframe.s.entry_prog
    grab set $w
}

# *******************************************************************
# Name:      make_transducer_modify
# Input:     String (old structure grammar file)
#            String (new structure grammar file)
# Output:    -
# Function:  Checks filenames. Calls make_copy and edit_file.
# *******************************************************************
proc make_transducer_modify { t1 t2 } {
     global struct gram_file2

     set t1 [file dirname $t1]/[file tail $t1]
     if [file isfile $t1] {
	 set t2 [def_dir $t2 $struct]
	 if {[string length $t2] > 0} {
	     set gram_file2 $t2
	     make_copy $t1 $t2
	     edit_file $t2
         }
    } else {
	tk_dialog .d {} "\"$t1\" isn't a regular file!" {} -1 OK 
    }
}

# *******************************************************************
# Name:      transducer_compare 
# Input:     String (help file)
# Output:    -
# Function:  Creates top-level window for asking structure grammar 
#            file, new structure grammar file and transformation
#            grammar file.
# *******************************************************************
proc transducer_compare helpfile {
    global win w
    global browse struct trgrams dir
    global trgram_file struct_file gram_file1 gram_file2

    set wi iframe
    catch {destroy $w}
    toplevel $w
    dpos $w
    wm title $w "Generate transducer-transformation grammar "
    wm iconname $w "Generate transducer-transformation grammar "

    frame $w.$wi -bd 1m
	frame $w.$wi.i -bd 2m
	label $w.$wi.i.label_doc -width 38 \
				 -text "Structure grammar file " \
				 -anchor w
	entry $w.$wi.i.entry_doc -relief sunken \
	    -textvariable gram_file1
	button $w.$wi.i.browse -text $browse \
	    -command { 
	     set gram_file1 [ create_browser $struct $gram_file1]
                       moveto_end $w.iframe.i.entry_doc }

	frame $w.$wi.s -bd 2m
	label $w.$wi.s.label_tex -width 38 \
				 -text "New structure grammar file " \
				 -anchor w
	entry $w.$wi.s.entry_tex -relief sunken \
	    -textvariable gram_file2 
	button $w.$wi.s.browse -text $browse \
	    -command { set gram_file2 [ create_browser $struct $gram_file2]
                       moveto_end $w.iframe.s.entry_tex }

	frame $w.$wi.k -bd 2m
	label $w.$wi.k.label_prog -width 38 \
				  -text "Transducer-Transformation grammar file " \
				  -anchor w
	entry $w.$wi.k.entry_prog -relief sunken \
	    -textvariable trgram_file 
	button $w.$wi.k.browse -text $browse \
	  -command { set trgram_file [ create_browser $trgrams $trgram_file]
                     moveto_end $w.iframe.k.entry_prog }
    
    pack $w.$wi -fill x
    pack $w.$wi.i.label_doc -side left 
    pack $w.$wi.i -fill x
    pack $w.$wi.i.entry_doc $w.$wi.i.browse -side left 
     
    pack $w.$wi -fill x
    pack $w.$wi.s.label_tex -side left 
    pack $w.$wi.s -fill x
    pack $w.$wi.s.entry_tex $w.$wi.s.browse -side left 
     
    pack $w.$wi -fill x
    pack $w.$wi.k.label_prog -side left 
    pack $w.$wi.k -fill x
    pack $w.$wi.k.entry_prog $w.$wi.k.browse -side left 
     
    button $w.go -text Go \
      -command {
       $w configure -cursor watch 
       make_transducer_compare $gram_file1 $gram_file2 $trgram_file 
       moveto_end $w.iframe.k.entry_prog
       $w configure -cursor arrow} \
      -width 8

    button $w.done -text Done -command {destroy $w} -width 8

    button $w.help -text Help -command  "open_help $helpfile $w" -width 8
    pack $w.go $w.help $w.done -side left -expand yes 

# Default values for the grammar files
    if {![file isfile $gram_file1]} { 
        set gram_file1 $struct_file }
    if {![file isfile $gram_file2]} { 
        set gram_file2 $struct_file }
    moveto_end $w.iframe.i.entry_doc
    moveto_end $w.iframe.s.entry_tex
    moveto_end $w.iframe.k.entry_prog
    grab $w
}
# *******************************************************************
# Name:      make_transducer_compare 
# Input:     String (old structure grammar file)
#            String (new structure grammar file)
#            String (new grammar file for the transformation)
# Output:    -
# Function:  If given filenames are Ok, calls prolog-predicate 
#            transform_compare_transducer.
# *******************************************************************
proc make_transducer_compare {t1 t2 t3} {
    global trgrams trgram_file 

    set t1 [file dirname $t1]/[file tail $t1]
    if [file isfile $t1] {
        set t2 [file dirname $t2]/[file tail $t2]
        if [file isfile $t1] {
            set t3 [def_dir $t3 $trgrams]
	    if {[string length $t3] > 0} {
		 set trgram_file $t3
                 p transform_compare_transducer('$t1','$t2','$t3')
            }
        } else {
   	    tk_dialog .d {} "\"$t2\" isn't a regular file!" {} -1 OK
        }
    } else {
	tk_dialog .d {} "\"$t1\" isn't a regular file!" {} -1 OK
    }
}

# ******************************************************************
# Name:     transducer_generate
# Input:    String (help file)
# Output:   -
# Function: Creates top-level window for asking transformation grammar
#           file and transformation program file
# ******************************************************************
proc transducer_generate helpfile {
    global win w browse trgrams trprogs
    global trgram_file trprog_file

    set wi iframe
    catch {destroy $w}
    toplevel $w
    dpos $w
    wm title $w "Generate transducer-transformation program"
    wm iconname $w "Generate transducer-transformation program"

    frame $w.$wi -bd 1m
	frame $w.$wi.i -bd 2m
	label $w.$wi.i.label_lay -width 38 \
				 -text "Transducer-Transformation grammar file " \
				 -anchor w
	entry $w.$wi.i.entry_lay -relief sunken \
	    -textvariable trgram_file 
	button $w.$wi.i.browse -text $browse \
	  -command {
	       set trgram_file [create_browser $trgrams $trgram_file]
	       moveto_end $w.iframe.i.entry_lay }

	frame $w.$wi.s -bd 2m
	label $w.$wi.s.label_prog -width 38 \
				  -text "Transducer-Transformation program file " \
				  -anchor w
	entry $w.$wi.s.entry_prog -relief sunken \
	    -textvariable trprog_file 
	button $w.$wi.s.browse -text $browse \
	    -command { 
	      set trprog_file [create_browser $trprogs $trprog_file]
	       moveto_end $w.iframe.s.entry_prog }
    
    pack $w.$wi -fill x
    pack $w.$wi.i.label_lay -side left
    pack $w.$wi.i -fill x
    pack $w.$wi.i.entry_lay $w.$wi.i.browse -side left
    pack $w.$wi.s.label_prog -side left 
    pack $w.$wi.s -fill x
    pack $w.$wi.s.entry_prog $w.$wi.s.browse -side left 
     
    button $w.go -text Go \
      -command { make_transducer_generate $trgram_file $trprog_file
                  moveto_end $w.iframe.s.entry_prog } \
      -width 8

    button $w.done -text Done -command {destroy $w} -width 8

    button $w.help -text Help -command  "open_help $helpfile $w" -width 8
    pack $w.go $w.help $w.done -side left -expand yes 

    moveto_end $w.iframe.i.entry_lay
    moveto_end $w.iframe.s.entry_prog
    grab set $w

}
# *******************************************************************
# Name:      make_transducer_generate  
# Input:     String (transformation grammar file)
#            String (transformation program file)
# Output:    -
# Function:  If given files are ok, calls prolog-predicate      
#            transform_generate_transducer
# *******************************************************************
proc make_transducer_generate {t1 t2} {
    global trprogs trprog_file
    set t1 [file dirname $t1]/[file tail $t1]
    if [file isfile $t1] {
	 set t2 [def_dir $t2 $trprogs]
	 if {[string length $t2] > 0} {
	     set trprog_file $t2
	     p transform_generate_transducer('$t1','$t2')
         }
    } else {
	tk_dialog .d {} "\"$t1\" isn't a regular file!" {} -1 OK 
    }

}

# *******************************************************************
# Name:      transducer_transform 
# Input:     String (help file)
# Output:    -
# Function:  Creates top-level window for asking transformation
#            program file, old document file and new document file
# *******************************************************************
proc transducer_transform helpfile {
    global win w
    global browse trgrams dir
    global trgram_file doc_file doc_file1 doc_file2

    set doc_file1 $doc_file
    set doc_file2 $doc_file
    set wi iframe
    catch {destroy $w}
    toplevel $w
    dpos $w
    wm title $w "Transform document using transducer"
    wm iconname $w "Transform document using transducer"

    frame $w.$wi -bd 1m
	frame $w.$wi.i -bd 2m
	label $w.$wi.i.label_doc -width 38 \
				 -text "Transducer-Transformation grammar file " \
				 -anchor w
	entry $w.$wi.i.entry_doc -relief sunken \
	    -textvariable trgram_file
	button $w.$wi.i.browse -text $browse \
	    -command { 
	     set trgram_file [ create_browser $trgrams $trgram_file]
             moveto_end $w.iframe.i.entry_doc }

	frame $w.$wi.s -bd 2m
	label $w.$wi.s.label_tex -width 38 \
				 -text "Old document file " \
				 -anchor w
	entry $w.$wi.s.entry_tex -relief sunken \
	    -textvariable doc_file1 
	button $w.$wi.s.browse -text $browse \
	    -command { set doc_file1 [ create_browser $docs $doc_file1]
                       moveto_end $w.iframe.s.entry_tex }

	frame $w.$wi.k -bd 2m
	label $w.$wi.k.label_gram -width 38 \
				  -text "New document file " \
				  -anchor w
	entry $w.$wi.k.entry_gram -relief sunken \
	    -textvariable doc_file2 
	button $w.$wi.k.browse -text $browse \
	  -command { set doc_file2 [ create_browser $docs $doc_file2]
                     moveto_end $w.iframe.k.entry_gram }

    pack $w.$wi -fill x
    pack $w.$wi.i.label_doc -side left 
    pack $w.$wi.i -fill x
    pack $w.$wi.i.entry_doc $w.$wi.i.browse -side left 

    pack $w.$wi -fill x
    pack $w.$wi.s.label_tex -side left 
    pack $w.$wi.s -fill x
    pack $w.$wi.s.entry_tex $w.$wi.s.browse -side left 
     
    pack $w.$wi -fill x
    pack $w.$wi.k.label_gram -side left 
    pack $w.$wi.k -fill x
    pack $w.$wi.k.entry_gram $w.$wi.k.browse -side left 
     
    button $w.go -text Go \
      -command {
       make_transducer_transform $trgram_file $doc_file1 $doc_file2 } \
      -width 8

    button $w.done -text Done -command {p cleanup_transform_tr
                                        destroy $w} -width 8

    button $w.help -text Help -command  "open_help $helpfile $w" -width 8
    pack $w.go $w.help $w.done -side left -expand yes 

    moveto_end $w.iframe.i.entry_doc
    moveto_end $w.iframe.s.entry_tex
    moveto_end $w.iframe.k.entry_gram
    grab $w
}
# *******************************************************************
# Name:      make_transducer_transform 
# Input:     String (transformation grammar file)
#            String (old document file)
#            String (new document file)
# Output:    -
# Function:  If given filenames are Ok, calls prolog-predicate 
#            transform_transducer.
# *******************************************************************
proc make_transducer_transform {t1 t2 t3} {
    global docs doc_file2 
    global trprogs

    set t1 [file dirname $t1]/[file tail $t1]
    if [file isfile $t1] {
        set t2 [file dirname $t2]/[file tail $t2]
        if [file isfile $t1] {
            set t3 [def_dir $t3 $docs]
	    if {[string length $t3] > 0} {
		 set doc_file2 $t3
# Name for the transformation program file (variable t4):
                 set t4 [file tail $t1].pl
                 set t4 [def_dir $t4 $trprogs]
                 p transform_transducer('$t1','$t2','$t3','$t4')
            }
        } else {
   	    tk_dialog .d {} "\"$t2\" isn't a regular file!" {} -1 OK
        }
    } else {
	tk_dialog .d {} "\"$t1\" isn't a regular file!" {} -1 OK
    }
}





















