# ************************************************************************
# syndoctree.tcl
# 
# Function:   Creates a top-level window and loads a document
# Date:       1.10.1995
# Author:     Paula Leinonen
#**************************************************************************

#**************************************************************************
# Global variables - declarations in file  syndocglobals.tcl  
#**************************************************************************

    set wi .win;		# Name of the window 
    set treehelp_file tree.hlp; # Name of the helpfile 
    set rulewin .rulewin; # window for selecting rule 
    set e .editor;        # text-widget

# ************************************************************************
# Name:       make_tree
# Input:      String (parse tree in ascii form) 
#	      String (title of the window in ascii form)
# Output:     -
# Function:   Creates text-widget for the document in the tree-mode.
# ************************************************************************
proc make_tree {treeChar document} {
     global wi wf
     global treehelp_file
     global SYNDOCHOME;	# Path for syndoc bitmap files
     			# (syndoc_arrowup, syndoc_arrowdown)
     global docText;	# Parse tree in the text widget (in ASCII)
     global rowNum;	# Number of the current row in the text 
			# of the text widget

    set docText $treeChar
    set rowNum 1
    catch {destroy $wi}
    toplevel $wi
    dpos $wi
    wm title $wi $document
    wm iconname $wi $document

    set wf iframe
    frame $wi.$wf -borderwidth 10
    pack $wi.$wf -side top -fill both -anchor nw -expand yes

# Button-frames:
    frame $wi.$wf.b1 -borderwidth 2 -relief sunken 
    frame $wi.$wf.b2 -borderwidth 2 -relief sunken
    frame $wi.$wf.b3 -borderwidth 2 -relief sunken
    frame $wi.$wf.b6 -borderwidth 2 -relief sunken
    frame $wi.$wf.b2.l
    frame $wi.$wf.b2.r
    frame $wi.$wf.b3.l
    frame $wi.$wf.b3.r
    frame $wi.$wf.b6.l
    frame $wi.$wf.b6.r

#Up-arrow-button (tree up):
    button $wi.$wf.b1.arrowup -bitmap @$SYNDOCHOME/syndoc_arrowup\
	       -command "p tree_upscreen"
#Down-arrow-button (tree down):
    button $wi.$wf.b1.arrowdown -bitmap @$SYNDOCHOME/syndoc_arrowdown\
	       -command "p tree_downscreen"

#Expand-button:
    button $wi.$wf.b2.l.expand -text Expand \
	       -command "p tree_expand" \
	       -width 8 
#Unexpand-button:
    button $wi.$wf.b2.l.unexpand -text "Unexpand" \
	       -command "p tree_unexpand" \
	       -width 8

#Left-button(Parent):
    button $wi.$wf.b3.l.left -text Parent -command "p tree_upper" -width 8 
#Child-button (same as expand):
    button $wi.$wf.b3.l.child -text Child -command "p tree_lower" -width 8

#Up-button(Previous):
    button $wi.$wf.b3.r.up -text Previous -command "p tree_up" -width 8
#Down-button (Next):
    button $wi.$wf.b3.r.down -text Next -command "p tree_down" -width 8

#Next-button (List Add):
    button $wi.$wf.b2.r.next -text "List Add" \
        -command "p tree_next" \
        -width 8 
#Delete-button (List Delete):
    button $wi.$wf.b2.r.delete -text "List Delete" \
        -command "p tree_del" \
        -width 8 

#Help-button:
    button $wi.$wf.b6.l.help -text Help \
	    -command "open_help $treehelp_file $wi" \
            -width 8
#Save-button:
    button $wi.$wf.b6.l.save -text Save -command "p tree_save" \
            -width 8
#Exit-button:
    button $wi.$wf.b6.r.exit -text Exit -command "p tree_save
						p tree_exit
					destroy $wi" \
            -width 8
#Cancel-button:
    button $wi.$wf.b6.r.cancel -text Cancel \
        -command "p tree_cancel
		  destroy $wi" -width 8



    frame $wi.$wf.tf -borderwidth 5
    text $wi.$wf.tf.t -yscrollcommand "$wi.$wf.tf.sy set" \
	              -xscrollcommand "$wi.$wf.tf.sx set" \
		      -wrap none -state normal 
    scrollbar $wi.$wf.tf.sy -command "$wi.$wf.tf.t yview"
    scrollbar $wi.$wf.tf.sx -orient horizontal -command "$wi.$wf.tf.t xview"           
#    pack $wi.$wf -fill x side bottom;         # kehys (iframe)  
    pack $wi.$wf.tf -fill both -anchor nw -side top -expand yes;                          # kehys tekstikentalle
    pack $wi.$wf.tf.sy -side right -fill y;      # scrollbar
    pack $wi.$wf.tf.sx -side bottom -fill x;     # scrollbar
    pack $wi.$wf.tf.t -fill both -side top -expand yes;    # teksti (message)

# Pack for buttons:
    pack $wi.$wf.b6 -side right -padx 4 -pady 4 -anchor s -expand yes
    pack $wi.$wf.b6.l -side left 
    pack $wi.$wf.b6.r -side right
    pack $wi.$wf.b2 -side right -padx 4 -pady 4 -anchor s -expand yes 
    pack $wi.$wf.b2.l -side left 
    pack $wi.$wf.b2.r -side right
    pack $wi.$wf.b3 -side right -padx 4 -pady 4 -anchor s -expand yes
    pack $wi.$wf.b3.l -side left 
    pack $wi.$wf.b3.r -side right

    pack $wi.$wf.b1 -side left -padx 4 -pady 4 -anchor s -expand yes

    pack $wi.$wf.b1.arrowup $wi.$wf.b1.arrowdown -side top -expand yes 

    pack $wi.$wf.b2.l.expand $wi.$wf.b2.l.unexpand -side top -expand yes 
    pack $wi.$wf.b2.r.next $wi.$wf.b2.r.delete \
	  -side top -expand yes 

    pack $wi.$wf.b3.l.left $wi.$wf.b3.l.child -side top -expand yes 
    pack $wi.$wf.b3.r.up $wi.$wf.b3.r.down -side top -expand yes 

    pack $wi.$wf.b6.l.save $wi.$wf.b6.l.help -side top -expand yes
    pack $wi.$wf.b6.r.cancel $wi.$wf.b6.r.exit -side top -expand yes 

# Inserts new text
    insert_text $treeChar 1 0 0 $rowNum

    focus $wi
    grab $wi

}; #end of procedure 

# ************************************************************************
# Name:       insert_text
# Input:      String (parse tree to be inserted (ascii))
#             Integer (expand button, 1 = enabled, 0 = disabled)
#             Integer ("list add" button, 1 = enabled, 0 = disabled)
#             Integer ("list delete" button, 1 = enabled, 0 = disabled)
#             Integer (current line)
# Output:     -
# Function:   Tree from ASCII to char. Inserts new tree to the text-
#             widget. Configure buttons to disabled or normal.
# ************************************************************************
proc insert_text {tree but1 but5 but6 rows} {
   global wi wf;	#Name of the window and the frame
   global docText;	# Text in the text widget (ascii)
   global rowNum;	# Number of the current row in the text 
			# of the text widget

    set docText $tree
    set rowNum $rows
   $wi.$wf.tf.t configure -state normal
# Clear old text 
   $wi.$wf.tf.t delete 1.0 end

# Insert new text
   # Muutetaan ascii-merkit tekstiksi
   foreach i $tree {
       $wi.$wf.tf.t insert end [format "%c" $i] 
   }
# Shows the current row in the text widget
   $wi.$wf.tf.t see $rowNum.0
   $wi.$wf.tf.t configure -state disabled 


   if {$but1 == 0} then {
       $wi.$wf.b2.l.expand configure -state disabled
   } else {
       $wi.$wf.b2.l.expand configure -state normal
   }
   if {$but5 == 0} then {
       $wi.$wf.b2.r.next configure -state disabled
   } else {
       $wi.$wf.b2.r.next configure -state normal
   }
   if {$but6 == 0} then {
       $wi.$wf.b2.r.delete configure -state disabled
   } else {
       $wi.$wf.b2.r.delete configure -state normal
   }
      
    
}

# ************************************************************************
# Name:       open_texteditor
# Input:      String (title of the editor window)
#	      String (text content of the node)
# Output:     -
# Function:   Moves ASCII from prolog to chars. Calls open_editor-proc.
#       
# ************************************************************************
proc open_texteditor { header oldtext } {
    global e;	# Name of the window

    open_editor $header
   # Tyhjennetaan vanha teksti
   $e.ef.t delete 1.0 end

# ascii to text
   foreach i $oldtext {
       $e.ef.t insert end [format "%c" $i] 
   }
}


# ************************************************************************
# Name:       open_editor
# Input:      String (title of the editor window)
# Output:     -
# Function:   Creates toplevel-window with text widget. 
# ************************************************************************
proc open_editor { header } {
    global e wi;	# Names of the windows
    toplevel $e
    dpos $e

# if nonterminal is in  [], remove them
    wm title $e [remove_brackets $header]

    frame $e.ef
    text $e.ef.t -yscrollcommand "$e.ef.s1 set" -setgrid true \
	      -xscrollcommand "$e.ef.s2 set"
    scrollbar $e.ef.s1 -command "$e.ef.t yview"           
    scrollbar $e.ef.s2 -orient horizontal -command "$e.ef.t xview"           


    pack $e.ef
    pack $e.ef.s1 -side right -fill y
    pack $e.ef.s2 -side bottom -fill x
    pack $e.ef.t -expand yes -fill both

    button $e.clear -text Clear \
       -command "$e.ef.t delete 1.0 end" \
       -width 8

    button $e.done -text Done \
       -command {set newtext [.editor.ef.t get 1.0 end]
		 set nt [remove_newline $newtext]
		 grab release .editor
		 grab $wi
		 p done('$nt'); destroy .editor} -width 8 

    button $e.cancel -text Cancel \
       -command "p editor_cancel; grab $wi; destroy $e" \
       -width 8
    pack $e.clear $e.done $e.cancel -side left -expand yes 
  
    focus $e.ef.t
    grab $e
}

# **********************************************************************
# Name:        remove_newline
# Input:       String (text to be editet)
# Output:      String (edited text)
# Function:    Removes last character from string.
# **********************************************************************
proc remove_newline { newt } {
    set withoutNl ""
    set withoutNl [string trim $newt]
    return $withoutNl
}

# **********************************************************************
# Name:        remove_brackets
# Input:       String (text to be editet)
# Output:      String (edited text)
# Function:    Removes brackets and formats from ascii to char.
# **********************************************************************
proc remove_brackets { newt } {
    set withoutb ""

    foreach i [string trim $newt \[\]] {
        append withoutb [format "%c" $i] 
    }
    return $withoutb
}

# **********************************************************************
# Name:        open_rulelist 
# Input:       String (title of the selection window)
# 	       String (list of rules)
# Output:      -
# Function:    Opens top-level window with list of rules. Returns
#              selected rule to calling prolog-program.
# **********************************************************************
proc open_rulelist { root rulesC } {
    global rulewin selectedRule
    set empty "1"

    toplevel $rulewin
    dpos $rulewin

# Title is in ascii-form, because tcl can't handle if chars [ or ] in name.
    foreach i $root {
	append root1 [format "%c" $i]
    }
    wm title $rulewin $root1 

    button $rulewin.close -text "Close" -command "set selectedRule $empty"
    pack $rulewin.close -side bottom -fill x

    scrollbar $rulewin.scroll -command "$rulewin.list yview"
    pack $rulewin.scroll -side right -fill y
    listbox $rulewin.list -yscroll "$rulewin.scroll set" -relief raised \
                                  -setgrid yes -width 0
    pack $rulewin.list -side left -fill both -expand yes
    #wm minsize $rulewin 5 5

    bind $rulewin.list <Double-Button-1> {set selectedRule [selection get]}

# Rule list from prolog is in ascii-form, because there can be newline in
# rulelist (tcl can't handle it)
    foreach i $rulesC {
	foreach j $i { 
	    append rules [format "%c" $j]
	}
        $rulewin.list insert end $rules
	set rules ""
    }

   set oldFocus [focus]
   grab $rulewin  
   focus $rulewin

   tkwait variable selectedRule

   select_rule $selectedRule $oldFocus
}

# **********************************************************************
# Name:        select_rule
# Input:       String String
# Output:      String (selected rule, returned to prolog by calling
#	       prolog-predicate selected)
# Function:    Destroys window with list of rules. Returns selected rule
#              to prolog.
# **********************************************************************
proc select_rule { selectedR oldF } {

    global rulewin

    grab release $rulewin
    destroy $rulewin
    focus $oldF
    grab $oldF
    p selected('$selectedR')

}




